/*******************************************************************************
 * @author Mohammad Ibadur Rehman (m.rehman@samsung.com)
 * @file This file implements Calendar Planner App function  
 * Copyright 2015 by Samsung Electronics, Inc.,
 * @license This software is the confidential and proprietary information of Samsung
 * Electronics, Inc. ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Samsung.
 ******************************************************************************/
var arrayfordisplay = [];
function mergeSort(arr) {
    if (arr.length < 2)
        return arr;
 
    var mid = parseInt(arr.length / 2);
    var l   = arr.slice(0, mid);
    var r  = arr.slice(mid, arr.length);
 
    return merge(mergeSort(l), mergeSort(r));
}
 
function merge(l, r) {
    var result = [];
 
    while (l.length && r.length) {
        if (l[0].start <= r[0].start) {
            result.push(l.shift());
        } else {
            result.push(r.shift());
        }
    }
 
    while (l.length)
        result.push(l.shift());
 
    while (r.length)
        result.push(r.shift());
 
    return result;
}

function bubbleSort(items) {
    var length = items.length;
    var swapcount = 1;
    while (swapcount)
    {
        swapcount = 0;
        for (var i = 0; i < length - 1; i++)
        {
            var a = items[i];
            var b = items[i + 1];
            var atime = a.timeStamp;
            var adate = new Date(atime);
            var anum = new Date(adate);
            var btime = b.timeStamp;
            var bdate = new Date(btime);
            var bnum = new Date(bdate);
            if (anum > bnum)
            {
                var temp = items[i];
                items[i] = items[i + 1];
                items[i + 1] = temp;

                swapcount = swapcount + 1;
            }
        }
    }
    return items;
}

var promise;
   
var Debug_tag = 'CAL : ';
var app = angular.module("SmartThings", ['angularRangeSlider', 'ngSanitize', 'ngTouch', 'ui.router', 'ngMaterial', 'datePicker','swipe']);
app.controller("calendarAppController", ["$scope", "$rootScope", "imageCache", "imageRequest", "SHPService", "$timeout", "$sce", "$interval", "$state", "REQUEST", calendarAppController]);

/**
 * @private
 * @function
 * @name calendarAppController
 *
 * @description This method is a constructor function for Air Purifier module and is used to define
 * properties and methods to module's scope (angular scope).
 */
function calendarAppController($scope, $rootScope, imageCache, imageRequest, SHPService, $timeout, $sce, $interval, $state, REQUEST, STCONST) {

    CALCONST = {
        GET_DEVICES: "getDevices",
        SCREEN_HOME : 1,
        SCREEN_ADD : 2,
        SCREEN_INVITEE : 3,
        SCREEN_ALERT : 4,
        SCREEN_REPEAT : 5,
        SCREEN_EVENT : 6,
        COLOR_v : '#c04d9c',
        COLOR_b : '#30a3d5',
        COLOR_g : '#2eb347',
        COLOR_y : '#f0b52c',
        COLOR_o : '#f77215',
        COLOR_r : '#eb2424',
        COLOR_i : '#4b0082',
        COLOR_gr : '#626769',
        GET_CALS : 100,
        CAL_REQUEST : "getCalendarsForFamily",
        EVENT_REQUEST : "getCalendarEventsForFamily",
        PROFILE_REQUEST : "getProfile",
        CREATE_REQUEST : "createEvent",
        DELETE_REQUEST : "deleteEvent",
        UPDATE_REQUEST : "updateEvent",
        RESPONSE_INVITATION : "responseInvitation",
        TOP_POINT : 720,
        BOTTOM_POINT : 1680,
        OOBE_BOTTOM_POINT : 1350,
        MID_POINT_TOP_DIR : 1600,
        MID_POINT_BOTTOM_DIR : 200,
        WEEK_VIEW_HEIGHT : 428,
        CALENDAR_VIEW_HEIGHT_5ROW : 1328,
        CALENDAR_VIEW_HEIGHT_6ROW : 1460,
        HOUR_VIEW_HEIGHT : 940,
        OOBE_CALENDAR_VIEW_HEIGHT_6ROW : 1250,
        OOBE_VIEW_HEIGHT : 1150,
        HOUR_VIEW_HEIGHT_6ROW : 760,
        HOUR_VIEW_HEIGHT_FULL : 1840,
        SHOW_TOAST : "showToast",
        OPEN_CALENDAR_SETTINGS : "openCalendarSettings"
    };

    colorList = [CALCONST.COLOR_r, CALCONST.COLOR_o, CALCONST.COLOR_y, CALCONST.COLOR_g, CALCONST.COLOR_b, CALCONST.COLOR_i, CALCONST.COLOR_v, CALCONST.COLOR_gr];

    eventBallList = ['red','orange','yellow','green','blue','indigo','violet','grey'];
    reloadOnNotification = false;
    var mendrianTimer ;
    var scrollFinishHour = false;
    var scrollFinishMinute = false;
    var baseFont;

    $scope.days = ["WEBMOB_fhub2_common_abbr_monday", "WEBMOB_fhub2_common_abbr_tuesday", "WEBMOB_fhub2_common_abbr_wednesday", "WEBMOB_fhub2_common_abbr_thursday", "WEBMOB_fhub2_common_abbr_friday", "WEBMOB_fhub2_common_abbr_saturday", "WEBMOB_fhub2_common_abbr_sunday"];
    monthName = ["WEBMOB_fhub2_common_january","WEBMOB_fhub2_common_february","WEBMOB_fhub2_common_march","WEBMOB_fhub2_common_april","WEBMOB_fhub2_common_may","WEBMOB_fhub2_common_june","WEBMOB_fhub2_common_july","WEBMOB_fhub2_common_august","WEBMOB_fhub2_common_september","WEBMOB_fhub2_common_october","WEBMOB_fhub2_common_november","WEBMOB_fhub2_common_december"];

    langArr = Object.keys(LANGUAGE_FILE.language);
    if (langArr.indexOf(LANG.toUpperCase()) !== -1) {
        $scope.translation = LANGUAGE_FILE["language"][LANG.toUpperCase()];
    } else if (langArr.indexOf(LANG.toUpperCase().substring(0, 2)) !== -1) {
        $scope.translation = LANGUAGE_FILE["language"][LANG.toUpperCase().substring(0, 2)];
    } else {
        $scope.translation = LANGUAGE_FILE["language"]["DEFAULT"];
    }
    if (LANG.toUpperCase().indexOf('EN') !== -1) {
        $scope.selectedLangEnglish = true;
    }

    // to check if input ot textara is focused
    function isTextInput(node) {
        return ['INPUT', 'TEXTAREA'].indexOf(node.nodeName) !== -1;
    }

    // remove focues on touch anywhere on the screen
    document.addEventListener('touchstart', function (e) {
        if (!isTextInput(e.target) && isTextInput(document.activeElement)) {
            document.activeElement.blur();
        }
    }, false);
    
    
    $scope.prevY = 0;
    $scope.isAppV2 = false;
    $scope.isLocal = false;
    $scope.pageLoaded = false;
    $scope.isAgendaView = false;
    $scope.isWeekView = false;
    $scope.firstHour = 0;
    $scope.lastHour = 24;
    $scope.currentDate = new Date();
    //$scope.prevSelectedDate;
    $scope.loadingBar = true;
    $scope.isTransparent = true;
    $scope.currentWeek = 0;
    $scope.isTodayBtnEnabled = false;
    $scope.isAllDayEvent = true;
    $scope.test = "test";
    $scope.isCalender = -1;
    $scope.isTimePicker = -1;
    $scope.showProfilePicker = false;
    $scope.daySelector = false;
    $scope.startDate = new Date();
    $scope.endDate = new Date();
    $scope.startsHour = $scope.startDate.getHours();
    $scope.endsHour = $scope.startDate.getHours() + 1;
    $scope.startsMinute = 0;
    $scope.endMinute = 0;
    $scope.isRepeatPopupOpen = false;
    $scope.isAlertPopupOpen = false;
    $scope.showOOBEScreen = false;
    $scope.visibleAddEventGridLayout = false;
    $scope.defaultCalendarId;
    $scope.currentScreen;
    $scope.profileObj;
    $scope.hourEventObj;
    $scope.checkResponse = [];
    $scope.currentHour;
    $scope.requestedDate;
    $scope.hourSelected;
    $scope.showDeclineToastOnDeleteSuccess = false;
    $scope.checkIfEventIsAlreadyAllDayEvent = false;
    $scope.endDateBeforeEditEvent = new Date();
    $scope.checkForDeletedEventId;
    $scope.repeatMin=1;
    $scope.repeatMax=365;
    $scope.isReceiveProfileNotification = false;

    $scope.hasProcessed = false;

    $scope.createEventData = {
        selectedUser : '',
        attendeeList : [],
        userId : '',
        summay : '',
        starts : undefined,
        ends :  undefined,
        allDay : false,

        location : '',
        desc : '',
        invitee : '',
        alert : '',
        recur : [],
        arrayDynamicallyAddedEmail : []
    };

    $scope.dialogPopup = {
        show: false,
        dialogPopupTitle: '',
        dialogPopupText: '',
        button1Title: '',
        button1Handler: function () {
            toggleBottomPopUpClass(false);
            $scope.dialogPopup.show = false;
        },
        button2Enabled: false,
        button2Title: '',
        button2Handler: function () {
        }
    };

    $scope.alertOption = [
        $scope.translation.WEBMOB_fhub2_calendar_alert_none,
        $scope.translation.WEBMOB_fhub2_calendar_alert_at_start_time,
        ($scope.translation.WEBMOB_fhub2_calendar_alert_minute_other).replace(/(\#.*?\#)/gi, "10"),
        $scope.translation.WEBMOB_fhub2_calendar_alert_day,
        $scope.translation.WEBMOB_fhub2_calendar_alert_week,
        $scope.translation.WEBMOB_fhub2_calendar_alert_custom
    ];
    $scope.repeateOptionLable = [
        $scope.translation.WEBMOB_fhub2_calendar_alert_none,
        $scope.translation.WEBMOB_fhub2_calendar_repeat_daily,
        $scope.translation.WEBMOB_fhub2_calendar_repeat_weekly,
        $scope.translation.WEBMOB_fhub2_calendar_repeat_monthly,
        $scope.translation.WEBMOB_fhub2_calendar_repeat_yearly,
        $scope.translation.WEBMOB_fhub2_calendar_alert_custom
    ];
    $scope.repeatUnitData = [
        $scope.translation.WEBMOB_fhub2_calendar_day,
        $scope.translation.WEBMOB_fhub2_calendar_alert_custom_weeks,
        $scope.translation.WEBMOB_fhub2_calendar_alert_custom_months,
        $scope.translation.WEBMOB_fhub2_common_year
    ];
    $scope.alertUnitData = [
        $scope.translation.WEBMOB_fhub2_calendar_alert_custom_mins,
        $scope.translation.WEBMOB_fhub2_calendar_alert_custom_hours,
        $scope.translation.WEBMOB_fhub2_calendar_alert_custom_weeks
    ];
    $scope.repeatDailyOptionLable = [
        $scope.translation.WEBMOB_fhub2_common_check_all,
        $scope.translation.WEBMOB_fhub2_common_Sunday,
        $scope.translation.WEBMOB_fhub2_common_monday,
        $scope.translation.WEBMOB_fhub2_common_Tuesday,
        $scope.translation.WEBMOB_fhub2_common_Wednesday,
        $scope.translation.WEBMOB_fhub2_common_Thursday,
        $scope.translation.WEBMOB_fhub2_common_Friday,
        $scope.translation.WEBMOB_fhub2_common_Saturday
    ];
    $scope.repeatMonthlyOptionLable = [
        $scope.translation.WEBMOB_fhub2_calendar_repeat_monthly_on_date,
        $scope.translation.WEBMOB_fhub2_calendar_repeat_monthly_on_month
    ];

    $scope.dayName = [$scope.translation.WEBMOB_fhub2_common_abbr_sunday, $scope.translation.WEBMOB_fhub2_common_abbr_monday,$scope.translation.WEBMOB_fhub2_common_abbr_tuesday,$scope.translation.WEBMOB_fhub2_common_abbr_wednesday,$scope.translation.WEBMOB_fhub2_common_abbr_thursday, $scope.translation.WEBMOB_fhub2_common_abbr_friday, $scope.translation.WEBMOB_fhub2_common_abbr_saturday];
    $scope.monthName = [$scope.translation.WEBMOB_fhub2_common_abbr_january, $scope.translation.WEBMOB_fhub2_common_abbr_february,$scope.translation.WEBMOB_fhub2_common_abbr_march,
        $scope.translation.WEBMOB_fhub2_common_abbr_april,$scope.translation.WEBMOB_fhub2_common_abbr_may,$scope.translation.WEBMOB_fhub2_common_abbr_june,$scope.translation.WEBMOB_fhub2_common_abbr_july,
        $scope.translation.WEBMOB_fhub2_common_abbr_august, $scope.translation.WEBMOB_fhub2_common_abbr_september,$scope.translation.WEBMOB_fhub2_common_abbr_october,$scope.translation.WEBMOB_fhub2_common_abbr_november,
        $scope.translation.WEBMOB_fhub2_common_abbr_december];
    $scope.isEdit = false;
    $scope.repeatDataStructure = {
        selected: 0,
        custom: {
            unitIndex: 0,
            value: 1
        },
        daily: {
            AL: false,
            SU: false,
            MO: false,
            TU: false,
            WE: false,
            TH: false,
            FR: false,
            SA: false
        },
        monthly: {
            onDate: false,
            onMonth: false
        }
    };
    $scope.alertDataStructure = {
        selected : 0,
        custom : {
            unitIndex : 0,
            value : 1
        }
    };
    $scope.dropDownSelection = [];
    $scope.eventActionMessage = '';

    $scope.parseStaticData = parseStaticData;
    $scope.parseDeviceData = parseDeviceData;
    $scope.updateDeviceData = updateDeviceData;
    $scope.startApp = startApp;
    $scope.closeApp = closeApp;
    $scope.requestFailHandler = requestFailHandler;
    $scope.requestResponseHandler =  requestResponseHandler;
    $scope.onBackPressed = onBackPressed;
    $scope.closeLoadingBar = closeLoadingBar;
    $scope.dismissLoadingBar = dismissLoadingBar;
    $scope.showConnectionFailurePopup = showConnectionFailurePopup;
    $scope.changeHourOrAgendaView = changeHourOrAgendaView;
    $scope.getEventTimeString = getEventTimeString;
    $scope.getEventDetailTimeString = getEventDetailTimeString;
    $scope.getProfileData = getProfileData;
    $scope.getProfileImage_calendarId = getProfileImage_calendarId;
    $scope.getProfileImage = getProfileImage;
    $scope.getEventBallImg = getEventBallImg;
    $scope.updateCalValue = updateCalValue;
    $scope.onTouchStart = onTouchStart;
    $scope.onTouchEnd = onTouchEnd;
    $scope.onTouchMove = onTouchMove;
    $scope.selectToday = selectToday;
    $scope.isTodayButtonEnable = isTodayButtonEnable;
    $scope.sendBroadCast = sendBroadCast;
    $scope.onPressed = onPressed;
    $scope.disableDate = disableDate;
    $scope.getTimeStamp = getTimeStamp;
    $scope.setSelectedAlert = setSelectedAlert;
    $scope.setSelectedRepeat = setSelectedRepeat;
    $scope.arrayAddFunction = arrayAddFunction;
    $scope.deleteIndex = deleteIndex;
    $scope.getEventDateString = getEventDateString;
    $scope.getCalendarEventData = getCalendarEventData;
    $scope.closePicker = closePicker;
    $scope.openPickerPopup = openPickerPopup;
    $scope.openTimePicker = openTimePicker;
    $scope.closeTimePopup = closeTimePopup;
    $scope.cancelAddEvent = cancelAddEvent;
    $scope.sendAddEventRequest = sendAddEventRequest;
    $scope.showDeletePopup = showDeletePopup;
    $scope.updateEvent = updateEvent;
    $scope.openEditPage = openEditPage;
    $scope.callEventEditDeletePage = callEventEditDeletePage;
    $scope.createEventOnDayView = createEventOnDayView;
    $scope.openCreateEventPage = openCreateEventPage;
    $scope.cancelRepeatPopup = cancelRepeatPopup;
    $scope.getDailyCheckedStatus = getDailyCheckedStatus;
    $scope.setDailyCheckedStatus = setDailyCheckedStatus;
    $scope.getMonthlyCheckedStatus = getMonthlyCheckedStatus;
    $scope.setMonthlyCheckedStatus = setMonthlyCheckedStatus;
    $scope.changeRepeatDurUnit = changeRepeatDurUnit;
    $scope.increaseRepeatValue = increaseRepeatValue;
    $scope.decreaseRepeatValue = decreaseRepeatValue;
    $scope.changeAlertDurUnit = changeAlertDurUnit;
    $scope.increaseAlertValue = increaseAlertValue;
    $scope.decreaseAlertValue = decreaseAlertValue;
    $scope.saveAlertCustom = saveAlertCustom;
    $scope.saveRepeatPopup = saveRepeatPopup;
    $scope.addInternalInvitee = addInternalInvitee;
    $scope.isLargeEvent = isLargeEvent;
    $scope.getTimeToEvent = getTimeToEvent;
    $scope.adjustCommonInterval = adjustCommonInterval;
    $scope.profileSelected = profileSelected;
    $scope.getAgendaTimeString = getAgendaTimeString;
    $scope.getHeaderMonthText = getHeaderMonthText;
    $scope.getInvidedString = getInvidedString;
    $scope.getAcceptedInviteeString = getAcceptedInviteeString;
    $scope.getMayBeInviteeString = getMayBeInviteeString;
    $scope.getDeclineInviteeString = getDeclineInviteeString;
    $scope.getNoResponseInviteeString = getNoResponseInviteeString;
    $scope.getInvideeString = getInvideeString;
    $scope.getAttendeeName = getAttendeeName;
    $scope.getAttendeeAvatar = getAttendeeAvatar;
    $scope.onTimePickerOpen = onTimePickerOpen;
    $scope.setTime = setTime;
    $scope.parseAlertData = parseAlertData;
    $scope.getAlertString = getAlertString;
    $scope.parseRepeatData = parseRepeatData;
    $scope.getByDayString = getByDayString;
    $scope.getCustomRepeatString = getCustomRepeatString;
    $scope.getRepeatStringOnEventDetailPage = getRepeatStringOnEventDetailPage;
    $scope.getRepeatStringOnAddEventView = getRepeatStringOnAddEventView;
    //$scope.onListTouchUp = onListTouchUp;
    $scope.showToast = showToast;
    $scope.getCurrentSelectedDate = getCurrentSelectedDate;
    $scope.shiftUp = shiftUp;
    $scope.shiftDown = shiftDown;
    $scope.removeFocus = removeFocus;
    $scope.openDetailPage = openDetailPage;
    $scope.getLocalDateString = getLocalDateString;
    $scope.getLocalTimeString = getLocalTimeString;
    $scope.closeProfilePopup = closeProfilePopup;
    $scope.showProfilePopup = showProfilePopup;
    $scope.onBlurPicker = onBlurPicker;
    $scope.getTodayDateIcon = getTodayDateIcon;
    $scope.onTouchStartWeekView = onTouchStartWeekView;
    $scope.onTouchMoveWeekView = onTouchMoveWeekView;
    $scope.showAddEventGrid = showAddEventGrid;
    $scope.showAddEventGridLayout = showAddEventGridLayout;
    $scope.filterEvent = filterEvent;
    $scope.calculateProfileNameWidth = calculateProfileNameWidth;
    $scope.calculateEventTableHeight = calculateEventTableHeight;
    $scope.openCalendarSettings = openCalendarSettings;
    $scope.sendInvitationRequest = sendInvitationRequest;
    $scope.isResponseBarEnabled = isResponseBarEnabled;
    $scope.showDeleteForInviteesAfterActionTaken = showDeleteForInviteesAfterActionTaken;
    $scope.showMayBeForInviteeAfterActionTaken = showMayBeForInviteeAfterActionTaken;
    $scope.showInviteesList = showInviteesList;
    $scope.dontShowOOBEAgain = dontShowOOBEAgain;
    $scope.hideAddEventGrid = hideAddEventGrid;
    $scope.getImage = getImage;
    $scope.imageURL_Request = imageURL_Request;
    $scope.keyupHandler = keyupHandler;
    $scope.closePopups=closePopups;
    //$scope.up_value=up_value;
    //$scope.down_value=down_value;

    $scope.mendrianUnits = [$scope.translation.WEBMOB_fhub2_common_am_capital, $scope.translation.WEBMOB_fhub2_common_pm_capital];
    $scope.selectedMendrian = $scope.translation.WEBMOB_fhub2_common_pm_capital;
    $scope.selectedHour = '00';
    $scope.selectedMinute = '00';

    $scope.hourList = ['01','02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12'];
    $scope.minuteList = [];
    $scope.profileObjForEventCreation;
    $scope.NoOfProfileFiltered = 0;
    $scope.showAcceptedInvitees = false;
    $scope.showMayBeInvitees = false;
    $scope.showDeclinedInvitees = false;
    $scope.showNoResponseInvitees = false;
    $scope.showExternalInviteesDivider = false;
    $scope.alertValList=[];
    $scope.selectedAlertVal=1;
    $scope.selectedAlertUnit=0;
    $scope.Alertlimit=99;
    $scope.repeatValList=[];
    $scope.selectedRepeatVal=1;
    $scope.selectedRepeatUnit=0;
    $scope.Repeatlimit=365;
    
    function keyupHandler(event) {
        if (event.target.id === "enterTitle") {
            $scope.createEventData.summay = event.target.value;
        } else if (event.target.id === "location") {
            $scope.createEventData.location = event.target.value;
        } else if (event.target.id === "note") {
            $scope.createEventData.desc = event.target.value;
        }
    }
    function getLocalStorageData(item) {
        if (typeof (Storage) !== "undefined") {
            return localStorage.getItem(item);
        } else {
            // Sorry! No Web Storage support..
        }
    }
    function setLocalStorageData(item, value) {
        if (typeof (Storage) !== "undefined") {
            localStorage.setItem(item, value);
        } else {
            // Sorry! No Web Storage support..
        }
    }
    function dontShowOOBEAgain(){
        $scope.showOOBEScreen = false;
        var calendarContainer = document.getElementById("calendarViewContainer");
        calendarContainer.style.height="18rem";
        setLocalStorageData('firstTimeLaunch', false);
    }
    function isResponseBarEnabled(event) {
        if(event.isOrganizer || event.isCancelled || event.attendees === null) {
            return false;
        }
        
        var attandeeList = event.attendees;
        //console.log(" isResponseBarEnabled ", attandeeList);
        var tempObj;
        for(var i = 0; i < attandeeList.length; i++) {
            tempObj = attandeeList[i];
            if(angular.isDefined(tempObj.email) && angular.isDefined($scope.profileObj.email) && tempObj.email.toUpperCase() === $scope.profileObj.email.toUpperCase() && (tempObj.responseStatus === 'needsAction' || tempObj.responseStatus === 'tentative')) {
                return true;
            }
        }

        return false;
    }
    function showDeleteForInviteesAfterActionTaken(event) {
        if(event.isOrganizer || event.isCancelled) {
            return true;
        }

        var attandeeList = event.attendees;
        //console.log(" isResponseBarEnabled ", attandeeList);
        var tempObj;
        for(var i = 0; i < attandeeList.length; i++) {
            tempObj = attandeeList[i];
            if(angular.isDefined(tempObj.email) &&
                angular.isDefined($scope.profileObj.email)
                && tempObj.email.toUpperCase() === $scope.profileObj.email.toUpperCase()
                && (tempObj.responseStatus !== 'needsAction' && tempObj.responseStatus !== 'tentative')) {
                return true;
            }
        }

        return false;
    }

    function showInviteesList(attandeeList) {
        for(var i = 0; i < attandeeList.length; i++) {
            tempObj = attandeeList[i];
            if(angular.isUndefined(tempObj.responseStatus) || tempObj.responseStatus === 'needsAction') {
                $scope.showNoResponseInvitees = true;
            }else if(tempObj.responseStatus === 'accepted') {
                $scope.showAcceptedInvitees = true;
            }else if(tempObj.responseStatus === 'tentative') {
                $scope.showMayBeInvitees = true;
            }else if( tempObj.responseStatus === 'declined') {
                $scope.showDeclinedInvitees = true;
            }
        }
    }
    
    function showMayBeForInviteeAfterActionTaken(event) {
        var attandeeList = event.attendees;
        //console.log(" isResponseBarEnabled ", attandeeList);
        var tempObj;
        for (var i = 0; i < attandeeList.length; i++) {
            tempObj = attandeeList[i];
            if (angular.isDefined(tempObj.email) &&
                    angular.isDefined($scope.profileObj.email)
                    && tempObj.email.toUpperCase() === $scope.profileObj.email.toUpperCase()
                    && tempObj.responseStatus === 'tentative') {
                return true;
            }
        }

        return false;
    }
    function calculateProfileNameWidth(width) {
        var profileSelectedCount = 0;
        for (var i = 0; i < $scope.profileData.length; i++) {
            if ($scope.profileData[i].isFilterSelected) {
                profileSelectedCount++;
            }
        }
        $scope.NoOfProfileFiltered = profileSelectedCount;
        return (width / ($scope.profileData.length - profileSelectedCount));
    }

    $scope.getProfileFilterOnCount = getProfileFilterOnCount;
    $scope.getProfileActiveIndex = getProfileActiveIndex;
    function getProfileFilterOnCount() {
        var filterOnCount = 0;
        $scope.profileData.forEach(function (profile) {
            if (profile.isFilterSelected)
                filterOnCount++;
        });
        return filterOnCount;
    }

    function getProfileActiveIndex(idx) {
        var filterOnIndex = 0;
        if (angular.isDefined($scope.profileData)) {
            for (var i = 0; i < $scope.profileData.length; i++) {
                if (idx < i)
                    break;
                if (!$scope.profileData[i].isFilterSelected)
                    filterOnIndex++;
            }
        }
        return filterOnIndex;
    }

    function calculateEventTableHeight() {
        var baseFontSize = parseFloat(document.getElementById("htmlTag").style.fontSize.replace('px', ''));
        var viewContainer = $('#hourViewContainer');
        var height = viewContainer.height();
        //apply monthView size refer to p10 of GUI guide v1.05
        var subHeight = 80;     //pointerBtn height
        if ($scope.getProfileFilterOnCount() !== $scope.profileData.length ||
            ($scope.getProfileFilterOnCount() === $scope.profileData.length && $scope.dropDownSelection.length === 0)) {
            subHeight += 176;   //ProfileName height
        }
        if ($scope.getProfileFilterOnCount() !== $scope.profileData.length) {
            subHeight += 64;   //allDay height
        }

        subHeight = (subHeight / 80) * baseFontSize;  //convert px to rem and calculate px according to basefont.
        height -= subHeight;
        return height;
    }

    function filterEvent(profileIndex) {
        $scope.profileData[profileIndex].isFilterSelected = !$scope.profileData[profileIndex].isFilterSelected;
        $rootScope.$broadcast('update', {'text' : $scope.currentDate});
        $rootScope.$broadcast('updateHourView', {'text' : $scope.currentDate});
        $rootScope.$broadcast('updateAgendaView', {'text' : $scope.currentDate});
    }

    function showAddEventGrid(profileIndex, hour){
        $scope.profileObjForEventCreation = profileIndex;
        $scope.hourSelected = hour;
        $scope.visibleAddEventGridLayout = true;
    }

    function hideAddEventGrid(){
        if($scope.visibleAddEventGridLayout){
            $scope.visibleAddEventGridLayout = false;
        }
    }
    function showAddEventGridLayout(profileIndex, hourIndex){
        var setVisibility = true;
        var tempProfileObj = $scope.profileData[profileIndex];
        //$scope.profileObj = $scope.profileData[profileIndex];
        //$scope.hourEventObj = $scope.profileObj.hourEvent[hourIndex];
        if (angular.isDefined(tempProfileObj) && (tempProfileObj.hourEvent.length > 0)) {
            var tempStartDate = new Date($scope.currentDate.getFullYear(), $scope.currentDate.getMonth(), $scope.currentDate.getDate());
            tempStartDate.setHours(hourIndex);
            tempStartDate.setMinutes(00);

            var tempEndDate = new Date(tempStartDate.getFullYear(), tempStartDate.getMonth(), tempStartDate.getDate());
            tempEndDate.setHours(tempStartDate.getHours() + 1);
            tempEndDate.setMinutes(tempStartDate.getMinutes());

//            for (var i = 0;i < tempProfileObj.hourEvent.length; i++){
//                if(( tempStartDate.toISOString() > tempProfileObj.hourEvent[i].event.event.starts && tempStartDate.toISOString() < tempProfileObj.hourEvent[i].event.event.ends) ||
//                        (tempEndDate.toISOString() > tempProfileObj.hourEvent[i].event.event.starts && tempEndDate.toISOString() < tempProfileObj.hourEvent[i].event.event.ends)){
//                    setVisibility = false;
//                }
//            }
        }
        return setVisibility &&($scope.hourSelected === hourIndex && $scope.visibleAddEventGridLayout && $scope.profileObjForEventCreation === profileIndex);

    }

    function openCalendarSettings(){
        SHPService.sendNativeCommand(CALCONST.OPEN_CALENDAR_SETTINGS,"",CALCONST.OPEN_CALENDAR_SETTINGS, false);
    }
    function onTouchStartWeekView(event) {
        var touch = event.touches[0];
        var x = touch.pageX;
        var y = touch.pageY;
        $scope.prevY = y;
    }

    function onTouchMoveWeekView(event) {
        var cHeight = 0, base = 0;
        var touch = event.changedTouches[0];
        var x = touch.pageX;
        var y = touch.pageY;
        cHeight = window.screen.availHeight;
        if (cHeight === 0 || cHeight === undefined) {
            cHeight = document.body.clientHeight;
        }

        var viewContainer;
        if ($scope.isAgendaView) {
            viewContainer = document.getElementById("agendaListContainer");
        } else {
            viewContainer = document.getElementById("hourViewContainer");
        }
        var calendarContainer = document.getElementById("calendarViewContainer");

        var top = viewContainer.getBoundingClientRect().top;
        top = top * (2560 / cHeight);
        var offset = $scope.prevY - y;
        if (offset < -50) { // Direction down
            $scope.isWeekView = false;
            moveToBottom(viewContainer, calendarContainer, top, cHeight);
            return;
        }
    }

    function getTodayDateIcon(isDisabled) {
        var path = '';
        var date = (new Date()).getDate();
        if(isDisabled) {
            path = "CAL/assets/img/date_icons/calendar_ic_day" + date + "_dim.png";
        } else {
            path = "CAL/assets/img/date_icons/calendar_ic_day" + date + "_nor.png";
        }
        return path;
    }

    function closeProfilePopup() {
        $scope.showProfilePicker = false;
    }
    function showProfilePopup() {
        removeFocus();
        $scope.showProfilePicker = true;
    }

    function getLocalTimeString(date) {
        var h = date.getHours();
        var m = prepandZero(date.getMinutes());

        var mer = h < 12 ? $scope.translation.WEBMOB_fhub2_common_am_capital : $scope.translation.WEBMOB_fhub2_common_pm_capital;
        if(h === 0) {
            h = 12;
        } else if(h > 12) {
            h = h-12;
        }
        h = prepandZero(h);

        return h + ':' + m + mer;
    }

    function getLocalDateString(date) {
        var string = '';
        switch (date.getDay()) {
            case 0:
                string = $scope.translation.WEBMOB_fhub2_common_abbr_sunday;
                break;

            case 1:
                string = $scope.translation.WEBMOB_fhub2_common_abbr_monday;
                break;

            case 2:
                string = $scope.translation.WEBMOB_fhub2_common_abbr_tuesday;
                break;

            case 3:
                string = $scope.translation.WEBMOB_fhub2_common_abbr_wednesday;
                break;

            case 4:
                string = $scope.translation.WEBMOB_fhub2_common_abbr_thursday;
                break;

            case 5:
                string = $scope.translation.WEBMOB_fhub2_common_abbr_friday;
                break;

            case 6:
                string = $scope.translation.WEBMOB_fhub2_common_abbr_saturday;
                break;
        }

        string = string + " " + prepandZero(date.getMonth() + 1) + '/' + prepandZero(date.getDate()) + '/' + date.getFullYear();
        return string;
    }

    function prepandZero(num) {
        return num < 10 ? '0' + num : num;
    }
    function openDetailPage(event) {
        var calArr, flag = true, profileIndex;
        for(var i = 0; i < $scope.profileData.length && flag; i++) {
            calArr = $scope.profileData[i].calendarId;
            for(var j = 0; j < calArr.length && flag; j++) {
                if(calArr[j] === event.calendar) {
                    profileIndex = i;
                    flag = false;
                }
            }
        }

        $scope.profileObj = $scope.profileData[profileIndex];
        $scope.hourEventObj = event;
        $scope.checkForDeletedEventId = event.id;
        parseAlertData($scope.hourEventObj);
        parseRepeatData($scope.hourEventObj);
        onPressed(CALCONST.SCREEN_EVENT);
    }

    function removeFocus() {
        document.activeElement.blur();
        SHPService.sendNativeCommand(REQUEST.CLOSE_KEYPAD, REQUEST.CLOSE_KEYPAD, "");
    }

    function shiftUp() {
        var elem = document.getElementById("addPage");
        elem.style.bottom = "0rem";
        elem.style.position = "absolute";

        //console.log("scrollListView");
        var initTopPosition = $('.container-main').offset().top;
        //console.log("initTopPosition ", initTopPosition);
        //console.log("scrollTop ", $(".enter-location").offset().top - initTopPosition);
        $('.container-main').animate({
                scrollTop: $(".enter-note").offset().top - initTopPosition
            },
            'slow');
    }

    function shiftDown() {
        var elem = document.getElementById("addPage");
        elem.style.position  = null;
        elem.style.bottom = null;
        $("addPage").css('bottom', '2rem');
        console.log(" shiftUp ");
    }

    function getCurrentSelectedDate() {
        console.log(" getCurrentSelectedDate ", $scope.createEventData.starts);
        console.log(" getCurrentSelectedDate ", $scope.createEventData.ends);

        return $scope.isCalender === 1 ? $scope.createEventData.starts : $scope.createEventData.ends;
    }

    function showToast() {
        SHPService.sendNativeCommand(CALCONST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_common_max_characters, CALCONST.SHOW_TOAST);
    }

    function parseRepeatData(event) {
        resetRepeatValue();

        if(angular.isUndefined(event) || event === null ||
            angular.isUndefined(event.recurrence) || event.recurrence === null)
            return;
        //"RRULE:FREQ=YEARLY"
        //"RRULE:FREQ=WEEKLY;BYDAY=SU"

        var repeat = {
            FREQ: null,
            INTERVAL: null,
            BYDAY: null,
            BYMONTHDAY: null
        };
        var recurrence = event.recurrence[0];
        var RRULE = (recurrence.split(":")[0] === "RRULE")? recurrence.split(":")[1] : null;
        if (RRULE !== null) {
            RRULE = RRULE.split(";");
            RRULE.forEach(function (rruleValue) {
                var rAttribute = rruleValue.split("=")[0];
                var rAttributeValue = rruleValue.split("=")[1];

                switch (rAttribute) {
                    case "FREQ":    //FREQ=MONTHLY
                        repeat.FREQ = rAttributeValue;
                        return;
                    case "INTERVAL":    //INTERVAL=5
                        repeat.INTERVAL = parseInt(rAttributeValue);
                        return;
                    case "BYDAY":   //BYDAY=1TU or BYDAY=TH or BYDAY=SU,MO,TU,WE,TH,FR,SA
                        var byDays = rAttributeValue.split(",");
                        repeat.BYDAY = byDays;
                        return;
                    case "BYMONTHDAY":  //BYMONTHDAY=5
                        repeat.BYMONTHDAY = rAttributeValue;
                        return;
                }
            });     //parse Recurrence values.

            if (repeat.INTERVAL !== null) { //Custom
                $scope.repeatDataStructure.selected = 5;
                $scope.repeatDataStructure.custom.value = parseInt(repeat.INTERVAL);
                switch (repeat.FREQ) {
                    case 'DAILY':
                        $scope.repeatDataStructure.custom.unitIndex = 0;
                        $scope.Repeatlimit=365;
                        break;
                    case 'WEEKLY':
                        $scope.repeatDataStructure.custom.unitIndex = 1;
                        $scope.Repeatlimit=52;
                        break;
                    case 'MONTHLY':
                        $scope.repeatDataStructure.custom.unitIndex = 2;
                        $scope.Repeatlimit=12;
                        break;
                    case 'YEARLY':
                        $scope.repeatDataStructure.custom.unitIndex = 3;
                        $scope.Repeatlimit=5;
                        break;
                }
                 $scope.selectedRepeatVal=$scope.repeatDataStructure.custom.value;
                 $scope.selectedRepeatUnit=$scope.repeatDataStructure.custom.unitIndex;
                 
            }
            else if (repeat.BYDAY !== null && repeat.BYDAY.length > 0 && repeat.FREQ !== 'MONTHLY') {   //Daily
                $scope.repeatDataStructure.selected = 1;
                setDaySelectedOnDailyPopup(repeat.BYDAY);
            }
            else if (repeat.FREQ === 'DAILY') { //Daily - all days
                $scope.repeatDataStructure.selected = 1;
                if (repeat.BYDAY === null) {
                    repeat.BYDAY = ["SU","MO","TU","WE","TH","FR","SA"];
                }
                setDaySelectedOnDailyPopup(repeat.BYDAY);
            }
            else if (repeat.FREQ === 'WEEKLY') {    //Weekly
                $scope.repeatDataStructure.selected = 2;
            }
            else if (repeat.FREQ === 'MONTHLY') {   //Monthly
                $scope.repeatDataStructure.selected = 3;
                if (repeat.BYDAY !== null) {    //onMonth
                    $scope.repeatDataStructure.monthly.onMonth = true;
                } else {    //onDate
                    $scope.repeatDataStructure.monthly.onDate = true;
                }
            }
            else if (repeat.FREQ === 'YEARLY') {    //Yearly
                $scope.repeatDataStructure.selected = 4;
            }
        }

    }
    
    function getByDayString() {
        var byDay = "";
        if($scope.repeatDataStructure.daily.SU)
            byDay += ($scope.translation.WEBMOB_fhub2_common_abbr_sunday + ",");
        if($scope.repeatDataStructure.daily.MO)
            byDay += ($scope.translation.WEBMOB_fhub2_common_abbr_monday + ",");
        if($scope.repeatDataStructure.daily.TU)
            byDay += ($scope.translation.WEBMOB_fhub2_common_abbr_tuesday + ",");
        if($scope.repeatDataStructure.daily.WE)
            byDay += ($scope.translation.WEBMOB_fhub2_common_abbr_wednesday + ",");
        if($scope.repeatDataStructure.daily.TH)
            byDay += ($scope.translation.WEBMOB_fhub2_common_abbr_thursday + ",");
        if($scope.repeatDataStructure.daily.FR)
            byDay += ($scope.translation.WEBMOB_fhub2_common_abbr_friday + ",");
        if($scope.repeatDataStructure.daily.SA)
            byDay += ($scope.translation.WEBMOB_fhub2_common_abbr_saturday + ",");
        return byDay;
    }
    
    function getCustomRepeatString() {
        var string = "";
        if ($scope.repeatDataStructure.selected === 5) {
            if ($scope.repeatDataStructure.custom.unitIndex === 0) {    //day
                string = $scope.translation.WEBMOB_fhub2_calendar_event_repeat_display_days_one;
                if ($scope.repeatDataStructure.custom.value > 1)
                    string = $scope.translation.WEBMOB_fhub2_calendar_event_repeat_display_days_other;
            }
            else if ($scope.repeatDataStructure.custom.unitIndex === 1) {       //week
                string = $scope.translation.WEBMOB_fhub2_calendar_event_repeat_display_weeks_one;
                if ($scope.repeatDataStructure.custom.value > 1)
                    string = $scope.translation.WEBMOB_fhub2_calendar_event_repeat_display_weeks_other;
            }
            else if ($scope.repeatDataStructure.custom.unitIndex === 2) {       //month
                string = $scope.translation.WEBMOB_fhub2_calendar_event_repeat_display_months_one;
                if ($scope.repeatDataStructure.custom.value > 1)
                    string = $scope.translation.WEBMOB_fhub2_calendar_event_repeat_display_months_other;
            }
            else if ($scope.repeatDataStructure.custom.unitIndex === 3) {       //year
                string = $scope.translation.WEBMOB_fhub2_calendar_event_repeat_display_years_one;
                if ($scope.repeatDataStructure.custom.value > 1)
                    string = $scope.translation.WEBMOB_fhub2_calendar_event_repeat_display_years_other;
            }
            string = string.replace(/(\#.*?\#)/gi, $scope.repeatDataStructure.custom.value.toString());
        }
        return string;
    }

    function getRepeatStringOnEventDetailPage() {
        var string = "";
        switch ($scope.repeatDataStructure.selected) {
            case 0: //None
                break;
            case 1: //Daily
                var byDays = getByDayString();
                if(byDays !== "") {
                    string = $scope.translation.WEBMOB_fhub2_calendar_event_repeat_display_day_of_week;
                    string = string.replace(/(\#.*?\#)/gi, byDays.slice(0, -1));
                }
                break;
            case 2: //Weekly
                string = $scope.translation.WEBMOB_fhub2_calendar_event_detail_repeat_weekly;
                break;
            case 3: //Monthly
                string = $scope.translation.WEBMOB_fhub2_calendar_event_detail_repeat_monthly;
                break;
            case 4: //Yearly
                string = $scope.translation.WEBMOB_fhub2_calendar_event_detail_repeat_yearly;
                break;
            case 5: //Custom
                string = getCustomRepeatString();
                break;
        }

        return string;
    }

    function getRepeatStringOnAddEventView() {
        var string = "";
        switch ($scope.repeatDataStructure.selected) {
            case 0: //None
                string = $scope.translation.WEBMOB_fhub2_calendar_alert_none;
                break;
            case 1: //Daily
                var byDays = getByDayString();
                if(byDays !== "") {
                    string = byDays.slice(0, -1);
                }
                break;
            case 2: //Weekly
                string = $scope.translation.WEBMOB_fhub2_calendar_repeat_weekly;
                break;
            case 3: //Monthly
                if ($scope.repeatDataStructure.monthly.onDate === true)
                    string = $scope.translation.WEBMOB_fhub2_calendar_repeat_monthly_on_date;
                if ($scope.repeatDataStructure.monthly.onMonth === true)
                    string = $scope.translation.WEBMOB_fhub2_calendar_repeat_monthly_on_month;
                break;
            case 4: //Yearly
                string = $scope.translation.WEBMOB_fhub2_calendar_repeat_yearly;
                break;
            case 5: //Custom
                string = getCustomRepeatString();
                break;
        }

        return string;
    }

    function setDaySelectedOnDailyPopup(byDays) {
        byDays.forEach(function(day) {
            switch (day) {
                case "SU":
                    $scope.repeatDataStructure.daily.SU = true;
                    break;
                case "MO":
                    $scope.repeatDataStructure.daily.MO = true;
                    break;
                case "TU":
                    $scope.repeatDataStructure.daily.TU = true;
                    break;
                case "WE":
                    $scope.repeatDataStructure.daily.WE = true;
                    break;
                case "TH":
                    $scope.repeatDataStructure.daily.TH = true;
                    break;
                case "FR":
                    $scope.repeatDataStructure.daily.FR = true;
                    break;
                case "SA":
                    $scope.repeatDataStructure.daily.SA= true;
                    break;
            }
        });
        isSelectedAllDays();
    }

    function parseAlertData(event) {
        resetAlertValue();
        var alerObj = event.reminders;
        if(alerObj !== null && angular.isDefined(alerObj) &&
            angular.isDefined(alerObj["overrides"]) && alerObj["overrides"].length > 0 &&
            angular.isDefined(alerObj.overrides[0].minutes) && alerObj.overrides[0].minutes !== null) {
            var minutes = parseInt(alerObj.overrides[0].minutes);
            if(minutes === 0) { // at start time
                $scope.alertDataStructure.selected = 1;
            } else if(minutes === 10) { // 10 minutes
                $scope.alertDataStructure.selected = 2;
            } else if(minutes === 1440) { // 1 day
                $scope.alertDataStructure.selected = 3;
            } else if(minutes === 10080) { // 1 Week
                $scope.alertDataStructure.selected = 4;
            } else {    //Custom
                $scope.alertDataStructure.selected = 5;
                $scope.Alertlimit=99;
                if(Math.floor(minutes / (1440 * 7)) > 1 && minutes % (1440 * 7) === 0) { // weeks
                    $scope.alertDataStructure.custom.unitIndex = 2;
                    $scope.alertDataStructure.custom.value = Math.floor(minutes / (1440*7));
                    $scope.Alertlimit=4;
                } else if(minutes % (60) === 0) { // Hours
                    $scope.alertDataStructure.custom.unitIndex = 1;
                    $scope.alertDataStructure.custom.value = Math.floor(minutes / (60));
                } else if(minutes >= 1) { // minutes
                    $scope.alertDataStructure.custom.unitIndex = 0;
                    $scope.alertDataStructure.custom.value = minutes;
                }
                $scope.selectedAlertVal=$scope.alertDataStructure.custom.value;
                 $scope.selectedAlertUnit=$scope.alertDataStructure.custom.unitIndex;
            }
        }
    }

    function getAlertString() {
        var string;
        switch ($scope.alertDataStructure.selected) {
            case 0:     //none
                string = $scope.translation.WEBMOB_fhub2_calendar_alert_none;
                break;
            case 1:     //at start time
                string = $scope.translation.WEBMOB_fhub2_calendar_alert_at_start_time;
                break;
            case 2:     //10 minutes
                string = $scope.translation.WEBMOB_fhub2_calendar_alert_minutes;
                string = string.replace(/(\#.*?\#)/gi, "10");
                break;
            case 3:     //1 day
                string = $scope.translation.WEBMOB_fhub2_calendar_alert_day;
                break;
            case 4:     // 1 week
                string = $scope.translation.WEBMOB_fhub2_calendar_alert_week;
                break;
            case 5:     //Custom
                if ($scope.alertUnitData[$scope.alertDataStructure.custom.unitIndex] === $scope.alertUnitData[2]) { //weeks
                    string = $scope.translation.WEBMOB_fhub2_calendar_event_alert_display_week_one;
                    if($scope.alertDataStructure.custom.value > 1) {
                        string = $scope.translation.WEBMOB_fhub2_calendar_event_alert_display_week_other;
                    }
                } else if($scope.alertUnitData[$scope.alertDataStructure.custom.unitIndex] === $scope.alertUnitData[1]) { // Hours
                    string = $scope.translation.WEBMOB_fhub2_calendar_event_alert_display_hour;
                    if($scope.alertDataStructure.custom.value > 1) {
                        string = $scope.translation.WEBMOB_fhub2_calendar_event_alert_display_hours;
                    }
                } else if($scope.alertUnitData[$scope.alertDataStructure.custom.unitIndex] === $scope.alertUnitData[0]) { // minutes
                    string = $scope.translation.WEBMOB_fhub2_calendar_event_alert_display_minute_one;
                    if($scope.alertDataStructure.custom.value > 1) {
                        string = $scope.translation.WEBMOB_fhub2_calendar_event_alert_display_minute_other;
                    }
                }
                string = string.replace(/(\#.*?\#)/gi, $scope.alertDataStructure.custom.value.toString());
                break;
        }
        return string;
    }

    function onTimePickerOpen(isStartTimePicker) {
        var elem;
        if(isStartTimePicker)
            elem = document.getElementById('start-time');
        else
            elem = document.getElementById('end-time');
        elem.click();
    }

    function getAttendeeName(email) {
        var name = '';
        $scope.profileData.forEach(function(value) {
            if(value.oid === email)
                name = value.screenName;
        });

        name = name === '' ? email : name;
        return name;
    }
    function getAttendeeAvatar(email) {
        var name = '';
        $scope.profileData.forEach(function(value) {
            if(value.oid === email)
                name = value.avatar;
        });
        return name;
    }

    function getInvidedString(attendees) {
        var count = 0;
        if (attendees !== null) {
            showInviteesList(attendees);
            count = attendees.length;
        }
        //count = count === undefined ? 0 : count;
        var lable = $scope.translation.WEBMOB_fhub2_calendar_event_detail_invited_sub_title;
        lable = lable.replace(/(\#.*?\#)/gi, count);
        return lable;
    }
    function getAcceptedInviteeString(attendees) {
        if (attendees == null) {
            return;
        }
        var count = attendees.count;
        count = count === undefined ? 0 : count;
        var lable = $scope.translation.WEBMOB_fhub2_calendar_event_detail_accept_sub_title;
        for(var i = 0; i< attendees.length; i++){
            if(attendees[i].responseStatus === "accepted")
                count++;
        }
        lable = lable.replace(/(\#.*?\#)/gi, count);
        if(count === 0){
            $scope.showAcceptedInvitees = false;
        }
        return lable;
    }
    function getMayBeInviteeString(attendees) {
        if (attendees == null) {
            return;
        }
        var count = attendees.count;
        count = count === undefined ? 0 : count;
        var lable = $scope.translation.WEBMOB_fhub2_calendar_event_detail_maybe_sub_title;
        for (var i = 0; i < attendees.length; i++) {
            if (attendees[i].responseStatus === "tentative")
                count++;
        }
        if (count === 0) {
            $scope.showMayBeInvitees = false;
        }
        lable = lable.replace(/(\#.*?\#)/gi, count);
        return lable;
    }
    function getDeclineInviteeString(attendees) {
        if (attendees == null) {
            return;
        }
        var count = attendees.count;
        count = count === undefined ? 0 : count;
        var lable = $scope.translation.WEBMOB_fhub2_calendar_event_detail_declined_sub_title;
        for (var i = 0; i < attendees.length; i++) {
            if (attendees[i].responseStatus === "declined")
                count++;
        }
        if (count === 0) {
            $scope.showDeclinedInvitees = false;
        }
        lable = lable.replace(/(\#.*?\#)/gi, count);
        return lable;
    }
    function getNoResponseInviteeString(attendees) {
        if (attendees == null) {
            return;
        }
        var count = attendees.count;
        count = count === undefined ? 0 : count;
        var lable = $scope.translation.WEBMOB_fhub2_calendar_event_detail_no_response_sub_title;
        for (var i = 0; i < attendees.length; i++) {
            if (angular.isUndefined(attendees[i].responseStatus) || attendees[i].responseStatus === "needsAction")
                count++;
        }
        if (count === 0) {
            $scope.showNoResponseInvitees = false;
        }
        lable = lable.replace(/(\#.*?\#)/gi, count);
        return lable;
    }
    function getInvideeString() {
        var count = $scope.createEventData.arrayDynamicallyAddedEmail.length;
        $scope.profileData.forEach(function(value) {
            if(value.selected === true)
                count++;
        });
        var lable = $scope.translation.WEBMOB_fhub2_calendar_event_invitees;
        lable = lable.replace(/(\#.*?\#)/gi, count);
        return lable;
    }

    function getHeaderMonthText() {

        var month = $scope.translation[monthName[$scope.currentDate.getMonth()]];
        var year = $scope.currentDate.getFullYear();
        return month  + " " + year;
    }

    function profileSelected(value) {
        $scope.currOption = $scope.dropDownSelection[value];
        $scope.showProfilePicker = false;
    }

    function adjustCommonInterval(data) {
        var sorted = [];
        var i;
        for (i = 0; i < data.length; i++) {
            var start = new Date(data[i].starts);
            var end = new Date(data[i].heightEnds);
            var topPos = start.getFullYear() * 366 * 24 * 60 * 31 + start.getMonth() * 24 * 60 * 31 + start.getDate() * 24 * 60 + start.getHours() * 60 + start.getMinutes();
            var height = end.getFullYear() * 366 * 24 * 60 * 31 + end.getMonth() * 24 * 60 * 31 + end.getDate() * 24 * 60 + end.getHours() * 60 + end.getMinutes();

            var durationObject = {start: topPos, event: data[i], end: height};
            sorted.push(durationObject);
        }
        sorted = mergeSort(sorted);
        var finalarray = [];
        finalarray.push({start: sorted[0].start, end: sorted[0].end, arr: [0]});
        for (i = 1; i < data.length; i++) {
            var length = finalarray.length;
            if (sorted[i].start < finalarray[length - 1].end) {
                finalarray[length - 1].end = Math.max(sorted[i].end,finalarray[length - 1].end);
                finalarray[length - 1].arr.push(i);
            } else {
                finalarray.push({start: sorted[i].start, end: sorted[i].end, arr: [i]});
            }
        }

        var eventobject = [];
        for (i = 0; i < finalarray.length; i++) {
            var len = finalarray[i].arr.length;
            var j;
            for (j = 0; j < len; j++) {
                eventobject.push({event: sorted[finalarray[i].arr[j]], count: len, index: j});
            }
        }
        return eventobject;
    }

    function formatAMPM(date) {
        var hours = date.getHours();
        var minutes = date.getMinutes();
        var ampm = hours >= 12 ? $scope.translation.WEBMOB_fhub2_common_pm : $scope.translation.WEBMOB_fhub2_common_am;
        hours = hours % 12;
        hours = hours ? hours : 12; // the hour '0' should be '12'
        minutes = minutes < 10 ? '0' + minutes : minutes;
        var strTime = hours + ':' + minutes + ' ' + ampm;
        return strTime;
    }


    function isLargeEvent(event, count) {
        var start = new Date(event.starts);
        var end = new Date(event.ends);
        var returnVal = 0;

        var topPos = start.getHours() * 60 + start.getMinutes();
        var height = end.getHours() * 60 + end.getMinutes();
        if (start.getDate() + 1 === end.getDate() && end.getHours() === 0) {
            height = 24 * 60;
        }

        height = height - topPos;

        if ((height) >= 30 && count === 2)
            returnVal = 1;
        else if ((height) >= 60)
            returnVal = 1;
        else
            returnVal = 0;
        return returnVal;
    }

    function addInternalInvitee(index) {
        $scope.profileData[index].selected = !$scope.profileData[index].selected;
    }

    function resetInternalInviteeSelectedStatus() {
        $scope.profileData.forEach(function(value) {
            value.isSelected = false;
        });
    }

    function changeRepeatDurUnit(dir) {
        if(dir === 0 && $scope.selectedRepeatUnit < $scope.repeatUnitData.length-1) {
            $scope.selectedRepeatUnit++;
        } else if(dir === 1 && $scope.selectedRepeatUnit > 0){
            $scope.selectedRepeatUnit--;
        }

        var unit = $scope.repeatUnitData[$scope.selectedRepeatUnit];
        if (unit === $scope.repeatUnitData[0]) {
            if ($scope.repeatDataStructure.custom.value > 365) {
                $scope.repeatDataStructure.custom.value = 365;
            }
            $scope.Repeatlimit=365;
        }
        else if (unit === $scope.repeatUnitData[1]) {
            if ($scope.repeatDataStructure.custom.value > 52) {
                $scope.repeatDataStructure.custom.value = 52;
            }
            $scope.Repeatlimit=52;
        }
        else if (unit === $scope.repeatUnitData[2]) {
           if ($scope.repeatDataStructure.custom.value > 24) {
                $scope.repeatDataStructure.custom.value = 24;
            }
            $scope.Repeatlimit=24;
        }
        else if (unit === $scope.repeatUnitData[3]) {
            if ($scope.repeatDataStructure.custom.value > 5) {
                $scope.repeatDataStructure.custom.value = 5;
            }
            $scope.Repeatlimit=5;
        }
    }



    function increaseRepeatValue() {
        var unit = $scope.repeatUnitData[$scope.repeatDataStructure.custom.unitIndex];
        if (unit === $scope.repeatUnitData[0] && $scope.repeatDataStructure.custom.value < 365) {
            $scope.repeatDataStructure.custom.value++;
        }
        else if (unit === $scope.repeatUnitData[1] && $scope.repeatDataStructure.custom.value < 52) {
            $scope.repeatDataStructure.custom.value++;
        }
        else if (unit === $scope.repeatUnitData[2] && $scope.repeatDataStructure.custom.value < 24) {
            $scope.repeatDataStructure.custom.value++;
        }
        else if (unit === $scope.repeatUnitData[3] && $scope.repeatDataStructure.custom.value < 5) {
            $scope.repeatDataStructure.custom.value++;
        }
    }

    function decreaseRepeatValue() {
        if($scope.repeatDataStructure.custom.value > 1) {
            $scope.repeatDataStructure.custom.value--;
        }
    }

    function changeAlertDurUnit(dir) {
        if(dir === 0 && $scope.selectedAlertUnit < $scope.alertUnitData.length-1) {
            $scope.selectedAlertUnit++;
        } else if(dir === 1 && $scope.selectedAlertUnit > 0){
            $scope.selectedAlertUnit--;
        }
        // In case of weeks: value should be 1~4
        if ($scope.alertUnitData[$scope.selectedAlertUnit] === $scope.alertUnitData[2]) {
            if($scope.alertDataStructure.custom.value > 4){
                $scope.alertDataStructure.custom.value = 4;
            }
            $scope.Alertlimit=4;
        }
        else
            $scope.Alertlimit=99;
    }
    /*custom alert new implementation*/
    function initAlertList(val) {
         $scope.alertValList=[];
        var listItemHeight = 2.7 * baseFont;
        for(i=0;i<99;i++){
            $scope.alertValList[i]=i+1;
        }
        $scope.selectedAlertVal = $scope.alertValList[val];
        $("#alertVal-box").val($scope.selectedAlertVal);
        $timeout(function () {
            var scrollTop = listItemHeight * val;
            $("#alertValDiv").scrollTop(scrollTop);
        }, 150);
        
        $timeout(function() {registerAlertValScroll();}, 200);
    }

    function registerAlertValScroll() {
        $("#alertValDiv").off("scroll", alertValScrollHndler);
        $("#alertValDiv").scroll(alertValScrollHndler);
    }

    function alertValScrollHndler() {
        var prevScrollTop = $("#alertValDiv").scrollTop();
        var dir = 0;
        var scrollTop = $("#alertValDiv").scrollTop();
        var listItemHeight = 2.7 * baseFont;
        dir = prevScrollTop > scrollTop ? 1 : 0;
        prevScrollTop = scrollTop;
        if (scrollFinishMinute) {
            scrollFinishMinute = false;
            return;
        }
        var index;
        if (dir === 0) {
            scrollTop = (Math.round(scrollTop / listItemHeight)) * listItemHeight;
            index = Math.round(scrollTop / listItemHeight);
            index = index >= $scope.alertValList.length ? $scope.alertValList.length - 1 : index;
        } else {
            scrollTop = (Math.round(scrollTop / listItemHeight)) * listItemHeight;
            index = Math.round(scrollTop / listItemHeight);
            index = index < 0 ? 0 : index;

        }

        $scope.selectedAlertVal = $scope.alertValList[index];
        $scope.alertDataStructure.custom.value=$scope.alertValList[index];
        $("#alertVal-box").val($scope.selectedAlertVal);
        if (promise) {
            $timeout.cancel(promise);
        }
        promise = $timeout(function () {
            $("#alertValDiv").scrollTop(scrollTop);
            scrollFinishMinute = true;
        }, 150);
        //$scope.$apply();
    }
    
    function initAlertUnitList(unit) {
        $scope.selectedAlertUnit = $scope.alertDataStructure.custom.unitIndex;;
        /*var listItemHeight = 2.7 * baseFont;
        $("#alertUnit-box").val($scope.alertUnitData[$scope.selectedAlertUnit]);
        $timeout(function () {
            var scrollTop = listItemHeight * unit;
            $("#alertUnitDiv").scrollTop(scrollTop);
        }, 50);

        $timeout(function() {registerAlertUnitScroll();}, 0);*/
    }

    function registerAlertUnitScroll() {
        $("#alertUnitDiv").off("scroll", alertUnitScrollHndler);
        $("#alertUnitDiv").scroll(alertUnitScrollHndler);
    }

    function alertUnitScrollHndler() {
        var prevScrollTop = $("#alertUnitDiv").scrollTop();
        var dir = 0;
        var scrollTop = $("#alertUnitDiv").scrollTop();
        var listItemHeight = 2.7 * baseFont;
        dir = prevScrollTop > scrollTop ? 1 : 0;
        prevScrollTop = scrollTop;
        if (scrollFinishMinute) {
            scrollFinishMinute = false;
            return;
        }
        var index;
        if (dir === 0) {
            scrollTop = (Math.floor(scrollTop / listItemHeight) + 1) * listItemHeight;
            index = Math.floor(scrollTop / listItemHeight);
            index = index >= $scope.alertUnitData.length ? $scope.alertUnitData.length - 1 : index;
        } else {
            scrollTop = (Math.floor(scrollTop / listItemHeight)) * listItemHeight;
            index = Math.floor(scrollTop / listItemHeight);
            index = index < 0 ? 0 : index;

        }

        $scope.selectedAlertUnit = index;
        $scope.alertDataStructure.custom.unitIndex=index;
        $("#alertUnit-box").val($scope.alertUnitData[$scope.selectedAlertUnit]);
        if(index===2)
            $scope.Alertlimit=4;
        else
            $scope.Alertlimit = 99;
        
        if (promise) {
            $timeout.cancel(promise);
        }
        promise = $timeout(function () {
            $("#alertUnitDiv").scrollTop(scrollTop);
            scrollFinishMinute = true;
        }, 50);
    }
     /*custom alert new implementation ends*/
     
    /*custom repeat new implementation starts*/
    function initRepeatList(val) {
         $scope.repeatValList=[];
        var listItemHeight = 2.7 * baseFont;
        for(i=0;i<365;i++){
            $scope.repeatValList[i]=i+1;
        }
        $scope.selectedRepeatVal = $scope.repeatValList[val];
        $("#repeatVal-box").val($scope.selectedRepeatVal);
        $timeout(function () {
            var scrollTop = listItemHeight * val;
            $("#repeatValDiv").scrollTop(scrollTop);
        }, 150);

        $timeout(function() {registerRepeatValScroll();}, 200);
    }

    function registerRepeatValScroll() {
        $("#repeatValDiv").off("scroll", repeatValScrollHndler);
        $("#repeatValDiv").scroll(repeatValScrollHndler);
    }
    
    function repeatValScrollHndler() {
        var prevScrollTop = $("#repeatValDiv").scrollTop();
        var dir = 0;
        var scrollTop = $("#repeatValDiv").scrollTop();
        var listItemHeight = 2.7 * baseFont;
        dir = prevScrollTop > scrollTop ? 1 : 0;
        prevScrollTop = scrollTop;
         if (scrollFinishMinute) {
            scrollFinishMinute = false;
            return;
        }
        var index;
        if (dir === 0) {
            scrollTop = (Math.round(scrollTop / listItemHeight)) * listItemHeight;
            index = Math.round(scrollTop / listItemHeight);
            index = index >= $scope.repeatValList.length ? $scope.repeatValList.length - 1 : index;
        } else {
            scrollTop = (Math.round(scrollTop / listItemHeight)) * listItemHeight;
            index = Math.round(scrollTop / listItemHeight);
            index = index < 0 ? 0 : index;

        }

        $scope.selectedRepeatVal = $scope.repeatValList[index];
        $scope.repeatDataStructure.custom.value=$scope.repeatValList[index];
        $("#repeatVal-box").val($scope.selectedRepeatVal);
        if (promise) {
            $timeout.cancel(promise);
        }
        promise = $timeout(function () {
            $("#repeatValDiv").scrollTop(scrollTop);
            scrollFinishMinute = true;
        }, 150);
        //$scope.$apply();
    }
    
    function initRepeatUnitList(unit) {
        $scope.selectedRepeatUnit = $scope.repeatDataStructure.custom.unitIndex;
//        var listItemHeight = 2.7 * baseFont;
//        $scope.selectedRepeatUnit = unit;
//        $("#repeatUnit-box").val($scope.repeatUnitData[$scope.selectedRepeatUnit]);
//        $timeout(function () {
//            var scrollTop = listItemHeight * unit;
//            $("#repeatUnitDiv").scrollTop(scrollTop);
//        }, 0);
//
//        $timeout(function() {registerRepeatUnitScroll();}, 0);
    }

    function registerRepeatUnitScroll() {
        $("#repeatUnitDiv").scroll(repeatUnitScrollHndler);
        $("#repeatUnitDiv").off("scroll", repeatUnitScrollHndler);
    }

    function repeatUnitScrollHndler() {
        var prevScrollTop = $("#repeatUnitDiv").scrollTop();
        var dir = 0;
        var scrollTop = $("#repeatUnitDiv").scrollTop();
        var listItemHeight = 2.7 * baseFont;
        dir = prevScrollTop >= scrollTop ? 1 : 0;
        prevScrollTop = scrollTop;
        if (scrollFinishMinute) {
            scrollFinishMinute = false;
            return;
        }       
        var index;
        if (dir === 0) {
            scrollTop = (Math.floor(scrollTop / listItemHeight) + 1) * listItemHeight;
            index = Math.floor(scrollTop / listItemHeight);
            index = index >= $scope.repeatUnitData.length ? $scope.repeatUnitData.length - 1 : index;
        } else {
            scrollTop = (Math.floor(scrollTop / listItemHeight)) * listItemHeight;
            index = Math.floor(scrollTop / listItemHeight);
            index = index < 0 ? 0 : index;

        }

        $scope.selectedRepeatUnit = index;
        $scope.repeatDataStructure.custom.unitIndex=index;
        $("#repeatUnit-box").val($scope.repeatUnitData[$scope.selectedRepeatUnit]);
        switch(index){
            case 0:  $scope.Repeatlimit=365; break;
            case 1:  $scope.Repeatlimit=52; break;
            case 2:  $scope.Repeatlimit=24; break;
            case 3:  $scope.Repeatlimit=5; break;
        }

        if (promise) {
            $timeout.cancel(promise);
        }
        
        promise = $timeout(function () {
            $("#repeatUnitDiv").scrollTop(scrollTop);
            scrollFinishMinute = true;
        }, 0);
        $scope.$apply();
//        
        
//        if ($scope.$root.$$phase !== '$apply' && $scope.$root.$$phase !== '$digest') {
//            $scope.$apply();
//        }
    }
    /*repeat custom new implementation ends*/
    
    $scope.$watch(function () {
        //console.log("$scope.selectedRepeatUnit", $scope.selectedRepeatUnit);
        return $scope.selectedRepeatUnit;
    }, function (newVal, oldVal) {
        if (newVal !== oldVal && angular.isDefined($scope.selectedRepeatUnit)) {
            $("#repeatUnit-box").val($scope.repeatUnitData[$scope.selectedRepeatUnit]);
        }
    });
    
    function increaseAlertValue() {
        // In case of weeks: value should be 1~4, rest cases 1~99
        if ($scope.alertUnitData[$scope.alertDataStructure.custom.unitIndex] === $scope.alertUnitData[2]) {
            if ($scope.alertDataStructure.custom.value < 4) {
                $scope.alertDataStructure.custom.value++;
            }
        } else if ($scope.alertDataStructure.custom.value < 99) {
            $scope.alertDataStructure.custom.value++;
        }
    }

    function decreaseAlertValue() {
        if($scope.alertDataStructure.custom.value > 1) {
            $scope.alertDataStructure.custom.value--;
        }
    }

    function saveAlertCustom() {
        $scope.alertDataStructure.selected = 5;
        $scope.alertDataStructure.custom.value=$scope.selectedAlertVal;
        $scope.alertDataStructure.custom.unitIndex=$scope.selectedAlertUnit;
        $scope.isAlertPopupOpen = false;
    }

    function saveRepeatPopup(selectedOption) {
        switch (selectedOption) {
            case 1: //Daily
                $scope.repeatDataStructure.selected = 1;
                if (!$scope.repeatDataStructure.daily.SU && !$scope.repeatDataStructure.daily.MO && !$scope.repeatDataStructure.daily.TU &&
                    !$scope.repeatDataStructure.daily.WE && !$scope.repeatDataStructure.daily.TH && !$scope.repeatDataStructure.daily.FR &&
                    !$scope.repeatDataStructure.daily.SA) {
                    cancelRepeatPopup();
                }
                break;
            case 3: //Monthly
                $scope.repeatDataStructure.selected = 3;
                if($scope.repeatDataStructure.monthly.onDate === false && $scope.repeatDataStructure.monthly.onMonth === false) {
                    cancelRepeatPopup();
                }
                break;
            case 5: //Custom
                $scope.repeatDataStructure.selected = 5;
                 $scope.repeatDataStructure.custom.value=$scope.selectedRepeatVal;
                 $scope.repeatDataStructure.custom.unitIndex=$scope.selectedRepeatUnit;
                break;
        }
        $scope.isRepeatPopupOpen = false;
    }

    function getMonthlyCheckedStatus(index) {
        var flag = false;
        if(index === 0) {
            flag = $scope.repeatDataStructure.monthly.onDate;
        } else {
            flag = $scope.repeatDataStructure.monthly.onMonth;
        }
        return flag;
    }

    function setMonthlyCheckedStatus(index) {
        if(index === 0) {
            $scope.repeatDataStructure.monthly.onDate = !$scope.repeatDataStructure.monthly.onDate;

            $scope.repeatDataStructure.monthly.onMonth = !$scope.repeatDataStructure.monthly.onDate;
        }
        else
        {
            $scope.repeatDataStructure.monthly.onMonth = !$scope.repeatDataStructure.monthly.onMonth;

            $scope.repeatDataStructure.monthly.onDate = !$scope.repeatDataStructure.monthly.onMonth;

        }
    }

    function getDailyCheckedStatus(index) {
        var flag = false;
        switch (index) {
            case 0:
                flag = $scope.repeatDataStructure.daily.AL;
                break;
            case 1:
                flag = $scope.repeatDataStructure.daily.SU;
                break;
            case 2:
                flag = $scope.repeatDataStructure.daily.MO;
                break;
            case 3:
                flag = $scope.repeatDataStructure.daily.TU;
                break;
            case 4:
                flag = $scope.repeatDataStructure.daily.WE;
                break;
            case 5:
                flag = $scope.repeatDataStructure.daily.TH;
                break;
            case 6:
                flag = $scope.repeatDataStructure.daily.FR;
                break;
            case 7:
                flag = $scope.repeatDataStructure.daily.SA;
                break;
        }
        return flag;
    }

    function setDailyCheckedStatus(index) {
        switch (index) {
            case 0:
                if ($scope.repeatDataStructure.daily.AL) {
                    toggleAllDays(false);
                }
                else {
                    toggleAllDays(true);
                }
                break;
            case 1:
                $scope.repeatDataStructure.daily.SU = !$scope.repeatDataStructure.daily.SU;
                break;
            case 2:
                $scope.repeatDataStructure.daily.MO = !$scope.repeatDataStructure.daily.MO;
                break;
            case 3:
                $scope.repeatDataStructure.daily.TU = !$scope.repeatDataStructure.daily.TU;
                break;
            case 4:
                $scope.repeatDataStructure.daily.WE = !$scope.repeatDataStructure.daily.WE;
                break;
            case 5:
                $scope.repeatDataStructure.daily.TH = !$scope.repeatDataStructure.daily.TH;
                break;
            case 6:
                $scope.repeatDataStructure.daily.FR = !$scope.repeatDataStructure.daily.FR;
                break;
            case 7:
                $scope.repeatDataStructure.daily.SA = !$scope.repeatDataStructure.daily.SA;
                break;
        }
        isSelectedAllDays();
    }
    
    function isSelectedAllDays() {
        if ($scope.repeatDataStructure.daily.SU && $scope.repeatDataStructure.daily.MO && $scope.repeatDataStructure.daily.TU &&
            $scope.repeatDataStructure.daily.WE && $scope.repeatDataStructure.daily.TH && $scope.repeatDataStructure.daily.FR &&
            $scope.repeatDataStructure.daily.SA)
            $scope.repeatDataStructure.daily.AL = true;
        else
            $scope.repeatDataStructure.daily.AL = false;
        return $scope.repeatDataStructure.daily.AL;
    }

    function toggleAllDays(bool) {
        if (bool) {
            $scope.repeatDataStructure.daily.AL = true;
            $scope.repeatDataStructure.daily.SU = true;
            $scope.repeatDataStructure.daily.MO = true;
            $scope.repeatDataStructure.daily.TU = true;
            $scope.repeatDataStructure.daily.WE = true;
            $scope.repeatDataStructure.daily.TH = true;
            $scope.repeatDataStructure.daily.FR = true;
            $scope.repeatDataStructure.daily.SA = true;
        }
        else {
            $scope.repeatDataStructure.daily.AL = false;
            $scope.repeatDataStructure.daily.AL = false;
            $scope.repeatDataStructure.daily.SU = false;
            $scope.repeatDataStructure.daily.MO = false;
            $scope.repeatDataStructure.daily.TU = false;
            $scope.repeatDataStructure.daily.WE = false;
            $scope.repeatDataStructure.daily.TH = false;
            $scope.repeatDataStructure.daily.FR = false;
            $scope.repeatDataStructure.daily.SA = false;
        }
    }

    function cancelRepeatPopup() {
        resetRepeatValue();
        $scope.isRepeatPopupOpen = false;
    }

    function resetRepeatValue() {
        $scope.repeatDataStructure = {
            selected: 0,
            custom: {
                unitIndex: 0,
                value: 1
            },
            daily: {
                AL: false,
                SU: false,
                MO: false,
                TU: false,
                WE: false,
                TH: false,
                FR: false,
                SA: false
            },
            monthly: {
                onDate: false,
                onMonth: false
            }
        };
        $scope.selectedRepeatVal=1;
        $scope.selectedRepeatUnit=0;
        $scope.Repeatlimit=365;
        scrollFinishMinute =false;
    }

    function resetAlertValue() {
        $scope.alertDataStructure = {
            selected : 0,
            custom : {
                unitIndex : 0,
                value : 1
            }
        };
        $scope.selectedAlertVal=1;
        $scope.selectedAlertUnit=0;
        $scope.Alertlimit=99;
        scrollFinishMinute =false;
    }
    
    function createEventOnDayView(profileIndex, hourIndex){
        $scope.profileObj = $scope.profileData[profileIndex];
        //$scope.hourEventObj = $scope.profileObj.hourEvent[hourIndex];
        if ($scope.createEventData.starts === undefined && $scope.createEventData.ends === undefined) {
            $scope.createEventData.starts = new Date($scope.currentDate.getFullYear(), $scope.currentDate.getMonth(), $scope.currentDate.getDate());
            $scope.createEventData.starts.setHours(hourIndex);
            $scope.createEventData.starts.setMinutes(00);
            $scope.createEventData.ends = new Date($scope.createEventData.starts.getFullYear(), $scope.createEventData.starts.getMonth(), $scope.createEventData.starts.getDate());
            $scope.createEventData.ends.setHours($scope.createEventData.starts.getHours() + 1);
            $scope.createEventData.ends.setMinutes($scope.createEventData.starts.getMinutes());
        }
        $scope.currOption = $scope.profileObj;
        openCreateEventPage();
    }

    function openCreateEventPage() {
        resetAlertValue();
        if($scope.createEventData.starts === undefined && $scope.createEventData.ends === undefined) {
            $scope.createEventData.starts = new Date($scope.currentDate.getFullYear(), $scope.currentDate.getMonth(),$scope.currentDate.getDate());
            $scope.createEventData.starts.setHours((new Date()).getHours());
            $scope.createEventData.starts.setMinutes((new Date()).getMinutes());
            $scope.createEventData.ends = new Date($scope.currentDate.getFullYear(), $scope.currentDate.getMonth(),$scope.currentDate.getDate());
            $scope.createEventData.ends.setHours($scope.createEventData.starts.getHours()+1);
            $scope.createEventData.ends.setMinutes($scope.createEventData.starts.getMinutes());
        }

        $scope.isEdit = false;
        onPressed(CALCONST.SCREEN_ADD);
    }

    function callEventEditDeletePage(profileIndex, hourIndex) {
        $scope.profileObj = $scope.profileData[profileIndex];
        $scope.hourEventObj = $scope.profileObj.hourEvent[hourIndex].event.event;
        if ($scope.profileObj.hourEvent[hourIndex].count > 4) {
            changeHourOrAgendaView();       //goto Agenda view
            $scope.isWeekView = true;
            showWeekView();
        } else {
            openDetailPage($scope.hourEventObj);
        }
        $scope.currOption = $scope.profileObj;
    }

    function openEditPage() {
        $scope.checkIfEventIsAlreadyAllDayEvent = false;
        $scope.endDateBeforeEditEvent = new Date();
        $scope.createEventData.starts = new Date($scope.hourEventObj.starts);
        $scope.createEventData.ends = new Date($scope.hourEventObj.ends);
        $scope.createEventData.summay = $scope.hourEventObj.summary;
        $scope.createEventData.allDay = $scope.hourEventObj.allDay;
        $scope.createEventData.location = $scope.hourEventObj.location;
        $scope.createEventData.desc = $scope.hourEventObj.description;
        
        if ($scope.createEventData.allDay) {
            $scope.createEventData.starts.setHours(0);
            $scope.createEventData.starts.setMinutes(0);
            $scope.createEventData.starts.setSeconds(0);
            $scope.createEventData.starts.setMilliseconds(0);

            $scope.createEventData.ends.setHours(0);
            $scope.createEventData.ends.setMinutes(0);
            $scope.createEventData.ends.setSeconds(0);
            $scope.createEventData.ends.setMilliseconds(0);
            
            $scope.checkIfEventIsAlreadyAllDayEvent = true;
            $scope.endDateBeforeEditEvent = $scope.createEventData.ends;
        }

        var tempFlag = false;
        if(angular.isDefined($scope.hourEventObj.attendees) && $scope.hourEventObj.attendees !== null) {
            $scope.hourEventObj.attendees.forEach(function (attendee) {
                tempFlag = false;
                $scope.profileData.forEach(function (value) {
                    if (angular.isDefined(value.email) && angular.isDefined(attendee.email) && value.email.toUpperCase() === attendee.email.toUpperCase()) {
                        value.selected = true;
                        tempFlag = true;
                    }
                });
                if (!tempFlag)
                    $scope.createEventData.arrayDynamicallyAddedEmail.push(attendee.email);

            });
        }

        parseAlertData($scope.hourEventObj);
        parseRepeatData($scope.hourEventObj);

        $scope.isEdit = true;
        onPressed(CALCONST.SCREEN_ADD);
        $scope.currOption = $scope.profileObj;
    }

    function deleteIndex(id){
        $scope.createEventData.arrayDynamicallyAddedEmail.splice(id,1);
    };

    function arrayAddFunction() {
        var elem = document.getElementById('reset-email');
        elem.blur();
        var email = elem.value;

        var regex = /^([a-zA-Z0-9_\.\-])+\@(([a-zA-Z0-9\-])+\.)+([a-zA-Z0-9]{2,6})+$/;
        if (!(regex.test(email))) {
            $timeout(function() { showInvalidEmailPopup(); }, 400);
            return;
        }

        $scope.createEventData.arrayDynamicallyAddedEmail.push(email);
        document.getElementById('reset-email').value="";
    };

    function showInvalidEmailPopup() {
        $scope.dialogPopup.show = true;
        toggleBottomPopUpClass(true);
        $scope.dialogPopup.dialogPopupTitle = "";
        $scope.dialogPopup.dialogPopupText = $scope.translation.WEBMOB_fhub2_calendar_validation_error_email;
        $scope.dialogPopup.button1Title = $scope.translation.WEBMOB_fhub2_common_ok_capital;
        $scope.dialogPopup.button1Handler = function () {
            $scope.dialogPopup.show = false;
        };
    }

    function openPickerPopup(value) {
        $scope.isCalender = value;
    }

    function closePicker(obj, isCancel) {

        if ($scope.isCalender === 1) {
            if(!isCancel) {
                $scope.startDate = (obj === undefined) ? $scope.startDate : obj;
                $scope.createEventData.starts = $scope.startDate;
                $scope.createEventData.ends.setDate($scope.createEventData.starts.getDate());
                $scope.createEventData.ends.setMonth($scope.createEventData.starts.getMonth());
                $scope.createEventData.ends.setFullYear($scope.createEventData.starts.getFullYear());

                //set ends time in advance
                $scope.createEventData.ends.setHours($scope.createEventData.starts.getHours() + 1);
                $scope.createEventData.ends.setMinutes($scope.createEventData.starts.getMinutes());

            }
        } else if ($scope.isCalender === 2) {
            if(!isCancel) {
                $scope.endDate = (obj === undefined) ? $scope.endDate : obj;
                $scope.createEventData.ends = $scope.endDate;
                $scope.createEventData.ends.setHours($scope.createEventData.starts.getHours()+1);
                $scope.createEventData.ends.setMinutes($scope.createEventData.starts.getMinutes()); 
            }
        }
        $scope.isCalender = -1;
    }

    function showInvalidDate() {
        //  console.log(" showConnectionFailurePopup ");
        $scope.dialogPopup.show = true;
        toggleBottomPopUpClass(true);
        $scope.dialogPopup.dialogPopupTitle = "";
        $scope.dialogPopup.dialogPopupText = $scope.translation.WEBMOB_fhub2_calendar_validation_error_time;
        $scope.dialogPopup.button1Title = $scope.translation.WEBMOB_fhub2_common_ok_capital;
        $scope.dialogPopup.button1Handler = function () {
            $scope.dialogPopup.show = false;
        };
    }

    function openTimePicker(value) {
        $("#HourDiv").off("scroll", hourScrollHndler);
        $("#MinuteDiv").off("scroll", minuteScrollHndler);

        $scope.isTimePicker = value;
        if(value === 1) {
            $scope.selectedHour = $scope.createEventData.starts.getHours() > 12 ? $scope.createEventData.starts.getHours() - 12 : $scope.createEventData.starts.getHours();
            $scope.selectedMinute = $scope.createEventData.starts.getMinutes();
            $scope.selectedMendrian = $scope.createEventData.starts.getHours() >= 12 ? $scope.translation.WEBMOB_fhub2_common_pm_capital : $scope.translation.WEBMOB_fhub2_common_am_capital;
        } else {
            $scope.selectedHour = $scope.createEventData.ends.getHours() > 12 ? $scope.createEventData.ends.getHours() - 12 : $scope.createEventData.ends.getHours();
            $scope.selectedMinute = $scope.createEventData.ends.getMinutes();
            $scope.selectedMendrian = $scope.createEventData.ends.getHours() >= 12 ? $scope.translation.WEBMOB_fhub2_common_pm_capital : $scope.translation.WEBMOB_fhub2_common_am_capital;
        }

        initMeridianList($scope.selectedMendrian);
        initHourList($scope.selectedHour);
        initMinuteList($scope.selectedMinute);

        $("#hour-box").on('input', function() {
            var regex=/^[0-9]+$/;

            if($(this).val() === '') {
                return;
            } else if(($(this).val()).match(regex) === null || $(this).val() == '00') {
                $(this).val('01');
            }

            $(this).val($(this).val() > 12 ? 12 : $(this).val());
            scrollFinishHour = true;
            $scope.selectedHour = $(this).val();

            var listItemHeight = 2.7 * baseFont;
            var scrollTop = listItemHeight * ($scope.selectedHour-1);
            $("#HourDiv").scrollTop(scrollTop);
        });

        $("#minute-box").on('input', function() {
            var regex=/^[0-9]+$/;

            if($(this).val() === '') {
                return;
            } else if(($(this).val()).match(regex) === null || $(this).val() == '00') {
                $(this).val('00');
            }

            $(this).val($(this).val() > 59 ? 59 : $(this).val());
            scrollFinishMinute = true;
            $scope.selectedMinute = $(this).val();
            var listItemHeight = 2.7 * baseFont;
            var scrollTop = listItemHeight * $scope.selectedMinute;
            $("#MinuteDiv").scrollTop(scrollTop);
            $scope.$apply();
        });
    }

    function initMeridianList(meridian) {
        var val = meridian === $scope.translation.WEBMOB_fhub2_common_am_capital ? 0 : 2;
        var listItemHeight = 2.7 * baseFont;
        var scrollTop = listItemHeight * val;
        $timeout(function() { $("#medrianDiv").scrollTop(scrollTop);}, 100);
        $timeout(function() {registerMeridianScroll();}, 0);
    }

    function registerMeridianScroll() {
        var  prevScrollTop = $("#medrianDiv").scrollTop();
        var dir = 0;
        $("#medrianDiv").scroll(function() {
            var scrollTop = $("#medrianDiv").scrollTop();
            var listItemHeight = 2.76 * baseFont;
            dir = prevScrollTop > scrollTop ? 1 : 0;
            prevScrollTop = scrollTop;
            if(mendrianTimer !== null) {
                clearTimeout(mendrianTimer);
            }

            mendrianTimer = setTimeout(function() {
                var index;
                if(dir === 0) {
                    scrollTop = (Math.floor(scrollTop/listItemHeight)+1)* listItemHeight;
                    index = Math.floor(scrollTop/listItemHeight);
                    index = index >= $scope.mendrianUnits.length ? $scope.mendrianUnits.length - 1 : index;
                } else {
                    scrollTop = (Math.floor(scrollTop/listItemHeight)) * listItemHeight;
                    index = Math.floor(scrollTop/listItemHeight);
                }
                $scope.selectedMendrian = $scope.mendrianUnits[index];
                $("#medrianDiv").animate({ scrollTop: scrollTop});
                $scope.$apply();
            }, 100);
        });
    }

    function initHourList(hours) {
        var listItemHeight = 2.7 * baseFont;
        var val = 12;
        if(hours === 0) {
            val = 12;
        } else if(hours > 12) {
            val = hours - 12;
        } else {
            val = hours;
        }
        $("#hour-box").val(prepandZero(val));
        $timeout(function() {
            var scrollTop = listItemHeight * (val-1);
            $("#HourDiv").scrollTop(scrollTop);
        }, 150);
        $timeout(function() {registerHourScroll();}, 200);
    }

//    function onListTouchUp() {
//        $("#HourDiv").off("scroll", hourScrollHndler);
//        registerHourScroll();
//    }

    function registerHourScroll() {
        $("#HourDiv").off("scroll", hourScrollHndler);
        $("#HourDiv").scroll(hourScrollHndler);
    }

    function hourScrollHndler() {
        var prevScrollTop = $("#HourDiv").scrollTop();
        var dir = 0;
        var scrollTop = $("#HourDiv").scrollTop();
        var listItemHeight = 2.7 * baseFont;
        dir = prevScrollTop >= scrollTop ? 1 : 0;
        prevScrollTop = scrollTop;

        if (scrollFinishHour) {
            scrollFinishHour = false;
            return;
        }

        var index;
        if (dir === 0) {
            scrollTop = (Math.floor(scrollTop / listItemHeight) + 1) * listItemHeight;
            index = Math.floor(scrollTop / listItemHeight);
            index = index >= $scope.hourList.length ? $scope.hourList.length - 1 : index;
        } else {
            scrollTop = (Math.floor(scrollTop / listItemHeight)) * listItemHeight;
            index = Math.floor(scrollTop / listItemHeight);
            index = index < 0 ? 0 : index;
        }

        $scope.selectedHour = $scope.hourList[index];
        $("#hour-box").val($scope.selectedHour);
        if (promise) {
            $timeout.cancel(promise);
        }
        promise = $timeout(function () {
            $("#HourDiv").scrollTop(scrollTop);
            scrollFinishHour = true;
        }, 100);
    }

    function initMinuteList(minutes) {
        var listItemHeight = 2.7 * baseFont;
        $scope.minuteList = [];
        for (var i = 0; i < 60; i++) {
            $scope.minuteList.push(prepandZero(i));
        }
        $scope.selectedMinute = $scope.minuteList[minutes];
        $("#minute-box").val($scope.selectedMinute);
        $timeout(function () {
            var scrollTop = listItemHeight * minutes;
            $("#MinuteDiv").scrollTop(scrollTop);
        }, 150);

        $timeout(function() {registerMinuteScroll();}, 200);
    }

    function registerMinuteScroll() {
        $("#MinuteDiv").off("scroll", minuteScrollHndler);
        $("#MinuteDiv").scroll(minuteScrollHndler);
    }

    function minuteScrollHndler() {
        var prevScrollTop = $("#MinuteDiv").scrollTop();
        var dir = 0;
        var scrollTop = $("#MinuteDiv").scrollTop();
        var listItemHeight = 2.7 * baseFont;
        dir = prevScrollTop >= scrollTop ? 1 : 0;
        prevScrollTop = scrollTop;
        if (scrollFinishMinute) {
            scrollFinishMinute = false;
            return;
        }
        var index;
        if (dir === 0) {
            scrollTop = (Math.floor(scrollTop / listItemHeight) + 1) * listItemHeight;
            index = Math.floor(scrollTop / listItemHeight);
            index = index >= $scope.minuteList.length ? $scope.minuteList.length - 1 : index;
        } else {
            scrollTop = (Math.floor(scrollTop / listItemHeight)) * listItemHeight;
            index = Math.floor(scrollTop / listItemHeight);
            index = index < 0 ? 0 : index;

        }

        $scope.selectedMinute = $scope.minuteList[index];
        $("#minute-box").val($scope.selectedMinute);
        if (promise) {
            $timeout.cancel(promise);
        }
        promise = $timeout(function () {
            $("#MinuteDiv").scrollTop(scrollTop);
            scrollFinishMinute = true;
        }, 100);
    }

    function onBlurPicker(isHour) {
        var val;
        if (isHour) {
            val = $("#hour-box").val();
            if(val === '') {
                $("#hour-box").val('01');
            } else if(val.length === 1 && val.length < 2)  {
                $("#hour-box").val(prepandZero(val));
            }
        } else {
            val = $("#minute-box").val();
            if(val.length === 1 && val.length < 2) {
                $("#minute-box").val(prepandZero(val));
            } else if(val === '') {
                $("#minute-box").val('00');
                initMinuteList(0);
            }
        }
    }

    function closeTimePopup() {
        $scope.isTimePicker = -1;
        $("#HourDiv").off("scroll", hourScrollHndler);
        $("#MinuteDiv").off("scroll", minuteScrollHndler);
    }

    function setTime() {
        var hour;
        if ($scope.selectedMendrian === $scope.translation.WEBMOB_fhub2_common_pm_capital) {
            if (parseInt($scope.selectedHour) !== 12) {
                hour = parseInt($scope.selectedHour) + 12;
            } else {
                hour = parseInt($scope.selectedHour);
            }
        } else {
            if (parseInt($scope.selectedHour) === 12) {
                hour = parseInt($scope.selectedHour) - 12;
            } else {
                hour = parseInt($scope.selectedHour);
            }
        }
        $scope.selectedHour = hour;
        if ($scope.isTimePicker === 1) {
            $scope.createEventData.starts.setHours(hour);
            $scope.createEventData.starts.setMinutes($scope.selectedMinute);
            $scope.createEventData.ends.setDate($scope.createEventData.starts.getDate());
            $scope.createEventData.ends.setMonth($scope.createEventData.starts.getMonth());
            $scope.createEventData.ends.setFullYear($scope.createEventData.starts.getFullYear());
            $scope.createEventData.ends.setHours($scope.createEventData.starts.getHours() + 1);
            $scope.createEventData.ends.setMinutes($scope.createEventData.starts.getMinutes());
        } else {
            $scope.createEventData.ends.setHours(hour);
            $scope.createEventData.ends.setMinutes($scope.selectedMinute);
        }
        $scope.isTimePicker = -1;
    }

    function cancelAddEvent() {
        SHPService.sendNativeCommand(REQUEST.CLOSE_KEYPAD, REQUEST.CLOSE_KEYPAD, "");
        $timeout(function() {$rootScope.$broadcast('update', {'text' : $scope.currentDate});}, 0);
        $timeout(function() {$rootScope.$broadcast('updateHourView', {'text' : $scope.currentDate});}, 0);
        $timeout(function() {$rootScope.$broadcast('updateAgendaView', {'text' : $scope.currentDate});}, 0);
        $timeout(function() {$rootScope.$broadcast('updateSelectionDate', {'text' : $scope.currentDate});}, 0);
        $scope.currOption = $scope.dropDownSelection[0];
        resetAddPageModel();
        onPressed(1);
    }
    
    function getStringFromArray() {
        var temp = [];
        if ($scope.repeatDataStructure.daily.SU) {
            temp.push("SU");
        }
        if ($scope.repeatDataStructure.daily.MO) {
            temp.push("MO");
        }
        if ($scope.repeatDataStructure.daily.TU) {
            temp.push("TU");
        }
        if ($scope.repeatDataStructure.daily.WE) {
            temp.push("WE");
        }
        if ($scope.repeatDataStructure.daily.TH) {
            temp.push("TH");
        }
        if ($scope.repeatDataStructure.daily.FR) {
            temp.push("FR");
        }
        if ($scope.repeatDataStructure.daily.SA) {
            temp.push("SA");
        }

        return temp.join(',');
    }

    function getCustomRule() {
        var temp = '';
        var unit = $scope.repeatUnitData[$scope.repeatDataStructure.custom.unitIndex];
        if (unit === $scope.repeatUnitData[0]) {    //custom-day
            temp += 'DAILY;INTERVAL=' + $scope.repeatDataStructure.custom.value;
        } else if (unit === $scope.repeatUnitData[1]) {    //custom-week
            temp += 'WEEKLY;INTERVAL=' + $scope.repeatDataStructure.custom.value;
        } else if (unit === $scope.repeatUnitData[2]) {    //custom-month
            temp += 'MONTHLY;INTERVAL=' + $scope.repeatDataStructure.custom.value;
        } else if (unit === $scope.repeatUnitData[3]) {    //custom-year
            temp += 'YEARLY;INTERVAL=' + $scope.repeatDataStructure.custom.value;
        }
        return temp;
    }

    function prepareRepeatRule() {
        var rRule = '';
        switch ($scope.repeatDataStructure.selected) {
            case 0:
                rRule = null;
                break;
            case 1: //Daily
                if($scope.repeatDataStructure.daily.AL) { // For All days
                    rRule = '["RRULE:FREQ=WEEKLY;BYDAY=SU,MO,TU,WE,TH,FR,SA"]';
                } else {
                    rRule = '["RRULE:FREQ=WEEKLY;BYDAY=' + getStringFromArray() + '"]';
                }
                break;
            case 2: //Weekly
                rRule = '["RRULE:FREQ=WEEKLY;"]';
                break;
            case 3: //Monthly
                if($scope.repeatDataStructure.monthly.onDate) {
                    rRule = '["RRULE:FREQ=MONTHLY"]';
                } else {
                    var checkDay = $scope.createEventData.starts.getDate();
                        checkDay = checkDay === 0 ? 6 : checkDay -1;
                    var dayAbbr = ['SU','MO','TU','WE','TH','FR','SA'],
                        day = $scope.createEventData.starts.getDay(),
                        weekNumber = [1,2,3,4,5],
                        week = weekNumber[0 | checkDay/7];
                    rRule = '["RRULE:FREQ=MONTHLY;BYDAY=' + week + dayAbbr[day] + '"]';
                    console.log("rRule " + rRule);
                }
                break;
            case 4: //Yearly
                rRule = '["RRULE:FREQ=YEARLY"]';
                break;
            case 5: //Custom
                rRule = '["RRULE:FREQ=' + getCustomRule() + '"]';
                break;
        }
        //console.log("recurring rules : " + rRule, $scope.repeatDataStructure.selected);
        return (rRule == null)? null : JSON.parse(rRule);
    }

    function prepareReminderRule() {
        //  console.log("prepareReminderRule");
        var rRule = '';
        switch ($scope.alertDataStructure.selected) {
            case 0:
                if($scope.currOption.emailType === 'outlook'){
                    rRule = null;
                }else{
                    rRule = '{"overrides": [], "useDefault": false}';
                }
                break;
            case 1: //at Start Time
                rRule = '{"overrides": [{"method": "email", "minutes": 0}], "useDefault": false}';
                break;
            case 2: //10 Minutes
                rRule = '{"overrides": [{"method": "email", "minutes": 10}], "useDefault": false}';
                break;
            case 3: //1 Day
                rRule = '{"overrides": [{"method": "email", "minutes": 1440}], "useDefault": false}';
                break;
            case 4: //1 Week
                rRule = '{"overrides": [{"method": "email", "minutes": 10080}], "useDefault": false}';
                break;
            case 5: //Custom
                var minutes = 0;
                if($scope.alertUnitData[$scope.alertDataStructure.custom.unitIndex] === $scope.alertUnitData[2]) {  //Weeks
                    minutes = $scope.alertDataStructure.custom.value * 1440 * 7;
                } else if($scope.alertUnitData[$scope.alertDataStructure.custom.unitIndex] === $scope.alertUnitData[1]) {   //Hours
                    minutes = $scope.alertDataStructure.custom.value * 60;
                } else { //Minutes
                    minutes = $scope.alertDataStructure.custom.value;
                }
                rRule = '{"overrides": [{"method": "email", "minutes": ' + minutes + '}], "useDefault": false}';
                break;
        }

        return (rRule == null)? null : JSON.parse(rRule);
    }

    function decodeHtmlEntity(html) {
        var txt = document.createElement("textarea");
        txt.innerHTML = html;
        return txt.value;
    }

    function decodeUnescape(str) {
        if (angular.isDefined(str) && str != "") {
            try {
                str = decodeURIComponent(str);
                return decodeHtmlEntity(str);
            }
            catch(err) {
                $scope.showConsoleLogs &&  console.log("In decodeUnescape Error::: str = " + str);
                $scope.showConsoleLogs &&  console.log("err = " + JSON.stringify(err));
                return decodeHtmlEntity(str);
            }
        }
        return "";
    }

    function sendAddEventRequest() {
        $scope.createEventData.summay = document.getElementById("enterTitle").value;
        if ($scope.createEventData.starts.getTime() > $scope.createEventData.ends.getTime()) {
            showInvalidDate();
            return;
        }
        if($scope.createEventData.summay === "" || $scope.createEventData.summay.length === 0){
            SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_calendar_event_title_incomplete_toast, "");
            return;
        }
        var attendessData = null;

        if ($scope.createEventData.arrayDynamicallyAddedEmail.length > 0) {
            $scope.createEventData.arrayDynamicallyAddedEmail.forEach(function (value) {
                attendessData = attendessData === null ? '' : attendessData;
                if (value !== null && value !== '')
                    ;
                attendessData += '{"email": "' + value + '"},';
            });
        }

        $scope.profileData.forEach(function(value) {
            if(value.selected) {
                attendessData = attendessData === null ? '' : attendessData;
                if((angular.isDefined(value.email)) && value.email !== null && value.email !== '')
                    attendessData += '{"email": "' + value.email + '"},';
            }
        });

        attendessData = attendessData === null ? [] : JSON.parse('['+attendessData.slice(0, attendessData.length-1)+']');

        var startsDate = $scope.createEventData.starts;
        var endsDate = $scope.createEventData.ends;

        // If allDay true, then send time in 00:00:00 format
        if ($scope.createEventData.allDay) {
            startsDate = new Date(Date.UTC($scope.createEventData.starts.getFullYear(), $scope.createEventData.starts.getMonth(), $scope.createEventData.starts.getDate(), 0, 0, 0, 0));
            endsDate = new Date(Date.UTC($scope.createEventData.ends.getFullYear(), $scope.createEventData.ends.getMonth(), $scope.createEventData.ends.getDate(), 0, 0, 0, 0));
            endsDate.setDate(endsDate.getDate() + 1);
        }

        var payload = {
            calendarId: $scope.currOption.defaultCalendarId,
            event: {
                starts: startsDate.toISOString(),
                ends: endsDate.toISOString(),
                allDay: $scope.createEventData.allDay,
                summary: $scope.createEventData.summay,
                description: $scope.createEventData.desc,
                location: $scope.createEventData.location,
                attendees: attendessData,
                recurrence: prepareRepeatRule(),
                reminders: prepareReminderRule(),
                isOrganizer: true
            }
        };

        console.log("payload param for add event ", payload);

//        var s = '{"calendarId":"58aa7d640693ce18773819b4","event":{"starts":"2017-02-20T00:00:00.000Z","ends":"2017-02-21T00:00:00.000Z","allDay":true,"summary":"Test20","description":"","location":"","attendees":[],"recurrence":null,"reminders":null}}'
//        payload = JSON.parse(s);
        $scope.checkResponse.push(CALCONST.CREATE_REQUEST);
        SHPService.sendNativeCommand("createEvent", payload, CALCONST.CREATE_REQUEST);
    }

    function showDeletePopup(checkForDeclinMessage) {
        $scope.dialogPopup.show = true;
        toggleBottomPopUpClass(true);
        $scope.dialogPopup.dialogPopupTitle = "";
        $scope.dialogPopup.dialogPopupText = $scope.translation.WEBMOB_fhub2_calendar_event_delete_popup;
        $scope.dialogPopup.button2Enabled = true;
        $scope.dialogPopup.button2Title = $scope.translation.WEBMOB_fhub2_common_ok;
        $scope.dialogPopup.button1Title = $scope.translation.WEBMOB_fhub2_common_cancel;
        $scope.dialogPopup.button2Handler = function () {
            $scope.dialogPopup.show = false;
            $scope.dialogPopup.button2Enabled = false;
            toggleBottomPopUpClass(false);
            deleteEvent(checkForDeclinMessage);
            //$scope.eventActionMessage = $scope.translation.WEBMOB_fhub2_calendar_event_detail_decline_toast;
        };
        $scope.dialogPopup.button1Handler = function () {
            $scope.dialogPopup.show = false;
            $scope.dialogPopup.button1Enabled = false;
            $scope.dialogPopup.button2Enabled = false;
            toggleBottomPopUpClass(false);
        };
    }

    function deleteEvent(checkForDeclinMessage) {
        if (checkForDeclinMessage === 'decline') {
            $scope.showDeclineToastOnDeleteSuccess = true;
            $scope.eventActionMessage = $scope.translation.WEBMOB_fhub2_calendar_event_detail_decline_toast;
        }
        var param = '{"calendarId" : "' + $scope.profileObj.defaultCalendarId + '", "eventId" : "' + $scope.hourEventObj.id + '"}';
        $scope.checkResponse.push(CALCONST.DELETE_REQUEST);
        SHPService.sendNativeCommand("deleteEvent", param, CALCONST.DELETE_REQUEST);
    }

    function updateEvent() {
        if ($scope.createEventData.starts.getTime() > $scope.createEventData.ends.getTime()) {
            showInvalidDate();
            return;
        }
        if ($scope.createEventData.summay === "" || $scope.createEventData.summay.length === 0) {
            SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_calendar_event_title_incomplete_toast, "");
            return;
        }
        var attendessData = null;

        if ($scope.createEventData.arrayDynamicallyAddedEmail.length > 0) {
            $scope.createEventData.arrayDynamicallyAddedEmail.forEach(function (value) {
                attendessData = attendessData === null ? '' : attendessData;
                if (value !== null && value !== '')
                    ;
                attendessData += '{"email": "' + value + '"},';
            });
        }

        $scope.profileData.forEach(function(value) {
            if(value.selected) {
                attendessData = attendessData === null ? '' : attendessData;
                if(value.email !== null && value.email !== '')
                    attendessData += '{"email": "' + value.email + '"},';
            }
        });

        attendessData = attendessData === null ? [] : JSON.parse('['+attendessData.slice(0, attendessData.length-1)+']');

        var startsDate = $scope.createEventData.starts;
        var tempEndDate = $scope.createEventData.ends;
        var endsDate = $scope.createEventData.ends;


        // If allDay true, then send time in 00:00:00 format
        if ($scope.createEventData.allDay) {
            startsDate = new Date(Date.UTC($scope.createEventData.starts.getFullYear(), $scope.createEventData.starts.getMonth(), $scope.createEventData.starts.getDate(), 0, 0, 0, 0));
            endsDate = new Date(Date.UTC($scope.createEventData.ends.getFullYear(), $scope.createEventData.ends.getMonth(), $scope.createEventData.ends.getDate(), 0, 0, 0, 0));
            if(Number($scope.endDateBeforeEditEvent) !== Number(tempEndDate)){
                endsDate.setDate(endsDate.getDate() + 1);
            }
        }

        var payload = {
            calendarId: $scope.currOption.defaultCalendarId,
            eventId: $scope.hourEventObj.id,
            event: {
                starts: startsDate.toISOString(),
                ends: endsDate.toISOString(),
                allDay: $scope.createEventData.allDay,
                summary: $scope.createEventData.summay,
                description: $scope.createEventData.desc,
                location: $scope.createEventData.location,
                attendees: attendessData,
                recurrence: prepareRepeatRule(),
                reminders: prepareReminderRule(),
                isOrganizer: true
            }
        };

        console.log("update  param ", payload);
        $scope.checkResponse.push(CALCONST.UPDATE_REQUEST);
        SHPService.sendNativeCommand("updateEvent", payload, CALCONST.UPDATE_REQUEST);
    }


    function sendInvitationRequest(event, response) {
        console.log(" sendInvitationRequest ");
        if(response === 'accept') {
            $scope.eventActionMessage = $scope.translation.WEBMOB_fhub2_calendar_event_detail_accept_toast;
        } else if(response === 'decline') {
            $scope.showDeletePopup('decline');
            //$scope.eventActionMessage = $scope.translation.WEBMOB_fhub2_calendar_event_detail_decline_toast;
            return;
        } else {
            $scope.eventActionMessage = $scope.translation.WEBMOB_fhub2_calendar_event_detail_maybe_toast;
        }

        if($scope.isStatic) {
            $scope.checkResponse.push(CALCONST.RESPONSE_INVITATION);
            requestResponseHandler('{"type":"responseInvitation", "status":200, "requestToken":"responseInvitation","data":{}}');
        } else {
            var payload = {
                calendarId: event.calendar,
                eventId: event.id,
                response : response
            };

            $scope.checkResponse.push(CALCONST.RESPONSE_INVITATION);
            SHPService.sendNativeCommand(CALCONST.RESPONSE_INVITATION, payload, CALCONST.RESPONSE_INVITATION);
        }
    }

    /***** Schedule Related methods  *********/
    function setSelectedAlert(selectedOption) {
        //$scope.prevSelected=$scope.alertDataStructure.selected ;
        if ($scope.alertDataStructure.selected !== selectedOption)
            resetAlertValue();
        $scope.alertDataStructure.selected = selectedOption;
        switch (selectedOption) {
            case 0: //None
                break;
            case 1: //At start time
                break;
            case 2: //10 minutes
                break;
            case 3: //1 day
                break;
            case 4: //1 week
                break;
            case 5: // Custom
                $scope.isAlertPopupOpen = true;
                initAlertList($scope.selectedAlertVal-1);
                initAlertUnitList($scope.selectedAlertUnit);
                break;
        }
    }

    function setSelectedRepeat(selectedOption) {
        if ($scope.repeatDataStructure.selected !== selectedOption)
            resetRepeatValue();
        $scope.repeatDataStructure.selected = selectedOption;
        switch (selectedOption) {
            case 0: //None
                break;
            case 1: //Daily
                $scope.isRepeatPopupOpen = true;
                break;
            case 2: //Weekly
                break;
            case 3: //Monthly
                $scope.isRepeatPopupOpen = true;
                break;
            case 4: //Yearly
                break;
            case 5: //Custom
                $scope.isRepeatPopupOpen = true;
                initRepeatList($scope.selectedRepeatVal-1);
                initRepeatUnitList($scope.selectedRepeatUnit);
                break;
        }
    }

    function disableDate() {
        var flag = false;
        if ($scope.currentScreen === CALCONST.SCREEN_HOME)
            flag = true;

        return flag;
    }


    function isTodayButtonEnable() {
        var today = new Date();
        return !($scope.currentDate.getDate() == today.getDate() && $scope.currentDate.getMonth() == today.getMonth() && $scope.currentDate.getFullYear() == today.getFullYear());
    }

    function changeHourOrAgendaView() {
        if(!$scope.isAgendaView) {
            $rootScope.$broadcast('updateAgendaView', {'text' : 'Hi There'});
        }
        else {
            $rootScope.$broadcast('updateHourView', {'text' : $scope.currentDate});
        }
        $scope.isAgendaView = !$scope.isAgendaView;
        $scope.visibleAddEventGridLayout = false;
        setViewContainerHeight();
    }

    $scope.$on('updateWeeks', function (event, weeks) {
        $scope.weeks = weeks;
        $scope.setMonthViewHeight($scope.isWeekView);
    });

    function onTouchStart(event) {
        //   console.log(" onTouchStart ");
        var touch = event.touches[0];
        var x = touch.pageX;
        var y = touch.pageY;
        $scope.prevY = y;
    }

    function onTouchMove(event) {
        var touch = event.touches[0];
        var x = touch.pageX;
        var y = touch.pageY;

        var cHeight = 0, base = 0;
        cHeight = window.screen.availHeight;
        if (cHeight === 0 || cHeight === undefined) {
            cHeight = document.body.clientHeight;
        }

        var viewContainer;
        if ($scope.isAgendaView) {
            viewContainer = document.getElementById("agendaListContainer");
        } else {
            viewContainer = document.getElementById("hourViewContainer");
        }
        var calendarContainer = document.getElementById("calendarViewContainer");
        var oobeContainer = document.getElementById("oobeContainer");
        if($scope.showOOBEScreen){
            oobeContainer.style.height= "14rem";
        }

        var top = viewContainer.getBoundingClientRect().top;
        top = top * (2560 / cHeight);

        var offset = $scope.prevY - y;

        if (offset > 0) { // Direction up
            if ((top - offset) <= CALCONST.TOP_POINT) {
                $scope.isWeekView = true;
                moveToTop(viewContainer, calendarContainer, top, cHeight);
                return;
            }
        } else {
            if ($scope.showOOBEScreen && !$scope.isWeekView) {
                $scope.isWeekView = false;
                return;
            }
            if ((top + offset) > CALCONST.BOTTOM_POINT) {
                $scope.isWeekView = false;
                moveToBottom(viewContainer, calendarContainer, top, cHeight);
                return;
            }

        }

        var a = calendarContainer.clientHeight - offset;
        var b = viewContainer.clientHeight + offset;

        var ratio = (2560 / cHeight);
        if (offset > 0) {
            a = a < (CALCONST.WEEK_VIEW_HEIGHT / ratio) ? (CALCONST.WEEK_VIEW_HEIGHT / ratio) : a;
            b = b > (CALCONST.HOUR_VIEW_HEIGHT_FULL / ratio) ? (CALCONST.HOUR_VIEW_HEIGHT_FULL / ratio) : b;
        } else {
            if ($scope.showOOBEScreen) {
                oobeContainer.style.height= "10rem";
                a = a > (CALCONST.OOBE_CALENDAR_VIEW_HEIGHT_6ROW / ratio) ? (CALCONST.OOBE_CALENDAR_VIEW_HEIGHT_6ROW / ratio) : a;
                b = b < (CALCONST.OOBE_VIEW_HEIGHT / ratio) ? (CALCONST.OOBE_VIEW_HEIGHT / ratio) : b;
            } else {
                a = a > (CALCONST.CALENDAR_VIEW_HEIGHT_6ROW / ratio) ? (CALCONST.CALENDAR_VIEW_HEIGHT_6ROW / ratio) : a;
                b = b < (CALCONST.HOUR_VIEW_HEIGHT_6ROW / ratio) ? (CALCONST.HOUR_VIEW_HEIGHT_6ROW / ratio) : b;

            }
        }

        calendarContainer.style.height = a;
        viewContainer.style.height = b;

        $scope.prevY = y;

        setViewContainerHeight();
    }

    function onTouchEnd(event) {
        var touch = event.touches[0] || event.targetTouches[0] || event.changedTouches[0];
        var x = touch.pageX;
        var y = touch.pageY;
        var offset = $scope.prevY-y;

        var cHeight = 0, base = 0;
        cHeight = window.screen.availHeight;
        if (cHeight === 0 || cHeight === undefined) {
            cHeight = document.body.clientHeight;
        }

        var viewContainer;
        if($scope.isAgendaView) {
            viewContainer = document.getElementById("agendaListContainer");
        } else {
            viewContainer = document.getElementById("hourViewContainer");
        }

        var calendarContainer = document.getElementById("calendarViewContainer");
        var top = viewContainer.getBoundingClientRect().top;

        top = top * (2560/cHeight);
        if(offset >= 0 ) { //Direction Up
            if(top < CALCONST.MID_POINT_TOP_DIR && !$scope.isWeekView) {
                moveToTop(viewContainer, calendarContainer, top, cHeight);
            } else if(top > CALCONST.MID_POINT_BOTTOM_DIR) {
                moveToBottom(viewContainer, calendarContainer, top, cHeight);
            }
        } else if (offset < 0 && top > CALCONST.MID_POINT) { //Direction Down
        }

        $scope.prevY = 0;
        setViewContainerHeight();
    }

    $scope.setMonthViewHeight = setMonthViewHeight;
    function setMonthViewHeight(isWeekView){
        var calendarContainer = document.getElementById("calendarViewContainer");

        var monthViewHeight = 0;
        if (isWeekView) {
            monthViewHeight = getWeekViewHeight();
        }
        else {
            monthViewHeight = getMonthViewHeight();
        }
        calendarContainer.style.height = monthViewHeight;

        setViewContainerHeight();
        $scope.isWeekView = isWeekView;
    }

    function setViewContainerHeight() {
        var baseFontSize = parseFloat(document.getElementById("htmlTag").style.fontSize.replace('px', ''));
        var calendarContainer = $('#calendarViewContainer');
        var viewContainer = $('#hourViewContainer');

        //calculate dayView & agendaView
        var viewContainerHeight = $('.cal-container').height() - calendarContainer.height();

        if($scope.isAgendaView) {
            viewContainer = $('#agendaViewContainer');
            var agendaListContainer = $('#agendaListContainer');

            //apply monthView size refer to p22 of GUI guide v1.05
            var agendaListContainerHeight = viewContainerHeight - ((156 / 80) * baseFontSize);
            agendaListContainer.height(agendaListContainerHeight);
            console.log("calculate agendaListContainer>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>",  agendaListContainerHeight);
        }
        viewContainer.height(viewContainerHeight);

        console.log('cal-container = ', $('.cal-container').height(), ' viewContainer.clientHeight = ', calendarContainer.height());
        console.log("calculate viewContainer>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>",  viewContainerHeight);

    }

    function getMonthViewHeight() {
        var baseFontSize = parseFloat(document.getElementById("htmlTag").style.fontSize.replace('px', ''));
        var height = 0;
        if (angular.isDefined($scope.weeks)) {
            //apply monthView size refer to p15~17 of GUI guide v1.05
            if (!$scope.showOOBEScreen)
                height += 208; //profile area
            height += 168; //top of month view
            height += (172 * $scope.weeks.length);
            height += 84;   //bottom margin
            console.log('MonthView height = ' + height + 'px / height = ' + height / 80 + 'rem');
            height = (height / 80) * baseFontSize;  //convert px to rem and calculate px according to basefont.
            console.log('getMonthViewHeight::: weeks = ',$scope.weeks, ' height = ', height);
        }
        return height;
    }

    function getWeekViewHeight() {
        var baseFontSize = parseFloat(document.getElementById("htmlTag").style.fontSize.replace('px', ''));
        //apply weekView size refer to p6 of GUI guide v1.05
        var height = (428 / 80) * baseFontSize;  //convert px to rem of weekView height and calculate px according to basefont.
        console.log('getWeekViewHeight::: height = ', height);
        return height;
    }

    function validMonthViewHeight(height) {
        return (angular.isDefined(height) && height >= 0 && height <= document.body.clientHeight);
    }

    function moveToTop(viewContainer, calendarContainer, t, cHeight) {
        var top = t;
        var weekViewHeight = getWeekViewHeight();
        console.log('moveToTop::: weekviewHeight', weekViewHeight, ' calendarContainer.clientHeight = ', calendarContainer.clientHeight);

        if(calendarContainer.clientHeight < weekViewHeight) {
            for (var i = calendarContainer.clientHeight; calendarContainer.clientHeight > weekViewHeight && validMonthViewHeight(calendarContainer.clientHeight); i++) {
                calendarContainer.style.height = calendarContainer.clientHeight - 1;
                viewContainer.style.height = viewContainer.clientHeight + 1;

                top = viewContainer.getBoundingClientRect().top;
                top = top * (2560 / cHeight);
            }
        }
        else {
            calendarContainer.style.height = viewContainer.clientHeight = weekViewHeight;
        }

        setMonthViewHeight(true);
    }

    function moveToBottom(viewContainer, calendarContainer, top, cHeight) {
        var monthViewHeight = getMonthViewHeight();
        console.log('moveToBottom::: monthViewHeight = ', monthViewHeight, ' calendarContainer.clientHeight = ', calendarContainer.clientHeight);

        if(calendarContainer.clientHeight < monthViewHeight) {
            for (var i = calendarContainer.clientHeight; calendarContainer.clientHeight < monthViewHeight && validMonthViewHeight(calendarContainer.clientHeight); i++) {
                calendarContainer.style.height = calendarContainer.clientHeight + 1;
                viewContainer.style.height = viewContainer.clientHeight - 1;

                top = viewContainer.getBoundingClientRect().top;
                top = top * (2560 / cHeight);
            }
        }
        else {
            calendarContainer.style.height = viewContainer.clientHeight = monthViewHeight;
        }

        setMonthViewHeight(false);
    }

    function selectToday() {
        $scope.visibleAddEventGridLayout = false;
        $scope.weeks =  [];
        $scope.calObj = new Date();
        var today = new Date();
        var tempDate = new Date();
        tempDate = $scope.currentDate;
        //if year changed, than get the next year event data
        if($scope.currentDate.getFullYear() !== today.getFullYear()) {
            $scope.currentDate = today;
            $rootScope.$broadcast('updateCalObj', {'text' : today});
            $scope.updateCalValue(today);
            $scope.getCalendarEventData();
            return;
        }

        $scope.currentDate =  today;
        $scope.updateCalValue(today);
        if(tempDate.getMonth() !== today.getMonth()) {
            $scope.getCalendarEventData();
        }
        $timeout(function() {$rootScope.$broadcast('update', {'text' : $scope.calObj});}, 10);
        $rootScope.$broadcast('updateHourView', {'text' : $scope.calObj});
        $rootScope.$broadcast('updateAgendaView', {'text' : $scope.calObj});
    }
    
    $scope.calendarsData = [];
    $scope.eventData = [];
    $scope.profileData = [];
    $scope.reserveProfileFilteredState = [];

    function init() {
    }


    /**
     * @function
     * @private
     * @name startApp
     * @param
     * @description All neccessory operation perform to launch home screen of app.
     **/
    function startApp() {
//        var hasLaunchedBefore = getLocalStorageData('firstTimeLaunch');
//        if (hasLaunchedBefore !== "false") {
//            $scope.showOOBEScreen = true;
//            setLocalStorageData('firstTimeLaunch', false);
//        }
        var cWidth = 0;
        if (cWidth === 0 || cWidth === undefined) {
            cWidth = document.body.clientWidth;
        }
        baseFont = (cWidth * 80)/1440;

        $state.transitionTo('HomePage');
        $scope.currentScreen = CALCONST.SCREEN_HOME;
        getCalendarsForFamily();
    }

    function getCalendarEventData() {
        if (!$scope.isStatic) {
            var timeMin, timeMax;
            $scope.requestedDate = $scope.currentDate;
            var prevMonth = $scope.requestedDate.getMonth(); // one previous month data as date object returns month from 0~11 format (i.e. 0 as January & 11 as December)
            var lastMonth = $scope.requestedDate.getMonth() + 4; // next 3 months from current date
            var prevYear = new Date($scope.requestedDate.getFullYear(), $scope.requestedDate.getMonth()-1, 1); // previous month date to be requested
            var nextYear = new Date($scope.requestedDate.getFullYear(), $scope.requestedDate.getMonth()+3, 1); // next month date to be requested

            if($scope.requestedDate.getFullYear() !== prevYear.getFullYear()){
                var prevYearMonth = prevYear.getMonth() +1; // added one as fhub server accepts request for month 1~12 but date object returns 0~11 format (i.e. 0 as January & 11 as December)
                timeMin = prevYear.getFullYear() + '-' + prevYearMonth + '-' + 01;
            }else{
                timeMin = $scope.requestedDate.getFullYear() + '-' + prevMonth + '-' + 01;
            }
            var yearMax = $scope.requestedDate.getFullYear() ;//+ 3; //Removed fetching 3 year data for calendar ..

            if($scope.requestedDate.getFullYear() !== nextYear.getFullYear()){
                var nextYearMonth = nextYear.getMonth() +1; // added one as fhub server accepts request for month 1~12 but date object returns 0~11 format (i.e. 0 as January & 11 as December)
                timeMax = nextYear.getFullYear() + '-' + nextYearMonth + '-' + daysInMonth(nextYearMonth , nextYear.getFullYear());
            }else{
                timeMax = yearMax + '-' + lastMonth + '-' + daysInMonth(lastMonth , yearMax);
            }

            var param = '{"timeMin" : "' + timeMin + '", "timeMax" : "' + timeMax + '"}';
            $scope.isTransparent = true;        //for Transparent dim screen.
            $scope.checkResponse.push(CALCONST.EVENT_REQUEST);
            SHPService.sendNativeCommand("getCalendarEventsForFamily", param, CALCONST.EVENT_REQUEST);
        } else {
            var calResponse = '{"type":"requestResponse","status":200,"requestToken":"getCalendarEventsForFamily","data":[{"calendar":"5810cef75a534b46391003ff","oid":"20171029_60o30e3474o30c1g60o30dr560","ends":"2017-11-27T11:30:00.000Z","starts":"2017-11-27T11:00:00.000Z","reminderMinutes":null,"attendees":[null],"recurrence":null,"recurringEventId":null,"recurs":false,"location":null,"description":null,"summary":"TWENTY MIN","allDay":false,"id":"5810cefaa481fb979b79dd10"},{"calendar":"5810cef75a534b46391003ff","oid":"20171029_60o30e3474o30c1g60o30dr560","ends":"2017-11-22T06:40:00.000Z","starts":"2017-11-22T05:30:00.000Z","reminderMinutes":null,"attendees":[null],"recurrence":null,"recurringEventId":null,"recurs":false,"location":null,"description":null,"summary":"TWENTY MIN","allDay":false,"id":"5810cefaa481fb979b79dd10"},{"calendar":"5810cef75a534b46391003ff","oid":"20171029_60o30e3474o30c1g60o30dr560","ends":"2017-11-22T06:30:00.000Z","starts":"2017-11-22T05:50:00.000Z","reminderMinutes":null,"attendees":[null],"recurrence":null,"recurringEventId":null,"recurs":false,"location":null,"description":null,"summary":"TWENTY MIN","allDay":false,"id":"5810cefaa481fb979b79dd10"},{"calendar":"5810cef75a534b46391003ff","oid":"20171029_60o30e3474o30c1g60o30dr560","ends":"2017-11-22T08:05:00.000Z","starts":"2017-11-22T07:20:00.000Z","reminderMinutes":null,"attendees":[null],"recurrence":null,"recurringEventId":null,"recurs":false,"location":null,"description":null,"summary":"99 MIN","allDay":false,"id":"5810cefaa481fb979b79dd10"},{"calendar":"5810cef75a534b46391003ff","oid":"20171029_60o30e3474o30c1g60o30dr560","ends":"2017-11-23T10:40:00.000Z","starts":"2017-11-23T10:00:00.000Z","reminderMinutes":null,"attendees":[null],"recurrence":null,"recurringEventId":null,"recurs":false,"location":null,"description":null,"summary":"FORTY MINUTES","allDay":false,"id":"5810cefaa481fb979b79dd10"},{"calendar":"5810cef75a534b46391003ff","oid":"20171029_60o30e3474o30c1g60o30dr560","ends":"2017-11-24T10:10:00.000Z","starts":"2017-11-24T09:00:00.000Z","reminderMinutes":null,"attendees":[null],"recurrence":null,"recurringEventId":null,"recurs":false,"location":null,"description":null,"summary":"ONE HOUR 10 MINUTES","allDay":false,"id":"5810cefaa481fb979b79dd10"},{"calendar":"5810cef75a534b46391003ff","oid":"20171029_60o30e3474o30c1g60o30dr560","ends":"2017-11-25T02:30:00.000Z","starts":"2017-11-25T00:00:00.000Z","reminderMinutes":null,"attendees":[null],"recurrence":null,"recurringEventId":null,"recurs":false,"location":null,"description":null,"summary":"ONE HOUR 10 MINUTES","allDay":false,"id":"5810cefaa481fb979b79dd10"},{"calendar":"5810cef75a534b46391003ff","oid":"20171029_60o30e3474o30c1g60o30dr560","ends":"2017-11-25T00:30:00.000Z","starts":"2017-11-25T00:00:00.000Z","reminderMinutes":null,"attendees":[null],"recurrence":null,"recurringEventId":null,"recurs":false,"location":null,"description":null,"summary":"ONE HOUR 10 MINUTES","allDay":false,"id":"5810cefaa481fb979b79dd10"},{"calendar":"5810cef75a534b46391003ff","oid":"20171029_60o30e3474o30c1g60o30dr560","ends":"2017-11-25T00:00:00.000Z","starts":"2017-11-24T00:00:00.000Z","reminderMinutes":null,"attendees":[null],"recurrence":null,"recurringEventId":null,"recurs":false,"location":null,"description":null,"summary":"New Holiday","allDay":true,"id":"5810cefaa481fb979b79dd10"},{"calendar":"5810cef75a534b46391003ff","oid":"20171029_60o30e3474o30c1g60o30dr560","ends":"2017-11-25T00:30:00.000Z","starts":"2017-11-24T23:00:00.000Z","reminderMinutes":null,"attendees":[null],"recurrence":null,"recurringEventId":null,"recurs":false,"location":null,"description":null,"summary":"ONE HOUR 10 MINUTES","allDay":false,"id":"5810cefaa481fb979b79dd10"},{"calendar":"5810cef75a534b46391003ff","oid":"20171029_60o30e3474o30c1g60o30dr560","ends":"2017-11-25T01:00:00.000Z","starts":"2017-11-25T00:00:00.000Z","reminderMinutes":null,"attendees":[null],"recurrence":null,"recurringEventId":null,"recurs":false,"location":null,"description":null,"summary":"Holiday","allDay":false,"id":"5810cefaa481fb979b79dd10"},{"calendar":"5810cef75a534b46391003ff","oid":"20171029_60o30e3474o30c1g60o30dr560","ends":"2017-11-24T22:30:00.000Z","starts":"2017-11-24T17:30:00.000Z","reminderMinutes":null,"attendees":[null],"recurrence":null,"recurringEventId":null,"recurs":false,"location":null,"description":null,"summary":"New Holiday","allDay":false,"id":"5810cefaa481fb979b79dd10"},{"calendar":"5810cef75a534b46391003ff","oid":"20171029_60o30e3474o30c1g60o30dr560","ends":"2017-11-16T00:00:00.000Z","starts":"2017-11-16T00:00:00.000Z","reminderMinutes":null,"attendees":[null],"recurrence":null,"recurringEventId":null,"recurs":false,"location":null,"description":null,"summary":"holiday Chaturdasi","allDay":true,"id":"5810cefaa481fb979b79dd10"},{"calendar":"5810cef75a534b46391003ff","oid":"20171029_60o30e3474o30c1g60o30dr560","ends":"2017-11-17T00:00:00.000Z","starts":"2017-11-17T00:00:00.000Z","reminderMinutes":null,"attendees":[null],"recurrence":null,"recurringEventId":null,"recurs":false,"location":null,"description":null,"summary":"Naraka Chaturdasi","allDay":false,"id":"5810cefaa481fb979b79dd10"},{"calendar":"5810cef75a534b46391003ff","oid":"20171030_60o30d9g6go30c1g60o30dr560","ends":"2017-10-31T00:00:00.000Z","starts":"2017-10-30T00:00:00.000Z","reminderMinutes":null,"attendees":[null],"recurrence":null,"recurringEventId":null,"recurs":false,"location":null,"description":null,"summary":"Diwali/Deepavali","allDay":true,"id":"5810cefaa481fb979b79dd11"},{"calendar":"5810cef75a534b46391003fd","oid":"20171031_60o30d9h70o30c1g60o30dr560","ends":"2017-11-01T00:00:00.000Z","starts":"2017-10-31T00:00:00.000Z","reminderMinutes":null,"attendees":[null],"recurrence":null,"recurringEventId":null,"recurs":false,"location":null,"description":null,"summary":"Govardhan Puja","allDay":true,"id":"5810cefaa481fb979b79dd12"},{"calendar":"5810cef75a534b46391003fd","oid":"20171101_60o30d9h74o30c1g60o30dr560","ends":"2017-11-01T14:00:00.000Z","starts":"2017-11-01T10:00:00.000Z","reminderMinutes":null,"attendees":[null],"recurrence":null,"recurringEventId":null,"recurs":false,"location":null,"description":null,"summary":"Bhai Duj","allDay":false,"id":"5810cefaa481fb979b79dd13"},{"calendar":"5810cef75a534b46391003fd","oid":"20171106_60o30e34c4o30c1g60o30dr560","ends":"2017-11-06T18:30:00.000Z","starts":"2017-11-06T17:30:00.000Z","reminderMinutes":null,"attendees":[null],"recurrence":null,"recurringEventId":null,"recurs":false,"location":null,"description":null,"summary":"Chhat Puja (Pratihar Sashthi/Surya Sashthi)","allDay":false,"id":"5810cefaa481fb979b79dd14"},{"calendar":"5810cef75a534b46391003fe","oid":"20171124_60o30d9hc4o30c1g60o30dr560","ends":"2017-11-25T00:00:00.000Z","starts":"2017-11-24T00:00:00.000Z","reminderMinutes":null,"attendees":[null],"recurrence":null,"recurringEventId":null,"recurs":false,"location":null,"description":null,"summary":"Guru Tegh Bahadur&#39;s Martyrdom Day","allDay":true,"id":"5810cefaa481fb979b79dd15"}]}';
            requestResponseHandler(JSON.parse(calResponse));
        }
    }

    // to check no. of days in given month of the year
    function daysInMonth(month, year) {
        return new Date(year, month, 0).getDate();
    }

    function getCalendarsForFamily() {
        if (!$scope.isStatic) {
            $scope.isTransparent = true;        //for Transparent dim screen.
            $scope.checkResponse.push(CALCONST.CAL_REQUEST);
            SHPService.sendNativeCommand("getCalendarsForFamily", null, CALCONST.CAL_REQUEST);
        } else {
            var calResponse = JSON.parse('{"type":"requestResponse","status":200,"requestToken":"getCalendarsForFamily","data":[{"createdAt":"2017-10-26T15:42:47.693Z","updatedAt":"2017-10-26T15:42:55.698Z","name":"samystar16@gmail.com","oid":"samystar16@gmail.com","owner":"2adc1400-04ff-4353-979d-3b27edd0bdea","canWrite":true,"subscriptionUUID":"6c24f7d1-e53e-4f5e-bfb8-f720f26ef8d1","isShared":false,"source":"GoogleCalendar","isDefault":true,"id":"5810cef75a534b46391003fd"},{"createdAt":"2017-10-26T15:42:47.698Z","updatedAt":"2017-10-26T15:42:56.111Z","name":"Contacts","oid":"#contacts@group.v.calendar.google.com","owner":"2adc1400-04ff-4353-979d-3b27edd0bdeb","canWrite":false,"isShared":false,"source":"GoogleCalendar","isDefault":true,"id":"5810cef75a534b46391003fe"},{"createdAt":"2017-10-26T15:42:47.700Z","updatedAt":"2017-10-26T15:42:56.372Z","name":"Holidays in India","oid":"en.indian#holiday@group.v.calendar.google.com","owner":"2adc1400-04ff-4353-979d-3b27edd0bdec","canWrite":false,"isShared":false,"source":"GoogleCalendar","isDefault":false,"id":"5810cef75a534b46391003ff"},{"createdAt":"2017-10-26T15:42:47.698Z","updatedAt":"2017-10-26T15:42:56.111Z","name":"Contacts","oid":"#contacts@group.v.calendar.google.com","owner":"2adc1400-04ff-4353-979d-3b27edd0bded","canWrite":false,"isShared":false,"source":"GoogleCalendar","isDefault":false,"id":"5810cef75a534b46391003fg"},{"createdAt":"2017-10-26T15:42:47.698Z","updatedAt":"2017-10-26T15:42:56.111Z","name":"Contacts","oid":"#contacts@group.v.calendar.google.com","owner":"2adc1400-04ff-4353-979d-3b27edd0bdee","canWrite":false,"isShared":false,"source":"GoogleCalendar","isDefault":false,"id":"5810cef75a534b46391003fh"},{"createdAt":"2017-10-26T15:42:47.698Z","updatedAt":"2017-10-26T15:42:56.111Z","name":"Contacts","oid":"#contacts@group.v.calendar.google.com","owner":"2adc1400-04ff-4353-979d-3b27edd0bdef","canWrite":false,"isShared":false,"source":"GoogleCalendar","isDefault":false,"id":"5810cef75a534b46391003fi"}]}');
            requestResponseHandler(calResponse);
        }
    }

    /**
     * Checks for overlapped data, event spread for two days and adjust event lenght to min length
     * @returns {undefined}
     */
    function processData() {
        processAllDayEvents();
        for (var i = 0; i < $scope.eventData.length; i++) {
//            if (!$scope.eventData[i].allDay) {

            var end = new Date($scope.eventData[i].ends);
            var start = new Date($scope.eventData[i].starts);
            $scope.eventData[i].heightEnds = end.toISOString();
            $scope.eventData[i].heightStarts = start.toISOString();
            
            if(end.getDate() === start.getDate() && end.getMonth() === start.getMonth() && end.getFullYear() === start.getFullYear())
                 continue;
            
            if(end.getMonth() === start.getMonth() && end.getFullYear() === start.getFullYear()){
                if(end.getDate() === start.getDate() + 1 && end.getHours() === 0 && end.getMinutes() === 0)
                    continue;
            }   
            end = ($scope.eventData[i].allDay)?end.setDate(end.getDate()-1):end;
            var end_hours = Number(end);
            var selected_day_end = new Date(start.getFullYear(), start.getMonth(), start.getDate());
            selected_day_end.setHours(23);
            selected_day_end.setMinutes(59);

            if (end_hours > Number(selected_day_end)) {
                var prevYear = start.getFullYear();
                var prevMonth = start.getMonth();
                var prevDate = start.getDate()+1;
                var prevHour = '00';
                var prevMinute = '00';
                var nextHour = '00';
                var nextMinute = '00';
                var tempEventData = angular.copy($scope.eventData[i]);
                var newEndTime = new Date(prevYear, prevMonth, prevDate, prevHour, prevMinute);
                $scope.eventData[i].ends = newEndTime.toISOString();
                $scope.eventData[i].heightEnds = newEndTime.toISOString();
                var newStartTime = new Date(prevYear, prevMonth, prevDate, nextHour, nextMinute);

                tempEventData.starts = newStartTime.toISOString();
                tempEventData.heightStarts = newStartTime.toISOString();
                $scope.eventData.push(tempEventData);
            }
        }

        for (var i = 0; i < $scope.eventData.length; i++) {
            var end = new Date($scope.eventData[i].ends);
            var start = new Date($scope.eventData[i].starts);
            var startNum = Number(start);
            var endNum = Number(end);
            var height = endNum - startNum;
            height = Math.floor(height / (1000 * 60));
            if (Math.floor(height / 30) === 0) {
                height = 30;
            } else {
                var dividend = Math.floor(height / 30);
                if (dividend * 30 === height)
                    ;
                else if (height >= dividend * 30 + 15)
                    height = (dividend + 1) * 30;
                else if (dividend * 30 + 15 > height)
                    height = (dividend) * 30;
            }
            var checkEnd = new Date(start.getFullYear(), start.getMonth(), start.getDate());
            checkEnd.setHours(24);
            var checkStart = new Date(start.getFullYear(), start.getMonth(), start.getDate());
            checkStart.setHours(0);
            checkStart.setMinutes(0);
            if (endNum === Number(checkEnd) && startNum !== Number(checkStart)) {
                $scope.eventData[i].heightStarts = (new Date(Number(checkEnd) - (height * 1000 * 60))).toISOString();
            } else if (Number(checkStart) === startNum && endNum !== Number(checkEnd)) {
                $scope.eventData[i].heightEnds = (new Date(Number(checkStart) + (height * 1000 * 60))).toISOString();
            } else {
                var newEndNum = startNum + height * 1000 * 60;
                var newStartNum;
                if (newEndNum > Number(checkEnd)) {
                    newEndNum = Number(checkEnd);
                    newStartNum = newEndNum - ((height - 1) * 1000 * 60);
                    var s = new Date(newStartNum);
                    var d = new Date(newEndNum);
                    $scope.eventData[i].heightStarts = s.toISOString();
                    $scope.eventData[i].heightEnds = d.toISOString();
                } else {
                    var d = new Date(newEndNum);
                    $scope.eventData[i].heightEnds = d.toISOString();
                }
            }
            $scope.eventData[i].summary = decodeUnescape($scope.eventData[i].summary);
            $scope.eventData[i].description = decodeUnescape($scope.eventData[i].description);
            $scope.eventData[i].location = decodeUnescape($scope.eventData[i].location);

            $scope.eventData[i].summary = $scope.eventData[i].summary === "null" ? "" : $scope.eventData[i].summary;
            $scope.eventData[i].description = $scope.eventData[i].description === "null" ? "" : $scope.eventData[i].description;
            $scope.eventData[i].location = $scope.eventData[i].location === "null" ? "" : $scope.eventData[i].location;
        }
        arrayfordisplay = [];
        if ($scope.eventData.length >= 1) {
            var map = {};
            for (i = 0; i < $scope.eventData.length; i++) {
                var profile = getProfileId($scope.eventData[i].calendar);
                if (map[profile] === undefined) {
                    map[profile] = Object.keys(map).length;
                }
            }

            var splitEvents = new Array(Object.keys(map).length);
            for (i = 0; i < Object.keys(map).length; i++) {
                splitEvents[i] = [];
            }
            //arrayfordisplay = [];
            for (i = 0; i < $scope.eventData.length; i++) {
                if ($scope.eventData[i].allDay) {
                    arrayfordisplay.push({count: 0, event: {end: 0, event: $scope.eventData[i], start: 0}, index: 0});
                } else {
                    var profileId = getProfileId($scope.eventData[i].calendar);
                    splitEvents[map[profileId]].push($scope.eventData[i]);
                }
            }

            for (i = 0; i < Object.keys(map).length; i++) {
                var local_arr = [];
                if (splitEvents[i].length > 0) {
                    local_arr = $scope.adjustCommonInterval(splitEvents[i]);
                    for (var j = 0; j < local_arr.length; j++) {
                        arrayfordisplay.push(local_arr[j]);
                    }
                }
            }
        }
    }

    function checkForDeletedEvent(eventData){
        var oldEventDate = JSON.stringify(eventData);
        if(angular.isDefined($scope.checkForDeletedEventId) && oldEventDate.indexOf($scope.checkForDeletedEventId.toString()) === -1 && $scope.currentScreen === CALCONST.SCREEN_EVENT){
            onPressed(CALCONST.SCREEN_HOME);
        }
    }
    
    function getProfileId(calId) {
        var profileId;
        $scope.profileData.forEach(function(value) {
            value.calendarId.forEach(function(calVal) {
                if(calVal === calId) {
                    profileId = value.userId;
                }
            });
        });
        return profileId;
    }

    function processAllDayEvents(){

        if($scope.hasProcessed)
        {
            return;
        }
        for (var i = 0; i < $scope.eventData.length; i++) {
            if ($scope.eventData[i].allDay){
                var userTimezoneOffset = new Date().getTimezoneOffset()*60000;
                $scope.eventData[i].starts = (new Date(new Date($scope.eventData[i].starts).getTime() + userTimezoneOffset)).toISOString();
                $scope.eventData[i].ends = (new Date(new Date($scope.eventData[i].ends).getTime() + userTimezoneOffset)).toISOString();
            }
        }
    }

    /**
     * @function
     * @private
     * @name requestFailHandler
     * @param {String} response
     *       JSON response received in case of request failed
     * @description Handling of request failed cases
     */
    function requestFailHandler(response) {
        if (response["type"] === "requestFailed" || response["type"] === "requestResponse") {
            $scope.checkResponse = [];
            $scope.loadingBar = false;

            if (angular.isDefined(response["status"]) && angular.isDefined(response["data"]) &&
                    angular.isDefined(response["data"]["error"]) && response["data"]["error"] === "Invalid_ID") {
                resetAddPageModel();
                resetInternalInviteeSelectedStatus();
                onPressed(CALCONST.SCREEN_HOME);

                SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_calendar_error_invalid_id, "");
            }
            // Update All views if any request failed to responed to show existing data
            processData();
            $timeout(function() {$rootScope.$broadcast('update', {'text' : $scope.currentDate});}, 30);
            $timeout(function() {$rootScope.$broadcast('updateHourView', {'text' : $scope.currentDate});}, 20);
            $timeout(function() {$rootScope.$broadcast('updateAgendaView', {'text' : $scope.currentDate});}, 10);
            
            // if (response["status"] === 500)
            //     SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_common_check_power_and_network, "");
            // else if (angular.isDefined(response["status"]) && angular.isDefined(response["data"]) && angular.isDefined(response["data"]["error"]))
            //     SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, response["status"] + ' - ' + response["data"]["error"], "");
            // else
            //     SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, response["status"], "");
        }
    }

    function requestResponseHandler(response) {
        var data = response;
        if (response['requestToken'] === REQUEST.GET_IMAGE_LIST) {
            if (angular.isDefined(response["data"])) {
                imageRequest.parseNativeResponse(response["data"]);
            }
        }

        if (data['requestToken'] === CALCONST.CAL_REQUEST) {
            console.log("<requestResponseHandler calendar data received>", response);
            if ($scope.calendarsData.length >= 0) {
                $scope.calendarsData = [];
            }
            $scope.calendarsData = data.data;
            updateProfileCalendarsIds();
            getCalendarEventData();
            dismissLoadingBar(CALCONST.CAL_REQUEST);
        } else if (data['requestToken'] === CALCONST.EVENT_REQUEST) {
            $scope.flag = true;
            if ($scope.eventData.length >= 0) {
                $scope.eventData = [];
                 $scope.hasProcessed = false;
            }
            $scope.eventData = data.data;
            checkForDeletedEvent($scope.eventData);
            processData();
            $scope.hasProcessed = true;
            dismissLoadingBar(CALCONST.EVENT_REQUEST);
            if ($scope.currentScreen !== CALCONST.SCREEN_ADD) {
                $scope.currOption = $scope.dropDownSelection[0];
            }
            $timeout(function () {$rootScope.$broadcast('update', {'text': $scope.currentDate});}, 30);
            $timeout(function() {$rootScope.$broadcast('updateHourView', {'text' : $scope.currentDate});}, 20);
            $timeout(function() {$rootScope.$broadcast('updateAgendaView', {'text' : $scope.currentDate});}, 10);

        } else if(data['requestToken'] === CALCONST.PROFILE_REQUEST) {
            console.log(" Profile Data Recieved", response);
            if($scope.profileData.length >= 0) {
                checkIfAnyProfileIsFiltered($scope.profileData);
                $scope.profileData = [];
            }
            fillProfileData(data.data);
            if ($scope.isReceiveProfileNotification) {
                $scope.isReceiveProfileNotification = false;
                getCalendarsForFamily();
            } else {
                startApp();
            }
            $scope.reserveProfileFilteredState = [];
            dismissLoadingBar(CALCONST.PROFILE_REQUEST);
            $timeout(function () {$rootScope.$broadcast('update', {'text': 'Hi There'});}, 1000);
            $timeout(function() {$rootScope.$broadcast('updateHourView', {'text' : $scope.currentDate});}, 20);
            $timeout(function() {$rootScope.$broadcast('updateAgendaView', {'text' : $scope.currentDate});}, 10);
        } else if(data['requestToken'] === CALCONST.CREATE_REQUEST) {
            console.log("Event Added, response received ");
            $scope.eventData = $scope.eventData.concat(data.data);
            resetAddPageModel();
            resetInternalInviteeSelectedStatus();
            dismissLoadingBar(CALCONST.CREATE_REQUEST);
            onPressed(CALCONST.SCREEN_HOME);
            processData();
            $timeout(function() {$rootScope.$broadcast('update', {'text' : $scope.currentDate});}, 30);
            $timeout(function() {$rootScope.$broadcast('updateHourView', {'text' : $scope.currentDate});}, 20);
            $timeout(function() {$rootScope.$broadcast('updateAgendaView', {'text' : $scope.currentDate});}, 10);
        } else if(data['requestToken'] === CALCONST.DELETE_REQUEST) {
            console.log("Event Deleted, response received");
            removeEvent($scope.hourEventObj);
            dismissLoadingBar(CALCONST.DELETE_REQUEST);
            onPressed(CALCONST.SCREEN_HOME);
            resetAddPageModel();
            processData();
            if($scope.showDeclineToastOnDeleteSuccess && $scope.eventActionMessage !== ""){
                console.log("showDeclineToastOnDeleteSuccess");
                SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.eventActionMessage, "");
                $scope.showDeclineToastOnDeleteSuccess = false;
                $scope.eventActionMessage = "";
            }
            $timeout(function() {$rootScope.$broadcast('update', {'text' : $scope.currentDate});}, 10);
            if(!$scope.isAgendaView)
                $timeout(function() {$rootScope.$broadcast('updateHourView', {'text' : $scope.currentDate});}, 10);
            else
                $timeout(function() {$rootScope.$broadcast('updateAgendaView', {'text' : $scope.currentDate});}, 10);
        } else if(data['requestToken'] === CALCONST.UPDATE_REQUEST) {
            updateEventInLocalData(data.data);
            resetAddPageModel();
            resetInternalInviteeSelectedStatus();
            onPressed(CALCONST.SCREEN_HOME);
            processData();
            $timeout(function() {$rootScope.$broadcast('update', {'text' : $scope.currentDate});}, 10);
            if(!$scope.isAgendaView)
                $timeout(function() {$rootScope.$broadcast('updateHourView', {'text' : $scope.currentDate});}, 10);
            else
                $timeout(function() {$rootScope.$broadcast('updateAgendaView', {'text' : $scope.currentDate});}, 10);
            dismissLoadingBar(CALCONST.UPDATE_REQUEST);
        } else if(data['requestToken'] === CALCONST.RESPONSE_INVITATION) {
            //getCalendarEventData();
            dismissLoadingBar(CALCONST.RESPONSE_INVITATION);
            SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.eventActionMessage, "");
            onPressed(CALCONST.SCREEN_HOME);
            $timeout(function() {$rootScope.$broadcast('update', {'text' : $scope.currentDate});}, 30);
            $timeout(function() {$rootScope.$broadcast('updateHourView', {'text' : $scope.currentDate});}, 20);
            $timeout(function() {$rootScope.$broadcast('updateAgendaView', {'text' : $scope.currentDate});}, 10);
        }
    }


    function resetAddPageModel() {
        $scope.createEventData.summay = '';
        $scope.createEventData.starts = undefined;
        $scope.createEventData.ends = undefined;
        $scope.createEventData.allDay = false;
        $scope.createEventData.location = '';
        $scope.createEventData.desc = '';
        $scope.createEventData.attendeeList = [];
        $scope.createEventData.arrayDynamicallyAddedEmail = [];
        $scope.createEventData.selectedUser = undefined;
        $scope.isEdit = false;
        resetRepeatValue();
        resetAlertValue();
        $scope.visibleAddEventGridLayout = false;

        $scope.profileData.forEach(function(value){
            value.selected = false;
        });

        //Make first element as default selection
        $scope.currOption = $scope.dropDownSelection[0];
    }

    function removeEvent(event) {
        var flag = false;
        for(var i = 0; i < $scope.eventData.length && !flag; i++) {
            if(event.id === $scope.eventData[i].id) {
                $scope.eventData.splice(i, 1);
                flag = true;
            }
        }
    }

    function updateEventInLocalData(event) {
        var flag = false;
        for(var i = 0; i < $scope.eventData.length && !flag; i++) {
            if(event.id === $scope.eventData[i].id) {
                $scope.eventData[i] = event;
                flag = true;
            }
        }
    }

    /**
     * @function
     * @private
     * @name showConnectionFailurePopup
     * @description To display connection failure popup, it is called from app.js
     */
    function showConnectionFailurePopup() {
        //  console.log(" showConnectionFailurePopup ");
        dismissLoadingBar('FAIL');
        $scope.checkResponse = [];
        $scope.dialogPopup.show = true;
        toggleBottomPopUpClass(true);
        $scope.dialogPopup.dialogPopupTitle = $scope.translation.WEBMOB_fhub2_common_connection_failure;
        $scope.dialogPopup.dialogPopupText = $scope.translation.WEBMOB_fhub2_common_check_power_and_network;
        $scope.dialogPopup.button1Title = $scope.translation.WEBMOB_fhub2_common_ok_capital;
        $scope.dialogPopup.button1Handler = function () {
            closeApp();
        };
    }

    function toggleBottomPopUpClass(popupName) {
        if (popupName) {
            $scope.BottomPopupDimBackgroundDiv = "BottomPopupDimBackgroundDivIn";
            $scope.BottomPopupDivCls = "BottomPopupBGIn";
        } else {
            $scope.BottomPopupDimBackgroundDiv = "BottomPopupDimBackgroundDivOut";
            $scope.BottomPopupDivCls = "BottomPopupBGOut";
        }
    }

    /**
     * @function
     * @private
     * @name closeLoadingBar
     * @description To close loading bar based on response
     */
    function closeLoadingBar() {
        $timeout(function () {
            $scope.checkResponse = [];
            $scope.loadingBar = false;
            $scope.isTransparent = false;
            SHPService.loadingFinished();
        }, 0);
    }

    /**
     * @function
     * @private
     * @name dismissLoadingBar
     * @description To dismiss loading bar based on response
     */
    function dismissLoadingBar(value) {
        if ($scope.checkResponse.indexOf(value) !== -1) {
            $scope.checkResponse.splice($scope.checkResponse.indexOf(value), 1);
            if ($scope.checkResponse.length === 0) {
                closeLoadingBar();
            }
        }
    }


    function onBackPressed() {
        removeFocus();
        if($scope.isCalender !== -1) {
            $scope.isCalender = -1;
            return;
        }

        if($scope.isTimePicker !== -1) {
            $scope.isTimePicker = -1;
            return;
        }

        if($scope.isAlertPopupOpen) {
            $scope.isAlertPopupOpen = false;
            return;
        }

        if($scope.isRepeatPopupOpen) {
            $scope.isRepeatPopupOpen = false;
            return;
        }

        if($scope.isCalender !== -1) {
            $scope.isCalender = -1;
            return;
        }

        if($scope.showProfilePicker) {
            $scope.showProfilePicker = false;
            return;
        }

        if($scope.dialogPopup.show) {
            $scope.dialogPopup.show = false;
            return;
        }

        switch ($scope.currentScreen) {
            case CALCONST.SCREEN_ADD :
                $timeout(function() {$rootScope.$broadcast('update', {'text' : $scope.currentDate});}, 0);
                $timeout(function() {$rootScope.$broadcast('updateHourView', {'text' : $scope.currentDate});}, 0);
                $timeout(function() {$rootScope.$broadcast('updateAgendaView', {'text' : $scope.currentDate});}, 0);
                $timeout(function() {$rootScope.$broadcast('updateSelectionDate', {'text' : $scope.currentDate});}, 0);
                $scope.currOption = $scope.dropDownSelection[0];
                resetAddPageModel();
                onPressed(CALCONST.SCREEN_HOME);
                break;

            case CALCONST.SCREEN_EVENT :
                resetAddPageModel();
                $timeout(function() {$rootScope.$broadcast('update', {'text' : $scope.currentDate});}, 10);
                if($scope.isAgendaView) {
                    $timeout(function() {$rootScope.$broadcast('updateAgendaView', {'text' : $scope.currentDate});}, 20);
                } else {
                    $timeout(function() {$rootScope.$broadcast('updateHourView', {'text' : null});}, 20);
                }
                onPressed(CALCONST.SCREEN_HOME);
                break;

            case CALCONST.SCREEN_HOME :
                closeApp();
                break;

            case CALCONST.SCREEN_INVITEE :
                onPressed(CALCONST.SCREEN_ADD);
                break;

            case CALCONST.SCREEN_ALERT :
                onPressed(CALCONST.SCREEN_ADD);
                break;

            case CALCONST.SCREEN_REPEAT :
                onPressed(CALCONST.SCREEN_ADD);
                break;
        }
    }

    function onPressed(screenName) {
        $scope.currentScreen = screenName;
        switch (screenName) {
            case CALCONST.SCREEN_ADD :
                $state.transitionTo('AddPage');
                break;

            case CALCONST.SCREEN_HOME :
                $scope.alertDataStructure.selected = 0;
                $state.transitionTo('HomePage');
                if($scope.isWeekView)
                    $timeout(function() {showWeekView();}, 10);
                break;

            case CALCONST.SCREEN_INVITEE :
                $state.transitionTo('InviteePage');
                break;

            case CALCONST.SCREEN_ALERT :
                $state.transitionTo('AlertPage');
                break;

            case CALCONST.SCREEN_REPEAT :
                $state.transitionTo('RepeatPage');
                break;

            case CALCONST.SCREEN_EVENT :
                $state.transitionTo('EventDetailPage');
                break;
        }
    }


    function showWeekView() {
        if($scope.isWeekView) {
            var cHeight = 0, base = 0;
            cHeight = window.screen.availHeight;
            if (cHeight === 0 || cHeight === undefined) {
                cHeight = document.body.clientHeight;
            }

            var viewContainer;
            if($scope.isAgendaView) {
                viewContainer = document.getElementById("agendaListContainer");
            } else {
                viewContainer = document.getElementById("hourViewContainer");
            }

            var calendarContainer = document.getElementById("calendarViewContainer");
            var top = viewContainer.getBoundingClientRect().top;

            top = top * (2560/cHeight);
            moveToTop(viewContainer, calendarContainer, top, cHeight);
        }
    }


    /**
     * @function
     * @private
     * @name closeApp
     * @description To close application, called from app.js
     */
    function closeApp() {
        SHPService.sendNativeCommand(REQUEST.CLOSE_WEB_APP, "", "");
    }


    function getEventTimeString(event, count) {
        if (event === undefined)
            return;

        var start = new Date(event.starts);
        var end = new Date(event.ends);
        var s = formatAMPM(start);
        var e = formatAMPM(end);

        s = s.split(" ");
        e = e.split(" ");
        var lable = '';
        if ($scope.profileData.length > 4 && $scope.profileData.length < 7) {
            lable = s[0] + s[1];
        } else if($scope.profileData.length > 5 && count > 1) {
            lable = '';
        } else if(count !== undefined && count > 1) {
            lable = s[0] + s[1];
        } else {
            lable = s[0] + s[1] + "-" + e[0] + e[1];
        }
        return lable;
    }

    function getEventDetailTimeString(event) {
        if (event === undefined)
            return;

        var start = new Date(event.starts);
        var end = new Date(event.ends);
        var s = formatAMPM(start);
        var e = formatAMPM(end);

        s = s.split(" ");
        e = e.split(" ");
        var lable = s[0] + s[1] + "-" + e[0] + e[1];
        //console.log(" getEventDetailTimeString ", lable);
        return lable;
    }

    $scope.isReadOnlyEvnet = isReadOnlyEvnet;
    function isReadOnlyEvnet(event) {
        if (angular.isDefined(event)) {
            for (var i=0; i < $scope.calendarsData.length; i++) {
                if (event.calendar == $scope.calendarsData[i].id && $scope.calendarsData[i].canWrite == false) {
                    return true;
                }
            }
        }
        return false;
    }

    function getAgendaTimeString(event) {
        if (event === undefined)
            return;
        var start = new Date(event.starts);
        var end = new Date(event.ends);
        var s = formatAMPM(start);
        var e = formatAMPM(end);

        s = s.split(" ");
        e = e.split(" ");
        var lable = '';
        lable = s[0] + s[1] + "-" + e[0] + e[1];

        return lable;
    }


    function getEventDateString(event) {
        if(event === undefined)
            return;
        var start = new Date(event.starts);
        var day = start.getDay() !== 0  ? start.getDay() : CALVIEWCONST.DAY_COUNT-1;
        var month = start.getMonth();
        return $scope.dayName[day] + "," + $scope.monthName[month] + " " + start.getDate() + "," + start.getFullYear();
    }

    function getProfileData(calendarId) {
        var profileData = {};
        $scope.profileData.forEach(function (profile, index) {
            var profile_calendarIds = profile.calendarId;
            profile_calendarIds.forEach(function (profile_calendarId) {
                if (profile_calendarId === calendarId) {
                    profileData = profile;
                }
            });
        });
        return profileData;
    }

    function getProfileImage_calendarId(calendarId) {
        var profileData = getProfileData(calendarId);
        return getProfileImage(profileData.userId);
    }

    function getProfileImage(userId) {
        var index = $scope.profileData.map(function (profile) {
            return profile.userId;
        }).indexOf(userId);
        if (index !== -1) {
            if ($scope.profileData[index].photo !== "" && getImage($scope.profileData[index].photo) !== "") {
                return getImage($scope.profileData[index].photo);
            }
            else {
                return $scope.profileData[index].avatar;
            }
        }

        return '';
    }

    function getEventBallImg(calId) {
        var eventBallImg, tempArr;
        $scope.profileData.forEach(function(profile, index) {
            tempArr = profile.calendarId;
            tempArr.forEach(function(value) {
                if(value == calId) {
                    if(!profile.isFilterSelected){
                        eventBallImg = profile.eventBall;
                    }else{
                        eventBallImg = 'circle_unselected';
                    }
                }
            });
        });
        return eventBallImg;
    }

    function getTimeStamp(date, month, year) {
        var current = new Date(year, month, date);
//        current.setYear(year);
//        current.setMonth(month);
//        current.setDate(date);
        return current.toISOString();
    }

    function getTimeToEvent(time) {
        var eventDate = new Date(time);
        var now = new Date();
        now.setMilliseconds(0);
        now.setSeconds(0);
        var lable = '';
        var divider = 1000 * 60 ;
        var min = (eventDate.getTime() - now.getTime()) / divider;

        if(min > 0 && min <= 99) {
            lable = $scope.translation.WEBMOB_fhub2_calendar_agenda_view_created_mins;
            lable = lable.replace(/(\#.*?\#)/gi, Math.floor(min));
        }
        return lable;
    }

    /**
     * @function
     * @private
     * @name parseStaticData
     * @description To parse static data.
     * To run the application in static mode in browser, add ng-init="parseStaticData" to body tag in html
     */
    function parseStaticData() {
        //  console.log("parseStaticData");
        $scope.isStatic = true;
        init();

        var profileData = JSON.parse('{"type":"requestResponse","status":200,"requestToken":"getProfile","data":{"profileList":[{"familyId":"330803c1-51e0-46de-bf11-93cdab71b925","userId":"2adc1400-04ff-4353-979d-3b27edd0bdea","screenName":"todo list testing","profileColor":"1","photo":"","avatar":"1"}, {"familyId":"330803c1-51e0-46de-bf11-93cdab71b925","userId":"2adc1400-04ff-4353-979d-3b27edd0bdeb","screenName":"CalendarAppTestingAccount","profileColor":"2","photo":"","avatar":"2"}, {"familyId":"330803c1-51e0-46de-bf11-93cdab71b925","userId":"2adc1400-04ff-4353-979d-3b27edd0bdec","screenName":"Ibad Rehman","profileColor":"3","photo":"","avatar":"3"},{"familyId":"330803c1-51e0-46de-bf11-93cdab71b925","userId":"2adc1400-04ff-4353-979d-3b27edd0bded","screenName":"Sachin Gupta","profileColor":"4","photo":"","avatar":"4"},{"familyId":"330803c1-51e0-46de-bf11-93cdab71b925","userId":"2adc1400-04ff-4353-979d-3b27edd0bdee","screenName":"Namit Bansal","profileColor":"5","photo":"","avatar":"5"},{"familyId":"330803c1-51e0-46de-bf11-93cdab71b925","userId":"2adc1400-04ff-4353-979d-3b27edd0bdef","screenName":"Gaurav Sharma","profileColor":"6","photo":"","avatar":"6"}]}}');
        $scope.checkResponse.push(CALCONST.PROFILE_REQUEST);
        requestResponseHandler(profileData);
    }


    /**
     * @function
     * @private
     * @name updateDeviceData
     * @param {String} data
     * data is passed to updateDeviceData which is in app.js
     * @description it gets called from nativeCallback function (app.js), it is for passing the data from native app to web app
     */
    function updateDeviceData(data) {
        //  console.log(" updateDeviceData ");
        SHPService.updateDeviceData($scope, data);
    };

    /**
     * @function
     * @private
     * @name parseDeviceData
     * @param {Object} device
     * getDevice response object
     * @description To parse getDevice response, it contains complete details of present device state
     */
    function parseDeviceData(device) {

        if (angular.isDefined(device.event) && device.event === "Notified"){
            if (angular.isDefined(device.service) && device.service === "Calendar"){ //This noitification will be recieved when adding event on google calendar
                getCalendarsForFamily();
            }

            if (angular.isDefined(device.service) && device.service === "Profile"){ //This noitification will be recieved when a profile is added
                $scope.isTransparent = true;        //for Transparent dim screen.
                $scope.isReceiveProfileNotification = true; // to check if profile notification is coming on Create/Delete/Update operations
                $scope.checkResponse.push(REQUEST.PROFILE_REQUEST);
                SHPService.sendNativeCommand("getProfile", null, REQUEST.PROFILE_REQUEST);
            }

//            if(angular.isDefined(device.CalendarEvent) && device.CalendarEvent !== ""){
//                getCalendarEventData();
//            }
            reloadOnNotification = true;
        }

//        if (angular.isDefined(device.event) && device.event === "Deleted"){
//            getCalendarEventData();
//        }
    }
    
    
    function checkIfAnyProfileIsFiltered(oldProfileData){
        
         oldProfileData.forEach(function(value) {
             if(value.isFilterSelected){
                var tempProfileData = {
                    userId : value.userId,
                    isFilterSelected: value.isFilterSelected
                };
               $scope.reserveProfileFilteredState.push(tempProfileData);  
             }
         });
        
    }
    function fillProfileData(response) {
        var profileList = response.profileList;
        var colorIndex,profileIndex, email = '', emailType = '',avatarType,max,isUnknown;
        profileList.forEach(function(value) {
            isUnknown=false;
            max=7;
            colorIndex = value.profileColor;
            profileIndex = value.avatar;
            if(profileIndex >= 1 && profileIndex <=8 && value.photo === ""){
                avatarType='character_ic_0';
                max=8;
            }
            else
                if(profileIndex == 9 && value.photo === "")
                    avatarType='family/family_ic_0';
                else
                    if(profileIndex >= 10 && profileIndex <= 16 && value.photo === "")
                        avatarType='family/family_ic_';
                        else
                            if(profileIndex >= 17 && profileIndex <= 24 && value.photo === "")
                                avatarType='animal/animal_ic_';
                            else
                                isUnknown=true;
            
            if(colorIndex > max)
                isUnknown=true;
            if(angular.isDefined(value.calendar) && value.calendar.length > 0) {
                email = value.calendar[0].email;
                emailType = value.calendar[0].type;
            } else {
                email = '';
                emailType = '';
            }
            
            var setProfileFilter = false;
            $scope.reserveProfileFilteredState.forEach(function(oldProfileValue) {
                if(oldProfileValue.userId === value.userId){
                    console.log("oldProfileValue.familyId", oldProfileValue.userId);
                    console.log("value.familyId", value.userId);
                    setProfileFilter = oldProfileValue.isFilterSelected;
                }
            });
            
            var profile = {
                familyId : value.familyId,
                owner : value.owner,
                calendarId : [],
                samsungAccount : '',
                oid : '',
                screenName: decodeUnescape(value.screenName),
                profileColor: colorList[colorIndex-1],
                photo: value.photo,
                eventBall : 'circle_' + eventBallList[colorIndex-1],
                avatar: 'common/img/Character_ic/'+ avatarType + value.avatar +'_'+ eventBallList[colorIndex-1] +'.png',
                hourEvent : [],
                allDayEvent : false,
                allDayEventName : '',
                source:'',
                userId: value.userId,
                selected : false,
                defaultCalendarId : '',
                isShared : false,
                email : email,
                emailType : emailType,
                isFilterSelected: setProfileFilter
            };

            if(isUnknown === true)
                profile.avatar='common/img/Character_ic/character_ic_unkown.png';
        
            $scope.profileData.push(profile);
            imageURL_Request(profile.photo);

        });
    }

    function updateProfileCalendarsIds() {
        $scope.dropDownSelection = $scope.dropDownSelection.length > 0  ? [] : $scope.dropDownSelection;
        $scope.profileData.forEach(function(profileItem) {
            var tempCalendarId = [];
            $scope.calendarsData.forEach(function(calendarItem) {
                if(profileItem.userId === calendarItem.owner) {
                    tempCalendarId.push(calendarItem.id);
                    if(calendarItem.isDefault === true && calendarItem.isShared === true) {
                        profileItem.defaultCalendarId = calendarItem.id;
                        profileItem.oid = calendarItem.oid;
                        profileItem.isShared = calendarItem.isShared;
                        profileItem.source = calendarItem.source;
                    }
                }
            });
            profileItem.calendarId = tempCalendarId;
        });

        $scope.profileData.forEach(function(profileItem) {
            if(profileItem.calendarId.length > 0 && profileItem.defaultCalendarId !== '')
                $scope.dropDownSelection.push(profileItem);
        });
        if($scope.dropDownSelection.length === 0 && getLocalStorageData('firstTimeLaunch') !== 'false'){
            $scope.showOOBEScreen = true;
        }else{
            $scope.showOOBEScreen = false;
        }
    }

    function updateCalValue(calObj) {
        $scope.currentDate = calObj;
    }


    function sendBroadCast(keyword, message) {
        $rootScope.$broadcast('keyword', {'param' : message});

//        if(keyword === 'updateCalendarView') {
//             $rootScope.$broadcast('update', {'text' : 'Hi There'});
//        } else if(keyword === 'updateHourView') {
//
//        } else {
//
//        }
    }

    function getImage (key) {
        if (angular.isDefined(imageCache) && angular.isDefined(imageCache.get(key))) {
            return imageCache.get(key);
        }
        return "";
    }

    function imageURL_Request (imageURL) {
        imageRequest.nativeRequest(imageURL);
    }
    
    function closePopups(){
        if($scope.isAlertPopupOpen){
            $scope.alertDataStructure.selected=0;
            //$scope.alertDataStructure.selected=$scope.prevSelected;
             $scope.isAlertPopupOpen=false;
        }       
        if($scope.isRepeatPopupOpen){
            $scope.repeatDataStructure.selected=0;
            $scope.isRepeatPopupOpen=false;
        } 
        if($scope.isCalender !== -1)
            $scope.isCalender=-1;    
        if($scope.isTimePicker !== -1)
            $scope.isTimePicker= -1;
        if($scope.dialogPopup.show)
            $scope.dialogPopup.show=false;
    }
};

/*
 * In old webkits textarea maxlength attribute is not supported,
 * To prevent user from entering text more than 32 characters we are forcefully preventing it. 
 */
app.directive('textareaLoaded', function (SHPService, REQUEST, $timeout) {
    return {
        restrict: 'A',
        link: function (scope, element) {
            $(element).bind('keypress keydown change', function(e){
                console.log("textareaLoaded ", this.maxLength);
                if (this.value.length >= this.maxLength && (e.keyCode !== 13 && e.keyCode !== 8)){
                    debugMessage(this.value.length);
                    e.preventDefault();
                    this.value = this.value.substr(0, this.maxLength);
                    
                    /*Move cursor to the front of input text*/
                    var currentText = $(this).val();
                    $(this).val('');
                    $(this).val(currentText);
                    
                    this.blur();
                    $timeout(function () {
                        SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, scope.translation.WEBMOB_fhub2_common_max_characters, "");
                    }, 500);

                    return false;
                }
            });
        }
    };
});


