/*******************************************************************************
 * @author Ajay Singh (ajay1.s@samsung.com)
 * @file This file implements Refrigerator controller function
 * Copyright 2016 by Samsung Electronics, Inc.,
 * @license This software is the confidential and proprietary information of Samsung
 * Electronics, Inc. ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with the terms
 * of the license agreement you entered into with Samsung.
 ******************************************************************************/
var app = angular.module("SmartThings", ['ngSanitize', 'pinch-zoom', 'ngQuickDate', 'ngImgCrop', 'ui.router', 'ngMaterial', 'swipe']);
app.controller("foodReminderController", ["$scope", "$state", "$q", "$filter", "imageCache", "imageRequest", "profileService", "timeStamp", "SHPService", "REQUEST", "staticData", "$timeout", "$compile", foodReminderController]);

/**
 * @private
 * @function
 * @name foodReminderController
 *
 * @description This method is a constructor function for Food Reminder module and is used to define
 * properties and methods to module's scope (angular scope).
 */
function foodReminderController($scope, $state, $q, $filter, imageCache, imageRequest, profileService, timeStamp, SHPService, REQUEST, staticData, $timeout, $compile) {
    /**
     * @enum {String} REFCONST
     * @readonly
     * @description  This is a global variable used to identify various constants of Food reminder module
     * @private
     */
    var cWidth, cHeight, firstLaunch = true, MAX_EXPIRY_YEAR = 21,
        debugMode = !false, deleteItemId = null, deleteMemoId = null, deleteOption, ENTER_KEYCODE = 13;
    var default_category_id = "OTHER";
    var default_master_category_id = "Other";
    var categoryMap = [
        {"Whisk_Category":"FRUITS AND VEGETABLES", "Category_ID":"Fruits_Vegetables"},
        {"Whisk_Category":"MEATS AND SEAFOOD", "Category_ID":"Meats_Seafood"},
        {"Whisk_Category":"BREAD AND BAKERY", "Category_ID":"Bread_Bakery"},
        {"Whisk_Category":"DAIRY AND EGGS", "Category_ID":"Dairy_Eggs_Cheese"},
        {"Whisk_Category":"CONDIMENTS", "Category_ID":"Condiments_Sauces"},
        {"Whisk_Category":"FROZEN", "Category_ID":"Frozen"},
        {"Whisk_Category":"HERBS AND SPICES", "Category_ID":"Seasoning_Spices_Baking"},
        {"Whisk_Category":"BAKING", "Category_ID":"Seasoning_Spices_Baking"},
        {"Whisk_Category":"BREAKFAST FOODS", "Category_ID":"Breakfast_Cereal"},
        {"Whisk_Category":"SNACKS", "Category_ID":"Cookies_Snacks_Candy"},
        {"Whisk_Category":"DRINKS", "Category_ID":"Beverages"},
        {"Whisk_Category":"CANNED FOODS", "Category_ID":"Canned_DryGoods"},
        {"Whisk_Category":"PASTA, RICE AND BEANS", "Category_ID":"Canned_DryGoods"},
        {"Whisk_Category":"PET CARE", "Category_ID":"PetCare"},
        {"Whisk_Category":"HOUSEHOLD", "Category_ID":"Household"},
        {"Whisk_Category":"HEALTH AND BEAUTY", "Category_ID":"PersonalCare"},
        {"Whisk_Category":"BABY CARE", "Category_ID":"BabyCare"},
        {"Whisk_Category":"OTHER", "Category_ID":"Other"}
    ];
    var BASE64 = "data:image/png;base64,";
    $scope.glazeImageForCrop;
    $scope.myCroppedImage = '';
    $scope.showConsoleLogs = !false;

    var langArr = Object.keys(LANGUAGE_FILE.language);
    if (langArr.indexOf(LANG.toUpperCase()) !== -1) {
        $scope.translation = LANGUAGE_FILE["language"][LANG.toUpperCase()];
    } else if (langArr.indexOf(LANG.toUpperCase().substring(0, 2)) !== -1) {
        $scope.translation = LANGUAGE_FILE["language"][LANG.toUpperCase().substring(0, 2)];
    } else {
        $scope.translation = LANGUAGE_FILE["language"]["DEFAULT"];
    }
    if (LANG.toUpperCase().indexOf('EN') !== -1) {
        $scope.selectedLangEnglish = true;
    }

    if (window.navigator.platform === 'iPhone' || window.navigator.platform === 'iPad') {
        $scope.isAppleDevice = true;
    }

    // to check if input or textara or md-autocomplete is focused
    function isTextInput(node) {
        return ['INPUT', 'TEXTAREA', 'MD-AUTOCOMPLETE-PARENT-SCOPE'].indexOf(node.nodeName) !== -1;
    }

    function isMdAutoCompleteResult(node) {
        return $(node).hasClass('md-virtual-repeat-container') || $(node).parents().hasClass('md-virtual-repeat-container');
    }

    // remove focues on touch anywhere on the screen
    document.addEventListener('touchmove', function (e) {
        if ($scope.isAppleDevice) {
            if (!(isTextInput(e.target) || isMdAutoCompleteResult(e.target)) && isTextInput(document.activeElement)) {
                removeFocus();
            }
        }
    }, false);
    
    
    var VIEW_INSIDE_SCREEN = {
        HOME_SCREEN: "HOME_SCREEN",
        FOOD_ITEM_SCREEN: "FOOD_ITEM_SCREEN",
        SEND_TO_SHOPPING_LIST_SCREEN: "SEND_TO_SHOPPING_LIST_SCREEN"
    };

    var VIEW_INSIDE_VIEW = {
        INSIDE_IMAGE: "INSIDE_IMAGE",
        FOOD_LIST: "FOOD_LIST"
    };
    $scope.currentScreen = VIEW_INSIDE_SCREEN.HOME_SCREEN;
    $scope.currentView = VIEW_INSIDE_VIEW.INSIDE_IMAGE;

    /** Starts to define the default values for FHub 2.0 **/
    $scope.isSupportShoppingList = true;
    $scope.isSupportFoodTag = false;
    $scope.isSupportMemoTag = true;
    $scope.MAX_MEMO_TAG = 50;
    $scope.MAX_FOOD_TAG = 0;
    $scope.MAX_FOOD_ITEM = 150;
    $scope.MAX_SHOPPING_LIST_TITLE = 24;       //check for Shopping List
    $scope.MAX_SHOPPING_LIST_ITEM = 50;       //check for Shopping List
    $scope.MAX_CHARACTER_TITLE = 32;//32;       //check for Shopping List
    $scope.MAX_CHARACTER_ITEM = 32;//32;
    $scope.MAX_PASTHISTORY_COUNT = 9;
    $scope.DEFAULT_DAYS = 14;
    $scope.LCD_GLAZE_RESOLUTION = {
        width: 1080,
        height: 1430
    };
    $scope.LCD_MEMO_TAG_SIZE = {
        width: 143,
        height: 143
    };
    $scope.LCD_EXPIRY_TAG_SIZE = {
        width: 195,
        height: 72
    };
    /** Ends to define the default values for FHub 2.0 **/

    {
        /*
         * Scope varibles and methods
         */
        $scope.LANGUAGE_FILE = LANGUAGE_FILE;
        $scope.profileService = profileService;
        $scope.imageCache = imageCache;
        $scope.SHPService = SHPService;
        $scope.loadingBar = false;
        $scope.loadingBar_noTimeOut = false;
        $scope.homePage = true;
        $scope.view = {
            insideImageView: true,
            foodListView: false
        };
        $scope.foodItemPage = false;
        $scope.helpPage = false;
        $scope.isStatic = false;
        $scope.isLocal = false;
        $scope.isParseTag = false;
        $scope.isCalender = false;
        $scope.daySelector = false;
        $scope.bOptionPopUp = false;
        $scope.addFoodItem = false;
        $scope.editFoodItem = false;
        $scope.checkGrazeImgCrop = false;
        $scope.zoomGlazeImg = false;
        $scope.enableDrag = false;
        $scope.confirmationPopUpVisible = false;
        $scope.confirmationPopUpTitle = "";
        $scope.confirmationPopupText = "";
        $scope.dragCheckProcessed = false;
        $scope.datepick = new Date();
        // $scope.AddFoodImage = "";
        $scope.draggedTag = "";
        $scope.sendToShoppingList = [];

        $scope.userId = "";
        $scope.checkResponse = [];
        $scope.checkResponse_noTimeOut = [];
        $scope.isSelectedMode = false;
        $scope.currentSelectedMode = "";
        $scope.FOODREMINDER_OPTIONS = {
            DELETE: $scope.translation.WEBMOB_fhub2_common_delete,
            MOVE_LOCATION: $scope.translation.WEBMOB_fhub2_food_reminder_move_location,
            SEND_TO_SHOPPING_LIST: $scope.translation.WEBMOB_fhub2_food_reminder_send_to_shopping_list
        };

        $scope.CHANGE_IMAGE_OPTION = [
            {"id": "GO_TO_CAMERA", "name": $scope.translation.WEBMOB_fhub2_food_reminder_change_image_option_go_to_camera},
            {"id": "GO_TO_GALLERY", "name": $scope.translation.WEBMOB_fhub2_food_reminder_change_image_option_go_to_gallery},
            {"id": "DEFAULT_COVER", "name": $scope.translation.WEBMOB_fhub2_food_reminder_change_image_option_default_cover},
            {"id": "VIEW_PICTURE", "name": $scope.translation.WEBMOB_fhub2_food_reminder_change_image_option_view_picture}
        ];

        $scope.insideFridgeData = {
            GlazeURL: [],
            doorOpenTime: "",
            drawSize: [],
            pastHistoryIdx: 0,
            maxPastHistory: 1,
            touched: false
        };

        $scope.Device = {
            FoodList: {
                "Fridge": [],
                "Tag_List": []
            },
            GlazeInfo: {},
            ShoppingList: {}
        };
        $scope.categoryDB = [];

        $scope.DefineLocations = [
            {"id": "STORAGE_FRIDGE", "name": decodeHtmlEntity($scope.translation.WEBMOB_fhub2_food_reminder_fridge), "whiskType": "fridge", "isExpand": true},
            {"id": "STORAGE_FREEZER", "name": decodeHtmlEntity($scope.translation.WEBMOB_fhub2_food_reminder_freezer), "whiskType": "freezer", "isExpand": true},
            {"id": "STORAGE_FLEX_ZONE", "name": decodeHtmlEntity($scope.translation.WEBMOB_fhub2_food_reminder_flex_zone), "whiskType": "flexzone", "isExpand": true},
            {"id": "STORAGE_FLEX_ZONE_18K", "name": decodeHtmlEntity($scope.translation.WEBMOB_fhub2_food_reminder_flex_zone_18K), "whiskType": "flexzone", "isExpand": true},
            {"id": "STORAGE_PANTRY", "name": decodeHtmlEntity($scope.translation.WEBMOB_fhub2_food_reminder_pantry), "whiskType": "pantry", "isExpand": true}
        ];
        $scope.locations = [];

        $scope.inputData = {
            id: "",
            category: decodeHtmlEntity($scope.translation.WEBMOB_fhub2_shopping_list_create_new_title),
            createCategory: "",
            name: "",
            entryDate: new Date(),
            expirationDate: getDefaultExpirationDate(),
            keepDays: 0,
            calculateDays: diffDays(new Date(), getDefaultExpirationDate()),
            storageType: (angular.isDefined($scope.locations[0]))? $scope.locations[0].id : '',
            is_expiry_tag: false,
            shoppingTitle: {
                groupId: "",
                category: "",
                count: 0
            },
            imageURL: "",
            isReminder: false,
            category_info: default_category_id,
            data_type: "",
            emart_item_id: "",
            emart_min_qty: "",
            site_no: "",
            recommended: {
                location: false,
                expiration: false
            },
            isAllowRecommend: {
                location: true,
                expiration: true
            },
            defaultLocation: ""         //Recommended the location 1st
        };

        $scope.placeHolder = decodeHtmlEntity($scope.translation.WEBMOB_fhub2_food_reminder_food_default);

        $scope.dialogPopup = {
            show: false,
            dialogPopupTitle: '',
            dialogPopupText: '',
            button1Title: '',
            button1Handler: function () {
                toggleBottomPopUpClass(false);
                $scope.dialogPopup.show = false;
            },
            button2Enabled: false,
            button2Title: '',
            button2Handler: function () {
            }
        };

        $scope.glazeProgress = {
            show: false,
            value: 0
        };
        $scope.memoCoordinates = [];

        $scope.init = init;
        $scope.updateDeviceData = updateDeviceData;
        $scope.parseLcdSpec = parseLcdSpec;
        $scope.parseDeviceData = parseDeviceData;
        $scope.parseStaticData = parseStaticData;
        $scope.parseFoodData = parseFoodData;
        $scope.isAllowLocationType = isAllowLocationType;
        $scope.addFoodData = addFoodData;
        $scope.updateFoodData = updateFoodData;
        $scope.parseShoppingData = parseShoppingData;
        $scope.showAddFoodItemPage = showAddFoodItemPage;
        $scope.deleteItem = deleteItem;
        $scope.deleteFoodItem = deleteFoodItem;
        $scope.showEditFoodItemPage = showEditFoodItemPage;
        $scope.keyupHandler = keyupHandler;
        $scope.showDayPopup = showDayPopup;
        $scope.showEntryDatePopup = showEntryDatePopup;
        $scope.showExpirationPopup = showExpirationPopup;
        $scope.showCalenderPopup = showCalenderPopup;
        $scope.cancelDate = cancelDate;
        $scope.cancelFood = cancelFood;
        $scope.getFoodListFormat = getFoodListFormat;
        $scope.sendSaveFoodCommand = sendSaveFoodCommand;
        $scope.sendEditFoodCommand = sendEditFoodCommand;
        $scope.home = home;
        $scope.lastUpdate = lastUpdate;
        $scope.onBackPressed = onBackPressed;
        $scope.closeLoadingBar = closeLoadingBar;
        $scope.dismissLoadingBar = dismissLoadingBar;
        $scope.setLoadingBar_noTimeOut = setLoadingBar_noTimeOut;
        $scope.dismissLoadingBar_noTimeOut = dismissLoadingBar_noTimeOut;
        $scope.closeApp = closeApp;
        $scope.showConnectionFailurePopup = showConnectionFailurePopup;
        $scope.closeConnectionFailurePopup = closeConnectionFailurePopup;
        $scope.requestFailHandler = requestFailHandler;
        $scope.requestResponseHandler = requestResponseHandler;
        $scope.requestProgressHandler = requestProgressHandler;
        $scope.getFoodList = getFoodList;
        $scope.backToHomePage = backToHomePage;
        $scope.isScreenScrollable = isScreenScrollable;
        $scope.deleteTag = deleteTag;
        $scope.closeAllPopUp = closeAllPopUp;
        $scope.closeSubPopUp = closeSubPopUp;
        $scope.showOptionPopup = showOptionPopup;
        $scope.closeOptionPopup = closeOptionPopup;
        $scope.selectIndexFromOptionMenu = selectIndexFromOptionMenu;
        $scope.enableSendToShoplist = enableSendToShoplist;
        $scope.isEmpty = isEmpty;
        $scope.isMaxItem = isMaxItem;
        $scope.foodListCount = foodListCount;
        $scope.isFoodListEmpty = isFoodListEmpty;
        $scope.isEmart = isEmart;
        $scope.isWoolworth = isWoolworth;
        $scope.closeConfirmationPopUp = closeConfirmationPopUp;
        $scope.showConfirmationPopUp = showConfirmationPopUp;
        $scope.onOkConfirmationPopUp = onOkConfirmationPopUp;
        $scope.onCancelConfirmationPopUp = onCancelConfirmationPopUp;
        $scope.parseGlazeInfo = parseGlazeInfo;
        $scope.decodeHtmlEntity = decodeHtmlEntity;
        $scope.encodeEscape = encodeEscape;
        $scope.decodeUnescape = decodeUnescape;
        $scope.expiryDigitCal = expiryDigitCal;
        $scope.longPressHandlerForCrop = longPressHandlerForCrop;
        $scope.showAddFoodItemPageFromCrop = showAddFoodItemPageFromCrop;
        $scope.goToFoodList = goToFoodList;
        $scope.goToInsideImage = goToInsideImage;
        $scope.goToFoodItem = goToFoodItem;
        $scope.goToSendToShoppingList = goToSendToShoppingList;
        $scope.showSendToShopListPopUpFromCrop = showSendToShopListPopUpFromCrop;
        $scope.sendShoppingListFromCreateNewListPopup = sendShoppingListFromCreateNewListPopup;
        $scope.getShoppingListFormat = getShoppingListFormat;
        $scope.sendToShoppingListTitleCommand = sendToShoppingListTitleCommand;
        $scope.sendToShoppingListItemCommand = sendToShoppingListItemCommand;
        $scope.sendToShoppingList = sendToShoppingList;
        $scope.selectedModeForDelete = selectedModeForDelete;
        $scope.selectedModeForSendToShoppingList = selectedModeForSendToShoppingList;
        $scope.longPressHandlerForDelete = longPressHandlerForDelete;
        $scope.selectedModeForMoveLocation = selectedModeForMoveLocation;
        $scope.moveLocationCommand = moveLocationCommand;
        $scope.deleteExpiryTag = deleteExpiryTag;
        $scope.diffDays = diffDays;
        $scope.getImage = getImage;
        $scope.imageURL_Request = imageURL_Request;
        $scope.glazeImageURL_Request = glazeImageURL_Request;
        $scope.renderGlazeImage = renderGlazeImage;
        $scope.deleteConfirmationCheck = deleteConfirmationCheck;
        $scope.removeFocus = removeFocus;
        $scope.reminderFilter = reminderFilter;
        $scope.checkFilter = checkFilter;

        //MEMO
        $scope.addMemoFromCrop = addMemoFromCrop;
        $scope.addMemoCommand = addMemoCommand;
        $scope.goToMemo = goToMemo;
        $scope.deleteMemo = deleteMemo;
        $scope.moveTag = moveTag;
        $scope.prepareMemoText = prepareMemoText;
        $scope.getTagType = getTagType;
        $scope.mobileTagSize = mobileTagSize;
        $scope.getTagResolution_MOBILE = getTagResolution_MOBILE;
        $scope.getTagResolution_LCD = getTagResolution_LCD;
        $scope.parseTagData = parseTagData;
        $scope.removeExistingTags = removeExistingTags;
    }

    $scope.initial_inputData = function () {
        console.log('initial_inputData');
        $scope.inputData = {
            id: "",
            category: decodeHtmlEntity($scope.translation.WEBMOB_fhub2_shopping_list_create_new_title),
            createCategory: "",
            name: "",
            entryDate: new Date(),
            expirationDate: getDefaultExpirationDate(),
            keepDays: 0,
            calculateDays: diffDays(new Date(), getDefaultExpirationDate()),
            storageType: (angular.isDefined($scope.locations[0]))? $scope.locations[0].id : '',
            is_expiry_tag: false,
            shoppingTitle: {
                groupId: "",
                category: "",
                count: 0
            },
            imageURL: "",
            isReminder: false,
            category_info: default_category_id,
            data_type: "",
            emart_item_id: "",
            emart_min_qty: "",
            site_no: "",
            recommended: {
                location: false,
                expiration: false
            },
            isAllowRecommend: {
                location: true,
                expiration: true
            },
            defaultLocation: ""         //Recommended the location 1st
        };
    };

    $scope.isDefinedList = isDefinedList;
    function isDefinedList(groupId) {
        return (angular.isDefined(groupId) && angular.isDefined($scope.Device.ShoppingList[groupId]));
    }

    function backToHomePage() {
        $scope.currentScreen = VIEW_INSIDE_SCREEN.HOME_SCREEN;
        changeScreen();
    }

    function goToInsideImage() {
        $scope.currentScreen = VIEW_INSIDE_SCREEN.HOME_SCREEN;
        $scope.currentView = VIEW_INSIDE_VIEW.INSIDE_IMAGE;
        closeCropScreen();
        changeScreen();
        $state.transitionTo('InsideImagePage', null, {reload: true});
        debugMessage("moveToInsideImagePage");
    }

    function goToFoodList() {
        $scope.currentScreen = VIEW_INSIDE_SCREEN.HOME_SCREEN;
        $scope.currentView = VIEW_INSIDE_VIEW.FOOD_LIST;
        changeScreen();
        $state.transitionTo('FoodListPage');
    }

    function goToSendToShoppingList() {
        $scope.currentScreen = VIEW_INSIDE_SCREEN.SEND_TO_SHOPPING_LIST_SCREEN;
        changeScreen();
        $state.transitionTo('SendToShoppingListPage');
    }

    function goToFoodItem() {
        $scope.currentScreen = VIEW_INSIDE_SCREEN.FOOD_ITEM_SCREEN;
        changeScreen();
        $state.transitionTo('FoodItemPage');
    }

    /**
     * @function
     * @private
     * @name changeScreen
     * @param
     * @description Change screen logic - visibility and invisibilty of pages
     */
    function changeScreen() {
        hideScreen();
        if ($scope.currentScreen === VIEW_INSIDE_SCREEN.HOME_SCREEN) {
            $scope.homePage = true;
            hideView();
            if ($scope.currentView === VIEW_INSIDE_VIEW.INSIDE_IMAGE) {
                $scope.view.insideImageView = true;
            }
            else if ($scope.currentView === VIEW_INSIDE_VIEW.FOOD_LIST) {
                $scope.view.foodListView = true;
            }
        } else if ($scope.currentScreen === VIEW_INSIDE_SCREEN.FOOD_ITEM_SCREEN) {
            $scope.foodItemPage = true;
        } else if ($scope.currentScreen === VIEW_INSIDE_SCREEN.SEND_TO_SHOPPING_LIST_SCREEN) {
            $scope.sendToShoppingListPage = true;
        } else {
            //NULL
        }
    }

    function hideScreen() {
        $scope.homePage = false;
        $scope.foodItemPage = false;
        $scope.sendToShoppingListPage = false;
        $scope.helpPage = false;
    }

    function hideView() {
        $scope.view.insideImageView = false;
        $scope.view.foodListView = false;
    }

    /**
     * @function
     * @private
     * @name init
     * @description after getting the getDevice response, it is called from app.js
     */
    function init() {
        // Future purpose
        debugMessage('called init');
        if ($scope.currentScreen === VIEW_INSIDE_SCREEN.HOME_SCREEN) {
            if ($scope.currentView === VIEW_INSIDE_VIEW.INSIDE_IMAGE)
                $scope.goToInsideImage();
            else if ($scope.currentView === VIEW_INSIDE_VIEW.FOOD_LIST)
                $scope.goToFoodList();
        }
    };

    $scope.showCropScreen = showCropScreen;
    $scope.closeCropScreen = closeCropScreen;
    function showCropScreen() {
        $scope.checkGrazeImgCrop = true;   //open image crop
    }

    function closeCropScreen() {
        $scope.checkGrazeImgCrop = false;   //close image crop
    }

    function longPressHandlerForCrop() {
       $scope.showConsoleLogs &&  console.log("longPressHandlerForCrop: event ", event);
        $scope.showConsoleLogs &&  console.log("longPressHandlerForCrop: " + "id: " + event.target.id);
        $scope.showConsoleLogs &&  console.log("longPressHandlerForCrop: " + "Class: " + $(event.target).attr('class'));

        var evt = event || window.event;
        $scope.showConsoleLogs &&  console.log("longPressHandlerForCrop: evt ", evt);
        positionXToCreateCrop = evt.pageX;
        positionYToCreateCrop = evt.pageY;
        $scope.showConsoleLogs &&  console.log("positionXToCreateCrop positionYToCreateCrop", positionXToCreateCrop + " " + positionYToCreateCrop);
        if (evt.target.id.indexOf("REF_GLAZE_") !== -1) {
            if ($scope.insideFridgeData.pastHistoryIdx === $scope.insideFridgeData.maxPastHistory) {
                showCropScreen();
                $scope.showConsoleLogs && console.log("longPressHandlerForCrop: " + "$scope.checkGrazeImgCrop: " + $scope.checkGrazeImgCrop);
            }
            else {
                SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, decodeHtmlEntity($scope.translation.WEBMOB_fhub2_food_reminder_prevent_crop_message), REQUEST.SHOW_TOAST);
            }
        }
    }

    /**
     * @function
     * @private
     * @name removeFocus
     * @param
     * @description
     **/
    function removeFocus() {
        debugMessage("removeFocus:::");
        document.activeElement.blur();
        SHPService.sendNativeCommand(REQUEST.CLOSE_KEYPAD, REQUEST.CLOSE_KEYPAD, "");
    }

    /**
     * @function
     * @private
     * @name updateDeviceData
     * @param {String} data
     * data is passed to updateDeviceData which is in app.js
     * @description it gets called from nativeCallback function (app.js), it is for passing the data from native app to web app
     */
    function updateDeviceData(data) {
        SHPService.updateDeviceData($scope, data);
    }

    function parseLcdSpec(data) {
        console.log("parseLcdSpec::: ", data);
        if (angular.isDefined(data)) {
            if (angular.isDefined(data["viewinside"])) {
                var viewinsideSpec = data["viewinside"];
                $scope.isSupportFoodTag = (angular.isDefined(viewinsideSpec["foodTag"])) ? !!viewinsideSpec["foodTag"] : $scope.isSupportFoodTag;
                $scope.MAX_MEMO_TAG = (angular.isDefined(viewinsideSpec["maxMemoTags"])) ? parseInt(viewinsideSpec["maxMemoTags"]) : $scope.MAX_MEMO_TAG;
                $scope.MAX_FOOD_TAG = (angular.isDefined(viewinsideSpec["maxFoodTags"])) ? parseInt(viewinsideSpec["maxFoodTags"]) : $scope.MAX_FOOD_TAG;
                $scope.MAX_FOOD_ITEM = (angular.isDefined(viewinsideSpec["maxItems"])) ? parseInt(viewinsideSpec["maxItems"]) : $scope.MAX_FOOD_ITEM;
                $scope.MAX_CHARACTER_ITEM = (angular.isDefined(viewinsideSpec["maxChar"])) ? parseInt(viewinsideSpec["maxChar"]) : $scope.MAX_CHARACTER_ITEM;
                $scope.MAX_PASTHISTORY_COUNT = (angular.isDefined(viewinsideSpec["maxHistoryCnt"])) ? parseInt(viewinsideSpec["maxHistoryCnt"]) : $scope.MAX_PASTHISTORY_COUNT;

                if (angular.isDefined(viewinsideSpec["glazeResolution"])) {
                    if (angular.isDefined(viewinsideSpec["glazeResolution"]["width"]))
                        $scope.LCD_GLAZE_RESOLUTION.width = parseInt(viewinsideSpec["glazeResolution"]["width"]);
                    if (angular.isDefined(viewinsideSpec["glazeResolution"]["height"]))
                        $scope.LCD_GLAZE_RESOLUTION.height = parseInt(viewinsideSpec["glazeResolution"]["height"]);
                }

                if (angular.isDefined(viewinsideSpec["memoTagSize"])) {
                    if (angular.isDefined(viewinsideSpec["memoTagSize"]["width"]))
                        $scope.LCD_MEMO_TAG_SIZE.width = parseInt(viewinsideSpec["memoTagSize"]["width"]);
                    if (angular.isDefined(viewinsideSpec["memoTagSize"]["height"]))
                        $scope.LCD_MEMO_TAG_SIZE.height = parseInt(viewinsideSpec["memoTagSize"]["height"]);
                }

                if (angular.isDefined(viewinsideSpec["expiryTagSize"])) {
                    if (angular.isDefined(viewinsideSpec["expiryTagSize"]["width"]))
                        $scope.LCD_EXPIRY_TAG_SIZE.width = parseInt(viewinsideSpec["expiryTagSize"]["width"]);
                    if (angular.isDefined(viewinsideSpec["expiryTagSize"]["height"]))
                        $scope.LCD_EXPIRY_TAG_SIZE.height = parseInt(viewinsideSpec["expiryTagSize"]["height"]);
                }
            }
            if (angular.isDefined(data["shoppinglist"])) {
                var shoppinglistSpec = data["shoppinglist"];
                $scope.isSupportShoppingList = (angular.isDefined(shoppinglistSpec["installed"])) ? !!shoppinglistSpec["installed"] : $scope.isSupportShoppingList;
                $scope.MAX_SHOPPING_LIST_TITLE = (angular.isDefined(shoppinglistSpec["maxLists"])) ? parseInt(shoppinglistSpec["maxLists"]) : $scope.MAX_SHOPPING_LIST_TITLE;
                $scope.MAX_SHOPPING_LIST_ITEM = (angular.isDefined(shoppinglistSpec["maxItems"])) ? parseInt(shoppinglistSpec["maxItems"]) : $scope.MAX_SHOPPING_LIST_ITEM;
                $scope.MAX_CHARACTER_TITLE = (angular.isDefined(shoppinglistSpec["maxChar_list"])) ? parseInt(shoppinglistSpec["maxChar_list"]) : $scope.MAX_CHARACTER_TITLE;
                $scope.MAX_CHARACTER_ITEM = (angular.isDefined(shoppinglistSpec["maxChar_item"])) ? parseInt(shoppinglistSpec["maxChar_item"]) : $scope.MAX_CHARACTER_ITEM;
            }
            if (angular.isDefined(data["memo"])) {
                var memoSpec = data["memo"];
                $scope.isSupportMemoTag = (angular.isDefined(memoSpec["installed"])) ? !!memoSpec["installed"] : $scope.isSupportMemoTag;
            }
        }
    }

    /**
     * @function
     * @private
     * @name parseGlazeInfo
     * @description parse GlazeInfo for setting glaze data.
     */
    function parseGlazeInfo(device) {
        console.log("***** parseGlazeInfo ********", device);
        $scope.Device.GlazeInfo = angular.copy(device);

        if (angular.isDefined(device.GlazeURL))
            $scope.Device.GlazeInfo.GlazeURL = device.GlazeURL;

        if (angular.isDefined(device.doorOpenedTime))
            $scope.Device.GlazeInfo.doorOpenedTime = device.doorOpenedTime;

        //Reverse History order
        $scope.Device.GlazeInfo.pastHistory_URL = [];
        if (angular.isDefined(device.pastHistory_URL)) {
            if ($scope.MAX_PASTHISTORY_COUNT !== device.pastHistory_URL.length) {
                var emptyCnt = $scope.MAX_PASTHISTORY_COUNT - device.pastHistory_URL.length;
                for (var cnt = 0; cnt<emptyCnt; cnt++) {
                    $scope.Device.GlazeInfo.pastHistory_URL.push(undefined);
                }
            }
            for (var idx = device.pastHistory_URL.length-1; idx>=0; idx--) {
                $scope.Device.GlazeInfo.pastHistory_URL.push(device.pastHistory_URL[idx]);
            }
        }

        //Reverse History order
        $scope.Device.GlazeInfo.pastHistory_doorOpenedTime = [];
        if (angular.isDefined(device.pastHistory_doorOpenedTime)) {
            if ($scope.MAX_PASTHISTORY_COUNT !== device.pastHistory_doorOpenedTime.length) {
                var emptyCnt = $scope.MAX_PASTHISTORY_COUNT - device.pastHistory_doorOpenedTime.length;
                for (var cnt = 0; cnt<emptyCnt; cnt++) {
                    $scope.Device.GlazeInfo.pastHistory_doorOpenedTime.push(undefined);
                }
            }
            for (var idx= device.pastHistory_doorOpenedTime.length-1; idx>=0; idx--) {
                $scope.Device.GlazeInfo.pastHistory_doorOpenedTime.push(device.pastHistory_doorOpenedTime[idx]);
            }
        }

        if (angular.isDefined($scope.Device.GlazeInfo.pastHistory_URL) && $scope.Device.GlazeInfo.pastHistory_URL.length > 0)
            $scope.insideFridgeData.maxPastHistory = $scope.Device.GlazeInfo.pastHistory_URL.length;
        else
            $scope.insideFridgeData.maxPastHistory = 1;

        if (angular.isDefined(device.drawSize))
            $scope.insideFridgeData.drawSize = $scope.Device.GlazeInfo.drawSize = device.drawSize;

        moveToMaxPastHistory();

        //initial Locations
        if (angular.isDefined($scope.Device.GlazeInfo["StorageModes"])) {
            var location;
            $scope.locations = [];
            $scope.Device.GlazeInfo["StorageModes"].forEach(function (modes) {
                if (modes === 'STORAGE_FRIDGE') {
                    location = {"id": "STORAGE_FRIDGE", "name": decodeHtmlEntity($scope.translation.WEBMOB_fhub2_food_reminder_fridge)};
                    $scope.locations.push(location);
                }
                else if (modes === 'STORAGE_FREEZER') {
                    location = {"id": "STORAGE_FREEZER", "name": decodeHtmlEntity($scope.translation.WEBMOB_fhub2_food_reminder_freezer)};
                    $scope.locations.push(location);
                }
                else if (modes === 'STORAGE_FLEX_ZONE') {
                    location = {"id": "STORAGE_FLEX_ZONE", "name": decodeHtmlEntity($scope.translation.WEBMOB_fhub2_food_reminder_flex_zone)};
                    $scope.locations.push(location);
                }
                else if (modes === 'STORAGE_FLEX_ZONE_18K') {
                    location = {"id": "STORAGE_FLEX_ZONE_18K", "name": decodeHtmlEntity($scope.translation.WEBMOB_fhub2_food_reminder_flex_zone_18K)};
                    $scope.locations.push(location);
                }
                else if (modes === 'STORAGE_PANTRY') {
                    location = {"id": "STORAGE_PANTRY", "name": decodeHtmlEntity($scope.translation.WEBMOB_fhub2_food_reminder_pantry)};
                    $scope.locations.push(location);
                }
            });
        }
        else {
            //if it is not supported 'StorageModes', show all locations.
            $scope.locations = [
                {"id": "STORAGE_FRIDGE", "name": decodeHtmlEntity($scope.translation.WEBMOB_fhub2_food_reminder_fridge)},
                {"id": "STORAGE_FREEZER", "name": decodeHtmlEntity($scope.translation.WEBMOB_fhub2_food_reminder_freezer)},
                {"id": "STORAGE_FLEX_ZONE", "name": decodeHtmlEntity($scope.translation.WEBMOB_fhub2_food_reminder_flex_zone)},
                {"id": "STORAGE_PANTRY", "name": decodeHtmlEntity($scope.translation.WEBMOB_fhub2_food_reminder_pantry)}
            ];
        }
    }

    function drawCropCanvas() {
        debugMessage("drawCropCanvas:::");
        html2canvas($("#frame"), {
            background: '#000000',
            logging: (debugMode) ? true : false,
            onrendered: function (canvas) {
                try {
                    $scope.glazeImageForCrop = canvas.toDataURL();
                }
                catch (err) {
                    console.log('html2canvas::: ', err);
                    $scope.glazeImageForCrop = getImage('REF_GLAZE_' + ($scope.insideFridgeData.GlazeURL.length - 1));
                }

                // console.log("drawCropCanvas::: " + $scope.glazeImageForCrop);
                glazeProgressIncrease(Math.ceil(36 / $scope.insideFridgeData["GlazeURL"].length));
                console.log("[Time]drawCropCanvas::: ", new Date());

                dismissLoadingBar_noTimeOut("REF_GLAZE");
            }
        });
    }

    /**
     * @function
     * @private
     * @name parseFoodData
     * @param
     * @description
     **/
    function parseFoodData(device) {
        debugMessage('parseFoodData');
        $scope.Device.FoodList = { //reinitialize foodlist
            "Fridge": [],
            "Tag_List": []
        };

        console.log("parseFoodData >>>>>>>>", device["FoodList"]);
        if (angular.isDefined(device["FoodList"])) {
            device["FoodList"].forEach(function (food) {
                addFoodData(food);
            });
        }

        if (firstLaunch) {
            $scope.homePage = true;
            firstLaunch = false;
        }

        //To remove splash screen this variable is defined in top of the controller
        //$scope.mainLoadingScreen = true;
        dismissLoadingBar(REQUEST.GET_FOOD_LIST);
    }

    $scope.requestGetMemo = requestGetMemo;
    function requestGetMemo(item) {
        if ($scope.isStatic || $scope.isLocal) {
            var response = staticData.responseGetMemo(item.memoId);
            requestResponseHandler(response);
            return;
        }
        SHPService.sendNativeCommand(REQUEST.GET_MEMO, {"memoId":item.memoId}, REQUEST.GET_MEMO, false);
    }
    $scope.parseMemoData = parseMemoData;
    function parseMemoData(data) {
        if (angular.isDefined(data) && angular.isDefined(data["Memo"])) {
            data["Memo"].forEach(function (memo) {
                console.log("parseMemoData:::", memo);
                if (angular.isDefined(memo) && memo !== null && angular.isDefined(memo.id)) {
                    var index = $scope.Device.FoodList.Tag_List.map(function (item) {return item.memoId;}).indexOf(memo.id);
                    if (index !== -1) {
                        if (angular.isDefined(memo.imageURL) && memo.imageURL !== "")
                            imageURL_Request(memo.imageURL);
                        $scope.Device.FoodList.Tag_List[index].Memo = memo;
                    }
                }
            });
        }
    }

    /** Starts Memo Tag UI & GUI **/
    $scope.receiver_avatar_div_width = function (length) {
        if (length === 0 || length === 1)
            return {};
        return {
            width: (0.525) + (0.25 * (length - 1)) + "rem"
        }
    };
    $scope.icon_overlap = function (index) {
        if (index === 0)
            return {};
        return {
                left: (-0.25 * index) + "rem"
        };
    };

    $scope.receiver_text_width = function (length) {
        if (length > 1) {
            var avatar_style = $scope.receiver_avatar_div_width(length);
            if (angular.isDefined(avatar_style) && angular.isDefined(avatar_style.width)) {
                return {
                    width: 'calc(100% - 0.2rem - 0.1rem - ' + avatar_style.width + ')'
                }
            }
        }
        return {};
    };

    $scope.getReceiverStr = function (receiverList) {
        var output = "";
        if (angular.isDefined(receiverList)) {
            receiverList.forEach(function (receiver, index) {
                output += profileService.getScreenName(receiver);
                if (index !== receiverList.length - 1)
                    output += ",";
            });
        }
        return output;
    };

    $scope.getWriterAndTimeStr = function (memoTag) {
        var output = "";

        if (angular.isDefined(memoTag) && angular.isDefined(memoTag.Memo) && angular.isDefined(memoTag.Memo.writer)) {
            var writerName = profileService.getScreenName(memoTag.Memo.writer);
            var displayTime = prepareMemoText(memoTag.inputDate);

            if (angular.isDefined(writerName) && writerName !== "")
                output = $scope.translation.FHUBMOB_fhub2_memo_writer.replace("#user#", writerName);
            if (angular.isDefined(displayTime) && displayTime !== "")
                output += (" " + displayTime);
        }

        return output;
    };
    /** Ends Memo Tag UI & GUI **/

    $scope.parseCategoryDB = parseCategoryDB;
    $scope.getCategoryInfo = getCategoryInfo;
    function parseCategoryDB(data) {
        if (angular.isDefined(data)) {
            $scope.categoryDB = [];
            data.forEach(function (category) {
                if (angular.isDefined(category.Order) && category.Order !== null && category.Order !== "")
                    category.Order = parseInt(category.Order);
                $scope.categoryDB.push(category);
            });
        }
    }

    $scope.convertToCategory_ID = convertToCategory_ID;
    function convertToCategory_ID(whisk_category_id) {
        //Convert Whisk Category id to Master Category
        for (var i = 0; i < categoryMap.length; i++) {
            var category = categoryMap[i];
            if (whisk_category_id === category.Whisk_Category)
                return category.Category_ID;
        }
        return default_master_category_id;
    }

    $scope.convertToWhisk_Category = convertToWhisk_Category;
    function convertToWhisk_Category(masterCategory_id) {
        //Convert Master Category to Whisk Category id
        for (var i = 0; i < categoryMap.length; i++) {
            var category = categoryMap[i];
            if (masterCategory_id === category.Category_ID)
                return category.Whisk_Category;
        }
        return default_category_id;
    }

    function getCategoryInfo(categoryId) {
        if (angular.isDefined(categoryId) && categoryId !== "") {
            for (var index = 0; index < $scope.categoryDB.length; index++) {
                var categoryInfo = $scope.categoryDB[index];
                if (categoryInfo.Category_ID === categoryId)
                    return categoryInfo;
            }
        }
        return undefined;
    }

    function parseShoppingData(device) {
        debugMessage("parseShoppingData");
        if (angular.isUndefined(device["ShoppingList"]) || angular.isUndefined($scope.Device.ShoppingList))
            return;

        if (angular.isDefined(device["ShoppingList"])) {
            device["ShoppingList"].forEach(function (deviceShoppingList) {

                if(deviceShoppingList.groupId === deviceShoppingList.id) {       //Title Info
                    deviceShoppingList.category = decodeUnescape(deviceShoppingList.category);
                    if (!angular.isDefined($scope.Device.ShoppingList[deviceShoppingList.groupId])) {
                        $scope.Device.ShoppingList[deviceShoppingList.groupId] = angular.copy(deviceShoppingList);
                        $scope.Device.ShoppingList[deviceShoppingList.groupId].items = [];
                        $scope.Device.ShoppingList[deviceShoppingList.groupId].isTemp = false;
                        debugMessage("parseShoppingData:: deviceShoppingList=" + JSON.stringify($scope.Device.ShoppingList[deviceShoppingList.groupId]));
                    } else {
                        deviceShoppingList.items = $scope.Device.ShoppingList[deviceShoppingList.groupId].items;
                        $scope.Device.ShoppingList[deviceShoppingList.groupId] = angular.copy(deviceShoppingList);
                        $scope.Device.ShoppingList[deviceShoppingList.groupId].isTemp = false;
                    }
                }
                else {      //Item Info
                    if (angular.isDefined(device["event"]) && device["event"] === "Notified") {
                        //if item is moved to other list, delete item before saved list
                        $.each($scope.Device.ShoppingList, function (groupId, items) {
                            if (deviceShoppingList.groupId === groupId)
                                return;
                            var index = $scope.Device.ShoppingList[groupId].items.map(function(item) {return item.id; }).indexOf(deviceShoppingList.id);
                            if (index !== -1) {
                                $scope.Device.ShoppingList[groupId].items.splice(index, 1);
                            }
                        });
                    }

                    // console.log("deviceShoppingList ", deviceShoppingItem);
                    //Title Info is not defined, create the temp title info.(isTemp = true)
                    if (angular.isUndefined($scope.Device.ShoppingList[deviceShoppingList.groupId])) {
                        $scope.Device.ShoppingList[deviceShoppingList.groupId] = angular.copy(deviceShoppingList);
                        $scope.Device.ShoppingList[deviceShoppingList.groupId].id = deviceShoppingList.groupId;
                        $scope.Device.ShoppingList[deviceShoppingList.groupId].category = decodeHtmlEntity($scope.translation.WEBMOB_fhub2_shopping_list_create_new_title);
                        $scope.Device.ShoppingList[deviceShoppingList.groupId].isDefault = deviceShoppingList.isDefault;
                        $scope.Device.ShoppingList[deviceShoppingList.groupId].items = [];
                        $scope.Device.ShoppingList[deviceShoppingList.groupId].isTemp = true;
                    }

                    //Save Item info.
                    if (angular.isDefined($scope.Device.ShoppingList[deviceShoppingList.groupId])) {
                        deviceShoppingList.name = decodeUnescape(deviceShoppingList.name);
                        deviceShoppingList.recipeName = decodeUnescape(deviceShoppingList.recipeName);

                        if (angular.isDefined(deviceShoppingList.food_position) && isAllowLocationType(deviceShoppingList.food_position)) {
                            if (deviceShoppingList.food_position.indexOf('STORAGE_FLEX_ZONE') !== -1) {
                                //integrated the flex zone type by StorageModes of GlazeInfo
                                for (var i = 0; i < $scope.locations.length; i++) {
                                    if ($scope.locations[i].id.indexOf('STORAGE_FLEX_ZONE') !== -1) {
                                        deviceShoppingList.food_position = $scope.locations[i].id;
                                    }
                                }
                            }
                        }
                        else {
                            deviceShoppingList.food_position = $scope.locations[0].id;
                        }

                        var index = $scope.Device.ShoppingList[deviceShoppingList.groupId].items.map(function(item) {return item.id; }).indexOf(deviceShoppingList.id);
                        if (index === -1){
                            $scope.Device.ShoppingList[deviceShoppingList.groupId].items.push(deviceShoppingList);
                        }
                        else {
                            $scope.Device.ShoppingList[deviceShoppingList.groupId].items[index] = angular.copy(deviceShoppingList);
                        }
                    }

                }
            });
        }
    }

    $scope.isMaximum_ShoppingList = isMaximum_ShoppingList;
    function isMaximum_ShoppingList() {
        var savedDefaultList = false;
        $.each($scope.Device.ShoppingList, function (groupId, list) {
            if (angular.isDefined(list.isDefault) && list.isDefault === true) {
                savedDefaultList = true;
                return;
            }
        });

        debugMessage('isMaximum_ShoppingList:::savedDefaultList = ' + savedDefaultList);
        if (savedDefaultList)
            return (Object.keys($scope.Device.ShoppingList).length >= ($scope.MAX_SHOPPING_LIST_TITLE + 1));
        else
            return (Object.keys($scope.Device.ShoppingList).length >= $scope.MAX_SHOPPING_LIST_TITLE);
    }

    $scope.isMaximum_ShoppingList_item = isMaximum_ShoppingList_item;
    function isMaximum_ShoppingList_item(groupId, addItemCount) {
        if (angular.isDefined($scope.Device.ShoppingList[groupId]) && groupId !== '')
            return (($scope.Device.ShoppingList[groupId].items.length + addItemCount) > $scope.MAX_SHOPPING_LIST_ITEM);
        else
            return addItemCount > $scope.MAX_SHOPPING_LIST_ITEM;
    }

    /**
     * @function
     * @private
     * @name deleteFoodData
     * @param
     * @description
     **/
    function deleteFoodData(device) {
        debugMessage('deleteFoodData');
        var delDone = false;
        device["FoodList"].forEach(function (deviceFoodList) {
            for (var index = 0; index < $scope.Device.FoodList.Fridge.length; index++) {
                if ($scope.Device.FoodList.Fridge[index].id === deviceFoodList.id) {
                    $scope.Device.FoodList.Fridge.splice(index, 1);

                    //in case of delete notification, if the same item is being edited in app then cancel the edit operation
                    if ($scope.inputData.id === deviceFoodList.id && $scope.editFoodItem)
                        $scope.editFoodItem && cancelFood(); //if edit food popup is open then cancel edit food
                    break;
                }
            }

            //CHECK IF MEMO TAG IS CREATED FOR DELETED ID
            for (var index = 0; index < $scope.Device.FoodList.Tag_List.length; index++) {
                if ($scope.Device.FoodList.Tag_List[index].id === deviceFoodList.id) {
                    $scope.Device.FoodList.Tag_List.splice(index, 1);
                    //DELETE MEMO TAG FROM UI [GAURAV]
                    $("#tag_element_" + deviceFoodList.id) && $("#tag_element_" + deviceFoodList.id).remove();
                    break;
                }
            }
        });
    }

    function isAllowLocationType(storageType) {
        if (angular.isDefined(storageType) && storageType !== null && storageType !== "") {
            return (storageType === "STORAGE_FRIDGE" || storageType === "STORAGE_FREEZER" || storageType === "STORAGE_PANTRY" ||
                storageType === "STORAGE_FLEX_ZONE" || storageType === "STORAGE_FLEX_ZONE_18K" || storageType === "STORAGE_TAG");
        }
        return false;
    }

    function addFoodData(item) {
        $scope.showConsoleLogs &&  console.log("item >>>>>>>>>>>>>> " +item);
        item.inputDate = timeStamp.convertDate(item.inputDate);
        item.entryDate = timeStamp.convertDate(item.entryDate);

        if (item.expirationDate !== "")
            item.expirationDate = timeStamp.convertDate(item.expirationDate);

        if (angular.isDefined(item.isReminder) && item.isReminder) {
            item.expirationDate_order = item.expirationDate;
        }
        else {
            //expirationDate_order: If item is saved isReminder = false, do not sort expirationDate.
            item.expirationDate_order = "";
        }

        item.keepDays = diffDays(item.entryDate, new Date());
        item.calculateDays = diffDays(new Date(), item.expirationDate);
        item.name = decodeUnescape(item.name);

        if (isAllowLocationType(item.storageType)) {
            if (item.storageType.indexOf('STORAGE_FLEX_ZONE') !== -1) {     //integrated the flex zone type by StorageModes of GlazeInfo
                for (var i = 0; i < $scope.locations.length; i++) {
                    if ($scope.locations[i].id.indexOf('STORAGE_FLEX_ZONE') !== -1) {
                        item.storageType = $scope.locations[i].id;
                    }
                }
            }
        }
        else {
            item.storageType = $scope.locations[0].id;
        }

        item.storageType_idx = getLocationIndex(item.storageType);
        if (angular.isUndefined(item.is_expiry_tag)) {
            item.is_expiry_tag = false;
        }
        if (angular.isUndefined(item.category_info)) {
            item.category_info = default_category_id;
        }
        if (angular.isDefined(item.Glaze)) {
            if (angular.isDefined(item.Glaze.depth)) {
                item.Glaze.depth = timeStamp.convertDate(item.Glaze.depth);
            } else {
                //if undefined depth, define to inputData.
                item.Glaze.depth = item.inputDate;
            }
        } else {
            item.Glaze = {
                shelfId: 0,
                tagPositionX: 0,
                tagPositionY: 0,
                depth: item.inputDate
            };
        }

        if (angular.isDefined(item.recommended)) {
            if (angular.isUndefined(item.recommended.location))
                item.recommended.location = false;
            if (angular.isUndefined(item.recommended.expiration))
                item.recommended.expiration = false;
        } else {
            item.recommended = {
                location: false,
                expiration: false
            };
        }

        //ADD to TAG_LIST
        if (item.storageType === 'STORAGE_TAG') {//parseMemo based on STORAGE_TYPE
            $scope.Device.FoodList.Tag_List.push(item);
            requestGetMemo(item);
            //ADD MEMO TAG TO UI [GAURAV]
            parseTagData(); //Recreate tags
        }else { //add to food list
            $scope.Device.FoodList.Fridge.push(item);
            imageURL_Request(item);
        }
    }

    /**
     * @function
     * @private
     * @name updateFoodData
     * @param
     * @description
     **/
    function updateFoodData(item) {
        var isUpdated = false;
        if (item.storageType === "STORAGE_TAG") {   // FOR MEMO TAGS
            //check if tag already exists, then get the index of that tag object
            var index = $scope.Device.FoodList.Tag_List.map(function(item) {return item.id; }).indexOf(item.id);
            $scope.showConsoleLogs &&  console.log("updateFoodData >>>>>>>>>>>>>" + index);
            if (index !== -1){//if tag already exist then update object
                var saved_item = $scope.Device.FoodList.Tag_List[index];
                //If same item is already saved, do not update again.
                if (item.Glaze.tagPositionX === saved_item.Glaze.tagPositionX &&
                    item.Glaze.tagPositionY === saved_item.Glaze.tagPositionY &&
                    angular.isDefined(item.Glaze.depth) && timeStamp.convertDate(item.Glaze.depth) === saved_item.Glaze.depth &&
                    timeStamp.convertDate(item.inputDate) === saved_item.inputDate)
                    return;

                var memo_data = angular.copy($scope.Device.FoodList.Tag_List[index].Memo);
                $scope.Device.FoodList.Tag_List[index] = angular.copy(item);
                $scope.Device.FoodList.Tag_List[index].Memo = memo_data;
                requestGetMemo(item);
                if (item.expirationDate !== ""){
                    $scope.Device.FoodList.Tag_List[index].expirationDate = timeStamp.convertDate(item.expirationDate);
                }
                else{
                    $scope.Device.FoodList.Tag_List[index].expirationDate = "";
                }
                $scope.Device.FoodList.Tag_List[index].entryDate = timeStamp.convertDate(item.entryDate);
                $scope.Device.FoodList.Tag_List[index].name = decodeUnescape(item.name);
                $scope.Device.FoodList.Tag_List[index].inputDate = timeStamp.convertDate(item.inputDate);
                $scope.Device.FoodList.Tag_List[index].keepDays = diffDays(item.entryDate, new Date());
                $scope.Device.FoodList.Tag_List[index].calculateDays = diffDays(new Date(), item.expirationDate);
                $scope.Device.FoodList.Tag_List[index].storageType_idx = getLocationIndex(item.storageType);
                if (angular.isDefined($scope.Device.FoodList.Tag_List[index].Glaze.depth)) {
                    $scope.Device.FoodList.Tag_List[index].Glaze.depth = timeStamp.convertDate($scope.Device.FoodList.Tag_List[index].Glaze.depth);
                } else {
                    //if undefined depth, define to inputData.
                    $scope.Device.FoodList.Tag_List[index].Glaze.depth = $scope.Device.FoodList.Tag_List[index].inputDate;
                }

                isUpdated = true;
                parseTagData(); //Recreate tags
            }
        }
        else{
            //check if tag already exists, then get the index of that tag object
            for (var fl = 0; fl < $scope.Device.FoodList.Fridge.length; fl++) {
                if ($scope.Device.FoodList.Fridge[fl].id === item.id) {
                    if (item.expirationDate !== "")
                        item.expirationDate = timeStamp.convertDate(item.expirationDate);
                    else
                        item.expirationDate = "";

                    if (angular.isDefined(item.isReminder) && item.isReminder) {
                        item.expirationDate_order = item.expirationDate;
                    }
                    else {
                        //expirationDate_order: If item is saved isReminder = false, do not sort expirationDate.
                        item.expirationDate_order = "";
                    }

                    item.entryDate = timeStamp.convertDate(item.entryDate);
                    item.name = decodeUnescape(item.name);
                    item.inputDate = timeStamp.convertDate(item.inputDate);
                    item.keepDays = diffDays(item.entryDate, new Date());
                    item.calculateDays = diffDays(new Date(), item.expirationDate);

                    if (isAllowLocationType(item.storageType)) {
                        if (item.storageType.indexOf('STORAGE_FLEX_ZONE') !== -1) {     //integrated the flex zone type by StorageModes of GlazeInfo
                            for (var i = 0; i < $scope.locations.length; i++) {
                                if ($scope.locations[i].id.indexOf('STORAGE_FLEX_ZONE') !== -1) {
                                    item.storageType = $scope.locations[i].id;
                                }
                            }
                        }
                    }
                    else {
                        item.storageType = $scope.locations[0].id;
                    }

                    item.storageType_idx = getLocationIndex(item.storageType);
                    if (angular.isUndefined(item.is_expiry_tag)) {
                        item.is_expiry_tag = false;
                    }
                    if (angular.isUndefined(item.category_info)) {
                        item.category_info = default_category_id;
                    }
                    if (angular.isDefined(item.Glaze.depth)) {
                        item.Glaze.depth = timeStamp.convertDate(item.Glaze.depth);
                    } else {
                        //if undefined depth, define to inputData.
                        item.Glaze.depth = item.inputDate;
                    }

                    if (angular.isDefined(item.recommended)) {
                        if (angular.isUndefined(item.recommended.location))
                            item.recommended.location = false;
                        if (angular.isUndefined(item.recommended.expiration))
                            item.recommended.expiration = false;
                    } else {
                        item.recommended = {
                            location: false,
                            expiration: false
                        };
                    }

                    $scope.Device.FoodList.Fridge[fl] = angular.copy(item);
                    imageURL_Request($scope.Device.FoodList.Fridge[fl]);

                    if (item.id === $scope.inputData.id && $scope.editFoodItem) {
                        $scope.inputData.name = $scope.Device.FoodList.Fridge[fl].name;
                        $scope.inputData.imageURL = $scope.Device.FoodList.Fridge[fl].imageURL;
                        $scope.inputData.isReminder = $scope.Device.FoodList.Fridge[fl].isReminder;
                        $scope.inputData.entryDate = $scope.Device.FoodList.Fridge[fl].entryDate;
                        $scope.inputData.expirationDate = $scope.Device.FoodList.Fridge[fl].expirationDate;
                        $scope.inputData.storageType = $scope.Device.FoodList.Fridge[fl].storageType;
                        $scope.inputData.storageType_idx = getLocationIndex($scope.Device.FoodList.Fridge[fl].storageType);
                    }
                    isUpdated = true;
                }
            }
        }
        return isUpdated;
    }

    /**
     * @function
     * @private
     * @name showAddFoodItemPage
     * @param
     * @description
     **/
    function showAddFoodItemPage() {
        if (isMaxItem()) {
            SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_food_reminder_add_max_items.replace("#num#", $scope.MAX_FOOD_ITEM.toString()), "");
            return;
        }

        $scope.addFoodItem = true;
        $scope.editFoodItem = false;
        $scope.daySelector = false;
        $scope.isCalender = false;
        $scope.daySelector = false;
        // $scope.AddFoodImage = "";
        $scope.selectedFoodData = undefined;
        $scope.initial_inputData();
        $scope.inputData.isReminder = true;

        goToFoodItem();

        $timeout(function () {
            $("#food_name").focus();
        }, 300);
    }

    function showAddFoodItemPageFromCrop() {
        if (isMaxItem()) {
            SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_food_reminder_add_max_items.replace("#num#", $scope.MAX_FOOD_ITEM.toString()), "");
            return;
        }

        $scope.addFoodItem = true;
        $scope.editFoodItem = false;
        $scope.daySelector = false;
        $scope.isCalender = false;
        $scope.daySelector = false;
        // $scope.AddFoodImage = $scope.myCroppedImage;
        $scope.selectedFoodData = undefined;
        $scope.initial_inputData();
        $scope.inputData.isAllowRecommend.location = false;     //do not recommend location automatically
        imageRequest.parseNativeResponse({"ImageList": [{imageURL: "newImage", imageData: $scope.myCroppedImage}]});
        $scope.inputData.imageURL = "newImage";
        $scope.inputData.data_type = "expiryTag";

        $scope.inputData.isReminder = true;
        if ($scope.isSupportFoodTag && !isMaxExpiryTag()) {
            $scope.inputData.is_expiry_tag = true;
        }

        closeCropScreen();
        goToFoodItem();

        $timeout(function () {
            $("#food_name").focus();
        }, 300);
    }

    $scope.move_shopping_list = [];
    $scope.isObjectCount = isObjectCount;
    $scope.setSelectedCategory = setSelectedCategory;
    function isObjectCount(obj) {
        if (angular.isDefined(obj))
            return Object.keys(obj).length;
        return 0;
    }

    function setSelectedCategory(lists, notSelectId) {
        if (isObjectCount(lists) === 0) {
            $scope.inputData.selectedCategory = '';
        }
        else {
            Object.keys(lists).forEach(function(key, index, array) {
                if (index === 0) {
                    if (angular.isDefined(notSelectId) && key === notSelectId) {
                        debugMessage("setSelectedCategory(true):::: " + notSelectId + " / " + index + " / " + array[1]);
                        $scope.inputData.selectedCategory = lists[array[1]];
                    }
                    else {
                        debugMessage("setSelectedCategory(false):::: " + notSelectId + " / " + index + " / " + array[0]);
                        $scope.inputData.selectedCategory = lists[array[0]];
                    }
                }
            });
        }
    }

    function showSendToShopListPopUpFromCrop() {
//        	console.log("croped image: " + newImage);
        $scope.addFoodItem = false;
        closeCropScreen();
        $scope.goToSendToShoppingList();
        $scope.editFoodItem = false;
        $scope.daySelector = false;
        $scope.isCalender = false;
        $scope.daySelector = false;
        // $scope.AddFoodImage = $scope.myCroppedImage;
        $scope.selectedFoodData = undefined;
        $scope.initial_inputData();
        imageRequest.parseNativeResponse({"ImageList": [{imageURL: "newImage", imageData: $scope.myCroppedImage}]});
        $scope.inputData.imageURL = "newImage";

        $scope.move_shopping_list = $filter('orderShoppingListBy')($scope.Device.ShoppingList, 'isDefault', 'groupId', false, $scope.Device.ShoppingList);
        setSelectedCategory($scope.move_shopping_list);

        $timeout(function () {
            $("#send_item_name").focus();
        }, 300);
    }

    function getTagType(element) {
        if (angular.isDefined(element)) {
            if (element.hasClass('tag_memo'))
                return 'memoTag';
            else if (element.hasClass('tag_food'))
                return 'expiryTag';
        }
        return undefined;
    }

    function mobileTagSize(tagType) {
        // console.log('mobileTagSize:::', tagType);
        var Mobile_tag_size = {
            width: 342,
            height: 342
        };

        if (angular.isDefined(tagType) && tagType === 'memoTag') {
            Mobile_tag_size = {
                width: 342,
                height: 342
            };
        }
        else if (angular.isDefined(tagType) && tagType === 'expiryTag') {
            Mobile_tag_size = {
                width: 258,
                height: 104
            };
        }

        var baseFontSize = parseFloat(document.getElementById("htmlTag").style.fontSize.replace('px', ''));
        return {
            width: (Mobile_tag_size.width / 80) * baseFontSize,
            height: (Mobile_tag_size.height / 80) * baseFontSize
        };  //convert px to rem of tag size and calculate px according to basefont.
    }

    function getTagResolution_MOBILE(tagType) {
        var MOBILE_resolution_width = cWidth;     //MOBILE Resoluton: 1440 * 1905
        var MOBILE_resolution_height = cHeight;
        var tag_resolution = {width: MOBILE_resolution_width, height: MOBILE_resolution_height};
        var convert_mobile_tag_size  = mobileTagSize(tagType);
        tag_resolution.width = MOBILE_resolution_width - convert_mobile_tag_size.width;
        tag_resolution.height = MOBILE_resolution_height - convert_mobile_tag_size.height;

        // console.log('getTagResolution_MOBILE:::', tagType, convert_mobile_tag_size, tag_resolution);
        return tag_resolution;
    }

    function getTagResolution_LCD(tagType) {
        var LCD_resolution_width = $scope.LCD_GLAZE_RESOLUTION.width;
        var LCD_resolution_height = $scope.LCD_GLAZE_RESOLUTION.height;
        var tag_resolution = {width: LCD_resolution_width, height: LCD_resolution_height};
        if (angular.isDefined(tagType) && tagType === 'memoTag') {
            tag_resolution.width -= $scope.LCD_MEMO_TAG_SIZE.width;
            tag_resolution.height -= $scope.LCD_MEMO_TAG_SIZE.height;
        }
        else if (angular.isDefined(tagType) && tagType === 'expiryTag') {
            tag_resolution.width -= $scope.LCD_EXPIRY_TAG_SIZE.width;
            tag_resolution.height -= $scope.LCD_EXPIRY_TAG_SIZE.height;
        }

        // console.log('getTagResolution_LCD:::', tagType, tag_resolution);
        return tag_resolution;
    }

    function addMemoFromCrop() {
        if ($scope.Device.FoodList.Tag_List.length >= $scope.MAX_MEMO_TAG){
            SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_food_reminder_overtag.replace("#num#", $scope.MAX_MEMO_TAG.toString()), "");
            return;
        }

        imageRequest.cutCircleImage('newImage', $scope.myCroppedImage);
        var asyncQuery = $timeout(function() {
            return getImage('newImage');
        },500);

        return asyncQuery.then(
            function successCallback(response_cropImage) {
                console.log("addMemoFromCrop:::successCallback");
                addMemoCommand(response_cropImage);
            },
            function errorCallback(response) {
                console.log("addMemoFromCrop:::errorCallback = ", response);
                addMemoCommand($scope.myCroppedImage);
            }
        );
    }

    /*
     * addMemoCommand
     * @description
     *  Send add memo request with the following details
     *
     *  "FoodList": [{
     *     "id":"201609211233000", //UTC TIMESTAMP
     *       "Glaze":{
     *           "shelfID":0,        //fixed only 0
     *           "tagPositionX":101, //range: 0~1080 positionX
     *           "tagPositionY":200  //range: 0~1430 positionY
     *       },
     *       "name":"tag",
     *       "category":" ",
     *       "quantity":1,
     *       "quantityUnit":"ea",
     *       "expirationDate":"",
     *       "inputDate":"",
     *       "storageType":"STORAGE_TAG",
     *       "isNewlyAdded":true,
     *       "imageIndex":"2",
     *       "imageURL":"data:image/png;base64,image data",
     *       "isReminder":false,
     *       "memoId":""
     * }]
     */
    function addMemoCommand(cropImage_src) {
        console.log("before memoCoordinates " + $scope.memoCoordinates);
        var tempPosX = $scope.memoCoordinates[0] <=60 ? 40 : 30;
        var tempPosY = $scope.memoCoordinates[1] <=60 ? 40 : 30;
        $scope.memoCoordinates[0] = $scope.memoCoordinates[0] > tempPosX ? $scope.memoCoordinates[0] - tempPosX : $scope.memoCoordinates[0];
        $scope.memoCoordinates[1] = $scope.memoCoordinates[1] > tempPosY ? $scope.memoCoordinates[1] - tempPosY : $scope.memoCoordinates[1];
        console.log("after $scope.memoCoordinates " + $scope.memoCoordinates);

        var Mobile_Resolution = getTagResolution_MOBILE('memoTag');
        var LCD_Resolution = getTagResolution_LCD('memoTag');
        var memoTag_info = {
            id: timeStamp.getDateFormat(new Date(), 'yyyyMMddHHmmss sss', 'UTC').replace(" ", ""),
            Glaze: {
                shelfID: 0,        //fixed only 0
                tagPositionX: ((parseInt($scope.memoCoordinates[0], 10) / Mobile_Resolution.width * scaleFactor) * LCD_Resolution.width) ,
                tagPositionY: ((parseInt($scope.memoCoordinates[1], 10) / Mobile_Resolution.height * scaleFactor) * LCD_Resolution.height),
                depth: timeStamp.getDateFormat(new Date(), 'yyyy-MM-ddTHH:mm:ss', 'UTC')
            },
            entryDate: new Date(),
            storageType: "STORAGE_TAG",
            imageURL: cropImage_src.replace(BASE64, "")
        };

        var payload = {
            "FoodList": [
                getFoodListFormat(memoTag_info)
            ]
        };
        $scope.showConsoleLogs &&  console.log("Add memo payload>>>>>>", payload);
        SHPService.sendNativeCommand(REQUEST.ADD_MEMO, payload, REQUEST.ADD_MEMO, false);
        closeCropScreen();
    }

    /*
     * goToMemo
     * @param {string} memoId (UTC DATE TIMESTAMP)
     * @param {string} itemId (UTC DATE TIMESTAMP)
     * @description
     *  Send request to native to open already saved memo 
     *  {
     *      "memoId":"201505200544123"
     *  }
     *  Response Sample (If memo is deleted from memo, requestFail will received)
     *  {
     *       "type":"requestFailed",
     *       "status":400,
     *       "requestToken": "gotoMemo",
     *       "data":{}
     *   }
     *  
     */
    function goToMemo(memoId, itemId) {
        $scope.dragCheckProcessed = false;
        console.log("goToMemo ",$(event.target).attr('class'));
        if ($(event.target).attr('class') === "tag_memo_close_div"){
            return;
        }
        debugMessage('GOTOMEMO------------------------');
        if (dragCheck === true){
            return;
        }
        var payload = {"memoId": memoId},
        e = window.event;
        e.stopPropagation();
        e.preventDefault();
        //sometimes at this point drag gets started on the element, revert the drag
        var element = $("#tag_element_" + itemId);
        element.draggable( "option", "revert", true );
        element.trigger('mouseup');
        $timeout(function(){
            SHPService.sendNativeCommand(REQUEST.GOTO_MEMO, payload, REQUEST.GOTO_MEMO, false);
        }, 500);
    }

    $scope.showEditFoodItemPageFromTag = function (item) {
        $scope.dragCheckProcessed = false;
        console.log("showEditFoodItemPageFromTag::: ", $(event.target).attr('class'));
        if ($(event.target).hasClass('tag_food_close_div')){
            return;
        }
        console.log('showEditFoodItemPageFromTag------------------------');
        if (dragCheck === true){
            return;
        }

        var e = window.event;
        e.stopPropagation();
        e.preventDefault();
        var element = $("#tag_element_" + item.id);
        //sometimes at this point drag gets started on the element, revert the drag
        element.draggable( "option", "revert", true );
        element.trigger('mouseup');

        showEditFoodItemPage(item);
    };

    /*
     * deleteMemo
     * @param {string} memoId (UTC DATE TIMESTAMP)
     * @description
     *  Send request to native to delete memo (Request is similar to delete foodList)
     *  {
     *      "memoId":"201505200544123"
     *  }     
     *  
     */
    function deleteMemo(){
        if (dragCheck === true){
            return;
        }
        if(!angular.isDefined(deleteItemId) || !angular.isDefined(deleteMemoId)){
            console.log("IDs are undefined.");
            return;
        }
        var payloadDeleteMemo = "/foodmanager/memo/?id=" + deleteMemoId,
        payloadDeleteFoodItem = "/foodmanager/foodlist/?id=" + deleteItemId,
        e = window.event;
        e.stopPropagation();
        e.preventDefault();
        
        deleteItemId = null;
        deleteMemoId = null;
        $scope.checkResponse.push(REQUEST.DELETE_MEMO);
        SHPService.sendNativeCommand(REQUEST.DELETE_MEMO, payloadDeleteMemo + '&sendPush=false',REQUEST.DELETE_MEMO);
       
        $scope.checkResponse.push(REQUEST.DELETE_FOOD_LIST);
        SHPService.sendNativeCommand(REQUEST.DELETE_FOOD_LIST, payloadDeleteFoodItem + '&sendPush=true', REQUEST.DELETE_FOOD_LIST);
        return false;
    }

    function moveTag(food_id){
        $scope.selectedTag = $("#tag_element_" + food_id);
        console.log("moveTag:::", food_id, $scope.selectedTag);
        var index = -1;
        var food_item = undefined;
        var tagType = undefined;
        if ($scope.selectedTag.hasClass('tag_memo')) {      //Memo Tag
            index = $scope.Device.FoodList.Tag_List.map(function(item) {return item.id; }).indexOf(food_id);
            if (index !== -1) {
                food_item = $scope.Device.FoodList.Tag_List[index];
            }
            tagType = 'memoTag';
        }

        if ($scope.selectedTag.hasClass('tag_food')) {      //Expiry Tag
            index = $scope.Device.FoodList.Fridge.map(function(item) {return item.id; }).indexOf(food_id);
            if (index !== -1) {
                food_item = $scope.Device.FoodList.Fridge[index];
            }
            tagType = 'expiryTag';
        }

        if (angular.isDefined(food_item)){
            console.log("toEditAnItem CWIDTH >>>> " + cWidth + "CHEIGHT >>>> " + cHeight);
            console.log("toEditAnItem top >>>> " + $scope.selectedTag.position().top + "left >>>> " + $scope.selectedTag.position().left);
            var Mobile_Resolution = getTagResolution_MOBILE(tagType);
            var LCD_Resolution = getTagResolution_LCD(tagType);
            var calcTagPositionX = (((($scope.selectedTag.position().left)/Mobile_Resolution.width) * LCD_Resolution.width));
            var calcTagPositionY = (((($scope.selectedTag.position().top)/Mobile_Resolution.height) * LCD_Resolution.height));

            if (angular.isDefined(food_item.Glaze)) {
                food_item.Glaze.shelfID = 0;
                food_item.Glaze.tagPositionX = (calcTagPositionX < 0)? 0 : calcTagPositionX;
                food_item.Glaze.tagPositionY = (calcTagPositionY < 0)? 0 : calcTagPositionY;
                food_item.Glaze.depth = timeStamp.getDateFormat(new Date(), 'yyyy-MM-ddTHH:mm:ss', 'UTC');
            }

            var payload = {
                "FoodList": [
                    getFoodListFormat(food_item)
            ]};
            $scope.checkResponse.push(REQUEST.EDIT_FOOD_LIST);
            SHPService.sendNativeCommand(REQUEST.EDIT_FOOD_LIST, payload, REQUEST.EDIT_FOOD_LIST);
        }
    }
    
    /*
     * prepareMemoText
     * @param {string} inputDate
     *  inputDate of memo (comes in add memo response)
     * @returns {String} memoText  
     * @description
     *  This method is used to format memo name based on the following rule
     *      - Created today and time diffrence is less than an hour - returns mins  e.g 15 mins ago
     *      - Created today and time diffrence is more than an hour - returns time in hr:min(AM|PM) e.g. 12:07PM
     *      - Older than one day - returns date (Month date, year) e.g. Sep 1, 2016
     */
    function prepareMemoText(inputDate) {
        try {
            var today = new Date(),
                memoText = "",
                inputDateObj = timeStamp.convertDate(inputDate),
                timeDiff = today.getTime() - inputDateObj.getTime(),
                calculatedDays = Math.floor(timeDiff / (1000 * 3600 * 24));
            // console.log(calculatedDays, timeDiff);
            // console.log("today.getDate()", today.getDate(), inputDateObj.getDate());
            if (calculatedDays === 0) {
                var calculatedHours = Math.floor(timeDiff / (1000 * 3600)),
                    calculatedMins = Math.floor(timeDiff / (1000 * 60));
                // hours = inputDateObj.getHours(),
                // minutes = inputDateObj.getMinutes();
                // console.log(hours, minutes, calculatedHours);
                if (calculatedHours === 0) {
                    // console.log("parse Tag::: calculatedMins = " + calculatedMins);
                    if (calculatedMins < 1)
                        memoText = $scope.translation.FHUBMOB_fhub2_time_just_now;
                    else if (calculatedMins === 1)
                        memoText = $scope.translation.FHUBMOB_fhub2_time_mins_ago_one.replace("#num#", calculatedMins.toString());
                    else {
                        memoText = $scope.translation.FHUBMOB_fhub2_time_mins_ago_many.replace("#num#", calculatedMins.toString());
                    }
                } else {
                    memoText = timeStamp.displayDate(inputDateObj, $scope.timeFormat);
                }
            } else {
                return memoText = timeStamp.displayDate(inputDate, $scope.dateFormat);
            }
        } catch (err) {
            // console.log('prepareMemoText:::', err);
        }

        return memoText;
    }
    
    /**    	 
    * @function
    * @private
    * @name parseTagData    * 
    * @description
    * Its responsible for creating memo tag [div] elements and append it to "container" div
    * Tag list is parsed from $scope.Device.FoodList.Tag_List
    **/
    function parseTagData() {
        console.log("parseTagData::: $scope.dragCheckProcessed ", $scope.dragCheckProcessed);
        if ($scope.dragCheckProcessed) {
            return;
        }

        // removeExistingTags();
        // $scope.Device.FoodList.Tag_List = $filter('orderTagListBy')($scope.Device.FoodList.Tag_List, 'Glaze', 'depth', false);
        // var tagList = $scope.Device.FoodList.Tag_List;
        //
        // console.log('tag created in parseTagData');
        // for (var i = 0; i < tagList.length; i++) {
        //     if (angular.isDefined(tagList[i]) && angular.isDefined(tagList[i].Glaze) && tagList[i].Glaze.shelfID !== -1) {
        //         cWidth = $("#frame").width();
        //         cHeight = $("#frame").height();
        //         createTag(tagList[i]);
        //     }
        // }
        $scope.isParseTag = false;
        $scope.enableDrag = true;
    }

    $scope.calculateTagPosition = calculateTagPosition;
    function calculateTagPosition (tag, index) {
        var position = {
            "top": 0,
            "left": 0,
            "z-index": 0
        };
        if (angular.isDefined(tag)) {
            var tag_element = $('#tag_element_' + tag.id);
            var tagType = $scope.getTagType(tag_element);

            if (angular.isDefined(tag) && angular.isDefined(tag.Glaze)) {
                var Mobile_Resolution = getTagResolution_MOBILE(tagType);
                var LCD_Resolution = getTagResolution_LCD(tagType);
                position["left"] = (tag.Glaze.tagPositionX * Mobile_Resolution.width / scaleFactor) / LCD_Resolution.width;
                position["top"] = (tag.Glaze.tagPositionY * Mobile_Resolution.height / scaleFactor) / LCD_Resolution.height;

                if (tag.storageType === "STORAGE_TAG")
                    position["z-index"] = index;
                else {
                    position["z-index"] = index + 100;
                }
                // console.log("calculateTagPosition::: tag id = ", tag.id, "top = ", position["top"], " left = ", position["left"], " z-index = ", position["z-index"]);
            }
        }

        return position;

    }

    $scope.isExpiryTag = isExpiryTag;
    function isExpiryTag (item) {
        if (angular.isUndefined(item) || angular.isUndefined(item.is_expiry_tag))
            return false;
        return item.is_expiry_tag;
    }

    /**
     * @function
     * @private
     * @name removeExistingTags
     * @description Remove all tags from glaze frame div
     **/
    function removeExistingTags() {
        debugMessage('Remove tags called');
        $("#frame .tag_memo").remove();
    }

    $scope.showSendToShoppingListPopup = showSendToShoppingListPopup;
    $scope.sendToShoppingListPopUp = false;
    function showSendToShoppingListPopup() {
        //Check item count for send to ShoppingList
        if ($scope.selectedItems.length > $scope.MAX_SHOPPING_LIST_ITEM) {
            SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_shopping_list_add_max_items.replace("#num#", $scope.MAX_SHOPPING_LIST_ITEM.toString()), "");
            return;
        }
        $scope.initial_inputData();
        $scope.sendToShoppingListPopUp = true;

        $scope.move_shopping_list = $filter('orderShoppingListBy')($scope.Device.ShoppingList, 'isDefault', 'groupId', false, $scope.Device.ShoppingList);
        setSelectedCategory($scope.move_shopping_list);
    }

    $scope.$watch(function () {return document.body.clientHeight;}, function (newValue, oldValue) {
        console.log("watch:::View Size is changed = " + oldValue + " to " + newValue);
    }, true);

    $scope.createNewListPopUp = false;
    $scope.showCreateNewListPopUp = showCreateNewListPopUp;
    $scope.closeCreateNewListPopUp = closeCreateNewListPopUp;
    function showCreateNewListPopUp() {
        removeFocus();
        $scope.inputData.createCategory = decodeHtmlEntity($scope.translation.WEBMOB_fhub2_shopping_list_create_new_title);
        $scope.createNewListPopUp = true;

        $timeout(function () {
            console.log('showCreateNewListPopUp:::newlistTitleID_createList-focus, select');
            $("#newlistTitleID_createList").focus();
            document.getElementById('newlistTitleID_createList').setSelectionRange(0, $scope.MAX_CHARACTER_ITEM);
        }, 0);
    }

    function closeCreateNewListPopUp() {
        removeFocus();
        $scope.inputData.createCategory = "";
        $scope.createNewListPopUp = false;
    }

    $scope.moveLocationPopup = false;
    $scope.showMoveLocationPopUp = showMoveLocationPopUp;
    $scope.closeMoveLocationPopup = closeMoveLocationPopup;
    function showMoveLocationPopUp() {
        debugMessage("showMoveLocationPopUp");
        $scope.inputData.storageType = (angular.isDefined($scope.locations[0]))? $scope.locations[0].id : '';
        $scope.moveLocationPopup = true;
    }

    function closeMoveLocationPopup() {
        $scope.inputData.storageType = (angular.isDefined($scope.locations[0]))? $scope.locations[0].id : '';
        $scope.moveLocationPopup = false;
    }

    $scope.changeImagePopup = false;
    $scope.showChangeImagePopup = showChangeImagePopup;
    $scope.closeChangeImagePopup = closeChangeImagePopup;
    $scope.changeImageCommand = changeImageCommand;
    $scope.hideChangeImageOption = hideChangeImageOption;

    function showChangeImagePopup() {
        $timeout(function () {
            $scope.changeImagePopup = true;
        }, 0);
    }

    function closeChangeImagePopup() {
        $timeout(function () {
            $scope.changeImagePopup = false;
        }, 0);
    }

    function changeImageCommand(option_id) {
        switch (option_id) {
            case $scope.CHANGE_IMAGE_OPTION[0].id:      //GO_TO_CAMERA
                SHPService.sendNativeCommand(REQUEST.GOTO_CAMERA, "", REQUEST.GOTO_CAMERA, false);
                break;
            case $scope.CHANGE_IMAGE_OPTION[1].id:      //GO_TO_GALLERY
                SHPService.sendNativeCommand(REQUEST.GOTO_GALLERY, "", REQUEST.GOTO_GALLERY, false);
                break;
            case $scope.CHANGE_IMAGE_OPTION[2].id:      //DEFAULT_COVER
                $scope.inputData.imageURL = "";
                // $scope.AddFoodImage = "";
                break;
            case $scope.CHANGE_IMAGE_OPTION[3].id:      //VIEW_PICTURE
                showViewPicturePopup();
                break;
            default:
                break;
        }
        closeChangeImagePopup();
    }

    function hideChangeImageOption(option_id) {
        switch (option_id) {
            case $scope.CHANGE_IMAGE_OPTION[0].id:      //GO_TO_CAMERA
            case $scope.CHANGE_IMAGE_OPTION[1].id:      //GO_TO_GALLERY
                if (isEmart($scope.inputData) || isWoolworth($scope.inputData))
                    return true;
                break;
            case $scope.CHANGE_IMAGE_OPTION[2].id:      //DEFAULT_COVER
                if (isEmptyString($scope.inputData.imageURL))
                    return true;
                if (isEmart($scope.inputData) || isWoolworth($scope.inputData))
                    return true;
                break;
            case $scope.CHANGE_IMAGE_OPTION[3].id:      //VIEW_PICTURE
                if (isEmptyString($scope.inputData.imageURL))
                    return true;
                break;
            default:
                break;
        }
        return false;
    }

    $scope.viewPicturePopup = false;
    $scope.showViewPicturePopup = showViewPicturePopup;
    $scope.closeViewPicturePopup = closeViewPicturePopup;

    function showViewPicturePopup() {
        $scope.viewPicturePopup = true;
    }

    function closeViewPicturePopup() {
        $scope.viewPicturePopup = false;
    }

    //yyyyMMddHHmmss sss = id format
    //yyyy-MM-ddTHH:mm:ss = time format of payload
    //dateFormat rule: Refer to https://docs.angularjs.org/api/ng/filter/date
    function getDateFormat(date, dateFormat, timezone) {
        //Checking the valid date.
        if (angular.isUndefined(date) || !(date instanceof Date && !isNaN(date.valueOf())))
            return "";

        //Checking the valid format.
        if (angular.isUndefined(dateFormat) || dateFormat === '')
            dateFormat = 'MMM d, yyyy, h:mm a';     //Default Format

        //Checking the valid timezone.
        if (angular.isDefined(timezone) && timezone !== '')
            return $filter('date')(date, dateFormat, timezone);
        else
            return $filter('date')(date, dateFormat);
    }

    $scope.getInputDate = getInputDate;
    function getInputDate() {
        var d_date = new Date();
        var day = d_date.getDate();
        var mnth = d_date.getMonth() + 1;
        var hour = d_date.getHours();
        var min = d_date.getMinutes();
        return d_date.getFullYear() + "-" + mnth + "-" + day + "T" + hour + ":" + min + ":00";
    }

    $scope.getDefaultExpirationDate = getDefaultExpirationDate;
    function getDefaultExpirationDate() {
        var date = new Date();
        date.setDate(date.getDate() + $scope.DEFAULT_DAYS);

        return date;
    }

    $scope.setExpirationDate = setExpirationDate;
    function setExpirationDate(startDate, days) {
        var date = new Date(startDate);
        date.setDate(date.getDate() + days);

        return date;
    }

    $scope.getLocationIndex = getLocationIndex;
    function getLocationIndex(storage_id) {
        for (var index = 0; index < $scope.DefineLocations.length; index++) {
            var location = $scope.DefineLocations[index];
            if (storage_id === location.id) {
                return index;
            }
        }
        return 0;
    }

    $scope.convertLocationIdToValue = convertLocationIdToValue;
    function convertLocationIdToValue(storage_id) {
        var location_idx = getLocationIndex(storage_id);
        return (angular.isDefined($scope.DefineLocations[location_idx].name))? $scope.DefineLocations[location_idx].name : "";
    }

    $scope.isExpandLocationList = function(storage_id) {
        var location_idx = getLocationIndex(storage_id);
        return $scope.DefineLocations[location_idx].isExpand;
    };

    $scope.toggleExpandLocationList = function (storage_id) {
        var location_idx = getLocationIndex(storage_id);
        $scope.DefineLocations[location_idx].isExpand = !$scope.DefineLocations[location_idx].isExpand;
    };

    $scope.getDaysOldString = function (keepDays) {
        if (angular.isDefined(keepDays) && Number.isInteger(keepDays)) {
            if (keepDays <= 0) {
                return $scope.translation.WEBMOB_fhub2_food_reminder_entered_today;
            }
            else if (keepDays === 1) {
                return $scope.translation.WEBMOB_fhub2_food_reminder_day_old;
            }
            else {
                return $scope.translation.WEBMOB_fhub2_food_reminder_days_old.replace("#num#", (keepDays).toString());
            }
        }
    };

    function getFoodListFormat(item) {
        if (angular.isUndefined(item.id)) {
            console.log('getShoppingListItemPayload::: Undefined id');
            return;
        }

        var glaze = {
            shelfID: 0,
            tagPositionX: 2000,
            tagPositionY: 2000,
            depth: timeStamp.getDateFormat(new Date(), 'yyyy-MM-ddTHH:mm:ss', 'UTC')
        };
        if (angular.isDefined(item.Glaze)) {
            glaze.shelfID = (angular.isDefined(item.Glaze.shelfID))? item.Glaze.shelfID : 0;
            glaze.tagPositionX = (angular.isDefined(item.Glaze.tagPositionX))? item.Glaze.tagPositionX : 2000;
            glaze.tagPositionY = (angular.isDefined(item.Glaze.tagPositionY))? item.Glaze.tagPositionY : 2000;
            glaze.depth = (angular.isDefined(item.Glaze.depth))? item.Glaze.depth : timeStamp.getDateFormat(new Date(), 'yyyy-MM-ddTHH:mm:ss', 'UTC');
        }

        var recommended = {
            location: false,
            expiration: false
        };
        if (angular.isDefined(item.recommended)) {
            recommended.location = (angular.isDefined(item.recommended.location))? item.recommended.location : false;
            recommended.expiration = (angular.isDefined(item.recommended.expiration))? item.recommended.expiration : false;
        }

        if (angular.isDefined(item.inputDate) && angular.isDefined(item.storageType) && item.storageType === "STORAGE_TAG")
            item.inputDate = timeStamp.getDateFormat(item.inputDate, 'yyyy-MM-ddTHH:mm:ss', 'UTC');
        else
            item.inputDate = timeStamp.getDateFormat(new Date(), 'yyyy-MM-ddTHH:mm:ss', 'UTC');

        return {
            userId: (angular.isDefined($scope.userId))? $scope.userId : "",
            id: item.id,
            category: encodeEscape(item.category),
            Glaze: glaze,
            is_expiry_tag: (angular.isDefined(item.is_expiry_tag))? item.is_expiry_tag : false,
            category_info: (angular.isDefined(item.category_info))? item.category_info : default_category_id,
            inputDate: item.inputDate,
            entryDate: (angular.isDefined(item.entryDate))? timeStamp.getDateFormat(item.entryDate, 'yyyy-MM-ddTHH:mm:ss', 'UTC') : "",
            expirationDate: (angular.isDefined(item.expirationDate))? timeStamp.getDateFormat(item.expirationDate, 'yyyy-MM-ddTHH:mm:ss', 'UTC') : "",
            name: encodeEscape(item.name),
            storageType: (angular.isDefined(item.storageType))? item.storageType : "STORAGE_FRIDGE",
            isReminder: (angular.isDefined(item.isReminder))? item.isReminder : false,
            imageURL: (angular.isDefined(item.imageURL))? item.imageURL : "",
            memoId: (angular.isDefined(item.memoId))? item.memoId : "",
            data_type: (angular.isDefined(item.data_type))? item.data_type : "",
            emart_item_id: (angular.isDefined(item.emart_item_id))? item.emart_item_id : "",
            emart_min_qty: (angular.isDefined(item.emart_min_qty))? item.emart_min_qty : "",
            site_no: (angular.isDefined(item.site_no))? item.site_no : "",
            recommended : recommended,
            quantity: (angular.isDefined(item.quantity))? item.quantity : 1,
            quantityUnit: (angular.isDefined(item.quantityUnit))? item.quantityUnit : "",
            isNewlyAdded: false
        };
    }

    /**
     * @function
     * @private
     * @name sendSaveFoodCommand
     * @param
     * @description
     **/
    function sendSaveFoodCommand() {
        if (isMaxItem()) {
            SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_food_reminder_add_max_items.replace("#num#", $scope.MAX_FOOD_ITEM.toString()), REQUEST.SHOW_TOAST);
            return;
        }
        var payload;

        checkElement($('#food_name')[0],$scope.MAX_CHARACTER_ITEM);
        $scope.inputData.name = ($scope.inputData.name.trim() === "") ? generateDefaultFoodTitle() : checkMaxFoodN($scope.inputData.name);
        // $scope.showConsoleLogs &&  console.log("scope.sendSaveFoodCommand: " + $scope.isStatic + " / " + $scope.AddFoodImage);

        var image = "";
        if (angular.isDefined($scope.inputData.imageURL)) {
            if ($scope.inputData.imageURL === "newImage")
                image = getImage($scope.inputData.imageURL).replace(BASE64, "");
            else
                image = $scope.inputData.imageURL;
        }

        var glaze = {
            shelfID: 0,        //fixed only 0
            tagPositionX: 2000,
            tagPositionY: 2000,
            depth: timeStamp.getDateFormat(new Date(), 'yyyy-MM-ddTHH:mm:ss', 'UTC')
        };

        if ($scope.inputData.is_expiry_tag) {   //expiry Tag
            console.log("before $scope.memoCoordinates ", $scope.memoCoordinates);
            console.log("sendSaveFoodCommand CWIDTH >>>> ", cWidth, " CHEIGHT >>>> ", cHeight);
            var tempPosX = $scope.memoCoordinates[0] <=60 ? 40 : 30;
            var tempPosY = $scope.memoCoordinates[1] <=60 ? 40 : 30;
            $scope.memoCoordinates[0] = $scope.memoCoordinates[0] > tempPosX ? $scope.memoCoordinates[0] - tempPosX : $scope.memoCoordinates[0];
            $scope.memoCoordinates[1] = $scope.memoCoordinates[1] > tempPosY ? $scope.memoCoordinates[1] - tempPosY : $scope.memoCoordinates[1];
            console.log("after $scope.memoCoordinates ", $scope.memoCoordinates);
            var Mobile_Resolution = getTagResolution_MOBILE('expiryTag');
            var LCD_Resolution = getTagResolution_LCD('expiryTag');
            glaze.tagPositionX = ((parseInt($scope.memoCoordinates[0], 10) / Mobile_Resolution.width * scaleFactor) * LCD_Resolution.width);
            glaze.tagPositionY = ((parseInt($scope.memoCoordinates[1], 10) / Mobile_Resolution.height * scaleFactor) * LCD_Resolution.height);
            console.log("sendSaveFoodCommand tagPositionX >>>> ", glaze.tagPositionX, " tagPositionY >>>> ", glaze.tagPositionY);
        }

        var item_info = {
            id: timeStamp.getDateFormat(new Date(), 'yyyyMMddHHmmss sss', 'UTC').replace(" ", ""),
            Glaze: glaze,
            is_expiry_tag: $scope.inputData.is_expiry_tag,
            category_info: $scope.inputData.category_info,
            entryDate: $scope.inputData.entryDate,
            expirationDate: $scope.inputData.expirationDate,
            name: $scope.inputData.name,
            storageType: $scope.inputData.storageType,
            isReminder: $scope.inputData.isReminder,
            imageURL: image,
            recommended: {
                location: $scope.inputData.recommended.location,
                expiration: $scope.inputData.recommended.expiration
            },
            data_type: $scope.inputData.data_type
        };

        payload = {
            "FoodList": [
                getFoodListFormat(item_info)
            ]
        };

        $scope.checkResponse.push(REQUEST.ADD_FOOD_LIST);
        SHPService.sendNativeCommand(REQUEST.ADD_FOOD_LIST, payload, REQUEST.ADD_FOOD_LIST);

        //if user add expiry tag, show Tags in Inside Fridge
        if ($scope.addFoodItem && $scope.inputData.is_expiry_tag)
            $scope.isShowTag = true;
        //close popup;
        cancelFood();
    }

    function closeConfirmationPopUp() {
        $scope.confirmationPopUpVisible = false;
        deleteItemId = null;
        deleteMemoId = null;
    }
    function deleteConfirmationCheck() {
        if (deleteOption === $scope.FOODREMINDER_OPTIONS.DELETE) {
            onOkConfirmationPopUp($scope.FOODREMINDER_OPTIONS.DELETE);
        }
        else if (deleteOption === 'DELETE_MEMO') {
            onOkConfirmationPopUp('DELETE_MEMO');
        }
        else if (deleteOption === 'DELETE_FOOD') {
            onOkConfirmationPopUp('DELETE_FOOD');
        }
    }
    function showConfirmationPopUp(option, itemId , memoId) {
        switch (option) {
            case $scope.FOODREMINDER_OPTIONS.DELETE://Delete foodItem
                deleteOption = option;
                $scope.confirmationPopUpTitle = "";
                if ($scope.selectedItems.length === 1)
                    // $scope.confirmationPopupText = $scope.translation.WEBMOB_fhub2_common_delete_item_popup_msg;
                    $scope.confirmationPopupText = $scope.translation.WEBMOB_fhub2_common_delete_selected_item;
                else
                    // $scope.confirmationPopupText = $scope.translation.WEBMOB_fhub2_common_delete_items_popup_msg.replace('#num#', $scope.selectedItems.length.toString());
                    $scope.confirmationPopupText = $scope.translation.WEBMOB_fhub2_common_delete_selected_items;
                break;
            //FALLTHROUGH
            case 'DELETE_MEMO': //Delete Memo
            case "DELETE_FOOD": //Delete Expiry Tag
                $scope.dragCheckProcessed = false;

                $timeout(function(){//REMOVE PRESS EFFECT FROM MEMO DIV
                    $("#tag_element_"+itemId).removeClass('tag_press');
                }, 100);
                if ($(event.target).attr('class') === "tag_memo_close_div") {//Delete Memo
                     deleteItemId = itemId; 
                     deleteMemoId = memoId;
                    deleteOption = option;
                    $scope.confirmationPopUpTitle = "";
                    $scope.confirmationPopupText = $scope.translation.WEBMOB_fhub2_food_reminder_delete_memo_popup;
                }
                else if ($(event.target).hasClass('tag_food_close_div')) {//Delete Expiry Tag
                    deleteItemId = itemId;
                    deleteOption = option;
                    $scope.confirmationPopUpTitle = "";
                    $scope.confirmationPopupText = $scope.translation.WEBMOB_fhub2_food_reminder_delete_expiry_tag_popup;
                }
                else {
                    return;
                }
                break;
            default:
                //do nothing
                break;
        }
        $scope.confirmationPopUpVisible = true;
    }

    function onCancelConfirmationPopUp() {
        if ($scope.inputData.id) {//check if delete tag id is set
            var tagElement = $("#delete_tag_div").find("#tag_element_" + $scope.inputData.id);
            if (tagElement.length) {
                var tagObject = $scope.Device.FoodList.Tag_List.filter(function (el) {
                    return (el.id === $scope.inputData.id);
                });
                if (tagObject.length > 0) {
                    tagElement.css({
                        "left": (parseInt(tagObject[0].Glaze.tagPositionX, 10) / 1080) * cWidth,
                        "top": (parseInt(tagObject[0].Glaze.tagPositionY, 10) / 503) * cHeight
                    });
                    var compiledTag = $compile(tagElement)($scope);
                    $("#REF_GLAZE_" + tagObject[0].Glaze.shelfID).append(compiledTag);
                    $("#delete_tag_div").find("#tag_element_" + $scope.inputData.id).remove();
                }
            }
        }
        closeConfirmationPopUp();
    }

    function onOkConfirmationPopUp(option) {
        switch (option) {
            case $scope.FOODREMINDER_OPTIONS.DELETE: //No popup in case food item is deleted, this case will not be executed
                deleteItem();
                break;
            case 2:
                deleteTag();
                break;
            case 'DELETE_MEMO': //Delete Memo:
                deleteMemo();
                break;
            case 'DELETE_FOOD': //Delete Expiry Tag
                deleteExpiryTag();
                break;
            default:
                //do nothing
                break;
        }
        closeConfirmationPopUp();
    }

    /**
     * @function
     * @private
     * @name deleteItem
     * @param
     * @description
     **/
    function deleteItem() {
        $scope.homePage = true;
        $scope.selectedMode_option = false;
        $scope.isSelectedMode = false;
        $scope.currentSelectedMode = "";
        $scope.addFoodItem = false;
        $scope.sendToShoppingListPopUp = false;
        $scope.moveLocationPopup = false;
        $scope.editFoodItem = false;
        $scope.daySelector = false;
        $scope.isCalender = false;
        var deleteIds = "";
        for (var i = 0; i < $scope.selectedItems.length; i++)
            deleteIds += $scope.selectedItems[i].id + ",";

        var payload = "/foodmanager/foodlist/?id=" + deleteIds.slice(0, -1);

        $scope.checkResponse.push(REQUEST.DELETE_FOOD_LIST);
        SHPService.sendNativeCommand(REQUEST.DELETE_FOOD_LIST, payload, REQUEST.DELETE_FOOD_LIST);
        closeSelectedMode();
    };

    /**
     * @function
     * @private
     * @name deleteFoodItem
     * @param
     * @description
     **/
    function deleteFoodItem(deleteID) {
        var delFood = {
            "FoodList": {
                "id": "",
                "category": "",
                "expirationDate": "",
                "imageIndex": "",
                "inputDate": "",
                "entryDate": "",
                "name": "",
                "quantityUnit": "",
                "storageType": "",
                "isNewlyAdded": true,
                "quantity": 0
            }
        };
        for (var fl = 0; fl < $scope.Device.FoodList.Fridge.length; fl++) {
            if ($scope.Device.FoodList.Fridge[fl].id === deleteID) {
                //$scope.Device.FoodList.Fridge.splice(fl,1);
                delFood.FoodList.id = $scope.Device.FoodList.Fridge[fl].id;
                delFood.FoodList.category = $scope.Device.FoodList.Fridge[fl].category;
                delFood.FoodList.expirationDate = $scope.Device.FoodList.Fridge[fl].expirationDate;
                delFood.FoodList.imageIndex = $scope.Device.FoodList.Fridge[fl].imageIndex;
                delFood.FoodList.inputDate = $scope.Device.FoodList.Fridge[fl].inputDate;
                delFood.FoodList.entryDate = $scope.Device.FoodList.Fridge[fl].entryDate;
                delFood.FoodList.name = $scope.Device.FoodList.Fridge[fl].name;
                delFood.FoodList.quantityUnit = $scope.Device.FoodList.Fridge[fl].quantityUnit;
                delFood.FoodList.storageType = $scope.Device.FoodList.Fridge[fl].storageType;
                delFood.FoodList.isNewlyAdded = $scope.Device.FoodList.Fridge[fl].isNewlyAdded;
                delFood.FoodList.quantity = $scope.Device.FoodList.Fridge[fl].quantity;
                delFood.FoodList.isReminder = $scope.Device.FoodList.Fridge[fl].isReminder;
                delFood.FoodList.imageURL = $scope.Device.FoodList.Fridge[fl].imageURL;
                break;
            }
        }
        deleteFoodData(delFood);
    }

    function showEditFoodItemPage(key) {
        debugMessage("showEditFoodItemPage::: ng-clicked");
        if ($scope.isSelectedMode) {
            // toggleCheckList(key);
            return;
        }

        $scope.selectedFoodData = undefined;
        $scope.initial_inputData();
        $scope.inputData.id = key.id;
        $scope.inputData.imageURL = key.imageURL;
        $scope.inputData.isAllowRecommend.location = false;     //do not recommend location automatically
        // $scope.AddFoodImage = "";
        $scope.daySelector = false;
        $scope.isCalender = false;
        $scope.addFoodItem = false;
        $scope.editFoodItem = true;
        goToFoodItem();

        for (var fl = 0; fl < $scope.Device.FoodList.Fridge.length; fl++) {
            if ($scope.Device.FoodList.Fridge[fl].id === key.id) {
                var item = angular.copy($scope.Device.FoodList.Fridge[fl]);
                console.log('editFoodItem::: ', item);
                $scope.inputData.name = item.name.trim();
                console.log('idx = ', fl, ' / name = ', $scope.inputData.name);
                $scope.inputData.entryDate = item.entryDate;
                if (item.expirationDate !== "")
                    $scope.inputData.expirationDate = item.expirationDate;
                else
                    $scope.inputData.expirationDate = getDefaultExpirationDate();
                $scope.inputData.calculateDays = diffDays(new Date(), $scope.inputData.expirationDate);

                // $scope.AddFoodImage = getImage(item.imageURL);

                $scope.inputData.isReminder = item.isReminder;
                $scope.inputData.storageType = item.storageType;
                $scope.inputData.storageType_idx = getLocationIndex(item.storageType);
                $scope.inputData.is_expiry_tag = item.is_expiry_tag;
                $scope.inputData.category_info = item.category_info;
                $scope.inputData.recommended = item.recommended;
                // if (item.recommended.location || item.recommended.expiration) {
                    requestQuery($scope.inputData.name, REQUEST.SEARCH_FOR_FOOD + '_ONE');
                // }
                $scope.inputData.data_type = item.data_type;
                $scope.inputData.emart_item_id = item.emart_item_id;
                $scope.inputData.emart_min_qty = item.emart_min_qty;
                $scope.inputData.site_no = item.site_no;
                break;
            }
        }
    }

    $scope.find_food_item = find_food_item;
    function find_food_item(array, str) {
        if (str !== "" && angular.isDefined(array) && array.length > 0) {
            var index = array.map(function (food_item) {
                return angular.isDefined(food_item.displayName) ? food_item.displayName.toLocaleLowerCase() : "";
            }).indexOf(str.toLocaleLowerCase());
            if (index !== -1) {
                $scope.selectedFoodData = array[index];
            }
            else
                $scope.selectedFoodData = undefined;
        }
    }

    /** Starts Check For the modified input fields **/
    $scope.modifyList = [];
    $scope.isAdded = isAdded;
    $scope.getModifyItem = getModifyItem;
    function clearModifyList() {
        $scope.modifyList = [];
    }

    function isAdded(item) {
        for (var i=0; i<$scope.modifyList.length; i++) {
            if ($scope.modifyList[i].id === item.id) {
                return true;
            }
        }
        return false;
    }

    function addModifyItem(item) {
        // console.log("addModifyItem");
        if (!isAdded(item))
            $scope.modifyList.push(item);
    }

    function removeModifyItem(item) {
        $scope.modifyList.forEach(function (modifyItem, index) {
            if (modifyItem.id === item.id)
                $scope.modifyList.splice(index, 1);
        });
    }

    function getModifyItem(id) {
        for (var i=0; i<$scope.modifyList.length; i++) {
            if ($scope.modifyList[i].id === id)
                return $scope.modifyList[i];
        }
        return undefined;
    }

    /** Ends Check For the modified input fields **/

    $scope.checkElement = checkElement;
    function checkElement(element, maxCharacter) {
        if (angular.isDefined(element)) {
            var previewItem = getModifyItem(element.id);
            if (angular.isDefined(previewItem) && element.value.length > maxCharacter) {
                $scope.inputData.name = element.value = previewItem.prevText;
                SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_common_max_characters, "");
            }
        }
    }

    $scope.checkMaxChar = checkMaxChar;
    function checkMaxChar(event, max_character, type) {
        console.log("checkMaxChar:::", event, max_character, type);

        if (angular.isUndefined(event) || (angular.isDefined(event) && event.keyCode !== ENTER_KEYCODE && event.keyCode !== 8)) {
            removeFocus();
            SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_common_max_characters, "");
        }
    }

    $scope.clearText = clearText;
    function clearText(element_id) {
        if ($scope.createNewListPopUp) {
            $scope.inputData.createCategory = "";
        }
        else if ($scope.currentScreen === VIEW_INSIDE_SCREEN.FOOD_ITEM_SCREEN || $scope.currentScreen === VIEW_INSIDE_SCREEN.SEND_TO_SHOPPING_LIST_SCREEN) {
            $scope.inputData.name = "";
        }

        $timeout(function () {
            if (angular.isDefined(element_id) && element_id !== null) {
                var element = $('#' + element_id);
                if (angular.isDefined(element) && element !== null) {
                    element.focus();
                }
            }
        }, 0);
    }

    $scope.focusHandler = focusHandler;
    function focusHandler(event) {
        debugMessage("****focusHandler****");
        if ($scope.loadingBar) {
            console.log("focusHandler::: Remove Focused  id = " + event.target.id );
            removeFocus();
            return;
        }

        console.log("event.target.id = " + event.target.id);
        if (angular.isDefined(event.target.id) && (event.target.id.indexOf('food_name') !== -1 || event.target.id.indexOf('send_item_name') !== -1)) {
            $(".md-scroll-mask").css({'display': 'block'});
            $("md-virtual-repeat-container").css({'display': 'block'});
        }

        if (angular.isDefined(event.target.id) && event.target.id !== '' && !getModifyItem(event.target.id)) {
            var item = {
                id: event.target.id,
                value: angular.isDefined(event.target.value)? event.target.value : "",
                prevText: angular.isDefined(event.target.value)? event.target.value : ""
            };

            addModifyItem(item);
            if (event.target.id === 'newlistTitleID_createList') {
                $timeout(function () {
                    console.log("~~~~~~~~~~~focusHandler($timeout)::: View Size = " + document.body.clientHeight);
                    $("body").height(document.body.clientHeight);
                }, 1000);
            }
        }
        console.log($scope.modifyList);
        console.log("***************************");
    }

    $scope.blurHandler = blurHandler;
    function blurHandler(event, max_character) {
        debugMessage("####blurHandler::: event.target.id = " + event.target.id + " / data = " + event.target.value + " / keyCode = " + event.keyCode);
        var previewItem = getModifyItem(event.target.id);

        if ($scope.loadingBar) {
            if (angular.isDefined(previewItem))
                removeModifyItem(previewItem);
            return;
        }

        if (angular.isDefined(event.target.id) && (event.target.id.indexOf('food_name') !== -1 || event.target.id.indexOf('send_item_name') !== -1)) {
            $(".md-scroll-mask").css({'display': 'none'});
            $("md-virtual-repeat-container").css({'display': 'none'});
        }

        if (angular.isDefined(event.target.value) && angular.isDefined(previewItem) && previewItem.id === event.target.id) {
            //Check Max Character
            if (previewItem.value !== event.target.value) {
                if (angular.isDefined(max_character) && event.target.value.length > max_character && (event.keyCode !== ENTER_KEYCODE && event.keyCode !== 8)) {
                    event.preventDefault();
                    $timeout(function () {
                        removeFocus();
                    }, 0);
                    console.log("blurHandler:::Check Max Character - prevTest = ", previewItem.prevText);
                    event.target.value = previewItem.prevText;
                    if (angular.isDefined(event.target.id)) {
                        if (event.target.id === 'newlistTitleID_createList') {
                            $scope.inputData.createCategory = event.target.value;
                        }
                        else if (event.target.id === 'food_name') {
                            $scope.inputData.name = event.target.value;
                        }
                        else if (event.target.id === 'send_item_name') {
                            $scope.inputData.name = event.target.value;
                        }
                    }
                    SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_common_max_characters, "");
                }
            }

            if (event.target.id === 'newlistTitleID_createList') {
                $timeout(function () {
                    console.log("~~~~~~~~~~~blurHandler($timeout)::: View Size = " + document.body.clientHeight);
                    $("body").height(document.body.clientHeight);
                }, 1000);
            }
            else if (event.target.id === 'food_name' || event.target.id === 'send_item_name') {
                if ($scope.editFoodItem && previewItem.value !== event.target.value) {
                    $scope.inputData.isAllowRecommend = {
                        location: false,
                        expiration: false
                    }
                }
                find_food_item($scope.queryItems, event.target.value);
                var foodData = updateFoodLifeByFoodDB($scope.inputData.entryDate);
                updateRecommendedInfo(foodData);
            }
            removeModifyItem(previewItem);
        }
        // console.log($scope.modifyList);
        console.log("#############################");
    }

    $scope.changeHandler = changeHandler;
    function changeHandler(element_id ,ngModel, max_character) {
        console.log("changeHandler:::", element_id, ngModel, max_character);

        if (angular.isUndefined(ngModel) || (angular.isDefined(ngModel) && ngModel.length > max_character)) {
            removeFocus();
        }

        var previewItem = getModifyItem(element_id);
        if (angular.isDefined(previewItem) && ngModel.length <= max_character) {
            previewItem.prevText = ngModel;
        }
    }

    /**
     * @function
     * @private
     * @name keyupHandler
     * @param
     * @description
     **/
    function keyupHandler(event, max_character, type) {
        console.log("keyupHandler ("+ type +"):::", event.target.value, event.target.value.length, ' /keyCode: ' + event.keyCode);
        if (angular.isUndefined(event))
            return;

        if (event.target.id === 'send_item_name')
            $scope.inputData.name = event.target.value;
        else if (event.target.id === 'food_name')
            $scope.inputData.name = event.target.value;
        else if (event.target.id === 'newlistTitleID_createList')
            $scope.inputData.createCategory = event.target.value;

        if (event.target.value.length > max_character && (event.keyCode !== ENTER_KEYCODE && event.keyCode !== 8)) {
            removeFocus();
        }

        var previewItem = getModifyItem(event.target.id);
        if (angular.isDefined(previewItem) && event.target.value.length <= max_character && (event.keyCode !== ENTER_KEYCODE && event.keyCode !== 8)) {
            previewItem.prevText = event.target.value;
        }

        if (event.keyCode === ENTER_KEYCODE) {
            removeFocus();
        }

    }

    /**
     * @function
     * @private
     * @name showDayPopup
     * @param
     * @description
     **/
    function showDayPopup() {
        $scope.daySelector = !$scope.daySelector;
        $scope.isCalender = false;
    }

    /**
     * @function
     * @private
     * @name FUNCTION NAME
     * @description FUNCTION DESCRIPTION TO BE ADDED HERE.
     */
    // $scope.$watch(function () {
    //     return $scope.datepick;
    // }, function () {
    //     if (angular.isDefined($scope.datepick)) {
    //         var one_day = 1000 * 60 * 60 * 24;
    //         // Convert both dates to milliseconds
    //         //var date1_ms = $scope.datepick.getTime();
    //         //var date2_ms = new Date().getTime();
    //         var today = new Date();
    //         var date1_ms = Date.UTC($scope.datepick.getFullYear(), $scope.datepick.getMonth(), $scope.datepick.getDate());
    //         var date2_ms = Date.UTC(today.getFullYear(), today.getMonth(), today.getDate());
    //         // Calculate the difference in milliseconds
    //         var difference_ms = date1_ms - date2_ms;
    //         // Convert back to days and check if selcted date is in future enable setDate button
    //         //also selected date year should not be more than 21 years from current year
    //         if (Math.round(difference_ms / one_day) > 0 && (Number($scope.datepick.getFullYear()) - Number(today.getFullYear()) <= MAX_EXPIRY_YEAR)) {
    //             $scope.set_date_container = "ripple-container";
    //         } else {
    //             $scope.set_date_container = "set_date_disabled no_click";
    //         }
    //     }
    // });

    function cancelDate() {
        debugMessage("cancelDate");
        removeFocus();

        $scope.isCalender = false;
        $scope.entryDatePopup = false;
        $scope.expirationPopup = false;

        if ($scope.draggedTag) {//remove +day tag in case calendar is opened after dragging +dayTag and user cancels that
            $scope.draggedTag.remove();
            $scope.draggedTag = "";
        }
        //reinitialize calendar
        // $scope.reinit();
    }

    function cancelFood() {
        if ($scope.isCalender || $scope.daySelector) {
            closeSubPopUp();
        }
        $scope.addFoodItem = false;
        $scope.editFoodItem = false;
        closeCropScreen();
        removeFocus();
        $scope.currentScreen = VIEW_INSIDE_SCREEN.HOME_SCREEN;
        if ($scope.currentView === VIEW_INSIDE_VIEW.INSIDE_IMAGE) {
            debugMessage('called onbackpressed');
            $scope.goToInsideImage();
        }
        else if ($scope.currentView === VIEW_INSIDE_VIEW.FOOD_LIST)
            $scope.goToFoodList();
    }

    /**
     * @function
     * @private
     * @name toEditAnItem
     * @param {Boolean} tag [true in case called from editTag ]
     * @description
     **/
    function toEditAnItem() {
        var item = undefined;
        var index = $scope.Device.FoodList.Fridge.map(function(item) {return item.id; }).indexOf($scope.inputData.id);
        if (index !== -1) {
            item = $scope.Device.FoodList.Fridge[index];
        }

        if (angular.isUndefined(item))
            return;

        //if foodname is kept blank in that case use name with default naming convention
        item.name = $scope.inputData.name = ($scope.inputData.name.trim() === "") ? generateDefaultFoodTitle() : checkMaxFoodN($scope.inputData.name);

        if (angular.isDefined($scope.inputData.imageURL)) {
            if ($scope.inputData.imageURL === "newImage")
                item.imageURL = getImage($scope.inputData.imageURL).replace(BASE64, "");
            else
                item.imageURL = $scope.inputData.imageURL;
        }

        item.id = $scope.inputData.id;
        item.is_expiry_tag = $scope.inputData.is_expiry_tag;
        item.category_info = $scope.inputData.category_info;
        item.entryDate = $scope.inputData.entryDate;
        item.expirationDate = $scope.inputData.expirationDate;
        item.storageType = $scope.inputData.storageType;
        item.isReminder = $scope.inputData.isReminder;
        if (angular.isDefined(item.recommended)) {
            item.recommended.location = $scope.inputData.recommended.location;
            item.recommended.expiration = $scope.inputData.recommended.expiration;
        }


        var payload = {
            "FoodList": [
                getFoodListFormat(item)
            ]
        };

        console.log("editItem >>>>>>>", payload);
        return payload;
    }

    function moveLocationCommand(location) {
        debugMessage("moveLocationCommand - inputData.storageType: " + $scope.inputData.storageType);
        var payload = {"FoodList": []};
        $scope.selectedItems.forEach(function (item) {
            item.storageType = location.id;
            var newItem = getFoodListFormat(item);
            payload["FoodList"].push(newItem);
        });

        $scope.checkResponse.push(REQUEST.EDIT_FOOD_LIST);
        SHPService.sendNativeCommand(REQUEST.EDIT_FOOD_LIST, payload, "moveLocation");
        $scope.closeSelectedMode();
        SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_common_moved_to_string.replace("#string1#", location.name), "");
    }

    function deleteExpiryTag() {        //The deleted Expiry Tag means that ExpiryTag option is OFF.
        if (dragCheck === true || angular.isUndefined(deleteItemId)){
            return;
        }

        var item = undefined;
        for (var i = 0; i < $scope.Device.FoodList.Fridge.length; i++) {
            if (deleteItemId === $scope.Device.FoodList.Fridge[i].id)
                item = $scope.Device.FoodList.Fridge[i];
        }
        if (angular.isUndefined(item))
            return;

        debugMessage("deleteExpiryTag - item: " + item);
        item.is_expiry_tag = false;

        var payload = {"FoodList": [
                getFoodListFormat(item)
            ]};

        $scope.checkResponse.push(REQUEST.EDIT_FOOD_LIST);
        SHPService.sendNativeCommand(REQUEST.EDIT_FOOD_LIST, payload, REQUEST.EDIT_FOOD_LIST);
    }

    /**
     * @function
     * @private
     * @name sendEditFoodCommand
     * @param {Boolean} tag [true in case called from editTag ]
     * @description
     **/
    function sendEditFoodCommand() {
        checkElement($('#food_name')[0],$scope.MAX_CHARACTER_ITEM);
        var payload = toEditAnItem();

        if (angular.isDefined(payload) && angular.isDefined(payload.FoodList)) {
            console.log("sendEditFoodCommand::: payload", payload);
            // if ($scope.AddFoodImage !== $scope.showDetailImg) {
            //     console.log("sendEditFoodCommand::new image is updated");
            //     payload.FoodList[0].imageURL = $scope.AddFoodImage;
            // } else {
            //     console.log("sendEditFoodCommand::image is not changed");
            // }

            $scope.checkResponse.push(REQUEST.EDIT_FOOD_LIST);
            SHPService.sendNativeCommand(REQUEST.EDIT_FOOD_LIST, payload, REQUEST.EDIT_FOOD_LIST);
        }
         //not required in case of tag drag, so added condition   
        cancelFood();
    }

    $scope.mainLoadingScreen = true;
    /**
     * @function
     * @private
     * @name parseDeviceData
     * @param {Object} device
     *       getDevice response object
     * @description To parse getDevice response, it contains complete details of present device state
     */
    function parseDeviceData(device) {
        debugMessage('parseDeviceData');

        if (angular.isDefined(device['GlazeInfo']) && angular.isUndefined(device['event'])) {    //Ignore notification of GlazeInfo
            if (!angular.isDefined(device['GlazeInfo']['GlazeURL']) || !angular.isDefined(device['GlazeInfo']['doorOpenedTime'])) {
                console.log("GlazeInfo Error:::showConnectionFailurePopup()" + JSON.stringify(device['GlazeInfo']));
                showConnectionFailurePopup();
            }
            else {
                if (angular.isDefined(device['event']) && device['event'] === 'Notified') {
                    console.log('parseDeviceData:::GlazeInfo:::Notified');
                    if ($scope.checkGrazeImgCrop)
                        closeCropScreen();
                }
                parseGlazeInfo(device['GlazeInfo']);
            }
        }

        if (angular.isDefined(device["CategoryDB"])) {
            parseCategoryDB(device["CategoryDB"]);
        }

        if (angular.isDefined(device["FoodList"])) {
            if (device['event'] === 'Deleted') {
                deleteFoodData(device);
            } else if (device['event'] === 'Created' || device['event'] === 'Notified' || device['event'] === 'Canceled') {
                device["FoodList"].forEach(function (deviceFood) {
                    if (!updateFoodData(deviceFood)) {
                        debugMessage("need to add");
                        addFoodData(deviceFood);
                    }
                    else
                        debugMessage("Update done");
                });
            }

            if($scope.currentScreen === VIEW_INSIDE_SCREEN.HOME_SCREEN && $scope.currentView === VIEW_INSIDE_VIEW.FOOD_LIST && $scope.isSelectedMode) {
                if (isEmpty($scope.Device.FoodList.Fridge, $scope.order.filterBy)) {
                    closeSelectedMode();
                }
                else {
                    $scope.sendToShoppingListPopUp = false;
                    $scope.createNewListPopUp = false;
                    $scope.moveLocationPopup = false;
                    closeConfirmationPopUp();
                    clearSelectedItems();
                }
            }
        }

        if (angular.isDefined(device["ShoppingList"])) {
            if (angular.isUndefined($scope.Device.ShoppingList))
                return;
            // debugMessage("parseDeviceData:: device=" + JSON.stringify(device));
            if (device['event'] === 'Deleted') {
                debugMessage("parseDeviceData:: (device['event'] === 'Deleted')");
                device["ShoppingList"].forEach(function (deviceShoppingList) {
                    if (angular.isDefined(deviceShoppingList.groupId) && angular.isDefined(deviceShoppingList.id) && (deviceShoppingList.groupId === deviceShoppingList.id)) {
                        console.log("Parse Delete ShoppingList(Title)::: list.groupId = " + deviceShoppingList.groupId);
                        delete $scope.Device.ShoppingList[deviceShoppingList.groupId];
                    }
                    else if (angular.isDefined(deviceShoppingList.groupId) && angular.isDefined($scope.Device.ShoppingList[deviceShoppingList.groupId])){
                        console.log("Parse Delete ShoppingList(Item)::: list.id = " + deviceShoppingList.id);
                        for (var index = 0; index < $scope.Device.ShoppingList[deviceShoppingList.groupId].items.length; index++) {
                            if ($scope.Device.ShoppingList[deviceShoppingList.groupId].items[index].id === deviceShoppingList.id) {
                                $scope.Device.ShoppingList[deviceShoppingList.groupId].items.splice(index, 1);
                                break;
                            }
                        }

                        //if list is created temporarily, remove list.
                        if ($scope.Device.ShoppingList[deviceShoppingList.groupId].isTemp && $scope.Device.ShoppingList[deviceShoppingList.groupId].items.length === 0) {
                            delete $scope.Device.ShoppingList[deviceShoppingList.groupId];
                        }
                    }
                });
            }
            else if (device['event'] === 'Created' || device['event'] === 'Notified') {
                debugMessage("parseDeviceData:: (device['event'] === " + device['event']);
                parseShoppingData(device);
            }
        }

        if (angular.isDefined(device["Memo"])) {
            if (device['event'] === 'Deleted') {

            }
            else if (device['event'] === 'Created' || device['event'] === 'Notified' || device['event'] === 'Canceled') {
                parseMemoData(device);
            }
        }

        if (firstLaunch) {
            $scope.homePage = true;
            firstLaunch = false;
        }
        //$scope.loadingBar = false;
        //To remove splash screen this variable is defined in top of the controller
        //$scope.mainLoadingScreen = true;
    }

    if ($scope.isStatic === true /*&& !$scope.deviceUuid*/) {
        debugMessage("parseStaticData");
        $scope.parseStaticData();
    }

    /**
     * @function
     * @private
     * @name parseStaticData
     * @description To parse static data.
     * To run the application in static mode in browser, add ng-init="parseStaticData" to body tag in html
     */
    function parseStaticData() {
        $scope.init();
        $scope.isLocal = true;
        // staticData.parseStaticData_FHub2();
        staticData.parseStaticData_FHub3();
    }

    function home() {
        $scope.homePage = true;
        $scope.addFoodItem = false;
        $scope.editFoodItem = false;
        $scope.daySelector = false;
        $scope.isCalender = false;
    }

    /* Start md-select(List DropDown) For Control h/w back key */
    $scope.is_mdSelect = false;
    $scope.show_mdSelect = show_mdSelect;
    $scope.hide_mdSelect = hide_mdSelect;
    function show_mdSelect() {
        debugMessage('show_mdSelect:::');
        $scope.is_mdSelect = true;
    }
    function hide_mdSelect() {
        debugMessage('hide_mdSelect:::');
        if (angular.isDefined($("md-backdrop"))) {
            debugMessage( "md-backdrop Handler for .select() called." );
            $("md-backdrop").click();
        }
        $scope.is_mdSelect = false;
    }
    /* End md-select(List DropDown) For Control h/w back key */

    /**
     * @function
     * @private
     * @name onBackPressed
     * @description On back pressed handling of any screen
     */
    function onBackPressed(event) {
        debugMessage("onBackPressed");

        //md-select(List DropDown) For Control h/w back key
        if ($scope.is_mdSelect)
            return hide_mdSelect();

        removeFocus();
        if ($scope.moveLocationPopup) {
            closeMoveLocationPopup();
            return;
        }
        if ($scope.daySelector) {
            $scope.daySelector = false;
            return;
        } else if ($scope.isCalender) {
            cancelDate();
            return;
        } else if ($scope.createNewListPopUp) {
            closeCreateNewListPopUp();
            return;
        } else if ($scope.sendToShoppingListPopUp) {
            $scope.sendToShoppingListPopUp = false;
            return;
        } else if ($scope.changeImagePopup) {
            closeChangeImagePopup();
            return;
        } else if ($scope.viewPicturePopup) {
            closeViewPicturePopup();
            return;
        } else if ($scope.confirmationPopUpVisible) {
            onCancelConfirmationPopUp();
            return;
        } else if ($scope.bOptionPopUp) {
            $scope.bOptionPopUp = false;
            return;
        } else if ($scope.zoomGlazeImg) {
            $scope.zoomGlazeImg = false;
        }
        if($scope.isSelectedMode) {
            closeSelectedMode();
            return;
        }

        if ($scope.currentScreen === VIEW_INSIDE_SCREEN.HOME_SCREEN) {
            if ($scope.checkGrazeImgCrop) {
                closeCropScreen();
            } else {
                closeApp();
            }
        } else {
            if ($scope.currentScreen === VIEW_INSIDE_SCREEN.FOOD_ITEM_SCREEN ||
                $scope.currentScreen === VIEW_INSIDE_SCREEN.SEND_TO_SHOPPING_LIST_SCREEN) {
                $scope.addFoodItem = false;
                $scope.editFoodItem = false;
                removeFocus();
            }

            $scope.currentScreen = VIEW_INSIDE_SCREEN.HOME_SCREEN;
            if ($scope.currentView === VIEW_INSIDE_VIEW.INSIDE_IMAGE) {
                debugMessage('called onbackpressed');
                $scope.goToInsideImage();
            }
            else if ($scope.currentView === VIEW_INSIDE_VIEW.FOOD_LIST)
                $scope.goToFoodList();
        }
    }

    /**
     * @function
     * @private
     * @name requestFailHandler
     * @param {String} response
     *       JSON response received in case of request failed
     * @description Handling of request failed cases
     */
    function requestFailHandler(response) {
        if (response["status"] === 500 && (response["type"] === "requestResponse" || response["type"] === "requestFailed")) {
            $scope.checkResponse = [];
            $scope.loadingBar = false;
            SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_common_check_power_and_network, "");
        }
        if (response["status"] === 404 && response["type"] === "requestFailed") {
            console.log("Image Not Found - ", JSON.stringify(response) , response);

            if (response["requestToken"] === REQUEST.GET_IMAGE_LIST + '_glaze') {
                showConnectionFailurePopup();
            }
        }
    }

    $scope.initial_getFoodList = false;
    function requestResponseHandler(response) {
        if (response['requestToken'] === REQUEST.GET_PROFILE) {
            profileService.parseProfileData(response["data"]);
            dismissLoadingBar(REQUEST.GET_PROFILE);
            SHPService.loadingFinished();
        }

        if (response['requestToken'] === REQUEST.GET_GLAZE_INFO) {
            if (angular.isDefined(response["data"]['GlazeInfo'])) {
                $scope.parseDeviceData(response["data"]);
                if (!$scope.initial_getFoodList) {
                    getFoodList();
                    $scope.initial_getFoodList = true;
                }
                dismissLoadingBar(REQUEST.GET_GLAZE_INFO);
            }
            else {
                showConnectionFailurePopup();
            }
        }

        if (response['requestToken'].indexOf(REQUEST.GET_IMAGE_LIST + '_glaze_') !== -1) {
            if (angular.isDefined(response["data"]) && angular.isDefined(response["data"]['ImageList'])) {
                response["data"]['ImageList'].forEach(function (image) {
                    if(angular.isDefined(image['imageURL']) && angular.isDefined(image['imageData']) && image['imageData'] === "") {
                        console.log("Image Not Found - ", JSON.stringify(response) , response);
                        showConnectionFailurePopup();
                    }
                });
                var splitArr = response['requestToken'].split("_");
                var glaze_idx = splitArr[splitArr.length - 1];
                console.log("splitArr = ", splitArr, "glaze_idx = ", glaze_idx);
                imageRequest.parseNativeResponse(response["data"], $scope.insideFridgeData, glaze_idx);
            }
        }

        if (response['requestToken'] === REQUEST.GET_IMAGE_LIST) {
            if (angular.isDefined(response["data"])) {
                imageRequest.parseNativeResponse(response["data"]);

                // if (angular.isDefined($scope.inputData.imageURL) && $scope.inputData.imageURL !== "") {
                //     var index = response["data"]["ImageList"].map(function (image) {
                //         return image.imageURL;
                //     }).indexOf($scope.inputData.imageURL);
                //     if (index !== -1) {
                        // $scope.AddFoodImage = getImage($scope.inputData.imageURL);
                    // }
                // }

            }
        }

        if (response['requestToken'] === REQUEST.GOTO_CAMERA || response['requestToken'] === REQUEST.GOTO_GALLERY) {
            if (angular.isDefined(response["data"]) && angular.isDefined(response["data"]["imageData"]) && response["data"]["imageData"] !== "") {
                var newImage = {
                        imageURL: "newImage",
                        imageData: response["data"]["imageData"]
                };
                imageCache.remove(newImage.imageURL);
                imageRequest.parseNativeResponse({"ImageList": [newImage]});
                // $scope.inputData.imageURL = $scope.AddFoodImage = response["data"]["imageData"];
                $scope.inputData.imageURL = newImage.imageURL;
            }
        }

        if (response['requestToken'] === REQUEST.GET_CATEGORY_DB) {
            $scope.parseDeviceData(response["data"]);
            dismissLoadingBar(REQUEST.GET_CATEGORY_DB);
            SHPService.loadingFinished();
        }

        if (response['requestToken'].indexOf(REQUEST.SEARCH_FOR_FOOD + '_QUERY_') !== -1) {
            if (angular.isDefined($scope.asyncQuery) && $scope.asyncQuery !== null) {
                $scope.asyncQuery.notify(response);
            }
        }
        else if (response['requestToken'] === REQUEST.SEARCH_FOR_FOOD + '_ONE') {
            find_food_item(response["data"]["SearchList"], $scope.inputData.name);
        }

        if (response['requestToken'] === REQUEST.GET_SHOPPING_LIST) {
            $scope.parseShoppingData(response["data"]);
            dismissLoadingBar(REQUEST.GET_SHOPPING_LIST);
        }
        if (response['requestToken'].indexOf(REQUEST.ADD_SHOPPING_LIST) !== -1) {
            $scope.parseDeviceData(response['data']);
            dismissLoadingBar(REQUEST.ADD_SHOPPING_LIST);
            closeAllPopUp();
            backToHomePage();
            //response['requestToken']: addShoppinglist_Title, addShoppinglist
            if (response['requestToken'] === REQUEST.ADD_SHOPPING_LIST) {
                SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_food_reminder_send_to_shopping_list_toast, REQUEST.SHOW_TOAST);
            }
        }

        if (response['requestToken'] === REQUEST.GET_MEMO) {
            parseMemoData(response['data']);
        }
        if (response['requestToken'] === REQUEST.ADD_MEMO) {
            $scope.showConsoleLogs &&  console.log(">>>>>>>>>>>> ADD MEMO RESPONSE", response);
            if (angular.isDefined(response['data']['event']) && response['data']['event'] === 'Created') {
                $scope.showConsoleLogs &&  console.log(">>>>>>>>>>>> Added Memo(Event:Created)", response);
                $scope.isShowTag = true;        //if user add memo tag, show Tags in Inside Fridge
                dismissLoadingBar(REQUEST.ADD_MEMO);
                $scope.parseDeviceData(response['data']);
                closeAllPopUp();
                backToHomePage();
                closeCropScreen();
                SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_food_reminder_send_to_memo_toast, REQUEST.SHOW_TOAST);
            } else {
                $scope.showConsoleLogs &&  console.log(">>>>>>>>>>>> Canceled the added Memo(Event:Canceled)", response);
                //if addMemo is canceled, close imageCrop mode.
                closeCropScreen();
            }
        }

        if (response['requestToken'] === REQUEST.GET_FOOD_LIST) {
            initialLaunch = false;
            $scope.init();
            parseFoodData(response["data"]);
            dismissLoadingBar(REQUEST.GET_FOOD_LIST);
        }
        else if (response['requestToken'] === REQUEST.ADD_FOOD_LIST) {
            //$scope.loadingBar = false;
            $scope.parseDeviceData(response['data']);
            dismissLoadingBar(REQUEST.ADD_FOOD_LIST);
            if ($scope.currentView === VIEW_INSIDE_VIEW.INSIDE_IMAGE)
                SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_food_reminder_add_food_item_toast, REQUEST.SHOW_TOAST);
            if ($scope.currentView === VIEW_INSIDE_VIEW.FOOD_LIST)
                SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_common_added, REQUEST.SHOW_TOAST);
        }
        else if (response['requestToken'] === REQUEST.EDIT_FOOD_LIST) {
            $scope.parseDeviceData(response['data']);
            dismissLoadingBar(REQUEST.EDIT_FOOD_LIST);
        }
        else if (response['requestToken'] === 'moveLocation') {
            $scope.showConsoleLogs &&  console.log("requestToken = moveLocation");
            $scope.parseDeviceData(response['data']);
            dismissLoadingBar(REQUEST.EDIT_FOOD_LIST);
        }
        else if (response['requestToken'] === REQUEST.DELETE_FOOD_LIST) {
            $scope.showConsoleLogs &&  console.log("requestToken = Deleted");
            $scope.parseDeviceData(response['data']);
            dismissLoadingBar(REQUEST.DELETE_FOOD_LIST);
        }
        else if (response['requestToken'] === REQUEST.DELETE_MEMO) {
            $scope.showConsoleLogs &&  console.log("requestToken = Deleted");
            $scope.parseDeviceData(response['data']);
            dismissLoadingBar(REQUEST.DELETE_MEMO);
        }
    }

    function requestProgressHandler(response) {

    }

    /**
     * @function
     * @private
     * @name getFoodList
     * @param
     * @description
     **/
    function getFoodList() {
        debugMessage("scope.getFoodList()");
        $scope.checkResponse.push(REQUEST.GET_PROFILE);
        $scope.checkResponse.push(REQUEST.GET_FOOD_LIST);
        $scope.checkResponse.push(REQUEST.GET_SHOPPING_LIST);

        SHPService.sendNativeCommand(REQUEST.GET_PROFILE, "", REQUEST.GET_PROFILE, false);
        SHPService.sendNativeCommand(REQUEST.GET_FOOD_LIST, "", REQUEST.GET_FOOD_LIST);
        SHPService.sendNativeCommand(REQUEST.GET_SHOPPING_LIST, "", REQUEST.GET_SHOPPING_LIST);
    }

    /**
     * @function
     * @private
     * @name showConnectionFailurePopup
     * @description To display connection failure popup, it is called from app.js
     */
    function showConnectionFailurePopup() {
        console.log('showConnectionFailurePopup:::');
        dismissLoadingBar('FAIL');
        dismissLoadingBar_noTimeOut('FAIL');
        $scope.dialogPopup.show = true;
        toggleBottomPopUpClass(true);
        $scope.dialogPopup.dialogPopupTitle = $scope.translation.WEBMOB_fhub2_common_connection_failure;
        $scope.dialogPopup.dialogPopupText = $scope.translation.WEBMOB_fhub2_common_check_power_and_network;
        $scope.dialogPopup.button1Title = $scope.translation.WEBMOB_fhub2_common_ok_capital;
        $scope.dialogPopup.button1Handler = function () {
            $scope.dialogPopup.show = false;
            SHPService.sendNativeCommand(REQUEST.CLOSE_WEB_APP, "", "");
        };
    }

    /**
     * @function
     * @private
     * @name closeConnectionFailurePopup
     * @description Close connection failure popup, it is called from app.js
     */
    function closeConnectionFailurePopup() {
        $scope.dialogPopup.show = false;
        toggleBottomPopUpClass(false);
    }

    /**
     * @function
     * @private
     * @name lastUpdate
     * @param
     * @description
     **/
    function lastUpdate(start_date) {
        var today = new Date();
        if (start_date === undefined) {
            start_date = "undefined";
            return "";
        }
        // var oneDay = (1000 * 3600 * 24); //m/s*sec*min*hour
        // var ct = (today.getTime() - start_date.getTime()) / oneDay;

        if (angular.isUndefined($scope.dateTimeFormat) || $scope.dateTimeFormat === null || $scope.dateTimeFormat === "")
            $scope.dateTimeFormat = "MMM dd, yyyy, h:mm a";

        return timeStamp.displayDate(start_date, $scope.dateTimeFormat);
    }

    $scope.getFoodLifeStr = function () {
        var entryDateStr = timeStamp.displayDate($scope.inputData.entryDate, $scope.dateFormat);
        var expirationDateStr = timeStamp.displayDate($scope.inputData.expirationDate, $scope.dateFormat);
        var recommendStr = ($scope.inputData.recommended.expiration)? "*" : "";
        return $scope.translation.WEBMOB_fhub2_calendar_event_table_display_times
            .replace("#time1#", entryDateStr)
            .replace("#time2#", expirationDateStr) + recommendStr;
    };

    /**
     * @function
     * @private
     * @name diffDays
     * @param
     * @description
     **/
    function diffDays(inputDate, expireDte) {
        if (angular.isUndefined(inputDate) || inputDate === "" ||
            angular.isUndefined(expireDte) || expireDte === "")
            return -999;

        //initial time for compare date.
        var inputDay = angular.copy(timeStamp.convertDate(inputDate));
        var expireDay = angular.copy(timeStamp.convertDate(expireDte));

        inputDay.setHours(0);
        inputDay.setMinutes(0);
        inputDay.setSeconds(0);
        inputDay.setMilliseconds(0);
        expireDay.setHours(0);
        expireDay.setMinutes(0);
        expireDay.setSeconds(0);
        expireDay.setMilliseconds(0);

        var timeDiff = expireDay.getTime() - inputDay.getTime();
        return Math.ceil(timeDiff / (1000 * 3600 * 24));
    }

    $scope.N_days_left = N_days_left;
    function N_days_left(expirationDate) {
        return diffDays(new Date(), expirationDate);
    }

    $scope.isShowTag = true;
    $scope.toggleShowTag = function () {
        $scope.isShowTag = !$scope.isShowTag;
    };

    $scope.isShowTagElement = function () {
        return $scope.currentView === VIEW_INSIDE_VIEW.INSIDE_IMAGE && !$scope.checkGrazeImgCrop &&
            $scope.isShowTag && $scope.insideFridgeData.pastHistoryIdx === $scope.insideFridgeData.maxPastHistory;
    };

    $scope.refreshGlazeData = refreshGlazeData;
    function refreshGlazeData() {
        $scope.Device.GlazeInfo.GlazeURL.forEach(function (glaze_url) {
            imageCache.remove(glaze_url);
        });

        $scope.checkResponse.push(REQUEST.GET_GLAZE_INFO);
        SHPService.sendNativeCommand(REQUEST.GET_GLAZE_INFO, {"pullServer":true}, REQUEST.GET_GLAZE_INFO);
    }

    /** Start GlazeProgress functions **/
    $scope.$on("progress-update", function () {
        try {
            $scope.$digest();
        } catch (err) {
            $scope.showConsoleLogs &&  console.log('$on::progress-update::: errorMessage = ' + JSON.stringify(err));
        }
    });

    $scope.$watch('glazeProgress.value', function () {
        if ($scope.glazeProgress.value >= 100) {
            debugMessage('$scope.glazeProgress.hide');
            initial_glazeProgress(false);   //hide Glaze progress bar
            succeedAllGlazeImages();
        } else if ($scope.glazeProgress.value >= 90) {
            $timeout(function () {
                drawCropCanvas();
            }, 1000);
        }
    });

    $scope.initial_glazeProgress = initial_glazeProgress;
    function initial_glazeProgress(isShowProgress) {
        $scope.glazeProgress.show = (angular.isDefined(isShowProgress))? isShowProgress : false;
        $scope.glazeProgress.value = 0;
    }

    $scope.glazeProgressIncrease = glazeProgressIncrease;
    function glazeProgressIncrease(increase) {
        // if ($scope.glazeProgress.show) {
            $scope.glazeProgress.value += increase;    //increase Glaze progress bar
            $scope.showConsoleLogs &&  console.log('increase Glaze progress bar:::' + increase +" / " + $scope.glazeProgress.value);
            $scope.$broadcast("progress-update");
        // }
    }
    /** End GlazeProgress functions **/

    /** Start pastHistory slider functions **/
    $scope.getNumber = function () {
        return (angular.isDefined($scope.MAX_PASTHISTORY_COUNT) && $scope.MAX_PASTHISTORY_COUNT > 0)? new Array($scope.MAX_PASTHISTORY_COUNT) : 0;
    };

    $scope.isDefinedHistory = function (index) {
        var pastHistory_URL = $scope.Device.GlazeInfo.pastHistory_URL;
        if (angular.isDefined(pastHistory_URL) && angular.isDefined(index)) {
            if (index >= 0 && index < $scope.insideFridgeData.maxPastHistory) {
                return angular.isDefined(pastHistory_URL[index]);
            }
        }
        return false;
    };

    $scope.setPastHistoryIdx = setPastHistoryIdx;
    function setPastHistoryIdx(index) {
        if (angular.isDefined(index) && index >= 0 && index <= $scope.insideFridgeData.maxPastHistory) {
            $scope.insideFridgeData.pastHistoryIdx = index;
            updateFridgeInside();
        }
    }
    
    $scope.updateFridgeInside = updateFridgeInside;
    function updateFridgeInside(/*type*/) {
        // console.log("updateFridgeInside::: pastHistoryIdx = ", $scope.insideFridgeData.pastHistoryIdx, type, $scope.glazeProgress.show);
        if ($scope.glazeProgress.show)
            return;
        var historyIndex = parseInt($scope.insideFridgeData.pastHistoryIdx);
        var pastHistory_URL = $scope.Device.GlazeInfo.pastHistory_URL;
        var pastHistory_doorOpenedTime = $scope.Device.GlazeInfo.pastHistory_doorOpenedTime[historyIndex];
        if ($scope.insideFridgeData.maxPastHistory === 1 || historyIndex === $scope.insideFridgeData.maxPastHistory) {
            console.log("moveToMaxPastHistory", historyIndex);
            moveToMaxPastHistory();
        }
        else if (angular.isDefined(pastHistory_URL) && angular.isDefined(pastHistory_URL[historyIndex]) && angular.isDefined(pastHistory_doorOpenedTime)) {
            callGlazeImages(pastHistory_URL[historyIndex], pastHistory_doorOpenedTime);
        }
    }

    $scope.moveToMaxPastHistory = moveToMaxPastHistory;
    function moveToMaxPastHistory() {
        $scope.insideFridgeData.pastHistoryIdx = $scope.insideFridgeData.maxPastHistory;
        callGlazeImages($scope.Device.GlazeInfo.GlazeURL, $scope.Device.GlazeInfo.doorOpenedTime);
    }

    $scope.callGlazeImages = callGlazeImages;
    function callGlazeImages(glazeURL_array, doorOpenedTime) {
        console.log("***** callGlazeImages ********",glazeURL_array, doorOpenedTime);
        console.log("currentScreen page " + $scope.currentScreen);
        if ($scope.insideFridgeData.GlazeURL === glazeURL_array)
            return;

        initial_glazeProgress(true);   //show Glaze progress bar
        setLoadingBar_noTimeOut("REF_GLAZE");
        $scope.insideFridgeData.doorOpenTime = timeStamp.convertDate(doorOpenedTime);
        $scope.insideFridgeData.GlazeURL = glazeURL_array;
        for (var glazeIdx = 0; glazeIdx < glazeURL_array.length; glazeIdx++) {
            console.log("[Time]startRenderGlazeImg::: GlazeInfo_" + glazeIdx , new Date());
            glazeProgressIncrease(Math.ceil(30/glazeURL_array.length));
            glazeImageURL_Request(glazeURL_array[glazeIdx], glazeIdx);
        }

        if ($scope.isStatic || $scope.isLocal)
            renderGlazeImage();
    }

    $scope.slider_fg_width = function () {
        return {
            width: 'calc((100%/' + $scope.insideFridgeData.maxPastHistory + ') * ' + $scope.insideFridgeData.pastHistoryIdx + ')'
        };
    };

    // $scope.bindSliderEvent = function () {
    //     console.log("bind #past_history");
    //     $("#past_history").bind({
    //         click: function (event) {
    //             console.log("clicked");
    //         },
    //         touchstart: function (event) {
    //             console.log("touchstart");
    //             $scope.insideFridgeData.touched = true;
    //         },
    //         touchmove: function (event) {
    //             console.log("touchmove");
    //         },
    //         touchend: function (event) {
    //             console.log("touchend", $scope.insideFridgeData.touched);
    //             if ($scope.insideFridgeData.touched)
    //                 updateFridgeInside('touchend');
    //             $timeout(function () {
    //                 $scope.insideFridgeData.touched = false;
    //             }, 0);
    //         },
    //         mousedown: function (event) {
    //             console.log("mousedown");
    //             $scope.insideFridgeData.touched = true;
    //         },
    //         mouseup: function (event) {
    //             console.log("mouseup", $scope.insideFridgeData.touched);
    //             if ($scope.insideFridgeData.touched)
    //                 updateFridgeInside('mouseup');
    //             $timeout(function () {
    //                 $scope.insideFridgeData.touched = false;
    //             }, 0);
    //         }
    //     });
    // };
    /** End pastHistory slider functions **/

    $scope.succeedAllGlazeImages = succeedAllGlazeImages;
    function succeedAllGlazeImages() {
        //PARSE TAGS AFTER GETTING GLAZE IMAGES [GAURAV]
        $scope.isParseTag = true;
        if ($scope.isParseTag) {
            parseTagData();
            $scope.isParseTag = false;
            debugMessage("succeedAllGlazeImages");
        }
    }

    /**
     * @function
     * @private
     * @name closeLoadingBar
     * @description To close loading bar based on response
     */
    function closeLoadingBar() {
        $timeout(function () {
            $scope.checkResponse = [];
            $scope.loadingBar = false;
            $scope.isTransparent = false;
            SHPService.loadingFinished();
        }, 0);
    }

    /**
     * @function
     * @private
     * @name dismissLoadingBar
     * @description To dismiss loading bar based on response
     */
    function dismissLoadingBar(value) {
        if ($scope.checkResponse.indexOf(value) !== -1) {
            $scope.checkResponse.splice($scope.checkResponse.indexOf(value), 1);

            if ($scope.checkResponse.length === 0) {
                closeLoadingBar();
            }
        } else {
            if (value === 'FAIL') {
                $timeout(function () {
                    $scope.checkResponse = [];
                    $scope.loadingBar = false;
                    $scope.isTransparent = false;
                }, 0);
            }
        }
    }

    function setLoadingBar_noTimeOut(value) {
        $scope.loadingBar_noTimeOut = true;
        $scope.checkResponse_noTimeOut.push(value);
    }

    function dismissLoadingBar_noTimeOut(value) {
        console.log("dismissLoadingBar_noTimeOut:::value = " + value);
        if ($scope.checkResponse_noTimeOut.indexOf(value) !== -1) {
            $scope.checkResponse_noTimeOut.splice($scope.checkResponse_noTimeOut.indexOf(value), 1);
            if ($scope.checkResponse_noTimeOut.length === 0) {
                $timeout(function () {
                    $scope.loadingBar_noTimeOut = false;
                    $scope.isTransparent = false;
                }, 0);
            }
        } else {
            if (value === 'FAIL') {
                $timeout(function () {
                    $scope.checkResponse_noTimeOut = [];
                    $scope.loadingBar_noTimeOut = false;
                    $scope.isTransparent = false;
                }, 0);
            }
        }
    }

    /**
     * @function
     * @private
     * @name closeApp
     * @description To close application, called from app.js
     */
    function closeApp() {
        SHPService.sendNativeCommand(REQUEST.CLOSE_WEB_APP, "", "");
    }

    /**
     * @function
     * @private
     * @name toggleBottomPopUpClass
     * @param {Boolean} popupName
     * @description Change bottom popup background class based on popup open or close.
     */
    function toggleBottomPopUpClass(popupName) {
        if (popupName) {
            $scope.BottomPopupDimBackgroundDiv = "BottomPopupDimBackgroundDivIn";
            $scope.BottomPopupDivCls = "BottomPopupBGIn";
        } else {
            $scope.BottomPopupDimBackgroundDiv = "BottomPopupDimBackgroundDivOut";
            $scope.BottomPopupDivCls = "BottomPopupBGOut";
        }
    }

    function isScreenScrollable() {
        try {
            if ($scope.loadingBar || $scope.bOptionPopUp || $scope.is_mdSelect || $scope.zoomGlazeImg ||
                    //$scope.view.insideImageView ||
                $scope.confirmationPopUpVisible || $scope.dialogPopup.show ||
                $scope.isCalender || $scope.daySelector ||
                $scope.sendToShoppingListPopUp || $scope.createNewListPopUp || $scope.moveLocationPopup) {
                return false;
            }
            else if ($scope.view.foodListView)
                return true;
            else {
                return true;
            }
        } catch(err) {
            debugMessage("[ERR]isScreenScrollable:::", err);
            return true;
        }
    }

    function deleteTag() {
        debugMessage("deleteTag clicked");
        deleteItem();
    }

    function showEntryDatePopup() {
        $scope.myDate = $scope.inputData.entryDate;
        $scope.entryDatePopup = true;
        showCalenderPopup();
    }

    function showExpirationPopup() {
        $scope.myDate = $scope.inputData.expirationDate;
        $scope.expirationPopup = true;
        showCalenderPopup();
    }

    function showCalenderPopup() {
        removeFocus();
        $scope.isCalender = !$scope.isCalender;
        $scope.daySelector = false;
        $scope.datepick = new Date();
        $scope.set_date_container = "set_date_disabled no_click";
    }

    function closeAllPopUp() {
        debugMessage("closeAllPopUp");
        if ($scope.isCalender) {
            cancelDate();
            return;
        } else if ($scope.daySelector) {
            $scope.daySelector = false;
            return;
        }

        $scope.addFoodItem = false;
        $scope.sendToShoppingListPopUp = false;
        $scope.zoomGlazeImg = false;
        $scope.editFoodItem = false;
        $scope.moveLocationPopup = false;
    }

    function closeSubPopUp() {
        $scope.isCalender = false;
        $scope.daySelector = false;
    }

    function closeOptionPopup() {
        $scope.bOptionPopUp = false;
    }

    function showOptionPopup() {
        $scope.bOptionPopUp = true;
    }

    function selectIndexFromOptionMenu(selectOption) {
        // var optionMenuIndex = items.id;
        debugMessage("Selected Option is " + selectOption);
        switch (selectOption) {
            case $scope.FOODREMINDER_OPTIONS.DELETE:
                $scope.selectedModeForDelete();
                break;
            case $scope.FOODREMINDER_OPTIONS.MOVE_LOCATION:
                $scope.selectedModeForMoveLocation();
                break;
            case $scope.FOODREMINDER_OPTIONS.SEND_TO_SHOPPING_LIST:
                $scope.selectedModeForSendToShoppingList();
                break;
            default:
                break;
        }
        closeOptionPopup();
    }

    $scope.selectedMode_option = false;
    function longPressHandlerForDelete(item) {
        debugMessage("longPressHandlerForDelete" + $scope.currentSelectedMode);
        if ($scope.currentSelectedMode === "") {
            $scope.selectedMode_option = true;
            $scope.selectedModeForDelete();
            if (angular.isDefined(item))
                setSelectedItems(item);
            else
                console.log('longPressHandlerForDelete::::item is undefined');
        }
    }

    function selectedModeForDelete() {
        debugMessage("selectedModeForDelete");
        $scope.isSelectedMode = true;
        $scope.currentSelectedMode = $scope.FOODREMINDER_OPTIONS.DELETE;
    }

    function selectedModeForSendToShoppingList() {
        debugMessage("selectedModeForSendToShoppingList");
        if ($scope.isSelectedMode) {        //if selectedMode is already actived, send requestCommand.
            showSendToShoppingListPopup();
            return;
        }
        $scope.isSelectedMode = true;
        $scope.currentSelectedMode = $scope.FOODREMINDER_OPTIONS.SEND_TO_SHOPPING_LIST;
    }

    function selectedModeForMoveLocation() {
        debugMessage("selectedModeForMoveLocation");
        if ($scope.isSelectedMode) {        //if selectedMode is already actived, send requestCommand.
            showMoveLocationPopUp();
            return;
        }
        $scope.isSelectedMode = true;
        $scope.currentSelectedMode = $scope.FOODREMINDER_OPTIONS.MOVE_LOCATION;
        $scope.showConsoleLogs &&  console.log("End selectedModeForMoveLocation " + $scope.currentSelectedMode);
    }

    $scope.isSelectedSameLocation = function (location_id) {
        if (angular.isUndefined(location_id))
            return false;
        var result = $filter('filter')($scope.selectedItems, {storageType: location_id});
        return result.length === $scope.selectedItems.length;
    };

    function getShoppingListFormat(item) {
        if (angular.isUndefined(item.groupId) || angular.isUndefined(item.id)) {
            console.log('getShoppingListItemPayload::: Undefined group or id');
            return;
        }

        var deals = {
            deals_id: "",
            price: "",
            storeName: "",
            deals_detail: "",
            expiry_date:"",
            api_name:"",
            clip_state:""
        };
        if (angular.isDefined(item.Deals)) {
            deals.deals_id = (angular.isDefined(item.Deals.deals_id))? item.Deals.deals_id : "";
            deals.price = (angular.isDefined(item.Deals.price))? item.Deals.price : "";
            deals.storeName = encodeEscape(item.Deals.storeName);
            deals.deals_detail = encodeEscape(item.Deals.deals_detail);
            deals.expiry_date = (angular.isDefined(item.Deals.expiry_date))? item.Deals.expiry_date : "";
            deals.api_name = (angular.isDefined(item.Deals.api_name))? item.Deals.api_name : "";
            deals.clip_state = (angular.isDefined(item.Deals.clip_state))? item.Deals.clip_state : "";
        }

        return {
            userId: (angular.isDefined($scope.userId))? $scope.userId : "",
            groupId: item.groupId,
            id: item.id,
            category: encodeEscape(item.category),
            isDefault: (angular.isDefined(item.isDefault))? item.isDefault : false,
            category_info: (angular.isDefined(item.category_info))? item.category_info : "",
            name: encodeEscape(item.name),
            imageURL: (angular.isDefined(item.imageURL))? item.imageURL : "",
            itemStatus: (angular.isDefined(item.itemStatus))? item.itemStatus : "1",
            storageType: timeStamp.getDateFormat(new Date(),'yyyy-MM-ddTHH:mm:ss', 'UTC'),
            order: (angular.isDefined(item.order))? item.order : item.id,
            recipeName: (angular.isDefined(item.recipeName))? item.recipeName : "",
            data_type: (angular.isDefined(item.data_type))? item.data_type : "",
            emart_item_id: (angular.isDefined(item.emart_item_id))? item.emart_item_id : "",
            emart_min_qty: (angular.isDefined(item.emart_min_qty))? item.emart_min_qty : "",
            site_no: (angular.isDefined(item.site_no))? item.site_no : "",
            food_position: (angular.isDefined(item.food_position))? item.food_position : $scope.locations[0].id,
            Deals: deals,
            quantity: (angular.isDefined(item.quantity)) ? item.quantity : 1,
            quantityUnit: (angular.isDefined(item.quantityUnit)) ? item.quantityUnit : "",
            isAdded: false
        };
    }

    function sendToShoppingListTitleCommand(groupId, category) {
        if (angular.isUndefined(groupId))
            return;

        debugMessage('sendToShoppingListTitleCommand::: category= ' + category);
        if (angular.isUndefined(category) || category.trim()==="")
            category = decodeHtmlEntity($scope.translation.WEBMOB_fhub2_shopping_list_create_new_title);
        else if (category.trim().length > $scope.MAX_CHARACTER_TITLE)
            category = category.trim().substr(0, $scope.MAX_CHARACTER_TITLE);
        else
            category = category.trim();

        var title_info = {
            groupId: groupId,
            id: groupId,
            category: category,
            order: groupId,
            food_position: ""
        };

        var payload = {
            "ShoppingList": [
                getShoppingListFormat(title_info)
            ],
            sendPush: false
        };

        $scope.checkResponse.push(REQUEST.ADD_SHOPPING_LIST);
        SHPService.sendNativeCommand(REQUEST.ADD_SHOPPING_LIST, payload, REQUEST.ADD_SHOPPING_LIST+'_Title');
    }

    function sendToShoppingListItemCommand(groupId, newId, item) {
        if (angular.isUndefined(item))
            return;
        if (angular.isUndefined(item.name) || item.name.trim() === "")
            item.name = generateDefaultFoodTitle();
        else if (item.name.trim().length > $scope.MAX_CHARACTER_ITEM && !isEmart(item) && !isWoolworth(item))
            item.name = checkMaxFoodN(item.name.trim().substr(0, $scope.MAX_CHARACTER_ITEM));
        else
            item.name = checkMaxFoodN(item.name.trim());

        var isDefault = false;
        if(angular.isDefined($scope.Device.ShoppingList[groupId]) && angular.isDefined($scope.Device.ShoppingList[groupId].isDefault)) {
            console.log('sendToShoppingListItemCommand::: isDefault = ' + $scope.Device.ShoppingList[groupId].isDefault);
            isDefault = $scope.Device.ShoppingList[groupId].isDefault;
        }

        if (angular.isDefined(item.imageURL)) {
            if (item.imageURL === "newImage")
                item.imageURL = getImage(item.imageURL).replace(BASE64, "");
        }

        if (angular.isDefined(item.storageType))
            item.food_position = item.storageType;

        item.groupId = groupId;
        item.id = newId;
        item.isDefault = isDefault;
        item.order = newId;
        item.category = "";

        return getShoppingListFormat(item);
    }

    function sendShoppingListFromCreateNewListPopup(createCategory) {
        var payload = {
            "ShoppingList": []
        };

        if (isMaximum_ShoppingList()) {
            SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_shopping_list_send_list_max_lists.replace("#num#", $scope.MAX_SHOPPING_LIST_TITLE.toString()), "");
            return;
        }

        debugMessage("sendShoppingListFromCreateNewListPopup:: input List- " + createCategory);
        var groupId = timeStamp.getDateFormat(new Date(), 'yyyyMMddHHmmss sss', 'UTC').replace(" ", "");
        sendToShoppingListTitleCommand(groupId, createCategory);

        if ($scope.sendToShoppingListPopUp) {
            if (isMaximum_ShoppingList_item(groupId, $scope.selectedItems)) {
                SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_shopping_list_send_max_items.replace("#num#", $scope.MAX_SHOPPING_LIST_ITEM.toString()), "");
                return;
            }
            $scope.selectedItems.forEach(function (item, index) {
                //create id
                var currentLocalDate = new Date();
                currentLocalDate.setTime(currentLocalDate.getTime() + (index + 1));
                var id = timeStamp.getDateFormat(currentLocalDate, 'yyyyMMddHHmmss sss', 'UTC').replace(" ", "");

                payload['ShoppingList'].push(sendToShoppingListItemCommand(groupId, id, item));
            });
        }
        if ($scope.sendToShoppingListPage) {
            if (isMaximum_ShoppingList_item(groupId, 1)) {
                SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_shopping_list_send_max_items.replace("#num#", $scope.MAX_SHOPPING_LIST_ITEM.toString()), "");
                return;
            }

            //create id
            var currentLocalDate = new Date();
            currentLocalDate.setTime(currentLocalDate.getTime() + 1);
            var id = timeStamp.getDateFormat(currentLocalDate, 'yyyyMMddHHmmss sss', 'UTC').replace(" ", "");

            // $scope.inputData.imageURL = $scope.AddFoodImage.replace("data:image/png;base64,", "");
            payload['ShoppingList'].push($scope.sendToShoppingListItemCommand(groupId, id, $scope.inputData));
        }

        $scope.checkResponse.push(REQUEST.ADD_SHOPPING_LIST);
        SHPService.sendNativeCommand(REQUEST.ADD_SHOPPING_LIST, payload, REQUEST.ADD_SHOPPING_LIST);
        closeSelectedMode();
        if ($scope.sendToShoppingListPopUp)
            goToFoodList();
        else if ($scope.sendToShoppingListPage) {
            debugMessage('called sendShoppingListFromCreateNewListPopup');
            goToInsideImage();
        }
    }

    function sendToShoppingList(selectedList) {
        var payload = {
            "ShoppingList": []
        };

        // $scope.inputData.shoppingTitle = JSON.parse($scope.inputData.shoppingTitle);
        // debugMessage("sendToShoppingList:: SelectedList= " + JSON.stringify($scope.inputData.shoppingTitle));
        var listInfo;
        if (angular.isDefined(selectedList)) {
            listInfo = selectedList;
        }
        else {
            listInfo = $scope.inputData.selectedCategory;
        }

        var groupId = listInfo.groupId;

        if ($scope.sendToShoppingListPopUp) {
            if (isMaximum_ShoppingList_item(groupId, $scope.selectedItems.length)) {
                SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_shopping_list_send_max_items.replace("#num#", $scope.MAX_SHOPPING_LIST_ITEM.toString()), "");
                return;
            }
            $scope.selectedItems.forEach(function (item, index) {
                //create id
                var currentLocalDate = new Date();
                currentLocalDate.setTime(currentLocalDate.getTime() + (index + 1));
                var id = timeStamp.getDateFormat(currentLocalDate, 'yyyyMMddHHmmss sss', 'UTC').replace(" ", "");

                payload['ShoppingList'].push(sendToShoppingListItemCommand(groupId, id, item));
            });
        }
        if ($scope.sendToShoppingListPage) {
            if (isMaximum_ShoppingList_item(groupId, 1)) {
                SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_shopping_list_send_max_items.replace("#num#", $scope.MAX_SHOPPING_LIST_ITEM.toString()), "");
                return;
            }
            checkElement($('#send_item_name')[0],$scope.MAX_CHARACTER_ITEM);

            var id = timeStamp.getDateFormat(new Date(), 'yyyyMMddHHmmss sss', 'UTC').replace(" ", "");
            // $scope.inputData.imageURL = $scope.AddFoodImage.replace("data:image/png;base64,", "");
            payload['ShoppingList'].push($scope.sendToShoppingListItemCommand(groupId, id, $scope.inputData));
        }

        $scope.checkResponse.push(REQUEST.ADD_SHOPPING_LIST);
        SHPService.sendNativeCommand(REQUEST.ADD_SHOPPING_LIST, payload, REQUEST.ADD_SHOPPING_LIST);
        closeSelectedMode();
        if ($scope.sendToShoppingListPopUp)
            goToFoodList();
        else if ($scope.sendToShoppingListPage) {
            debugMessage('called sendToShoppingList');
            goToInsideImage();
        }
    }

    $scope.isEmptyString = isEmptyString;
    function isEmptyString(str) {
        if (angular.isUndefined(str))
            return true;
        return str.trim() === '';
    }

    function isEmpty(obj, filter) {
        if (angular.isUndefined(filter))
            return Object.keys(obj).length === 0;
        else {
            var count = 0;
            $.each(obj, function (index, item) {
                if (reminderFilter(item))
                    count++;
            });
            return count === 0;
        }
    }

    function isMaxItem() {
        if ($scope.Device && $scope.Device.FoodList && $scope.Device.FoodList.Fridge) {
            return $scope.Device.FoodList.Fridge.length >= $scope.MAX_FOOD_ITEM;
        }
        return false;
    }

    $scope.isMaxExpiryTag = isMaxExpiryTag;
    function isMaxExpiryTag() {
        if ($scope.Device && $scope.Device.FoodList && $scope.Device.FoodList.Fridge) {
            var filtered = $filter('filter')($scope.Device.FoodList.Fridge, {'is_expiry_tag': true});
            return filtered.length >= $scope.MAX_FOOD_TAG;
        }
        return false;
    }

    $scope.changed_isReminderOption = function () {
        removeFocus();
        // if (isAllowExpiryTag($scope.inputData) && $scope.addFoodItem && $scope.inputData.isReminder && isMaxExpiryTag()) {
        //     SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_food_reminder_add_max_expiry_tag.replace("#num#", $scope.MAX_FOOD_TAG.toString()), "");
        //     $scope.inputData.isReminder = false;
        //     $scope.inputData.is_expiry_tag = false;
        // }
    };

    $scope.changed_expiryTagOption = function () {
        removeFocus();
        if (isAllowExpiryTag($scope.inputData) && $scope.inputData.is_expiry_tag && isMaxExpiryTag()) {
            SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_food_reminder_add_max_expiry_tag.replace("#num#", $scope.MAX_FOOD_TAG.toString()), "");
            $scope.inputData.is_expiry_tag = false;
        }
    };

    $scope.changed_locationOption = function () {
        console.log("changed_locationOption:::", $scope.inputData.storageType);
        $scope.inputData.isAllowRecommend.location = false;
        var foodData = updateFoodLifeByFoodDB($scope.inputData.entryDate);
        updateRecommendedInfo(foodData);
    };

    function foodListCount(filter) {
        if (angular.isDefined(filter)) {
            if (isAllowLocationType(filter)) {
                var length = 0;
                $scope.Device.FoodList.Fridge.forEach(function (item) {
                    if (filter === item.storageType)
                        length++;
                });
                return length;
            }
            else if (filter === $scope.translation.WEBMOB_fhub2_food_reminder_expiration) {
                var length = 0;
                $scope.Device.FoodList.Fridge.forEach(function (item) {
                    if (!!item.isReminder === true)
                        length++;
                });
                return length;
            }
            else if (filter === "STORAGE_TAG") {
                return $scope.Device.FoodList.Tag_List.length;
            }
        }
        return $scope.Device.FoodList.Fridge.length;
    }

    function enableSendToShoplist() {
        var enable = false;
        var arr = $scope.sendToShoppingList.filter(function (el) {
            return (el.isSelected === true);
        });

        if (arr.length > 20) {//if selected more than display toast
            SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, $scope.translation.WEBMOB_fhub2_shopping_list_add_max_items.replace("#num#", $scope.MAX_SHOPPING_LIST_ITEM.toString()), "");
        } else if (arr.length > 0) {
            enable = true;
        } else {
            //do nothing
        }

        return enable;
    }

    function isFoodListEmpty(index) {
        var returnVal = false;
        if (((index === 0 || index === 1) && $scope.Device.FoodList.Fridge.length === 0) || (index === 2 && $scope.Device.FoodList.Tag_List.length === 0)) {
            returnVal = true;
        }

        return returnVal;
    }

    function isEmart(item) {
        if (angular.isDefined(item.data_type)) {
            if (item.data_type === 'emart')
                return true;
        }
        return false;
    }

    function isWoolworth(item) {
        if (angular.isDefined(item.data_type)) {
            if (item.data_type === 'woolworths')
                return true;
        }
        return false;
    }

    $scope.isAllowExpiryTag = isAllowExpiryTag;
    function isAllowExpiryTag(item) {
        if (angular.isDefined(item.data_type)) {
            return ($scope.isSupportFoodTag && item.data_type === 'expiryTag');
        }
        return false;
    }

    function encodeEscape(str) {
        if (angular.isDefined(str) && str !== "") {
            str = str.replace(/\n/g, "")
                .replace(/\r/g, "")
                .replace(/\t/g, "")
                .replace(/\b/g, "")
                .replace(/\f/g, "");
            return encodeURIComponent(str);
        }
        return "";
    }

    function decodeHtmlEntity(html) {
        var txt = document.createElement("textarea");
        txt.innerHTML = html;
        return txt.value.trim();
    }

    function decodeUnescape(str) {
        if (angular.isDefined(str) && str.length > 0 && str.trim() !== "") {
            try {
                str = decodeURIComponent(str);
                return decodeHtmlEntity(str);
            }
            catch(err) {
                $scope.showConsoleLogs &&  console.log("In decodeUnescape Error::: str = " + str);
                $scope.showConsoleLogs &&  console.log("err = " + JSON.stringify(err));
                return decodeHtmlEntity(str);
            }
        }
        return "";
    }

    $scope.checkMaxFoodN = checkMaxFoodN;
    function checkMaxFoodN(foodName) {
        var defaultName = decodeHtmlEntity($scope.translation.WEBMOB_fhub2_food_reminder_food_default);
        var itemName = foodName.split(" "),
            title = itemName[0],
            index = (itemName.length > 1) ? itemName[1] : 0;

        //Check Food N name
        if (itemName.length === 2 && defaultName === title && Number.isInteger(parseInt(index), 10)) {
            if (parseInt(index, 10) >= 1000000000000000)      //if N is over 16 digit.
                index = 999999999999999;        //set max N
            return title + ' ' + index;
        }

        return foodName;
    }

    function generateDefaultFoodTitle() {
        var listName = decodeHtmlEntity($scope.translation.WEBMOB_fhub2_food_reminder_food_default);
        var maxIndex = 0;

        $scope.Device.FoodList.Fridge.forEach(function (item) {
            var itemName = item.name.split(" "),
                title = itemName[0],
                index = (itemName.length > 1) ? itemName[1] : 0;
            if (title === listName && parseInt(maxIndex, 10) < parseInt(index, 10)) {
                maxIndex = parseInt(index, 10);
            }
        });

        maxIndex++;

        return checkMaxFoodN(listName + ' ' + maxIndex);
    }

    function expiryDigitCal(pos, days) {
        switch (pos) {
            case 1:
                if (days < 100) {
                    return -1;
                } else {
                    return parseInt(days / 100, 10);
                }
            case 2:
                if (days < 10) {
                    return -1;
                } else {
                    return parseInt((days % 100) / 10, 10);
                }
            case 3:
                return parseInt((days % 10), 10);
            default:
                return -1;
        }
    }

    /**
     * Sort the food list with the following criteria
     *  - Sort based on remaining days
     *  - If remainimng days are same, Sort based on string
     * @returns {VOID}
     */
    function sortFoodList() {
        $scope.Device.FoodList.Fridge.sort(function (x, y) {
            var n = parseInt(x.calculateDays, 10) - parseInt(y.calculateDays, 10); // calculate remaining days
            if (n !== 0) {//remaining days are not same
                return n;
            } else {//sort based on name (decode string first)
                var xName = x.name;
                var yName = y.name;
                if (xName > yName) {
                    return 1;
                } else if (xName < yName) {
                    return -1;
                } else {
                    // names must be equal to b
                    return 0;
                }
            }
        });
    }

    $scope.order = {
        sortBy: $scope.translation.WEBMOB_fhub2_food_reminder_expiration,
        isReverse: false,
        filterBy: $scope.translation.WEBMOB_fhub2_food_reminder_filter_all_items
    };

    $scope.setSortBy = function (sortType) {
        if (sortType === $scope.order.sortBy)
            $scope.order.isReverse = !$scope.order.isReverse;
        else {
            $scope.order.sortBy = sortType;
            $scope.order.isReverse = false;
        }
    };

    $scope.sortListBy = function (item) {
        if ($scope.order.sortBy === $scope.translation.WEBMOB_fhub2_food_reminder_sort_date) {
            return item.entryDate;
        }
        else if ($scope.order.sortBy === $scope.translation.WEBMOB_fhub2_food_reminder_sort_location) {
            return item.name;
        }
        return 0;
    };

    $scope.setFilterBy = function (filterType) {
        $scope.order.filterBy = filterType;
    };

    function reminderFilter (item) {
        // debugMessage("reminderFilter :: $scope.filterBy = " + $scope.order.filterBy);
        if ($scope.order.filterBy === $scope.translation.WEBMOB_fhub2_food_reminder_filter_all_items)
            return true;
        else if ($scope.order.filterBy === $scope.translation.WEBMOB_fhub2_food_reminder_expiration)
            return ((!!item.isReminder) === true);
        else {
            //Filter By location
            if ($scope.order.filterBy === "STORAGE_FRIDGE")
                return (item.storageType === "STORAGE_FRIDGE");
            else if ($scope.order.filterBy === "STORAGE_FREEZER")
                return (item.storageType === "STORAGE_FREEZER");
            else if ($scope.order.filterBy === "STORAGE_FLEX_ZONE")
                return (item.storageType === "STORAGE_FLEX_ZONE");
            else if ($scope.order.filterBy === "STORAGE_FLEX_ZONE_18K")
                return (item.storageType === "STORAGE_FLEX_ZONE_18K");
            else if ($scope.order.filterBy === "STORAGE_PANTRY")
                return (item.storageType === "STORAGE_PANTRY");
        }
        return false;
    }

    function checkFilter(filter, item) {
        if (angular.isDefined(filter) && angular.isDefined(item)) {
            if (filter === $scope.translation.WEBMOB_fhub2_food_reminder_filter_all_items)
                return true;
            else if (filter === $scope.translation.WEBMOB_fhub2_food_reminder_expiration && (!!item.isReminder))
                return true;
            else if (isAllowLocationType(filter)) {
                if (filter === item.storageType)
                    return true
            }
        }
        return false;
    }

    $scope.orderByFoodList = orderByFoodList;
    function orderByFoodList(isReverse) {
        if ($scope.order.sortBy === $scope.translation.WEBMOB_fhub2_food_reminder_expiration) {
            //expirationDate_order: If item is saved isReminder = false, do not sort expirationDate.
            if (isReverse)
                return ['+isReminder', '+expirationDate_order', '+entryDate', '+name'];
            else
                return ['-isReminder', '+expirationDate_order', '+entryDate', '+name'];
        }

        if ($scope.order.sortBy === $scope.translation.WEBMOB_fhub2_food_reminder_sort_location) {
            if (isReverse)
                return ['-storageType_idx', '+name'];
            else
                return ['+storageType_idx', '+name'];
        }

        if ($scope.order.sortBy === $scope.translation.WEBMOB_fhub2_shopping_list_find_deals_sort_by_name) {
            if (isReverse)
                return ['-name', '+entryDate'];
            else
                return ['+name', '+entryDate'];
        }
    }

    $scope.showSubTitle = function (prevList, curList, index, key) {
        if (angular.isDefined(curList) && angular.isDefined(index) && angular.isDefined(key)) {
            if (!isEmpty(curList[key])) {
                if (index === 0) {
                    return true;
                }
                else if (angular.isDefined(prevList) && !isEmpty(prevList[key]) && prevList[key] !== curList[key]) {
                    return true;
                }
            }
        }
        return false;
    };

    /*
     CheckBox Function for Delete
     */
    $scope.setSelectedItems = setSelectedItems;
    $scope.deleteSelectedItems = deleteSelectedItems;
    $scope.clearSelectedItems = clearSelectedItems;
    $scope.isCheckedItems = isCheckedItems;
    $scope.toggleCheckList = toggleCheckList;
    $scope.isChecked = isChecked;
    $scope.toggleSelectAllListCheckBox = toggleSelectAllListCheckBox;
    $scope.isAllCheckBoxSelected = isAllCheckBoxSelected;
    $scope.closeSelectedMode = closeSelectedMode;

    $scope.isSelectedMode = false;
    $scope.selectedItems = [];

    function closeSelectedMode() {
        $scope.homePage = true;
        $scope.selectedMode_option = false;
        $scope.sendToShoppingListPopUp = false;
        $scope.createNewListPopUp = false;
        $scope.moveLocationPopup = false;

        $scope.isSelectedMode = false;
        $scope.currentSelectedMode = "";
        $scope.clearSelectedItems();
        removeFocus();
    }

    function setSelectedItems(item) {
        // debugMessage("setSelectedItems:::"+JSON.stringify(item));
        if (!isChecked(item))
            $scope.selectedItems.push(item);
    }

    function deleteSelectedItems(item) {
        debugMessage("deleteSelectedItems:::");
        $scope.selectedItems.forEach(function (selectedItem, index) {
            if (selectedItem.id === item.id)
                $scope.selectedItems.splice(index, 1);
        });
    }

    function clearSelectedItems() {
        debugMessage("clearSelectedItems:::");
        $scope.selectedItems = [];
    }

    function isCheckedItems() {
        return ($scope.selectedItems.length > 0);
    }

    function toggleCheckList(item) {
        if (!isChecked(item))
            setSelectedItems(item);
        else
            deleteSelectedItems(item);
        isAllCheckBoxSelected();
    }

    function isChecked(item) {
        for (var i = 0; i < $scope.selectedItems.length; i++) {
            if ($scope.selectedItems[i].id === item.id)
                return true;
        }
        return false;
    }

    function toggleSelectAllListCheckBox() {
        debugMessage("toggleSelectAllListCheckBox:::");
        if ($scope.isAllCheckBoxSelected())
            $scope.clearSelectedItems();
        else {
            $scope.Device.FoodList.Fridge.forEach(function (item) {
                if (checkFilter($scope.order.filterBy, item))
                    setSelectedItems(item);
            });
        }

        $scope.DefineLocations.forEach(function (location) {
            location.isExpand = true;
        });

        $scope.isAllCheckBoxSelected();
    }

    function isAllCheckBoxSelected() {
        return ($scope.selectedItems.length === foodListCount($scope.order.filterBy));
    }

    $scope.getCategoryImage = getCategoryImage;
    function getCategoryImage(categoryId) {
        var mater_category_id = convertToCategory_ID(categoryId);
        switch (mater_category_id) {
            case 'Fruits_Vegetables':
                return 'fridge_produce';
            case 'Meats_Seafood':
                return 'fridge_meat';
            case 'Bread_Bakery':
                return 'flexzone_bread';
            case 'Dairy_Eggs_Cheese':
                return 'flexzone_dairy';
            case 'Condiments_Sauces':
                return 'fridge_ketchup';
            case 'Frozen':
                return 'flexzone_frozen';
            case 'Seasoning_Spices_Baking':
                return 'freezer_spices';
            case 'Breakfast_Cereal':
                return 'flexzone_cereal';
            case 'Cookies_Snacks_Candy':
                return 'freezer_snacs';
            case 'Beverages':
                return 'flexzone_drinks';
            case 'Canned_DryGoods':
                return 'flexzone_cans';
            case 'PetCare':
                return 'flexzone_pets';
            case 'Household':
                return 'flexzone_household';
            case 'PersonalCare':
                return 'flexzone_personal';
            case 'BabyCare':
                return 'flexzone_baby';
            case 'Other':
                return 'flexzone_other';
        }
        return 'flexzone_other';
    }

    function getImage (key) {
        if (angular.isUndefined(key) || key === "")
            return "";
        if ($scope.isStatic) {
            return './REF_ViewInside/assets/img/main_rf_img_03.png';
        }
        if (angular.isDefined(imageCache) && angular.isDefined(imageCache.get(key))) {
            return imageCache.get(key);
        }
        return "";
    }

    $scope.getProfileImage = function (user_id) {
        // console.log("getProfileImage::: ", user_id, profileService.getProfileImage(user_id));
        return profileService.getProfileImage(user_id);
    };

    function imageURL_Request(item) {
        console.log("imageURL_Request:::" + JSON.stringify(item));
        if (angular.isDefined(item) && angular.isDefined(item.imageURL))
            imageRequest.nativeRequest(item.imageURL);
        else if (angular.isDefined(item))
            imageRequest.nativeRequest(item);
    }

    function glazeImageURL_Request(glazeURL, glaze_idx) {
        console.log("glazeImageURL_Request:::" + glazeURL);
        if ($scope.isStatic || $scope.isLocal)
            return;
        imageRequest.nativeRequest(glazeURL, true, glaze_idx);
    }

    /*
     * Custom event listener for ngRenderFinished, used for glaze area initialization for Inside Image page
     */
    $scope.$on('ngRenderFinished', function () {
        console.log('ngRenderFinished');
        cWidth = $("#frame").width();
        cHeight = $("#frame").height();
        renderGlazeImage();
        parseTagData(); //Recreate tags
    });

    function renderGlazeImage() {
        var count = 0;
        $scope.insideFridgeData.GlazeURL.forEach(function (url, index) {
            console.log("renderGlazeImage::: ", index, url);
            if ($scope.isStatic || $scope.isLocal) {
                console.log("****renderGlazeImage::: ", index, url);
                $timeout(function () {
                    $('#REF_GLAZE_' + index).css({'background': 'url(./REF_ViewInside/assets/img/camera_fullSize/' + url + ') no-repeat'})
                        .css({'background-size': '100% 100%'});
                }, 0);
                count++;
            }
            else if (document.getElementById('REF_GLAZE_' + index) !== null && getImage('REF_GLAZE_' + index) !== "") {
                console.log("****renderGlazeImage::: ", 'REF_GLAZE_' + index, index, url);
                $timeout(function () {
                    $('#REF_GLAZE_' + index).css({'background': 'url(' + getImage('REF_GLAZE_' + index) + ') no-repeat'})
                                            .css({'background-size': '100% 100%'});
                }, 0);
                count++;
            }
        });
        if ($scope.loadingBar_noTimeOut && count === $scope.insideFridgeData["GlazeURL"].length)
            glazeProgressIncrease(60);
    }


    /** Start AutoComplete func to Search for Food Data **/
    $scope.queryItems = [];
    $scope.requestQuery = requestQuery;
    function requestQuery(str, request_token) {
        if (!angular.isDefined(str) || str.length < 1) {
            return;
        }
        else if ($scope.isStatic || $scope.isLocal) {
            $timeout(function () {
                var response = staticData.responseAutoComplete(str);
                responseAutoCompleteFromFoodDB(response["data"]);
            }, 0);  //test for delay
        }
        else {
            request_token = angular.isUndefined(request_token)? REQUEST.SEARCH_FOR_FOOD + "_QUERY_" + str : request_token;
            var payload = {
                search : str
            };
            SHPService.sendNativeCommand(REQUEST.SEARCH_FOR_FOOD, payload, request_token, false);
        }
    }

    $scope.responseAutoCompleteFromFoodDB = responseAutoCompleteFromFoodDB;
    function responseAutoCompleteFromFoodDB(data) {
        console.log("responseAutoCompleteFromFoodDB:::", data);
        if (angular.isDefined(data) && angular.isDefined(data["SearchList"]) && data["SearchList"].length > 0) {
            return data["SearchList"];
        }
        else {
            return [];
        }
    }

    $scope.asyncQuery = null;
    $scope.querySearch = function (query) {
        if (!angular.isDefined(query) || query.length < 1) {
            return [];
        }

        requestQuery(query);
        if (angular.isUndefined($scope.asyncQuery) || $scope.asyncQuery === null) {
            $scope.asyncQuery = $q.defer();
        }

        return $scope.asyncQuery.promise.then(
            function successCallback(response) {
                console.log("successCallback = ", response);
                $timeout(function () {
                    $(".md-scroll-mask").bind({
                        click: function () {
                            console.log("Clicked .md-scroll-mask");
                            removeFocus();
                            $(".md-scroll-mask").css({'display': 'none'});
                            $("md-virtual-repeat-container").css({'display': 'none'});
                        }
                    });

                    //RF00039359: moved the scroll position to top
                    var scroll_element = $('.md-virtual-repeat-scroller');
                    if (angular.isDefined(scroll_element) && scroll_element !== null && scroll_element.length > 0) {
                        scroll_element[0].scrollTop = 0;
                    }
                }, 0);

                $scope.asyncQuery = null;
                return response;
            },
            function errorCallback(response) {
                console.log("errorCallback = ", response);
                return [];
            },
            function notifyCallback(response) {
                console.log('notifyCallback = ', response);
                try {
                    var splitArr = response['requestToken'].split(REQUEST.SEARCH_FOR_FOOD + "_QUERY_");
                    var query = splitArr[splitArr.length - 1];

                    if (angular.isDefined(query) && query !== null && query === $scope.inputData.name) {
                        $scope.queryItems = $scope.responseAutoCompleteFromFoodDB(response["data"]);
                        console.log('notifyCallback - resolve = ', $scope.queryItems);
                        $scope.asyncQuery.resolve($scope.queryItems);
                    }
                } catch (err) {
                    console.log("asyncQuery - notifyCallback error = ", err);
                }
            }
        );
    };

    $scope.searchTextChanged = function (str) {
        console.log('searchTextChanged:::', str);
    };

    $scope.selectedFoodData = undefined;
    $scope.searchTextSelected = function (selectedData) {
        console.log('searchTextSelected:::', selectedData);
        if ($scope.editFoodItem) {
            $scope.inputData.isAllowRecommend = {
                location: false,
                expiration: false
            }
        }

        if (angular.isDefined(selectedData)) {
            $scope.selectedFoodData = selectedData;

            if (angular.isDefined(selectedData['metadata'])) {
                var foodData = updateFoodLifeByFoodDB($scope.inputData.entryDate);
                updateRecommendedInfo(foodData);
            }
        }
    };

    $scope.ValidURL = ValidURL;
    function ValidURL(str) {
        console.log("ValidURL = ", str);
        var pattern = new RegExp('^((http|https)?:\\/\\/)?'+ // protocol
            '((([a-z\\d]([a-z\\d-]*[a-z\\d])*)\\.?)+[a-z]{2,}|'+ // domain name
            '((\\d{1,3}\\.){3}\\d{1,3}))'+ // OR ip (v4) address
            '(\\:\\d+)?(\\/[-a-z\\d%_.~+]*)*'+ // port and path
            '(\\?[;&a-z\\d%_.~+=-]*)?'+ // query string
            '(\\#[-a-z\\d_]*)?$','i'); // fragment locator
        return pattern.test(str);
    }


    $scope.getLocationString = function (storage_id) {
        var output = convertLocationIdToValue(storage_id);
        if ($scope.inputData.defaultLocation === storage_id && $scope.inputData.recommended.location) {
            output += '*';
        }
        return output;
    };

    $scope.updateRecommendedInfo = updateRecommendedInfo;
    function updateRecommendedInfo(foodData) {
        $scope.inputData.recommended = foodData.recommended;
        $scope.inputData.defaultLocation = foodData.location;

        if ($scope.addFoodItem || $scope.sendToShoppingListPage ||
            ($scope.editFoodItem && (foodData.recommended.location || foodData.recommended.expiration) &&
            ($scope.inputData.isAllowRecommend.location || $scope.inputData.isAllowRecommend.expiration))) {
            var selectedData = $scope.selectedFoodData;
            if (angular.isDefined(selectedData)) {
                try {
                    if (($scope.inputData.imageURL !== "newImage" || !ValidURL($scope.inputData.imageURL)) &&
                        $scope.inputData.imageURL !== selectedData.imageUrl) {
                        $scope.inputData.imageURL = selectedData.imageUrl;
                        imageURL_Request(selectedData.imageUrl);
                    }
                } catch (err) {
                    console.log('[Error]updateRecommendedInfo:::' + err);
                    $scope.inputData.imageURL = '';
                }
                if (angular.isDefined(selectedData['metadata']) && angular.isDefined(selectedData['metadata'].category))
                    $scope.inputData.category_info = selectedData['metadata'].category;
            }
        }

        if (foodData.recommended.location && $scope.inputData.isAllowRecommend.location && $scope.inputData.isAllowRecommend.expiration) {
            $scope.inputData.storageType = foodData.location;
        }

        if (foodData.recommended.expiration && $scope.inputData.isAllowRecommend.expiration) {
            $scope.inputData.isReminder = true;
            $scope.inputData.expirationDate = foodData.expirationDate;
            $scope.inputData.calculateDays = diffDays(new Date(), foodData.expirationDate);
        }
    }

    $scope.updateFoodLifeByFoodDB = updateFoodLifeByFoodDB;
    function updateFoodLifeByFoodDB(entryDate) {
        var output_foodData = {
            recommended: {
                expiration: false,
                location: false
            },
            location: "",
            expirationDate: getDefaultExpirationDate()
        };

        console.log("updateFoodLifeByFoodDB:::", $scope.inputData.storageType);
        if (diffDays(new Date(), entryDate) === 0 &&
            ($scope.inputData.isAllowRecommend.location || $scope.inputData.isAllowRecommend.expiration) &&
            angular.isDefined($scope.selectedFoodData) &&
            angular.isDefined($scope.selectedFoodData['metadata']) &&
            angular.isDefined($scope.selectedFoodData['metadata']['storing'])) {
            var storing = $scope.selectedFoodData['metadata']['storing'];

            var locationIdx = getLocationIndex($scope.inputData.storageType);
            var whiskType = $scope.DefineLocations[locationIdx].whiskType;
            var metricVal = ["Days", "Weeks", "Months", "Years", "Infinitely", "NotRecommended"];

            if (angular.isDefined(storing['defaultLocation']) && storing['defaultLocation'] !== null && storing['defaultLocation'] !== "") {
                var index = $scope.DefineLocations.map(function (location) {return location.whiskType;}).indexOf(storing['defaultLocation'].toLocaleLowerCase());
                if (index !== -1 && angular.isDefined(storing[$scope.DefineLocations[index].whiskType])) { //Recommended the location 1st
                    var defaultLocation_whiskType = $scope.DefineLocations[index].whiskType;
                    output_foodData.location = $scope.DefineLocations[index].id;

                    if ($scope.inputData.isAllowRecommend.location && $scope.inputData.isAllowRecommend.expiration) {
                        output_foodData.recommended.location = true;
                        whiskType = defaultLocation_whiskType;
                    }
                    else if (defaultLocation_whiskType === whiskType) {
                        output_foodData.recommended.location = true;
                    }
                }
            }

            if (angular.isDefined(storing[whiskType])) {
                var lifeInfo = storing[whiskType];
                if (angular.isDefined(lifeInfo.metric) && lifeInfo.metric !== metricVal[4] && lifeInfo.metric !== metricVal[5]) {
                    var calcDays = 1;
                    if (lifeInfo.metric === metricVal[0]) {        //Days
                        calcDays = lifeInfo.min;
                    }
                    else if (lifeInfo.metric === metricVal[1]) {        //Weeks
                        calcDays = lifeInfo.min * 7;
                    }
                    else if (lifeInfo.metric === metricVal[2]) {        //Months
                        calcDays = lifeInfo.min * 30;
                    }
                    else if (lifeInfo.metric === metricVal[3]) {        //Years
                        calcDays = lifeInfo.min * 365;
                    }
                    console.log("updateFoodLifeByFoodDB::: recommended.expiration = true", lifeInfo.min + lifeInfo.metric, calcDays);
                    output_foodData.recommended.expiration = true;
                    output_foodData.expirationDate = setExpirationDate(entryDate, calcDays);
                }
            }
        }

        return output_foodData;
    }
    /** End AutoComplete func to Search for Food Data **/


    /** Start Graph picker wheel **/
    // $scope.$on('ngRepeatFinished_Graph', function () {
    //     console.log('ngRepeatFinished_Graph');
    //     // SwipeService.bind('bind_swipe', FoodGraphService);
    // });

    $scope.MAX_DISPLAY_DAYS = 1000;
    // $scope.getGraphValues = function () {
    //     return new Array($scope.MAX_DISPLAY_DAYS + 1);
    // };

    $scope.down_expirationDate = function () {
        console.log("down_expirationDate1");
        var n_days = N_days_left($scope.inputData.expirationDate);
        if (n_days > 1 && n_days <= $scope.MAX_DISPLAY_DAYS) {
            console.log("down_expirationDate2::: [before]expirationDate = ", $scope.inputData.expirationDate);
            $scope.inputData.expirationDate = setExpirationDate($scope.inputData.expirationDate, -1);
            console.log("down_expirationDate3::: [after]expirationDate = ", $scope.inputData.expirationDate);

            $scope.inputData.isAllowRecommend = {
                location: false,
                expiration: false
            };

            $scope.inputData.recommended = {
                location: false,
                expiration: false
            };
        }
    };

    $scope.up_expirationDate = function () {
        console.log("up_expirationDate1");
        var n_days = N_days_left($scope.inputData.expirationDate);
        if (n_days > 0 && n_days < $scope.MAX_DISPLAY_DAYS - 1) {
            console.log("up_expirationDate2::: [before]expirationDate = ", $scope.inputData.expirationDate);
            $scope.inputData.expirationDate = setExpirationDate($scope.inputData.expirationDate, +1);
            console.log("up_expirationDate3::: [after]expirationDate = ", $scope.inputData.expirationDate);

            $scope.inputData.isAllowRecommend = {
                location: false,
                expiration: false
            };

            $scope.inputData.recommended = {
                location: false,
                expiration: false
            }
        }
    };
    /** End Graph picker wheel **/
}

var counter = 0; //incremental value to create unique id's for each dragged tag
var dragCheck = false; // Flag to check drag state
var scaleFactor = 1; // used for tag position calculation when glaze area is zoomed.
var positionX = 0; // X position to create canvas same as pinch zoom.
var positionY = 0; // Y position to create canvas same as pinch zoom.
var positionXToCreateCrop = 0; // X position to create canvas same as pinch zoom.
var positionYToCreateCrop = 0; // Y position to create canvas same as pinch zoom.

app.directive('dragableBtnsReady', function ($compile, $timeout) {
    return {
        restrict: 'A',
        link: function (scope, element) {
            //This wil help making sure that the droppable area works with a sacled container.
            $.ui.ddmanager.prepareOffsets = function (t, event) {
                var i, j, m = $.ui.ddmanager.droppables[t.options.scope] || [], type = event ? event.type : null, list = ( t.currentItem || t.element ).find(":data(ui-droppable)").addBack();
                droppablesLoop: for (i = 0; i < m.length; i++) {
                    if (m[i].options.disabled || ( t && !m[i].accept.call(m[i].element[0], ( t.currentItem || t.element )) )) {
                        continue;
                    }
                    for (j = 0; j < list.length; j++) {
                        if (list[j] === m[i].element[0]) {
                            m[i].proportions().height = 0;
                            continue droppablesLoop;
                        }
                    }
                    m[i].visible = m[i].element.css("display") !== "none";
                    if (!m[i].visible) {
                        continue;
                    }
                    if (type === "mousedown") {
                        m[i]._activate.call(m[i], event);
                    }
                    m[i].offset = m[i].element.offset();
                    m[i].proportions({
                        width: (m[i].element[0].id !== 'delete_tag_div') ? m[i].element[0].offsetWidth * scaleFactor : m[i].element[0].offsetWidth,
                        height: (m[i].element[0].id !== 'delete_tag_div') ? m[i].element[0].offsetHeight * scaleFactor : m[i].element[0].offsetHeight
                    });
                }
            };
            //Make element draggable
            $(element).draggable({
                containment: 'frame-container',                
                delay: 400,                
                refreshPositions: true,
                revert: 'invalid',                
                start: function (event, ui) {
                    scope.showConsoleLogs &&  console.log('DRAG START');
                    if (!scope.enableDrag) {//disable drag till the graze image response is loaded and tags are loaded
                        event.preventDefault();
                        event.stopPropagation();
                        return;
                    }
                    event.stopImmediatePropagation();// avoide parent click function
                    // On drag start set tha flag to true
                    dragCheck = true;
                    scope.dragCheckProcessed = dragCheck;
                    
                    /*To correct position of helper on scaled container*/
                    if (scaleFactor > 1) {
                        ui.position.left = 0;
                        ui.position.top = 0;
                        var element = $(this);
                    }
                },
                stop: function (event, ui) {
                    // On stop of dragging reset the flag back to false
                    scope.showConsoleLogs &&  console.log('DRAG STOP');
                    dragCheck = false;
                    scope.dragCheckProcessed = dragCheck;
                },
                drag: function (event, ui) {
                    //TODO - for future use
                    scope.showConsoleLogs &&  console.log('DRAGGING');
                    var tagType = scope.getTagType(ui.helper);
                    var tagSize = scope.mobileTagSize(tagType);
                    if (ui.position.left + tagSize.width < 0 || ui.position.top + tagSize.height < 0) {
                        scope.showConsoleLogs &&  console.log('REVERT CALLED');
                        $(this).draggable('option', 'revert', 'invalid');
                        return false;
                    }
                    /*To correct position of helper on scaled container*/
                    if (scaleFactor > 1) {
                        var changeLeft = ui.position.left - ui.originalPosition.left,
                        newLeft = ui.originalPosition.left + changeLeft / scaleFactor,
                        changeTop = ui.position.top - ui.originalPosition.top,
                        newTop = ui.originalPosition.top + changeTop / scaleFactor;
                        ui.position.left = newLeft;
                        ui.position.top = newTop;
                    }
                }
            });
            
            $(element).on('touchstart', function(){
                $(this).addClass('tag_press');
                // scope.dragCheckProcessed = true;	//Commented to prevent true case, when user try pinchZoom in iOS.
            }).on('touchend', function(){
                $(this).removeClass('tag_press');
                $(this).trigger('mouseup');
            });
        }
    };
});

app.directive('dropableReady', function ($compile, $timeout) {
    return {
        restrict: 'A',
        link: function (scope, element) {
            //Make element droppable
            scope.showConsoleLogs &&  console.log("DROPPABLE>>>>>>>>>>" + element);
            $(element).droppable({
                hoverClass: "drop-hover",
                drop: function (ev, ui) {
                    var leftPos = (ui.offset.left - ($(this).offset().left) > 0) ? ((ui.offset.left - $(this).offset().left) / scaleFactor) : 0;
                    var topPos = (ui.offset.top - ($(this).offset().top) > 0) ? ((ui.offset.top - $(this).offset().top) / scaleFactor) : 0;

                    scope.showConsoleLogs &&  console.log("DROP",leftPos,  topPos,  $(ui.helper).width(), $(ui.helper).height());

                    var tagType = scope.getTagType(ui.helper);
                    var tagSize = scope.mobileTagSize(tagType);
                    if(leftPos + tagSize.width > $(this).width()) {
                        leftPos = Math.abs($(this).width() - tagSize.width);
                    }

                    if(topPos + tagSize.height > $(this).height()) {
                        topPos = Math.abs($(this).height() - tagSize.height);
                    }
                    if (leftPos + tagSize.width < 0 || topPos + tagSize.height < 0) {
                        //$(this).trigger('mouseup');
                        scope.showConsoleLogs &&  console.log('DROP REVERT CALLED');
                        ui.draggable.draggable('option', 'revert', 'invalid');
                        return false;
                    }
                    if (ui.helper.hasClass('dropped')) {//check if drop is already called, just in case to prevent drop called multiple times due to overlapping dropables
                        scope.showConsoleLogs &&  console.log('DROP dropped');
                        return false;
                    }
                    
                    ui.helper.addClass("dropped");//add ummy class just to prevent multiple drop calls                    
                    var element = $(ui.draggable).clone();
                    console.log(element, element.attr('memo-id'), element.attr('id').split('_')[2]);
                    element.css({"position": 'absolute', "left": leftPos, "top": topPos, "transform-origin": "center center"});
                    
                    // var content = $compile(element)(scope);
                    // $(this).append(content);
                    // $(ui.draggable).remove();
                    element.removeClass('ui-draggable-dragging');
                    //send sync tag position payload
                    var itemId = element.attr('id').split('_')[2];
		            var selectedTag = $("#tag_element_" + itemId);
		            if (angular.isDefined(selectedTag))
                    	selectedTag.css({"position": 'absolute', "left": leftPos, "top": topPos, "transform-origin": "center center"});

                    console.log(element, itemId);
                    // CALL EDIT TAG AND SEND REQUEST FOR EDIT FOODITEM [GAURAV]
                    scope.moveTag(itemId);
                    $timeout(function(){
                        $("#tag_element_"+itemId).removeClass('tag_press');
                        $("#tag_element_"+itemId).removeClass('dropped');
                       // scope.loadingBar = true;
                    }, 0);
                }
            });
        }
    };
});

// pressableElement: pressable-element
app.directive('pressableElement', function ($timeout) {
    return {
        restrict: 'A',
        link: function ($scope, $elm, $attrs) {
            $elm.bind('mousedown', function (evt) {
                $scope.longPress = true;
                $scope.click = true;

                // onLongPress: on-long-press
                $timeout(function () {
                    $scope.click = false;
                    if ($scope.longPress && $attrs.onLongPress) {
                        $scope.$apply(function () {
                            $scope.$eval($attrs.onLongPress, {$event: evt});
                        });
                    }
                }, $attrs.timeOut || 600); // timeOut: time-out

                // onTouch: on-touch
                if ($attrs.onTouch) {
                    $scope.$apply(function () {
                        $scope.$eval($attrs.onTouch, {$event: evt});
                    });
                }
            });

            $elm.bind('mouseup', function (evt) {
                $scope.longPress = false;

                // onTouchEnd: on-touch-end
                if ($attrs.onTouchEnd) {
                    $scope.$apply(function () {
                        $scope.$eval($attrs.onTouchEnd, {$event: evt});
                    });
                }

                // onClick: on-click
                if ($scope.click && $attrs.onClick) {
                    $scope.$apply(function () {
                        $scope.$eval($attrs.onClick, {$event: evt});
                    });
                }
            });
        }
    };
});

app.directive('onFinishRender', function () {
    return {
        restrict: 'A',
        link: function (scope, element, attr) {
            if (scope.$last === true) {
                element.ready(function () {
                    console.log("calling onFinishRender");
                    scope.$emit('ngRenderFinished');
                });
            }
        }
    }
});

// app.directive('endRepeatGraph', ['$timeout', function ($timeout) {
//     return {
//         restrict: 'A',
//         link: function (scope, element, attr) {
//             if (scope.$last === true) {
//                 $timeout(function () {
//                     scope.$emit('ngRepeatFinished_Graph');
//                 });
//             }
//         }
//     }
// }]);