(function() {
  var app;

  app = angular.module("ngQuickDate", []);

  app.provider("ngQuickDateDefaults", function() {
    return {
      options: {
        dateFormat: 'M/d/yyyy',
        timeFormat: 'h:mm a',
        labelFormat: null,
        placeholder: 'Click to Set Date',
        hoverText: null,
        buttonIconHtml: null,
        closeButtonHtml: '&times;',
        nextLinkHtml: 'Next &rarr;',
        prevLinkHtml: '&larr; Prev',
        disableTimepicker: false,
        disableClearButton: false,
        defaultTime: null,
        dayAbbreviations: ["sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"],
        dateFilter: null,
        parseDateFunction: function(str) {
          var seconds;
          seconds = Date.parse(str);
          if (isNaN(seconds)) {
            return null;
          } else {
            return new Date(seconds);
          }
        }
      },
      $get: function() {
        return this.options;
      },
      set: function(keyOrHash, value) {
        var k, v, _results;
        if (typeof keyOrHash === 'object') {
          _results = [];
          for (k in keyOrHash) {
            v = keyOrHash[k];
            _results.push(this.options[k] = v);
          }
          return _results;
        } else {
          return this.options[keyOrHash] = value;
        }
      }
    };
  });

  app.directive("quickDatepicker", [
    'ngQuickDateDefaults', '$filter', '$sce', '$timeout', 'SHPService', 'REQUEST', function(ngQuickDateDefaults, $filter, $sce, $timeout, SHPService, REQUEST) {
      return {
        restrict: "E",
        require: "?ngModel",
//        scope: {
//          dateFilter: '=?',
//          onChange: "&",
//          onCancel : "&",
//          required: '@'
//        },
        scope : false,
        replace: true,
        link: function(scope, element, attrs, ngModelCtrl) {
          var dateToString, datepickerClicked, datesAreEqual, datesAreEqualToMinute, debounce, getDaysInMonth, initialize, parseDateString, refreshView, setCalendarDate, setConfigOptions, setInputFieldValues, setupCalendarView, stringToDate;
          initialize = function() {
            var templateDate;
            setConfigOptions();
//            scope.toggleCalendar(false);
            scope.weeks = [];
            scope.inputDate = null;
            scope.inputTime = null;
            scope.invalid = true;
            scope.selected = {year: null};
            // scope.selected.year = null;
//            scope.selectedDate = null;
            scope.selectedDate = -1;
            scope.prevDisable = false;
            scope.nextDisable = false;
            scope.years = [];
            scope.min_year = 1900;
            scope.max_year = 2100;
            scope.showYearList = false;
            if(scope.entryDatePopup) {
              setCalendarDate(scope.inputData.entryDate);
              ngModelCtrl.$setViewValue(scope.inputData.entryDate);
            }
            else if(scope.expirationPopup) {
              scope.recommended = {
                  location: false,
                  expiration: false
              };
              scope.isAllowRecommend_exp = false;
              scope.entryDate = scope.inputData.entryDate;
              setCalendarDate(scope.inputData.expirationDate);
              ngModelCtrl.$setViewValue(scope.inputData.expirationDate);
            }
            else {
              setCalendarDate();
              ngModelCtrl.$setViewValue(new Date());
            }

            if (!scope.defaultTime) {
              templateDate = new Date();
              scope.datePlaceholder = $filter('date')(templateDate, scope.dateFormat);
              scope.timePlaceholder = $filter('date')(templateDate, scope.timeFormat);
            }

            scope.selected.year = ngModelCtrl.$modelValue.getFullYear().toString();

            for (var i = scope.min_year; i <= scope.max_year; i++) {
              scope.years.push(i.toString());
            }

            return refreshView();
          };

          scope.reinit = function(){
            var curDate = new Date();
            $timeout(function(){
                scope.selected.year = curDate.getFullYear();
            }, 0);

            ngModelCtrl.$setViewValue(curDate);
          };
          scope.isDisabledYear = function (year) {
            console.log("isDisabledYear");
            var today = new Date();
            if(scope.entryDatePopup)
              return year > today.getFullYear();
            else if(scope.expirationPopup)
              return year < today.getFullYear();
          };
          scope.filterDisabledDate = function (d) {
            var today = new Date();
            if(scope.entryDatePopup)
              return d.getTime() > today.getTime();
            else if(scope.expirationPopup)
              return d.getTime() <= today.getTime();
          };
          setConfigOptions = function() {
            var key, value;
            for (key in ngQuickDateDefaults) {
              value = ngQuickDateDefaults[key];
              if (key.match(/[Hh]tml/)) {
                scope[key] = $sce.trustAsHtml(ngQuickDateDefaults[key] || "");
              } else if (!scope[key] && attrs[key]) {
                scope[key] = attrs[key];
              } else if (!scope[key]) {
                scope[key] = ngQuickDateDefaults[key];
              }
            }

            scope.labelFormat = (LANG.toUpperCase() === "KO-KR" || LANG.toUpperCase() === "ZH-CN")?'MMMM d yyyy':'MMMM d. yyyy';

            if (attrs.iconClass && attrs.iconClass.length) {
              return scope.buttonIconHtml = $sce.trustAsHtml("<i ng-show='iconClass' class='" + attrs.iconClass + "'></i>");
            }
          };

          refreshView = function() {
            var date;
            date = ngModelCtrl.$modelValue ? parseDateString(ngModelCtrl.$modelValue) : new Date();
            setupCalendarView();
            var dateSplit = $filter('date')(date, scope.labelFormat).toString().split(" ");
            if(LANG.toUpperCase() === "KO-KR" || LANG.toUpperCase() === "ZH-CN"){
                scope.mainButtonStr = dateSplit[2] + scope.translation.WEBMOB_fhub2_common_year + " " + scope.translation['WEBMOB_fhub2_common_' + dateSplit[0].toLowerCase()] + " " + dateSplit[1] + scope.translation.WEBMOB_fhub2_common_date_day;
            }else{
                dateSplit[0] = scope.translation['WEBMOB_fhub2_common_' + dateSplit[0].toLowerCase()];//Applying localization for Month text
                scope.mainButtonStr = dateSplit.join(" ");
            }
            return scope.invalid = ngModelCtrl.$invalid;
          };
          setInputFieldValues = function(val) {
            if (val != null) {
              scope.inputDate = $filter('date')(val, scope.dateFormat);
              return scope.inputTime = $filter('date')(val, scope.timeFormat);
            } else {
              scope.inputDate = null;
              return scope.inputTime = null;
            }
          };
          setCalendarDate = function(val) {
            var d;
            if (val == null) {
              val = null;
            }
            d = val != null ? new Date(val) : new Date();
            if (d.toString() === "Invalid Date") {
              d = new Date();
            }
            d.setDate(1);
            return scope.calendarDate = new Date(d);
          };
          setupCalendarView = function() {
            var curDate, d, day, daysInMonth, numRows, offset, row, selected, time, today, weeks, _i, _j, _ref;
            offset = scope.calendarDate.getDay();

            daysInMonth = getDaysInMonth(scope.calendarDate.getFullYear(), scope.calendarDate.getMonth());
            numRows = Math.ceil((offset + daysInMonth) / 7);
            weeks = [];
            curDate = new Date(scope.calendarDate);
            curDate.setDate(curDate.getDate() + (offset * -1));
            for (row = _i = 0, _ref = numRows - 1; 0 <= _ref ? _i <= _ref : _i >= _ref; row = 0 <= _ref ? ++_i : --_i) {
              weeks.push([]);
              for (day = _j = 0; _j <= 6; day = ++_j) {
                d = new Date(curDate);
                if (scope.defaultTime) {
                  time = scope.defaultTime.split(':');
                  d.setHours(time[0] || 0);
                  d.setMinutes(time[1] || 0);
                  d.setSeconds(time[2] || 0);
                }
                selected = ngModelCtrl.$modelValue && d && datesAreEqual(d, ngModelCtrl.$modelValue);
                // today = datesAreEqual(d, new Date());

                if ((selected === undefined) || selected) {
                  scope.selectedDate = d.getDate();
                }

                // if (selected === undefined) {
                //   selected = today;
                // }

                weeks[row].push({
                    date: d,
                    selected: selected,
                    // disabled: typeof scope.dateFilter === 'function' ? !scope.dateFilter(d) : false,
                    disabled: scope.filterDisabledDate(d),
                    other: d.getMonth() !== scope.calendarDate.getMonth(),
                    today: datesAreEqual(d, new Date())
                });
                curDate.setDate(curDate.getDate() + 1);
              }
            }

            return scope.weeks = weeks;
          };
          ngModelCtrl.$parsers.push(function(viewVal) {
            if (scope.required && (viewVal == null)) {
              ngModelCtrl.$setValidity('required', false);
              return null;
            } else if (angular.isDate(viewVal)) {
              ngModelCtrl.$setValidity('required', true);
              return viewVal;
            } else if (angular.isString(viewVal)) {
              ngModelCtrl.$setValidity('required', true);
              return scope.parseDateFunction(viewVal);
            } else {
              return null;
            }
          });
          ngModelCtrl.$formatters.push(function(modelVal) {
            if (angular.isDate(modelVal)) {
              return modelVal;
            } else if (angular.isString(modelVal)) {
              return scope.parseDateFunction(modelVal);
            } else {
              return void 0;
            }
          });
          dateToString = function(date, format) {
            return $filter('date')(date, format);
          };
          stringToDate = function(date) {
            if (typeof date === 'string') {
              return parseDateString(date);
            } else {
              return date;
            }
          };
          parseDateString = ngQuickDateDefaults.parseDateFunction;
          datesAreEqual = function(d1, d2, compareTimes) {
            if (compareTimes == null) {
              compareTimes = false;
            }
            if (compareTimes) {
              return (d1 - d2) === 0;
            } else {
              d1 = stringToDate(d1);
              d2 = stringToDate(d2);
              return d1 && d2 && (d1.getYear() === d2.getYear()) && (d1.getMonth() === d2.getMonth()) && (d1.getDate() === d2.getDate());
            }
          };
          datesAreEqualToMinute = function(d1, d2) {
            if (!(d1 && d2)) {
              return false;
            }
            return parseInt(d1.getTime() / 60000) === parseInt(d2.getTime() / 60000);
          };
          getDaysInMonth = function(year, month) {
            return [31, ((year % 4 === 0 && year % 100 !== 0) || year % 400 === 0 ? 29 : 28), 31, 30, 31, 30, 31, 31, 30, 31, 30, 31][month];
          };
          debounce = function(func, wait) {
            var args, context, later, result, timeout, timestamp;
            timeout = args = context = timestamp = result = null;
            later = function() {
              var last;
              last = +new Date() - timestamp;
              if (last < wait && last > 0) {
                return timeout = setTimeout(later, wait - last);
              } else {
                return timeout = null;
              }
            };
            return function() {
              context = this;
              args = arguments;
              timestamp = +new Date();
              if (!timeout) {
                timeout = setTimeout(later, wait);
                result = func.apply(context, args);
                context = args = null;
              }
              return result;
            };
          };
          ngModelCtrl.$render = function() {
            setCalendarDate(ngModelCtrl.$viewValue);
            return refreshView();
          };
          ngModelCtrl.$viewChangeListeners.unshift(function() {
            setCalendarDate(ngModelCtrl.$viewValue);
            refreshView();
            if (scope.onChange) {
              return scope.onChange();
            }
          });
          scope.$watch('calendarShown', function(newVal, oldVal) {
            var dateInput;
            if (newVal) {
              dateInput = angular.element(element[0].querySelector(".quickdate-date-input"))[0];
              return dateInput.select();
            }
          });
          scope.toggleCalendar = debounce(function(show) {
//            if (isFinite(show)) {
//              return scope.calendarShown = show;
//            } else {
//              return scope.calendarShown = !scope.calendarShown;
//            }
          }, 150);
          scope.pigDate = function(date, closeCalendar) {
            console.log("pigDate");
            return true;
          };
          scope.setPickDate = function(date, closeCalendar) {
            console.log("setPickDate", date);
            if (closeCalendar === null) {
              closeCalendar = true;
            }

            if (typeof scope.dateFilter === 'function' && !scope.dateFilter(date)) {
              return false;
            }

            if (angular.isDefined(date) || date !== null)
              ngModelCtrl.$setViewValue(date);

            if (scope.expirationPopup) {
                scope.recommended.location = false;
                scope.recommended.expiration = false;
                scope.isAllowRecommend_exp = false;
            }

            if (closeCalendar) {
              scope.toggleCalendar(false);
            }
            return true;
          };
          scope.isPickDate = function (d) {
            return datesAreEqual(ngModelCtrl.$modelValue, d, null);
          };
          scope.isActivePeriod = function (d) {
            if (!datesAreEqual(scope.entryDate, d, false) && !datesAreEqual(ngModelCtrl.$modelValue, d, false))
              return (scope.entryDate < d && d < ngModelCtrl.$modelValue);
            return false;
          };
          scope.selectDateFromInput = function(closeCalendar) {
            var err, tmpDate, tmpDateAndTime, tmpTime;
            if (closeCalendar === null) {
              closeCalendar = false;
            }
            try {
              tmpDate = parseDateString(scope.inputDate);
              if (!tmpDate) {
                throw 'Invalid Date';
              }
              if (!scope.disableTimepicker && scope.inputTime && scope.inputTime.length && tmpDate) {
                tmpTime = scope.disableTimepicker ? '00:00:00' : scope.inputTime;
                tmpDateAndTime = parseDateString("" + scope.inputDate + " " + tmpTime);
                if (!tmpDateAndTime) {
                  throw 'Invalid Time';
                }
                tmpDate = tmpDateAndTime;
              }
              if (!datesAreEqualToMinute(ngModelCtrl.$viewValue, tmpDate)) {
                if (!scope.setPickDate(tmpDate, false)) {
                  throw 'Invalid Date';
                }
              }
              if (closeCalendar) {
                scope.toggleCalendar(false);
              }
              scope.inputDateErr = false;
              return scope.inputTimeErr = false;
            } catch (_error) {
              err = _error;
              if (err === 'Invalid Date') {
                return scope.inputDateErr = true;
              } else if (err === 'Invalid Time') {
                return scope.inputTimeErr = true;
              }
            }
          };
          scope.onDateInputTab = function() {
            if (scope.disableTimepicker) {
              scope.toggleCalendar(false);
            }
            return true;
          };
          scope.onTimeInputTab = function() {
            scope.toggleCalendar(false);
            return true;
          };
          scope.nextMonth = function() {
            var d = new Date(new Date(scope.calendarDate).setMonth(scope.calendarDate.getMonth() + 1));
            d.setDate(scope.selectedDate);
            console.log(d.getMonth() + " / " + new Date().getMonth());

            if (d.getFullYear() <= scope.max_year) {
                scope.selected.year = d.getFullYear();
                ngModelCtrl.$modelValue = d;
                setCalendarDate(d);
                return refreshView();
            }
          };
          scope.isNextDisable = function () {
            if(scope.calendarDate.getFullYear() ===  scope.max_year && scope.calendarDate.getMonth() === 11) {
              return true;
            }
            return false;
          };
          scope.isPrevDisable = function () {
            if(scope.calendarDate.getFullYear() ===  scope.min_year && scope.calendarDate.getMonth() === 0) {
              return true;
            }
            return false;
          };
          scope.prevMonth = function() {
            var d = new Date(new Date(scope.calendarDate).setMonth(scope.calendarDate.getMonth() - 1));
            d.setDate(scope.selectedDate);
            console.log(d.getMonth() + " / " + new Date().getMonth());

            if (d.getFullYear() >=  scope.min_year) {
                scope.selected.year = d.getFullYear();
                ngModelCtrl.$modelValue = d;
                setCalendarDate(d);
                return refreshView();
            }
          };
           scope.openYearList = function() {
              scope.showYearList = !scope.showYearList;
          };
          scope.setSelectedYear = function(year) {
              scope.selected.year = year;
              scope.showYearList = false;
              var d = new Date(new Date(ngModelCtrl.$modelValue).setFullYear(parseInt(scope.selected.year)));
              d.setDate(scope.selectedDate);
              ngModelCtrl.$modelValue = d;
              setCalendarDate(d);
              scope.hide_mdSelect();
              return refreshView();
          };
          scope.clear = function() {
            return scope.setPickDate(null, true);
          };
          scope.close = function() {
              scope.cancelDate();
          };
          scope.savePickedDate = function() {
            var d = new Date(scope.calendarDate);
            d.setDate(scope.selectedDate);
            // if(datesAreEqual(d, new Date())) {
            //     SHPService.sendNativeCommand(REQUEST.SHOW_TOAST, scope.translation.WEBMOB_device_ref_food_reminder_error_calrendar_today, REQUEST.SHOW_TOAST);
            //     return;
            // }
            console.log("ngModelCtrl.$viewValue: " + ngModelCtrl.$viewValue + " / scope.selectedDate: "+ scope.selectedDate);
            if (scope.entryDatePopup) {
              scope.inputData.entryDate = ngModelCtrl.$viewValue;
            }
            else if (scope.expirationPopup) {
              if (scope.diffDays(scope.entryDate, scope.inputData.entryDate) !== 0)
                  scope.inputData.entryDate = scope.entryDate;
              scope.inputData.expirationDate = ngModelCtrl.$viewValue;
              scope.inputData.calculateDays = scope.diffDays(new Date(), scope.inputData.expirationDate);
              scope.inputData.recommended = {
                  location: scope.recommended.location,
                  expiration: scope.recommended.expiration
              };

              if (scope.isAllowRecommend_exp) {
                  scope.inputData.isAllowRecommend.expiration = true;
              }
              else {
                  scope.inputData.isAllowRecommend = {
                      location: false,
                      expiration: false
                  };
              }
            }

            scope.datepick = d;
            scope.setPickDate(ngModelCtrl.$viewValue);
            scope.cancelDate();

          };
          scope.isDisplayToday = function (day) {
              var d = new Date(day);
              d.setDate(day);
              if(datesAreEqual(d, new Date(), null)) {
                  console.log(day);
                  return true;
              }
              return false;
          };
          scope.showChangeEntryDateBtn = function () {
            if (scope.editFoodItem) {
              return !datesAreEqual(new Date, scope.entryDate);
            }
            return false;
          };
          scope.changeEntryDateToToday = function () {
              scope.entryDate = new Date();
              var foodData = scope.updateFoodLifeByFoodDB(scope.entryDate);
              console.log("changeEntryDateToToday:: entryDate = ", scope.entryDate, "foodData = ", foodData);

              if (foodData.recommended.expiration) {
                  scope.setPickDate(foodData.expirationDate, false);
                  scope.recommended.location = foodData.recommended.location;
                  scope.recommended.expiration = foodData.recommended.expiration;
                  scope.isAllowRecommend_exp = true;
                  SHPService.sendNativeCommand(REQUEST.SHOW_TOAST,
                      scope.translation.WEBMOB_fhub2_food_reminder_changed_expiration_date_by_recommend
                          .replace('#num#', scope.diffDays(new Date(), ngModelCtrl.$viewValue))
                          .replace('#string1#', scope.convertLocationIdToValue(scope.inputData.storageType)),
                      REQUEST.SHOW_TOAST);
              }
              else {
                  scope.setPickDate(scope.getDefaultExpirationDate(), false);
                  SHPService.sendNativeCommand(REQUEST.SHOW_TOAST,
                      scope.translation.WEBMOB_fhub2_food_reminder_changed_expiration_date_by_default
                          .replace('#num#', scope.DEFAULT_DAYS),
                      REQUEST.SHOW_TOAST);
              }
          };
          scope.isEntryDate = function (day) {
              return datesAreEqual(day, scope.entryDate);
          };

          return initialize();
        },
        templateUrl: 'REF_ViewInside/views/ng-quick-date.html'
      };
    }
  ]);

  app.directive('ngEnter', function() {
    return function(scope, element, attr) {
      return element.bind('keydown keypress', function(e) {
        if (e.which === 13) {
          scope.$apply(attr.ngEnter);
          return e.preventDefault();
        }
      });
    };
  });

  app.directive('onTab', function() {
    return {
      restrict: 'A',
      link: function(scope, element, attr) {
        return element.bind('keydown keypress', function(e) {
          if ((e.which === 9) && !e.shiftKey) {
            return scope.$apply(attr.onTab);
          }
        });
      }
    };
  });

}).call(this);
