/**
 * Created by jji.yeon on 2017-06-15.
 */
app.factory('imageCache', ['$cacheFactory', function ($cacheFactory) {
    return $cacheFactory('img-cache');
}]);

app.service('imageRequest', ['$http', '$timeout', 'imageCache', 'SHPService', 'REQUEST', function ($http, $timeout, imageCache, SHPService, REQUEST) {
    var BASE64 = 'data:image/png;base64,';

    function _arrayBufferToBase64(buffer) {
        var binary = '';
        var bytes = new Uint8Array(buffer);
        var len = bytes.byteLength;
        for (var i = 0; i < len; i++) {
            binary += String.fromCharCode(bytes[i]);
        }
        return window.btoa(binary);
    }

    function resizeImage(imgURL, src) {
        try {
            var image = new Image();   // Create new img element
            image.addEventListener('load', function () {
                /** Start Resize image **/
                console.log("***[Real]imageSize***","width: " + image.width, ", height: " + image.height);
                var resizeRatio = 0.5;
                var resizeCanvas = document.createElement('canvas');
                var resizeCanvas_context = resizeCanvas.getContext('2d');

                /// set size proportional to image
                if (image.width >= 400 || image.height >= 400) {
                    resizeCanvas.width = image.width * resizeRatio;
                    resizeCanvas.height = image.height * resizeRatio;
                }
                else {
                    resizeCanvas.width = image.width;
                    resizeCanvas.height = image.height;
                }
                resizeCanvas_context.drawImage(image, 0, 0, resizeCanvas.width, resizeCanvas.height);
                /** End Resize image **/

                if(angular.isDefined(imageCache)) {
                    console.log("put into imageCache::: id = ", imgURL);
                    $timeout(function () {
                        imageCache.put(imgURL, resizeCanvas.toDataURL());
                    }, 0);
                }

            }, false);
            image.src = src;
        } catch (e) {
            console.log('resizeImage(Canvas Error) = ' + e);
        }
    }


    function resizeGlazeImage(imgURL, src, glazeInfo, glaze_idx) {
        var scope = angular.element(document.getElementById("bodyTag")).scope();

        try {
            var image = new Image();   // Create new img element
            image.addEventListener('load', function() {
                console.log("*****************Starts resizeGlazeImage*****************");
                /** Start Resize image **/
                var resizeRatio = 0.5;
                var resizeCanvas = document.createElement('canvas');
                var resizeCanvas_context = resizeCanvas.getContext('2d');

                /// set size proportional to image
                resizeCanvas.width = image.width * resizeRatio;
                resizeCanvas.height = image.height * resizeRatio;
                resizeCanvas_context.drawImage(image, 0, 0, resizeCanvas.width, resizeCanvas.height);
                /** End Resize image **/

                /** Start Draw Image(rotate, cut) **/
                var canvas = document.createElement('canvas');
                canvas.setAttribute('width', resizeCanvas.width.toString());
                canvas.setAttribute('height', resizeCanvas.height.toString());

                var canvas_context = canvas.getContext('2d');
                canvas_context.width = resizeCanvas.width;
                canvas_context.height = resizeCanvas.height;

                //Calculate draw size by proportion
                var startX = 0, startY = 0, rotate = 0;
                var width = resizeCanvas.width, height = resizeCanvas.height;
                if (angular.isDefined(glazeInfo.drawSize) && angular.isDefined(glazeInfo.drawSize[glaze_idx])) {
                    var drawSize = glazeInfo.drawSize[glaze_idx];
                    if (angular.isDefined(drawSize.startX))
                        startX = parseFloat((drawSize.startX * resizeRatio).toFixed(2));

                    if (angular.isDefined(drawSize.startY))
                        startY = parseFloat((drawSize.startY * resizeRatio).toFixed(2));

                    if (angular.isDefined(drawSize.width))
                        width = parseFloat((drawSize.width * resizeRatio).toFixed(2));

                    if (angular.isDefined(drawSize.height))
                        height = parseFloat((drawSize.height * resizeRatio).toFixed(2));

                    if (angular.isDefined(drawSize.rotate))
                        rotate = drawSize.rotate;
                }


                if (rotate === 90 || rotate === 270) {
                    canvas_context.save();

                    // translate to center-canvas
                    // the origin [0,0] is now center-canvas
                    canvas_context.translate(canvas_context.width/2, canvas_context.height/2);

                    // roate the canvas by degree
                    canvas_context.rotate(rotate * Math.PI / 180);

                    //re-calculate x position
                    startX = parseFloat((resizeCanvas.height - (startX + width)).toFixed(2));

                    //check max image size
                    if ((startX + width) > resizeCanvas.height)
                        width = Math.abs(resizeCanvas.height - startX);

                    if ((startY + height) > resizeCanvas.width)
                        height = Math.abs(resizeCanvas.width - startY);

                    // draw the image
                    // since images draw from top-left offset the draw by 1/2 width & height after rotate canvas
                    canvas_context.drawImage(resizeCanvas, startY, startX, height, width, -canvas_context.height/2, -canvas_context.width/2, canvas_context.height, canvas_context.width);
                    // canvas_context.drawImage(image, 0, 0, image.width, image.height, -canvas_context.height/2, -canvas_context.width/2, canvas_context.height, canvas_context.width);
                    canvas_context.restore();

                }
                else {
                    //check max image size
                    if ((startX + width) > resizeCanvas.width)
                        width = Math.abs(resizeCanvas.width - startX);

                    if ((startY + height) > resizeCanvas.height)
                        height = Math.abs(resizeCanvas.height - startY);

                    // draw the image
                    canvas_context.drawImage(resizeCanvas, startX, startY, width, height, 0, 0, canvas_context.width, canvas_context.height);
                }
                /** End Draw Image(rotate, cut) **/

                if(angular.isDefined(imageCache)) {
                    console.log("put into imageCache::: id = ", imgURL, 'REF_GLAZE_' + glaze_idx);
                    $timeout(function () {
                        imageCache.put('REF_GLAZE_' + glaze_idx, canvas.toDataURL());
                    }, 0);
                }

                console.log("***[Real]call_image***",'path = ' + glazeInfo["GlazeURL"][glaze_idx]);
                console.log("***[Real]imageSize***","width: " + image.width, ", height: " + image.height);
                console.log("***resizeCanvas***","width: " + resizeCanvas.width, ", height: " + resizeCanvas.height);
                console.log("***canvasSize***","width: " + canvas_context.width, ", height: " + canvas_context.height);
                console.log("***drawSize***","startX = ",startX," / startY = ",startY," / width = ", width, " / height = ", height," / rotate: " + rotate);

                $timeout(function () {
                    scope.renderGlazeImage();
                }, 0);

                console.log("[Time]resizeGlazeImage::: GlazeInfo_"+glaze_idx ,new Date());
                console.log("*****************Ends resizeGlazeImage*****************");
            }, false);
            image.src = src;
        } catch (e) {
            console.log('resizeGlazeImage(Canvas Error) = ' + e);
        }
    }

    this.cutCircleImage = function(imgURL, src) {
        try {
            var image = new Image();   // Create new img element
            image.addEventListener('load', function() {
                /** Start Draw Image(circle) **/
                var canvas = document.createElement('canvas');
                canvas.setAttribute('width', image.width.toString());
                canvas.setAttribute('height', image.height.toString());

                var canvas_context = canvas.getContext('2d');
                canvas_context.width = image.width;
                canvas_context.height = image.height;

                //Calculate draw size by proportion
                var width = image.width, height = image.height;
                var radius = width/2;

                canvas_context.save();
                canvas_context.beginPath();
                canvas_context.arc(width/2, height/2, radius, 0, Math.PI*2, true);
                canvas_context.closePath();
                canvas_context.clip();

                // draw the image
                canvas_context.drawImage(image, 0, 0, width, height);
                /** End Draw Image(circle) **/

                if(angular.isDefined(imageCache)) {
                    console.log("put into imageCache::: id = ", imgURL);
                    $timeout(function () {
                        imageCache.put(imgURL, canvas.toDataURL());
                    }, 0);
                }
            }, false);
            image.src = src;
        } catch (e) {
            console.log('cutCircleImage(Canvas Error) = ' + e);
        }
    };


    this.httpRequest = function (item, request) {
        // console.log("imageURL_Request:::"+JSON.stringify(item));
        if (request !== null) {
            $http(request).then(
                function successCallback(response) {
                    // this callback will be called asynchronously
                    // when the response is available
                    console.log("successCallback::::", response);
                    var base64_data = _arrayBufferToBase64(response.data);
                    resizeImage(item.imageURL, BASE64 + base64_data);

                }, function errorCallback(response) {
                    // called asynchronously if an error occurs
                    // or server returns response with an error status.
                    console.log("errorCallback::::" + JSON.stringify(response), response);
                    // resizeImage(item.imageURL, "");
                });
        }
    };
    
    this.nativeRequest = function (imageURL, isGlaze, glaze_idx) {
        var payload = {"imageURL" : []};
        if (angular.isDefined(imageURL) && typeof(imageURL) === 'object') {        //such as ['URL_1', 'URL_2', 'URL_3'...]
            imageURL.forEach(function (url) {
                if (url !== "")
                    payload.imageURL.push(url);
            });
        }
        else if (angular.isDefined(imageURL) && typeof(imageURL) === 'string') {
            if (imageURL !== "")
                payload.imageURL.push(imageURL);
        }

        if (payload.imageURL.length !== 0) {
            if (angular.isDefined(isGlaze) && isGlaze && angular.isDefined(glaze_idx))
                SHPService.sendNativeCommand(REQUEST.GET_IMAGE_LIST, payload, REQUEST.GET_IMAGE_LIST + '_glaze_' + glaze_idx, false);
            else
                SHPService.sendNativeCommand(REQUEST.GET_IMAGE_LIST, payload, REQUEST.GET_IMAGE_LIST, false);
        }
    };

    this.parseNativeResponse = function (device, glazeInfo, glaze_idx) {
        console.log('parseNativeResponse:::', device);
        if (angular.isDefined(device['ImageList'])) {
            device['ImageList'].forEach(function (image) {
                if(angular.isDefined(image['imageURL']) && angular.isDefined(image['imageData']) && image['imageData'] !== "") {
                    if (angular.isUndefined(glazeInfo) && angular.isUndefined(glaze_idx))
                        resizeImage(image['imageURL'], image['imageData']);
                    else
                        resizeGlazeImage(image['imageURL'], image['imageData'], glazeInfo, glaze_idx);
                }
            });
        }
    }


}]);