/**
 * Created by jji.yeon on 2017-11-22.
 */
app.service('profileService', ['$timeout', 'imageCache', 'imageRequest', function ($timeout, imageCache, imageRequest) {

    var scope = angular.element(document.getElementById("bodyTag")).scope();
    var colorList = ["red", "orange", "yellow", "green", "blue", "indigo", "violet", "grey"];
    var profileData = [];

    this.getProfileData = getProfileData;
    this.parseProfileData = parseProfileData;
    this.getScreenName = getScreenName;
    this.getAvatar = getAvatar;
    this.getProfileColor = getProfileColor;
    this.getProfileImage = getProfileImage;

    function getProfileData(user_id) {
        if (angular.isDefined(user_id) && user_id !== '') {
            if (user_id === 'all')
                return profileData;

            for (var i = 0; i < profileData.length; i++) {
                if (profileData[i].userId === user_id) {
                    return profileData[i];
                }
            }
        }

        return undefined;
    }

    function parseProfileData(data) {
        profileData = [];
        if (angular.isUndefined(data))
            return;
        data["profileList"].forEach(function (value) {
            var profile = {
                familyId: value.familyId,
                screenName: scope.decodeUnescape(value.screenName),
                profileColor: colorList[value.profileColor - 1],
                photo: value.photo,
                avatar: value.avatar,
                userId: value.userId
            };
            profileData.push(profile);
            imageRequest.nativeRequest(profile.photo);
        });
        console.log("parseProfileData::: ", profileData);
    }

    function getScreenName(user_id) {
        if (angular.isDefined(user_id) && user_id !== '') {
            var index = profileData.map(function (profile) {
                return profile.userId;
            }).indexOf(user_id);
            if (index !== -1)
                return profileData[index].screenName;
        }
        return "";
    }

    function getAvatar(user_id) {
        if (angular.isDefined(user_id) && user_id !== '') {
            for (var i = 0; i < profileData.length; i++) {
                if (profileData[i].userId === user_id) {
                    return profileData[i].avatar;
                }
            }
        }
        return "";
    }

    function getProfileColor(user_id) {
        if (angular.isDefined(user_id) && user_id !== '') {
            for (var i = 0; i < profileData.length; i++) {
                if (profileData[i].userId === user_id) {
                    return profileData[i].profileColor;
                }
            }
        }
        return "";
    }

    function getImage (key) {
        if (angular.isDefined(imageCache) && angular.isDefined(imageCache.get(key))) {
            return imageCache.get(key);
        }
        return "";
    }

    function getProfileImage(user_id) {
        if (angular.isDefined(user_id) && user_id !== '') {
            var index = profileData.map(function (profile) {
                return profile.userId;
            }).indexOf(user_id);
            if (index !== -1) {
                if (profileData[index].photo !== "" && getImage(profileData[index].photo) !== "") {
                    return getImage(profileData[index].photo);
                }
                else {
                    /**
                     * [avatar]
                     * character: 1~8, family: 9~16, animal: 17~24
                     * [color]
                     * 1: red, 2: orange, 3: yellow, 4: green, 5: blue, 6: indigo, 7: violet, (8: grey - deleted)
                     */
                    var avatarIdx = parseInt(getAvatar(user_id));
                    var colorName = getProfileColor(user_id);

                    if (avatarIdx <= 8) {
                        return 'common/img/Character_ic/character_ic_0' + avatarIdx + '_' + colorName + '.png';
                    }
                    else if (avatarIdx >= 9 && avatarIdx <= 16) {
                        if (avatarIdx === 9)
                            return 'common/img/Character_ic/family/family_ic_0' + avatarIdx + '_' + colorName + '.png';
                        else
                            return 'common/img/Character_ic/family/family_ic_' + avatarIdx + '_' + colorName + '.png';
                    }
                    else if (avatarIdx >= 17 && avatarIdx <= 24) {
                        return 'common/img/Character_ic/animal/animal_ic_' + avatarIdx + '_' + colorName + '.png';
                    }
                    else    //Unknown
                        return 'common/img/Character_ic/character_ic_unkown.png';
                }
            }
        }
        return '';
    }

}]);