app.service("SwipeService", ['$swipe', function ($swipe) {
    var scope = angular.element(document.getElementById("bodyTag")).scope();
    var activeIndex = 0;
    var startPos = null, endPos = null;
    var cWidth = $('body').width(), cHeight = $('body').height();

    this.getActiveIndex = getActiveIndex;
    this.onSwipeLeft = onSwipeLeft;
    this.onSwipeRight = onSwipeRight;
    this.slideTo = slideTo;
    this.updateTransition = updateTransition;
    this.setSwiper_lists_width = setSwiper_lists_width;

    /**
     * @function
     * @name setActiveIndex
     * @param index is means page index.
     * @description  index begins at 0.
     **/
    function setActiveIndex(index) {
        activeIndex = index;
    }

    /**
     * @function
     * @name getActiveIndex
     * @description  activeIndex begins at 0.
     **/
    function getActiveIndex() {
        return activeIndex;
    }

    function getWidth(index) {
        if (angular.isDefined(index) && index !== null && index !== "")
            return $('#swipe_'+index).width();
        else {
            var baseFontSize = parseFloat(document.getElementById("htmlTag").style.fontSize.replace('px', ''));
            return (1000 / 80) * baseFontSize;     //492
        }
    }

    function onSwipeLeft () {
        var pageIndex = getActiveIndex();
        if (pageIndex !== -1 && pageIndex < scope.getSwipeListLength() - 1) {
            slideTo(pageIndex + 1);
        } else {
            slideTo(pageIndex);
        }
    }

    function onSwipeRight () {
        var pageIndex = getActiveIndex();
        if (pageIndex !== -1 && pageIndex > 0) {
            slideTo(pageIndex - 1);
        } else {
            slideTo(pageIndex);
        }
    }

    function slideTo (pageIndex, isAnimation) {
        if (angular.isUndefined(isAnimation))
            isAnimation = true;

        console.log('slideTo::: pageIndex = ', pageIndex);
        if (pageIndex !== -1 && pageIndex >= 0 && pageIndex < scope.getSwipeListLength()) {
            setActiveIndex(pageIndex);
            scope.$emit('refreshSwiperIndex');
            updateTransition(0, isAnimation);
        }
    }

    function updateTransition (moveToIndex, distance, isAnimation) {
        var moveToIndex = getActiveIndex();
        setSwiper_lists_width(scope.getSwipeListLength());

        if (angular.isDefined(isAnimation) && isAnimation)
            $("#swiper_area").addClass('swiper_transition');
        else
            $("#swiper_area").removeClass('swiper_transition');

        if (angular.isUndefined(distance))
            distance = 0;
        var translateX = (moveToIndex * - getWidth(getActiveIndex())) + distance;
        // console.log("[SwipeService]updateTransition:::moveToIndex = ", moveToIndex, " distance = ", distance, " isAnimation = "+ isAnimation);
        $("#swiper_area").css("transform","translateX(" + translateX + "px)");
    }

    function setSwiper_lists_width(length) {
        var swiper_container_width = $(".swiper_container").width();
        if (angular.isDefined(length) && angular.isDefined(swiper_container_width)) {
            $(".swiper_lists_div").width(swiper_container_width * length);
        }
    }


    /**
     * @function
     * @name startHandler
     * @param receive as a parameter a coordinates object of the form { x: 150, y: 310 } and the raw event.
     * @description  startHandler is called on either mousedown, touchstart or pointerdown.
     *                After this event, $swipe is watching for touchmove, mousemove or pointermove events.
     *                These events are ignored until the total distance moved in either dimension exceeds a small threshold.
     **/
    this.startHandler = function (coordinates, raw) {
        console.log("[SwipeService]startHandler:::coordinates = ", coordinates, " / raw.originalEvent = ", raw.originalEvent, " / raw.type = ", raw.type);
        console.log("[SwipeService]startHandler:::startPos = ", startPos, " / endPos = ",endPos);
        if (angular.isDefined(raw.type) && raw.type !== "touchstart")
            return;
        startPos = coordinates;
        endPos = null;
    };

    /**
     * @function
     * @name moveHandler
     * @param receive as a parameter a coordinates object of the form { x: 150, y: 310 } and the raw event.
     * @description  moveHandler is called on mousemove, touchmove and pointermove after the above logic has determined that a swipe is in progress.
     * Once this threshold is exceeded, either the horizontal or vertical delta is greater.
     *    If the horizontal distance is greater, this is a swipe and move and end events follow.
     *    If the vertical distance is greater, this is a scroll, and we let the browser take over. A cancel event is sent.
     **/
    this.moveHandler = function (coordinates, raw) {
        // console.log("**********[SwipeService]moveHandler:::coordinates = ", coordinates, " / raw.originalEvent = ", raw.originalEvent, " / raw.type = ", raw.type);
        // console.log("**********[SwipeService]moveHandler:::startPos = ", startPos, " / endPos = ",endPos);
        if (startPos === null)
            return;
        if (angular.isDefined(raw.type) && raw.type !== "touchmove")
            return;
        if (endPos === null)
            endPos = coordinates;

        var pageIndex = getActiveIndex();
        var horizontal_distance = Math.abs(startPos.x - coordinates.x);
        var vertical_distance = Math.abs(startPos.y - coordinates.y);

        if (Math.abs(horizontal_distance) > Math.abs(vertical_distance)) {
            // console.log("^^^^^^[SwipeService]moveHandler:::pageIndex = ", pageIndex, "horizontal_distance = ", horizontal_distance);
            if (startPos.x > coordinates.x) {
                //drag left
                // console.log("^^^^^^[SwipeService]moveHandler:::~~move Left~~");
                updateTransition(pageIndex, -horizontal_distance, false);

            }
            else if (startPos.x < coordinates.x) {
                //drag right
                // console.log("^^^^^^[SwipeService]moveHandler:::~~move Right~~");
                updateTransition(pageIndex, horizontal_distance, false);
            }
        }

        endPos = coordinates;
    };

    /**
     * @function
     * @name endHandler
     * @param receive as a parameter a coordinates object of the form { x: 150, y: 310 } and the raw event.
     * @description  endHandler is called when a swipe is successfully completed with a touchend, mouseup or pointerup.
     * Once this threshold is exceeded, either the horizontal or vertical delta is greater.
     *    If the horizontal distance is greater, this is a swipe and move and end events follow.
     *    If the vertical distance is greater, this is a scroll, and we let the browser take over. A cancel event is sent.
     **/
    this.endHandler = function (coordinates, raw) {
        console.log("[SwipeService]endHandler:::coordinates = ", coordinates, " / raw.originalEvent = ", raw.originalEvent, " / raw.type = ", raw.type);
        console.log("[SwipeService]endHandler:::startPos = ", startPos, " / endPos = ",endPos);
        if (startPos === null || endPos === null)
            return;

        endPos = coordinates;

        var horizontal_distance = Math.abs(startPos.x - endPos.x);
        var vertical_distance = Math.abs(startPos.y - endPos.y);
        var width = getWidth(getActiveIndex());

        if (horizontal_distance > vertical_distance) {
            console.log("[SwipeService]endHandler:::move Page");
            if (startPos.x > endPos.x && horizontal_distance >= width / 2 ||
                startPos.x > endPos.x && horizontal_distance >= 100) {
                //drag left
                onSwipeLeft();
            }
            else if (startPos.x < endPos.x && horizontal_distance >= width / 2 ||
                startPos.x < endPos.x && horizontal_distance >= 100) {
                //drag right
                onSwipeRight();
            }
            else {
                slideTo(getActiveIndex());
            }
        }

        startPos = null;
        endPos = null;
    };

    /**
     * @function
     * @name cancelHandler
     * @param receives the raw event as its single parameter.
     * @description  cancelHandler is called either on a touchcancel or pointercancel from the browser, or when we begin scrolling as described above.
     **/
    this.cancelHandler = function (raw) {
        console.log("[SwipeService]cancelHandler:::raw.originalEvent = ", raw.originalEvent, " / raw.type = ", raw.type);
        console.log("[SwipeService]cancelHandler:::startPos = ", startPos, " / endPos = ",endPos);
        slideTo(getActiveIndex());
        startPos = null;
        endPos = null;
    };

    /**
     * @function
     * @name bind
     * @param receives the elementId as its single parameter.
     * @description  bind is called to bind swipe event handler.
     **/
    this.bind = function (elementId) {
        var element = $('#' + elementId);
        if (angular.isDefined(element) && element !== null) {
            $swipe.bind(element, {'start': this.startHandler});
            $swipe.bind(element, {'move': this.moveHandler});
            $swipe.bind(element, {'end': this.endHandler});
            $swipe.bind(element, {'cancel': this.cancelHandler});
        }
    };

}]);